//  AEdit 0.5.1 -:-  (C)opyright 2000-2001 Kristian Van Der Vliet
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <interface/Window.h>
#include <interface/Menu.h>
#include <interface/MenuItem.h>
#include <storage/FilePanel.h>
#include <interface/Alert.h>
#include <interface/Font.h>
#include <interface/TextView.h>
#include <app/Application.h>

#include <app/Message.h>

#include <fstream.h>
#include <strings.h>
#include <stdio.h>

#include "messages.h"
#include "aedit.h"
#include "buttonbar.h"

const char* DEFAULT_FONT_FAMILY = "Lucida Sans Typewriter";
const char* DEFAULT_FONT_STYLE = "Regular";
const float DEFAULT_FONT_SIZE = 8;

using namespace os;


AEditWindow::AEditWindow(const CRect& cFrame) : CWindow(cFrame, "AEdit V0.5.1")
{
	// Create a menu bar

	sMenuFrame = Bounds();
	sMenuFrame.bottom=MENU_OFFSET;

	CMenu* pcMenuBar = new CMenu(sMenuFrame, "main_menu", CF_FOLLOW_LEFT | CF_FOLLOW_RIGHT, WID_FULL_UPDATE_ON_H_RESIZE, ITEMS_IN_ROW);

	// Create the menus within the bar

	CMenu* pcFileMenu = new CMenu(CRect(0,0,1,1),"File", ITEMS_IN_COLUMN);
	pcFileMenu->AddItem("New", new CMessage( ID_FILE_NEW));
	pcFileMenu->AddItem("Open", new CMessage( ID_FILE_LOAD));
	pcFileMenu->AddItem("Save", new CMessage( ID_FILE_SAVE));
	pcFileMenu->AddItem("Save As...", new CMessage( ID_FILE_SAVE_AS));
	pcFileMenu->AddItem(new CSeparatorItem());
	pcFileMenu->AddItem("Exit", new CMessage (ID_EXIT));

	pcMenuBar->AddItem(pcFileMenu);

	CMenu* pcEditMenu = new CMenu(CRect(0,0,1,1), "Edit", ITEMS_IN_COLUMN);
	pcEditMenu->AddItem("Cut", new CMessage( ID_EDIT_CUT));
	pcEditMenu->AddItem("Copy", new CMessage( ID_EDIT_COPY));
	pcEditMenu->AddItem("Paste", new CMessage( ID_EDIT_PASTE));
	pcEditMenu->AddItem(new CSeparatorItem());
	pcEditMenu->AddItem("Select All", new CMessage(ID_EDIT_SELECT_ALL));

	pcMenuBar->AddItem(pcEditMenu);

	// Create the Find menu

	CMenu* pcFindMenu = new CMenu(CRect(0,0,1,1), "Find", ITEMS_IN_COLUMN);
	pcFindMenu->AddItem("Find", new CMessage (ID_FIND_FIND));
	pcFindMenu->AddItem("Replace", new CMessage (ID_FIND_REPLACE));
	pcFindMenu->AddItem(new CSeparatorItem());
	pcFindMenu->AddItem("Goto Line", new CMessage(ID_FIND_GOTO));

	pcMenuBar->AddItem(pcFindMenu);

	// Create Font Menu

	CMenu* pFontMenu = new CMenu(CRect(0,0,1,1), "Font", ITEMS_IN_COLUMN);

	// Add Size Submenu

	CMenu* pSizeMenu = new CMenu(CRect(0,0,1,1), "Size", ITEMS_IN_COLUMN);
	float sz,del=1.0;
	CMessage *pSizeMsg;
	char szLbl[8];

	for (sz=5.0; sz < 30.0; sz += del)
	{
		pSizeMsg = new CMessage(ID_FONT_SIZE);
		pSizeMsg->AddFloat("size",sz);
		sprintf(szLbl,"%.1f",sz);
		pSizeMenu->AddItem(szLbl,pSizeMsg);

		if (sz == 10.0)
			del = 2.0;
		else if (sz == 16.0)
			del = 4.0;
   }

	pFontMenu->AddItem(pSizeMenu);
	pFontMenu->AddItem(new CSeparatorItem());

	// Add Family and Style Menu

	int fc = CFont::GetFamilyCount();
	int sc,i=0,j=0;
	uint32 nFlags;
	char zFamily[FONT_FAMILY_LENGTH],zStyle[FONT_STYLE_LENGTH];
	CMenu *pFamilyMenu;
	CMessage *pFontMsg;

	for (i=0; i<fc; i++)
	{
		if (CFont::GetFamilyInfo(i,zFamily) == 0)
		{
			sc = CFont::GetStyleCount(zFamily);
			pFamilyMenu = new CMenu(CRect(0,0,1,1),zFamily,ITEMS_IN_COLUMN);

			for (j=0; j<sc; j++)
			{
				if (CFont::GetStyleInfo(zFamily,j,zStyle,&nFlags) == 0)
				{
					pFontMsg = new CMessage(ID_FONT_FAMILY_AND_STYLE);
					pFontMsg->AddString("family",zFamily);
					pFontMsg->AddString("style",zStyle);
					pFamilyMenu->AddItem(zStyle,pFontMsg);
				}
			}

		pFontMenu->AddItem(pFamilyMenu);
		}
	}

	pcMenuBar->AddItem(pFontMenu);   

   // Add the Help menu

	CMenu* pcHelpMenu = new CMenu(CRect(0,0,1,1), "Help", ITEMS_IN_COLUMN);
	pcHelpMenu->AddItem("About", new CMessage( ID_HELP_ABOUT));

	pcMenuBar->AddItem(pcHelpMenu);
	pcMenuBar->SetTargetForItems(this);

	// Add the menubar to the window

	AddChild(pcMenuBar);

	// Now we can create the text view

	sWindowBounds = Bounds();
	sWindowBounds.left=0;
	sWindowBounds.top=(MENU_OFFSET+1)+(BUTTON_HEIGHT+2);

	pcTextView = new EditTextView(sWindowBounds,"text_view", "", CF_FOLLOW_ALL, WID_WILL_DRAW);   

	pcTextView->SetMultiLine(true);

	// Change the font

	CFont* pcAppFont = new CFont ( );
	//pcAppFont->SetFamilyAndStyle( "Lucida Sans Typewriter", "Regular" );
	pcAppFont->SetFamilyAndStyle( DEFAULT_FONT_FAMILY, DEFAULT_FONT_STYLE );
	pcAppFont->SetSize( DEFAULT_FONT_SIZE );

	m_cFamily.assign(DEFAULT_FONT_FAMILY);
	m_cStyle.assign(DEFAULT_FONT_STYLE);
	m_vSize = DEFAULT_FONT_SIZE;

	pcTextView->SetFont(pcAppFont);
	pcAppFont->Release();

	// Now add the textview to the window

	AddChild(pcTextView);

	// Create the button bar for the top of the window

	pcButtonBar=new ButtonBar(CRect(0,0,sWindowBounds.right,BUTTON_HEIGHT),"button bar");
	pcButtonBar->MoveBy(0,MENU_OFFSET+1);

	AddChild(pcButtonBar);

	pcTextView->MakeFocus();
	SetSizeLimits(210.0,175.0,4096.0,4096.0);

	// Reset some internal variables

	strcpy(pzStorePath,"");  							// Reset the stored filename
	strcpy(pcWindowTitle,"AEdit V0.5.1 : New File");	// Reset the window title
	strcat(pcWindowTitle,"\0");
	SetTitle(pcWindowTitle);								// Set the window title

	bHasFind=false;	// User has not used the find dialog yet

}

void AEditWindow::DispatchMessage( CMessage* pcMsg, CHandler* pcHandler)
{
   CWindow::DispatchMessage(pcMsg, pcHandler);
}

void AEditWindow::MessageReceived(CMessage* pcMessage)
{
	
	switch(pcMessage->what) //Get the message code from the message
	{
		case ID_FILE_NEW:
		{
			pcTextView->Set("");
			strcpy(pzStorePath,"");				// Reset the stored filename
			strcpy(pcWindowTitle,"AEdit V0.5.1 : New File");
			strcat(pcWindowTitle,"\0");
			SetTitle(pcWindowTitle);				// Reset the window title
			pcTextView->MakeFocus();
			break;
		}

		case ID_FILE_LOAD:
		{
			pcLoadRequester = new CFilePanel(CFilePanel::LOAD_REQ, new CMessenger(this), getenv("$HOME"));
			pcLoadRequester->Show();
			pcLoadRequester->Activate();
			break;
		}

		case M_LOAD_REQUESTED:
		{
			if(pcMessage->FindString( "file/path", &pzFPath) == 0){
				strcpy(pzStorePath,pzFPath);	// Store the file path
				Load((char*)pzFPath);
				strcpy(pcWindowTitle,"AEdit V0.5.1 : ");
				strcat(pcWindowTitle,pzFPath);
				strcat(pcWindowTitle,"\0");
				SetTitle(pcWindowTitle);
				Activate();
				pcTextView->MakeFocus();
			}

			break;
		}

		case ID_FILE_SAVE:
		{
			if(strcmp(pzStorePath,"")){		// If a path has been stored
				Save(pzStorePath);			// Save using the previously stored path
			}
			else{
				pcMessage->what = ID_FILE_SAVE_AS;	// The file hasn't been created yet
				MessageReceived(pcMessage);				// So we'll pop a "Save As" requester instead
			}

			break;
		}

		case ID_FILE_SAVE_AS:
		{
			pcSaveRequester = new CFilePanel(CFilePanel::SAVE_REQ, new CMessenger(this), getenv("$HOME"));
			pcSaveRequester->Show();
			pcSaveRequester->Activate();
			break;
		}

		case M_SAVE_REQUESTED:
		{
			if(pcMessage->FindString( "file/path", &pzFPath) == 0)
			{
				strcpy(pzStorePath,pzFPath);  // Store the file path
				Save(pzFPath);
				strcpy(pcWindowTitle,"AEdit V0.5.1 : ");
				strcat(pcWindowTitle,pzFPath);
				strcat(pcWindowTitle,"\0");
				SetTitle(pcWindowTitle);
				Activate();
				pcTextView->MakeFocus();
			}

			break;
		}


		case M_FILE_REQUESTER_CANCELED:
		{
			Activate();
			pcTextView->MakeFocus();
			break;
		}

		case ID_VOID:  //Do nothing, this is a blank menu item to act as a spacer
			break;
	
		case ID_EXIT:
		{
			OkToQuit();
			break;
		}

		case ID_EDIT_CUT:
		{
			pcTextView->Cut();
			pcTextView->MakeFocus();
			break;
		}

		case ID_EDIT_COPY:
		{
			pcTextView->Copy();
			pcTextView->MakeFocus();
			break;
		}

		case ID_EDIT_PASTE:
		{
			pcTextView->Paste();
			pcTextView->MakeFocus();
			break;
		}

		case ID_EDIT_SELECT_ALL:
		{
			pcTextView->SelectAll();
			break;
		}

		case ID_FIND_FIND:
		{
			pcFindDialog=new FindDialog(CRect(200,200,450,310),this);
			pcFindDialog->Show();
			pcFindDialog->Activate();

			break;
		}

		case ID_FIND_REPLACE:
		{
			pcReplaceDialog=new ReplaceDialog(CRect(200,200,450,310),this);
			pcReplaceDialog->Show();
			pcReplaceDialog->Activate();

			break;
		}

		case ID_FIND_GOTO:
		{
			pcGotoDialog=new GotoDialog(CRect(200,200,450,250),this);
			pcGotoDialog->Show();
			pcGotoDialog->Activate();

			break;
		}

		case ID_BUT_GOTO:
		{

			if(!pcMessage->FindInt32("goto_lineno",&iGotoLineNo,0))
			{
				pcTextView->SetCursor(0,iGotoLineNo,false,false);
			}

			break;
		}

		case ID_HELP_ABOUT:
		{
			ShowAbout();
			break;
		}
	 case ID_FONT_SIZE:
	     {
		float size;

		if (pcMessage->FindFloat("size",&size) == 0) {
		   CFont *font = new CFont();
		   font->SetFamilyAndStyle(m_cFamily.c_str(),m_cStyle.c_str());
		   font->SetSize(size);
		   pcTextView->SetFont(font);
		   font->Release();

		   m_vSize = size;
		}
		break;
	     }
	 case ID_FONT_FAMILY_AND_STYLE:
	     {
		const char *family,*style;
				
		if (pcMessage->FindString("family",&family) == 0 &&
		    pcMessage->FindString("style",&style) == 0) {
		   CFont *font = new CFont();
		   font->SetFamilyAndStyle(family,style);
		   font->SetSize(m_vSize);
		   pcTextView->SetFont(font);
		   font->Release();
		   
		   m_cFamily.assign(family);
		   m_cStyle.assign(style);
		}
		break;
	     }

		case ID_TEST:		// Used for various testing during development, 
							// feel free to use for temporary stuff if you're 
							// hacking this source, but don't use it for 
							// anything permanent!
		{
			break;
		}

		case ID_BUT_FIND:
		{

			if(!pcMessage->FindString("find_text",&sFindText,0) && !pcMessage->FindBool("case_check_state",&bCaseSensitive,0))
			{
				vLines=pcTextView->GetBuffer();
				iTotalLines=vLines.size();

				for(iCurrentLine=0;iCurrentLine<iTotalLines;iCurrentLine++){
					iTextPosition=pcTextView->Find(pcTextView->GetBuffer()[iCurrentLine], sFindText, bCaseSensitive, 0);

					if(iTextPosition!=-1){	// Find() returns -1 if the text isn't found
						pcTextView->SetCursor(iTextPosition,iCurrentLine, false, false);
						pcTextView->SetCursor((iTextPosition+sFindText.size()),iCurrentLine, true, false);

						bHasFind=true;

						break;
					}

				}	// End of for() loop

			}	// End of if()
			
			break;

		}	// End of case block

		case ID_BUT_FIND_NEXT:
		{
			if(bHasFind)
			{
				iTextPosition+=sFindText.size();	// We want to start the search from just past the last instance.
														// There is a lot of fudging in this block of code :)

				for(iCurrentLine;iCurrentLine<iTotalLines;iCurrentLine++)
				{
					iTextPosition=pcTextView->Find(pcTextView->GetBuffer()[iCurrentLine], sFindText, bCaseSensitive, iTextPosition);

					if(iTextPosition!=-1){	// Find() returns -1 if the text isn't found
						pcTextView->SetCursor(iTextPosition,iCurrentLine, false, false);
						pcTextView->SetCursor((iTextPosition+sFindText.size()),iCurrentLine, true, false);

						break;
					}
					else
					{
						iTextPosition=0;		// This is a big fudge (See the call to EditText::Find() above!)
					}

				}	// End of for() loop

			}	// End of if()

			break;
		}

		case ID_BUT_REPLACE:
		{
			if(bHasFind){
				if(!pcMessage->FindString("replace_text",&sReplaceText,0))
				{
					pcTextView->Delete();
					pcTextView->Insert(sReplaceText.c_str(),false);

					MessageReceived(new CMessage(ID_BUT_FIND_NEXT));

				}	// End of if()

			}	// End of outer if()

			break;
		}

		case ID_BUT_FIND_CLOSE:
		{
			// Reset the variables used to indicate the last positions
			// we searched from / found text at.

			bHasFind=false;
			iTextPosition=0;
			iCurrentLine=0;

			// Close the Window

			pcFindDialog->Close();
			pcTextView->MakeFocus();

			break;
		}

		case ID_BUT_REPLACE_CLOSE:
		{
			// Reset the variables used to indicate the last positions
			// we searched from / found text at.

			bHasFind=false;
			iTextPosition=0;
			iCurrentLine=0;

			// Close the Window

			pcReplaceDialog->Close();
			pcTextView->MakeFocus();

			break;
		}

		case ID_BUT_GOTO_CLOSE:
		{
			// Just close the Goto window

			pcGotoDialog->Close();
			pcTextView->MakeFocus();
			break;
		}

		default:
		{
			CWindow::MessageReceived( pcMessage );
			break;
		}

	}	// End of switch block
    
}

bool AEditWindow::OkToQuit( void )
{
	CApplication::c_app->PostMessage(M_QUIT);
	return( true );
}

void AEditWindow::LoadAtStart(char* cFilename)
{
	strcpy(pzStorePath,cFilename);	// Store the file path
	Load(cFilename);
	strcpy(pcWindowTitle,"AEdit V0.5.1 : ");
	strcat(pcWindowTitle,cFilename);
	strcat(pcWindowTitle,"\0");
	SetTitle(pcWindowTitle);
}

void AEditWindow::ShowAbout()
// Show an about window
{
	CAlert* sAbout = new CAlert("About AEdit 0.5.1", "Aedit 0.5.1\n\nText editor for AtheOS\nWritten by Kristian Van Der Vliet, 2000-2001\n\nAEdit is released under the Gnu Public Licencse (GPL)\nPlease see the file COPYING, distributed with AEdit, for\nmore information\n", "OK");
	sAbout->Go(new CInvoker());
}

void AEditWindow::Load(char* cFileName)
// Load requested file
{
	uint32 iFileSize=0;

	struct stat sFileStat;

	if(!stat(cFileName, &sFileStat))
	{
		if(sFileStat.st_mode & S_IFREG)
		{
			// File is O.K, we can get the size & load it

			iFileSize=sFileStat.st_size;

			char cBuffer [iFileSize+1];
			unsigned long iCharCount;

			FILE *fInFile=fopen(cFileName,"r");

			if(fInFile){
				for(iCharCount=0;iCharCount<=iFileSize;iCharCount++){
					cBuffer [iCharCount]=fgetc(fInFile);
				}
			}

			fclose(fInFile);

			cBuffer[--iFileSize]=0;  // This stops the string running off of the end and producing garbage.

			pcTextView->Clear();
			pcTextView->Set(cBuffer);
		}
		else{
			// Pop error window, file is not a "regular" file

			CAlert* sFileAlert = new CAlert("AEdit", "Unable to open file", "OK");
			sFileAlert->Go();
		}
	}
	else
	{
			// Pop an error window, file does not exist?

			CAlert* sFileAlert = new CAlert("AEdit", "Unable to open file", "OK");
			sFileAlert->Go();
	}

}

void AEditWindow::Save(const char *cFileName)
// Save the text in the textview buffer to a file
{
	static std::string cBuffer;
	TextView::buffer_type vLines;
   
	int iLineNo, iLineTotal=0;

	vLines=pcTextView->GetBuffer();
	iLineTotal=vLines.size();

	std::ofstream foutfile(cFileName);
   
	if(foutfile)
	{
		for(iLineNo=0; iLineNo<iLineTotal; iLineNo++)
		{
			cBuffer=pcTextView->GetBuffer()[iLineNo];
			foutfile << cBuffer << std::endl;
		}
	}
	else
	{
		//Open an error window

		CAlert* sFileAlert = new CAlert("AEdit", "Unable to save file", "OK");
		sFileAlert->Go();
	}

	foutfile.close();

}

