//  AEdit 0.5.1 -:-  (C)opyright 2000-2001 Kristian Van Der Vliet
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "edittext.h"
#include "messages.h"
#include <string>				// C++ STD::strings
#include <string.h>			// C strings
#include <interface/Window.h>
#include <interface/Menu.h>
#include <interface/MenuItem.h>
#include <ctype.h>

using namespace os;


EditTextView::EditTextView (const CRect& cWindowBounds, char* pzTitle, char* pzBuffer, uint32 nResizeMask , uint32 nFlags) : TextView (cWindowBounds, pzTitle, pzBuffer, nResizeMask , nFlags)
{
	// Create a context menu
	// Moved into the constructor in 0.3.1

	sContextMenu=new CMenu(CRect(0,0,10,10),"",ITEMS_IN_COLUMN);
	sContextMenu->AddItem("Cut", new CMessage( ID_EDIT_CUT));
	sContextMenu->AddItem("Copy", new CMessage( ID_EDIT_COPY));
	sContextMenu->AddItem("Paste", new CMessage( ID_EDIT_PASTE));
	sContextMenu->AddItem(new CSeparatorItem());
	sContextMenu->AddItem("Select All", new CMessage( ID_EDIT_SELECT_ALL));
}

void EditTextView::AttachedToWindow(void)
{
	sContextMenu->SetTargetForItems(Window());
}

void EditTextView::MouseDown(CPoint cPosition)
{
/*
	int32 nButtons;

	Window->CurrentMessage()->FindInt32("buttons", &nButtons);
	
	if(nButtons == 2)
	{
		sContextMenu->Open( ConvertToScreen(cPosition) );	// Open the menu where the mouse is
	}
	else
*/
	{
		TextView::MouseDown( cPosition);	// Otherwise, pass the event back to the base class
	}
}

void EditTextView::KeyDown(const char* cString, const char* cRawString, uint32 nQualifiers)
{
	if(nQualifiers & QUAL_CTRL)
	{
		if(!(nQualifiers & QUAL_REPEAT))
		{
			switch(*cString)
			{
				case 24:	// Cut (Ctrl+X)
				{
					Window()->MessageReceived(new CMessage(ID_EDIT_CUT));
					break;
				}

				case 3:	// Copy (Ctrl+C)
				{
					Window()->MessageReceived(new CMessage(ID_EDIT_COPY));
					break;
				}

				case 22:	// Paste (Ctrl+V)
				{
					Window()->MessageReceived(new CMessage(ID_EDIT_PASTE));
					break;
				}

				case 15:	// Open (Ctrl+O)
				{
					Window()->MessageReceived(new CMessage(ID_FILE_LOAD));
					break;
				}

				case 19:	// Save (Ctrl+S)
				{
					Window()->MessageReceived(new CMessage(ID_FILE_SAVE));
					break;
				}

				case 1:	// Save As (Ctrl+A)
				{
					Window()->MessageReceived(new CMessage(ID_FILE_SAVE_AS));
					break;
				}

				case 6:	// Find (Ctrl+F)
				{
					Window()->MessageReceived(new CMessage(ID_FIND_FIND));
					break;
				}

				case 18:	// Replace (Ctrl+R)
				{
					Window()->MessageReceived(new CMessage(ID_FIND_REPLACE));
					break;
				}

				case 7:	// Goto Line (Ctrl+G)
				{
					Window()->MessageReceived(new CMessage(ID_FIND_GOTO));
					break;
				}

				case 17:	// Exit (Ctrl+Q)
				{
					Window()->MessageReceived(new CMessage(ID_EXIT));
					break;
				}

			}	// End of switch() block

		}	// End of if() testing the repeat

	}
	else
	{
		TextView::KeyDown(cString,cRawString,nQualifiers);
	}
}

int EditTextView::Find(std::string sOriginal, std::string sFind, bool bCaseSensitive, int iStartPosition)
{
// bCaseSensitive == True, do a case sensitive search
// bCaseSensitive == False, do a case insensitive search

	int iPosition1, iPosition2=0;

	const char* cOriginal=sOriginal.c_str();
	const char* cFind=sFind.c_str();
	bool bMatched=true;

	iPosition1=iStartPosition;

	if(bCaseSensitive)
	{
		for( ; iPosition1<=sOriginal.size(); iPosition1++)
		{
			if(cOriginal[iPosition1]==cFind[0]){

				bMatched=true;
				for(iPosition2=0;iPosition2<sFind.size();iPosition2++){
					if(cOriginal[iPosition1+iPosition2]!=cFind[iPosition2]){
						bMatched=false;
						break;
					}
				}

				if(bMatched)
					return(iPosition1);

			}	// End of if()
		}	// End of for()
	}	// End of outer if()

	else{	// Do a case insensitive search

		for(;iPosition1<=sOriginal.size();iPosition1++)
		{
			if(tolower(cOriginal[iPosition1]) == tolower(cFind[0]))
			{

				bMatched=true;
				for(iPosition2=0;iPosition2<sFind.size();iPosition2++){
					if(tolower(cOriginal[iPosition1+iPosition2])!=tolower(cFind[iPosition2])){
						bMatched=false;
						break;
					}
				}

				if(bMatched)
					return(iPosition1);

			}	// End of if()
		}	// End of for()

	}	// End of else() block

	return(-1);
}
