//  AEdit 0.5.1 -:-  (C)opyright 2000-2001 Kristian Van Der Vliet
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <string>
#include <layoutview.h>
#include "findlayout.h"

#include "messages.h"

using namespace os;


ReplaceLayoutView::ReplaceLayoutView(const CRect &cFrame) : LayoutView( cFrame, "", NULL, CF_FOLLOW_NONE)
{

	// Make the base Vertical LayoutNode

	pcHLayoutNode= new HLayoutNode("main_layout_node");

	// Create the InputNode

	pcInputNode= new VLayoutNode("input_layout_node");

	pcInputNode->AddChild(new VLayoutSpacer("spacer", 5.0f, 5.0f, pcInputNode, 1.0f));

	pcFindLabel=new CStringView(CRect(0,0,0,0),"find_label","Find",CF_FOLLOW_NONE,WID_WILL_DRAW);
	pcInputNode->AddChild(pcFindLabel);

	pcFindTextView= new TextView(CRect(0,0,0,0), "find_text_view", NULL, CF_FOLLOW_NONE);
	pcInputNode->AddChild(pcFindTextView);

	pcInputNode->AddChild(new VLayoutSpacer("spacer", 5.0f, 5.0f, pcInputNode, 1.0f));

	pcReplaceLabel=new CStringView(CRect(0,0,0,0),"replace_label","Replace with",CF_FOLLOW_NONE,WID_WILL_DRAW);
	pcInputNode->AddChild(pcReplaceLabel);

	pcReplaceTextView= new TextView(CRect(0,0,0,0), "replace_text_view", NULL, CF_FOLLOW_NONE);
	pcInputNode->AddChild(pcReplaceTextView);

	pcInputNode->AddChild(new VLayoutSpacer("spacer", 5.0f, 5.0f, pcInputNode, 1.0f));

	pcCaseCheckBox=new CCheckBox(CRect(0,0,0,0),"case_checkbox", "Case Sensitive Search", new CMessage(ID_VOID), CF_FOLLOW_NONE, WID_WILL_DRAW);
	pcInputNode->AddChild(pcCaseCheckBox);

	pcInputNode->SameWidth("find_label","find_text_view","replace_label","replace_text_view","case_checkbox",NULL);

	// Add a spacer to force the edit box to the top

	pcInputNode->AddChild(new VLayoutSpacer("spacer", 1));

	// Add a spacer on the far left

	pcHLayoutNode->AddChild(new HLayoutSpacer("spacer", 5.0f, 5.0f, pcHLayoutNode, 1.0f));
	pcHLayoutNode->AddChild(pcInputNode);

	// Add a spacer between the Edit box & the Buttons

	pcHLayoutNode->AddChild(new HLayoutSpacer("spacer", 5.0f, 5.0f, pcHLayoutNode, 1.0f));

	// Create the ButtonNode

	pcButtonNode= new VLayoutNode("button_layout_node");

	pcFindButton=new CButton(CRect(0,0,0,0), "find_button", "Find", new CMessage(ID_BUT_FIND), CF_FOLLOW_NONE);
	pcButtonNode->AddChild(pcFindButton);
	pcButtonNode->AddChild(new VLayoutSpacer("spacer", 5.0f, 5.0f, pcButtonNode, 1.0f ) );

	pcNextButton=new CButton(CRect(0,0,0,0), "find_next_button", "Find Next", new CMessage(ID_BUT_FIND_NEXT), CF_FOLLOW_NONE);
	pcButtonNode->AddChild(pcNextButton);
	pcButtonNode->AddChild(new VLayoutSpacer("spacer", 5.0f, 5.0f, pcButtonNode, 1.0f ) );

	pcReplaceButton=new CButton(CRect(0,0,0,0), "replace_button", "Replace", new CMessage(ID_BUT_REPLACE), CF_FOLLOW_NONE);
	pcButtonNode->AddChild(pcReplaceButton);
	pcButtonNode->AddChild(new VLayoutSpacer("spacer", 5.0f, 5.0f, pcButtonNode, 1.0f ) );

	pcCloseButton=new CButton(CRect(0,0,0,0), "close_button", "Close", new CMessage(ID_BUT_REPLACE_CLOSE), CF_FOLLOW_NONE);
	pcButtonNode->AddChild(pcCloseButton);
	pcButtonNode->AddChild(new VLayoutSpacer("spacer", 5.0f, 5.0f, pcButtonNode, 1.0f ) );

	pcButtonNode->SameWidth("find_button","find_next_button","replace_button","close_button", NULL);

	pcHLayoutNode->AddChild(pcButtonNode);

	// Add a spacer on the far right

	pcHLayoutNode->AddChild(new HLayoutSpacer("spacer", 5.0f, 5.0f, pcHLayoutNode, 1.0f));

	// Add the LayoutNodes to the layoutview

	SetRoot(pcHLayoutNode);

	// Set the focus

	pcFindTextView->MakeFocus();

}

std::string ReplaceLayoutView::GetFindText(void)
{
	return(pcFindTextView->GetBuffer()[0]);
}

std::string ReplaceLayoutView::GetReplaceText(void)
{
	return(pcReplaceTextView->GetBuffer()[0]);
}

bool ReplaceLayoutView::GetCaseCheckState(void)
{
	if(pcCaseCheckBox->Value()){
		return(true);
	}

	return(false);
}


