/*

	RLFSearch.cpp

	(c) 1998-99 Robert Lanziner-Furtenbach

*/

#include "RLFDebug.h"
#include "RLFSearch.h"
#include "RLFFields.h"

#include <interface/StringView.h>
#include <Box.h>
#include <interface/Button.h>

const char *FindText		= "Find";
const char *ReplaceText		= "ReplaceWith";

#define Border	10			// border around window-content 
#define X1off	16			// offsets for window relative to calling window 
#define Y1off	16
#define X2off	16
#define	Y2off	16
#define DeltaY	25			// space between elements in y-direction

RLFSearch::RLFSearch (BRect frame, const char *title, const char *text, BHandler *_handler, bool do_search )
	: BWindow ( BRect ( frame.left+X1off, frame.top+Y1off, frame.left+X2off, frame.top+Y2off), "Search",
	B_MODAL_WINDOW, B_NOT_RESIZABLE ), priv_do_search (do_search)
{
	DEBUG_PRINT ( "RLFSearch constructor" );

	BRect r = Bounds();													// bounds of window

	BView *View = new BView ( r, "", B_FOLLOW_ALL_SIDES, B_WILL_DRAW ); // create view within window bounds
	View->SetViewColor ( 216, 216, 216 );								// set viewcolor to gray
	AddChild ( View );													// add view to window
	
	r.Set ( Border, Border, Border+250, Border+14 );					// position for dialog-title
	View->AddChild ( new BStringView ( r, "", title ) );				// create and add title to view

	r.OffsetBy ( 0, DeltaY );											// calculate position for box
	BRect r2 = r;
	r2.bottom = r2.top+1;
	View->AddChild ( new BBox ( r2 ) );									// create and add box to view

// create TextControl for findtext
	r.OffsetBy ( 0, DeltaY/2 );
	View->AddChild ( find = new BTextControl ( r, "", "Find:", text, NULL ) );
	find->SetDivider ( 70 );											// where to begin textfield after description
	find->SetModificationMessage ( new BMessage ( Msg_Find_Change ) );	// send message if changed
	find->MakeFocus();													// put cursor and input to this field

	if ( !priv_do_search ) {											// replace ?
	// create TextControl for replacetext
		r.OffsetBy ( 0, DeltaY );
		View->AddChild ( replacewith = new BTextControl ( r, "", "Replace with:", "", NULL ) );
		replacewith->SetDivider ( 70 );									// where to begin textfield after description
	}

	r2.Set ( 0, 0, 0, 0 );
	float	width;
	float	height;
	float	w;

	if ( priv_do_search ) {												// search ?
		View->AddChild ( search = new BButton ( r2, "", "Search", new BMessage ( Msg_Search ) ) );	// add search-button
		search->GetPreferredSize ( &width, &height );					// get size of button for positioning and window resize
		if ( strlen ( text ) == 0 ) search->SetEnabled ( false );		// disable button
	} else {															// replace ?
		View->AddChild ( replace = new BButton ( r2, "", "Replace", new BMessage ( Msg_Replace ) ) );	// add replace-button 
		replace->GetPreferredSize ( &width, &height );					// get size of button for positioning and window resize
		if ( strlen ( text ) == 0 ) replace->SetEnabled ( false );

		View->AddChild ( replaceall = new BButton ( r2, "", "Replace All", new BMessage ( Msg_ReplaceAll ) ) );	// add replaceall-button
		replaceall->GetPreferredSize ( &w, &height );					// get size of button for positioning and window resize
		if ( strlen ( text ) == 0 ) replaceall->SetEnabled ( false );
		if ( w > width ) width = w;
	}

	View->AddChild ( cancel = new BButton ( r2, "", "Cancel", new BMessage ( Msg_Cancel ) ) );
	cancel->GetPreferredSize ( &w, &height );							// get size of button for window resize
	if ( w > width ) width = w;

	r = find->Frame();													// calulate size and position of the buttons
	r.left = r.right + 2*Border;
	r.top = Border;
	if ( priv_do_search ) {
		search->MoveTo ( r.left, r.top );
		search->ResizeTo ( width, height );
		if ( strlen ( text ) > 0 ) search->MakeDefault ( true );
	} else {
		replace->MoveTo ( r.left, r.top );
		replace->ResizeTo ( width, height );
		if ( strlen ( text ) > 0 ) replace->MakeDefault ( true );
		r.top += height+5;
		replaceall->MoveTo ( r.left, r.top );
		replaceall->ResizeTo ( width, height );
	}
	r.top += height+5;
	cancel->MoveTo ( r.left, r.top );
	cancel->ResizeTo ( width, height );
	if ( strlen ( text ) == 0 ) cancel->MakeDefault ( true );

	r = cancel->Frame();												// calculate needed size of window
	r2 = find->Frame();
	r = r | r2;
	DEBUG_EXEC ( r.PrintToStream() );
	DEBUG_PRINTF (( "W=%.2f H=%.2f", r.Width(), r.Height() ));
	ResizeTo ( r.right+Border, r.bottom+Border );						// and set the new size

	handler = _handler;

	Show();
}

void RLFSearch::MessageReceived ( BMessage *message )
{
	DEBUG_PRINT ( "RLFSearch MessageReceived" );
	DEBUG_EXEC ( message->PrintToStream() );

	switch ( message->what ){
		case Msg_Find_Change:
			if ( priv_do_search ) {
				search->SetEnabled ( true );
				search->MakeDefault ( true );
			} else {
				replace->SetEnabled ( true );
				replaceall->SetEnabled ( true );
				replace->MakeDefault ( true );
			}
			break;
		case Msg_Search :
			DEBUG_PRINT ( "Search" );
			ExtractToMsg ( new BMessage ( Msg_Search ) );
			break;
		case Msg_Replace :
			DEBUG_PRINT ( "Replace" );
			ExtractToMsg ( new BMessage ( Msg_Replace ) );
			break;
		case Msg_ReplaceAll :
			DEBUG_PRINT ( "ReplaceAll" );
			ExtractToMsg ( new BMessage ( Msg_ReplaceAll ) );
			break;
		case Msg_Cancel :
			DEBUG_PRINT ( "Cancel" );
			Close();
			break;
		default:
			BWindow::MessageReceived ( message );
			break;
	}

}

void RLFSearch::DispatchMessage ( BMessage *message, BHandler *handler )
// dispatches all messages the window gets
// if ESCAPE is pressed alter message to Msg_Cancel (with Closes Win)
{
	int8	key;
	if ( message->what == B_KEY_DOWN ) {
		if ( message->FindInt8 ( "byte", 0, &key ) == B_OK ) {
			if ( key == B_ESCAPE ) {
				message->MakeEmpty();
				message->what = Msg_Cancel;
			}
		}
	}
	BWindow::DispatchMessage ( message, handler );	// call inherited func
}

void RLFSearch::ExtractToMsg ( BMessage *message )
// adds FindText (and ReplaceText) to message and sends it to handler
{
	message->AddString ( FindText, find->Text() );
	if (!priv_do_search) {
		message->AddString ( ReplaceText, replacewith->Text() );
	}
	handler->Looper()->PostMessage ( message, handler );
	delete ( message );
	PostMessage ( Msg_Cancel );
}
