/*

	RLFSetting.cpp

	(c) 1998-99 Robert Lanziner-Furtenbach

*/


#include "RLFDebug.h"
#include "RLFSetting.h"
#include <Path.h>
#include <Application.h>
#include <FindDirectory.h>
#include <File.h>

#define Border 10

const char *LineLength		= "LineLength";
const char *FontFamily		= "FontFamily";
const char *FontSize		= "FontSize";

RLFSetting::RLFSetting (BRect frame, BMessage *_ValueMsg, BMessage *_DefaultMsg )
				: BWindow(BRect ( frame.left+16, frame.top+40, frame.left+350+16-1, frame.top+120+40-1), "CodeWalker Settings",
				B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE ),
				ValueMsg ( *_ValueMsg ), DefaultMsg ( *_DefaultMsg )
{
	DEBUG_PRINT ( "RLFSetting constructor" );
	DEBUG_EXEC ( ValueMsg.PrintToStream() );
	DEBUG_EXEC ( DefaultMsg.PrintToStream() );
	BRect r = Bounds();
	
	BView *View = new BView ( r, "", B_FOLLOW_ALL_SIDES, B_WILL_DRAW );
	View->SetViewColor ( 216, 216, 216 );
	AddChild ( View );

	r.left = Border;
	r.top = Border;
	r.right = r.left+150;
	r.bottom = r.top;
	View->AddChild ( linelen = new BTextControl ( r, "", "Line-Length:", "", NULL ) );
	linelen->SetModificationMessage ( new BMessage ( Msg_Length_Changed ) );
	linelen->SetDivider ( 60 );
	linelen->MakeFocus();

	r.top += 25;
	View->AddChild ( FontField = new RLFFontField ( r ) );
	FontField->SetDivider ( 60 );

	r.top += 25;
	View->AddChild ( FontSizeField = new RLFFontSize ( r, 6, 24 ) );
	FontSizeField->SetDivider ( 60 );

	r.Set (0,0,0,0);
	float	width;
	float	height;
	float	w;

	View->AddChild ( apply = new BButton ( r, "", "Apply", new BMessage ( Msg_Apply ) ) );
	apply->ResizeToPreferred();
	apply->GetPreferredSize ( &width, &height );

	View->AddChild ( revert = new BButton ( r, "", "Revert", new BMessage ( Msg_Revert ) ) );
	revert->GetPreferredSize ( &w, &height );
	if ( w > width ) width = w;

	View->AddChild ( Bdefault = new BButton ( r, "", "Default", new BMessage ( Msg_Default ) ) );
	Bdefault->GetPreferredSize ( &w, &height );
	if ( w > width ) width = w;

	View->AddChild ( cancel = new BButton ( r, "", "Cancel", new BMessage ( Msg_Cancel ) ) );
	Bdefault->GetPreferredSize ( &w, &height );
	if ( w > width ) width = w;

	r = Bounds();
	r.left = r.right - width - Border;
	r.top  += Border;
	apply->MoveTo ( r.left, r.top );
	apply->ResizeTo ( width, height );
	apply->MakeDefault ( true );

	r.top += height+5;
	revert->MoveTo ( r.left, r.top );
	revert->ResizeTo ( width, height );

	r.top += height+5;
	Bdefault->MoveTo ( r.left, r.top );
	Bdefault->ResizeTo ( width, height );

	r.top += height+5;
	cancel->MoveTo ( r.left, r.top );
	cancel->ResizeTo ( width, height );

	r = cancel->Frame();
	DEBUG_EXEC ( r.PrintToStream() );
	DEBUG_PRINTF (( "W=%.2f H=%.2f", r.Width(), r.Height() ));
	ResizeTo ( r.right+Border, r.bottom+Border );

	InitByMsg ( &ValueMsg );
	PostMessage ( Msg_Disable_Fields );

	Show();
}

void RLFSetting::DispatchMessage ( BMessage *message, BHandler *handler )
{
	int8	key;
	if ( message->what == B_KEY_DOWN ) {
		if ( message->FindInt8 ( "byte", 0, &key ) == B_OK ) {
			if ( key == B_ESCAPE ) {
				message->MakeEmpty();
				message->what = Msg_Cancel;
			}
		}
	}
	BWindow::DispatchMessage ( message, handler );
}

void RLFSetting::MessageReceived ( BMessage *message )
{
	BMessage *msg;
	DEBUG_PRINT ( "RLFSetting MessageReceived" );
	DEBUG_EXEC ( message->PrintToStream() );

	switch ( message->what ){
		case Msg_Length_Changed :
		case Msg_Size_Changed :
		case Msg_Font_Changed :
			apply->SetEnabled ( true );
			revert->SetEnabled ( true );
			apply->MakeDefault ( true );
			break;
		case Msg_Apply :
			DEBUG_PRINT ( "Apply" );
			msg = new BMessage ( DefaultMsg.what );
			if ( ExtractToMsg ( msg ) ) {
				ValueMsg = *msg;
				be_app->PostMessage ( msg );
				PostMessage ( Msg_Disable_Fields );
			}
			delete msg;
			break;
		case Msg_Revert :
			DEBUG_PRINT ( "Revert" );
			InitByMsg ( &ValueMsg );
			PostMessage ( Msg_Disable_Fields );
			break;
		case Msg_Default :
			DEBUG_PRINT ( "Default" );
			InitByMsg ( &DefaultMsg );
			be_app->PostMessage ( &DefaultMsg );
			PostMessage ( Msg_Disable_Fields );
			break;
		case Msg_Cancel :
			DEBUG_PRINT ( "Cancel" );
			Close();
			break;
		case Msg_Disable_Fields:
			cancel->MakeDefault ( true );
			apply->SetEnabled ( false );
			revert->SetEnabled ( false );
			break;
		default:
			BWindow::MessageReceived ( message );
			break;
	}

}

bool RLFSetting::InitByMsg ( BMessage *message )
{
	char 	s [30];
	const char 	*s2;
	int32	i;
	int32	j;
	font_family family;

	DEBUG_PRINT ( "RLFSetting InitByMsg" );

	if ( message->FindInt32 ( "LineLength", &i ) == B_OK ) {
		sprintf ( s, "%li", i );
		linelen->SetText(s);
	} else {
		DEBUG_PRINT ( "no LineLength");
	}
	if ( message->FindString ( FontFamily, &s2 ) == B_OK ) {
		strcpy ( family, s2 );
		FontField->SetFontFamily ( family );
	} else {
		DEBUG_PRINT ( "no FontFamily" );
	}
	if ( message->FindInt32 ( FontSize, &j ) == B_OK ) {
		FontSizeField->SetFontSize ( (int16)j );
	} else {
		DEBUG_PRINT ( "no FontSize" );
	}
	return (true);
}

bool RLFSetting::ExtractToMsg ( BMessage *message )
{
	int32		i;
	int32		j;
	char		s[30];
	font_family	family;

	DEBUG_PRINT ( "RLFSetting ExtractToMsg" );

	strcpy ( s, linelen->Text() );
	if ( sscanf ( s, "%li", &i ) == 0 ){
		DEBUG_PRINTF (( "unable to convert linelength-str=%s", s ));
		linelen->MakeFocus();
		return (false);
	}
	message->AddInt32 ( LineLength, i );
	FontField->GetFontFamily ( &family );
	message->AddString ( FontFamily, family );
	FontSizeField->GetFontSize ( &j );
	DEBUG_PRINTF (( "FontSize=%li", j ));
	message->AddInt32 ( FontSize, j );

	return (true);
}

void DefaultSettings ( BMessage *message )
{
	font_family	family;
	font_style	style;
	message->what = Msg_Preferences;
	message->AddInt32 ( LineLength, 255 );
	be_fixed_font->GetFamilyAndStyle ( &family, &style );
	message->AddString ( FontFamily, family );
	message->AddInt32 ( FontSize, (int32)be_fixed_font->Size() );
	DEBUG_EXEC ( message->PrintToStream() );
}

void LoadSettings ( BMessage *message ) 							// load the preferences 
{
	BPath		path;
	status_t	status;
	BMessage	local;

	DEBUG_PRINT ( "LoadSettings" );

	DefaultSettings ( message );

	status = find_directory ( B_USER_SETTINGS_DIRECTORY, &path );
	if ( status != B_OK ) { DEBUG_PRINT ( "find_directory not OK" ); return; }
	if ( path.Append ( "CodeWalker" ) != B_NO_ERROR ) { DEBUG_PRINT ("path.Append not OK"); return; }
	BFile file ( path.Path(), B_READ_ONLY );
	if ( file.InitCheck() != B_NO_ERROR ) { DEBUG_PRINT ("file.InitCheck not OK"); return; }
	if ( message->Unflatten ( &file ) != B_OK ) { DEBUG_PRINT ("Unflatten not OK"); }
	DEBUG_EXEC ( message->PrintToStream() );
}

void SaveSettings ( BMessage *message ) 							// save the preferences
{
	BPath		path;
	status_t	status;

	DEBUG_PRINT ( "RLFApp SavePref" );
	status = find_directory ( B_USER_SETTINGS_DIRECTORY, &path );
	if ( status != B_OK ) { DEBUG_PRINT ("find_directory not OK"); return; }
	if ( path.Append ( "CodeWalker" ) != B_NO_ERROR ) { DEBUG_PRINT ("path.Append not OK"); return; }
	BFile file ( path.Path(), B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE );
	if ( file.InitCheck() != B_NO_ERROR ) { DEBUG_PRINT ("file.InitCheck not OK"); return; }
	if ( message->Flatten ( &file ) != B_OK ) { DEBUG_PRINT ("Flatten not OK"); }
	DEBUG_EXEC ( message->PrintToStream() );
}

