/*
	
	RLFTextView.cpp
	
	(c) 1998-99 Robert Lanziner-Furtenbach
	
*/

#include "RLFDebug.h"
#include "RLFTextView.h"
#include "RLFWindow.h"
#include "RLFSetting.h"
#include "RLFSearch.h"

#include <stdlib.h>
#include <string.h>
#include <app/Application.h>
#include <PrintJob.h>
#include <app/Message.h>
#include <storage/File.h>

RLFTextView::RLFTextView(BRect frame, BMessage *ValueMsg, BStringView *StatView, entry_ref *ref ) :
	BTextView ( frame, "RLFView", BRect ( 0, 0, 1000, 10 ),
	B_FOLLOW_ALL_SIDES, B_WILL_DRAW ),
	unchanged (true ),
	StatusView (StatView)
{
	SetDoesUndo( true );
	SetStylable (false );		// no different styles possible
	PrintSettings = NULL;		// clear pointer to settings
	AdjustView ( ValueMsg );	// adjust the view to the values in the message, like FontFamily, FontSize, LineLength
	LoadFile ( ref );			// Load file if possible
	LastSearchString = (char *) malloc (1);
	*LastSearchString = 0x0;
	LastReplaceString = (char *) malloc (1);
	*LastReplaceString = 0x0;
}


void RLFTextView::MessageReceived(BMessage *message)
{
	int32		start;
	int32		finish;
	char		stext [200] = "Find in \"";
	char		rtext [200] = "Find & Replace in \"";

	DEBUG_PRINT ( "RLFTextView" );
	DEBUG_EXEC ( message->PrintToStream() );

 	switch ( message->what ){
   		case B_SIMPLE_DATA:							// was there a drag'n'drop ?
   		case B_REFS_RECEIVED:
			if (message->WasDropped() && message->HasRef("refs")) {
				message->what = B_REFS_RECEIVED;	// overwrite in case it is B_SIMPLE_DATA (dropped)
				be_app->PostMessage(message);		// forward ref-message to Application
   			}
			break;
		case Msg_Preferences:						// changing layout of textview
			AdjustView ( message );				// adjust the view to the values in the message, like FontFamily, FontSize, LineLength
			break;
		case Msg_Search:
			{
				const char *txt;
				message->FindString ( FindText, &txt );
				free ( LastSearchString );
				LastSearchString = strdup ( txt );
				Find ( LastSearchString );
			}
			break;
		case Msg_Replace:
		case Msg_ReplaceAll:
			{
				const char *ftxt;
				const char *rtxt;
				message->FindString ( FindText, &ftxt );
				message->FindString ( ReplaceText, &rtxt );
				free ( LastSearchString );
				LastSearchString = strdup ( ftxt );
				free ( LastReplaceString );
				LastReplaceString = strdup ( rtxt );
				if ( message->what == Msg_Replace )
					Replace ( LastSearchString, LastReplaceString );
				else
					ReplaceAll ( LastSearchString, LastReplaceString );
			}
			break;
		case Edit_Find:
			strcat ( stext, Window()->Title() );
			strcat ( stext, "\"" );
			GetSelection ( &start, &finish );
			if ( start == finish ) 
				new RLFSearch ( Frame(), stext, LastSearchString, this, true );
			else {
				char *txt = (char *) malloc ( finish-start );
				GetText ( start, finish-start, txt );
				new RLFSearch ( Frame(), stext, txt, this, true );
				free ( txt );
			}
			break;
		case Edit_Find_Again:
			Find ( LastSearchString );
			break;
		case Edit_Replace:
			strcat ( rtext, Window()->Title() );
			strcat ( rtext, "\"" );
			GetSelection ( &start, &finish );
			if ( start == finish ) 
				new RLFSearch ( Frame(), rtext, LastSearchString, this, false );
			else {
				char *txt = (char *) malloc ( finish-start );
				GetText ( start, finish-start, txt );
				new RLFSearch ( Frame(), rtext, txt, this, false );
				free ( txt );
			}
			break;
		case Edit_Replace_Same:
			Replace ( LastSearchString, LastReplaceString );
			break;
		case File_PageSetup:
			DEBUG_PRINT ( "PageSetup" );
			PageSetup ( Window()->Title() );					// PageSetup for Print
			break;
		case File_Print:
			DEBUG_PRINT ( "Print" );
			Print ( Window()->Title() );						// Print Text
			break;
   		default:
   			// Call inherited if we didn't handle the message
   			BTextView::MessageReceived(message);
   			break;
	}
}

void RLFTextView::Pulse ( void )
	// pulse is used to show the current linenumber in the statusview
{
	char s [30];
	sprintf ( s, "Line %li", CurrentLine()+1 );				// put current linenumber into s
	StatusView->SetText ( s );								// change text in status to s
	BTextView::Pulse();										// call original pulse
}

void RLFTextView::InsertText ( const char *text, int32 length, int32 offset, const text_run_array *runs )
	// used to log changes
{
	if ( unchanged == true ) {								// if text was original or saved
		unchanged = false;									// change flag
		BMessage msg ( Msg_Save_Enable );					// create message: text has been changed
		Window()->PostMessage ( &msg );						// post the message to parent
	}
	BTextView::InsertText ( text, length, offset, runs );	// call original InsertText
	
}

void RLFTextView::DeleteText ( int32 start, int32 finish )
	// used to log changes
{
	if ( unchanged == true) {								// if text was original or saved
		unchanged = false;									// change flag
		BMessage msg ( Msg_Save_Enable );					// create message: text has been changed
		Window()->PostMessage ( &msg );						// post the message to parent
	}
	BTextView::DeleteText ( start, finish );				// call original DeleteText
}

void RLFTextView::AdjustView ( BMessage *message )
	// adjust the view to the values in the message, like FontFamily, FontSize, LineLength
{
	int32		i;
	int32		j;
	BFont		font;
	const char	*s;

	DEBUG_PRINT ( "RLFTextView AdjustView" );
	DEBUG_EXEC ( message->PrintToStream() );
	
	if ( message->what == Msg_Preferences ) {
			if ( message->FindString ( FontFamily, &s ) == B_OK ) {		// is there a fontfamily in the message
				font.SetFamilyAndStyle ( s, NULL );						// change the fontfamily
			}
			if ( message->FindInt32 ( FontSize, &j ) == B_OK ) {		// is there a fontsize in the message
				font.SetSize ( j );										// change the fontsize
			}
			if ( message->FindInt32 ( LineLength, &i ) == B_OK ) {		// is there a linelength in the message
				SetTextRect ( BRect ( 0, 0, font.StringWidth ( "M" ) * (float)i + 1, 1 ) );
																		// change the text-rectangle to width of "M"*linelength
			}
			SetFontAndColor ( &font, B_FONT_ALL, NULL );	// set new font
	}
}

status_t RLFTextView::PageSetup( const char *DocumentName )
	// PageSetup and Print are based on the article in BNewsletter Volume II Issue 18 "Proper Printing" from Benoit Schillings
{
	status_t	result;
	BPrintJob	printJob ( DocumentName );										// create a new printjob with the Documentname
	if ( PrintSettings != NULL )												// if there are "saved" settings
		printJob.SetSettings ( new BMessage (*PrintSettings) );					// set the "saved" settings
	if ( ( result = printJob.ConfigPage () ) == B_OK ) {						// if calling the config-page-window is ok
		delete PrintSettings;
		PrintSettings = printJob.Settings();									// remember the settings
	}
	return (result);															// return the result of the ConfigPage-call
}

void RLFTextView::Print ( const char *DocumentName )
{
	DEBUG_PRINT ( "RLFTextView Print" );
	BPrintJob	printJob ( DocumentName );										// create a new printjob with the Documentname
	if ( PrintSettings == NULL ) {												// if there are no actual printsettings
		if ( PageSetup ( DocumentName ) != B_NO_ERROR )							// call pagesettings
			return;
	}	// in the original here is an "else", but i think SetSettings has to be called anyways
	
	printJob.SetSettings ( new BMessage (*PrintSettings) );						// set the "saved" settings

	if ( printJob.ConfigJob() == B_OK ) {										// if calling the config-job-window is ok
		delete PrintSettings;
		PrintSettings = printJob.Settings();									// remember the settings
		BRect	PrintRect	= printJob.PrintableRect();							// get printable rectangle on paper
		BRect	DocRect		= TextRect();										// get rectangle from BTextView for printing
		DEBUG_EXEC ( PrintRect.PrintToStream() );
		int32 i = (int32) ( PrintRect.Height() / LineHeight() );							// how many lines fit on paper
		DEBUG_PRINTF ( ( "Lines=%li\n", i ) );
		PrintRect.bottom = ( i * LineHeight() ) + PrintRect.top;				// shrink printable rectangle to fit lines
		DEBUG_EXEC ( PrintRect.PrintToStream() );
		DEBUG_EXEC ( DocRect.PrintToStream() );
		int32	pages		= (int32) (( DocRect.Height() + PrintRect.Height() - 1 ) / PrintRect.Height());
																				// max number of pages to be printed
		int32	firstpage	= printJob.FirstPage ();							// first page to print
		int32	lastpage	= printJob.LastPage ();								// last page to print
		if ( lastpage > pages ) lastpage = pages;								// adjust lastpage to the reality, it could be MAX(int32)
		DEBUG_PRINTF ( ( "Pages=%li\n", pages ) );
		printJob.BeginJob ();													// start the printing
		for ( int32 actpage = firstpage; actpage <= lastpage; actpage++ ) {		// do all wanted pages
			if ( printJob.CanContinue() ) {										// is printing possible ?
				PrintRect.OffsetTo ( 0, (actpage-1) * PrintRect.Height() );		// adjust printable rectangle to the document-part for actual page
				printJob.DrawView ( this, PrintRect, BPoint (0.0, 0.0 ) );		// print the view
				printJob.SpoolPage ();											// this page is done
			} else
				break;															// if printing not possible then end for-loop
		}
		printJob.CommitJob();													// commit, that the spooled paged now have to be printed
	}
}

void RLFTextView::LoadFile ( entry_ref *file_ref )
{
	BDirectory	dir;
	BFile		file;
	if ( file_ref != NULL ) {											// if ref for window
		DEBUG_PRINT ( "RLFTextView LoadFile" );
		if ( dir.SetTo ( file_ref ) != B_OK ) {							// if ref is a directory then ignore ref
			if ( (file.SetTo ( file_ref, B_READ_WRITE ) ) == B_OK ) {	// look if file exists
				off_t length;
				file.GetSize ( &length );								// get file length
				if ( length > 0 ) {										// only if text in file
					SetText ( &file, 0, length );						// set content of file as text
				}
			} else {
				DEBUG_PRINTF (( "Can't open file: %s", file_ref->name ));
			}
		} else {
			DEBUG_PRINTF (( "Can't open directory as file: %s", file_ref->name ));
		}
	}	
}

bool RLFTextView::Find ( const char *txt )
{
	int32	i;
	int32	len;
	int32	start;
	int32	finish;
	int32	ind;
	int32	txtlen = strlen(txt);

	if (txtlen == 0) return (false);
	GetSelection( &start, &finish );
	DEBUG_PRINTF (( "Start=%li Finish=%li", start, finish ));
	len = TextLength();
	DEBUG_PRINTF (( "TextLen=%li", len ));
	i = start;
	ind = 0;
	while (true) {
		i++;
		if ( i == start ) return (false);				// exit if nothing found
		if ( i >= len ) {
			i = ind = 0;								// wrap if end of text
			if ( i == start ) return (false);			// exit if nothing found
		}
		if ( ByteAt(i) == txt[ind] ) {
			ind++;
			if ( ind == txtlen ) {					// jearrr!! found
				Select ( i-ind+1, i+1 );
				ScrollToSelection();
				return (true);
			}
		} else {
			ind = 0;
		}
	}
}

void RLFTextView::Replace ( const char *findtext, const char *replacetext )
{
	int32 start;
	int32 finish;
	int32 txtlen = strlen(replacetext);
	if ( Find ( findtext ) ) {
		GetSelection ( &start, &finish );
		Delete();
		Insert ( replacetext );
		Select ( start, start + txtlen );
	}
}

void RLFTextView::ReplaceAll ( const char *findtext, const char *replacetext )
{
	int32 start;
	int32 finish;
	int32 txtlen = strlen(replacetext);
	while ( Find ( findtext ) ) {
		GetSelection ( &start, &finish );
		Delete();
		Insert ( replacetext );
		Select ( start, start + txtlen );
	}
}
