/*

	RLFWindow.cpp

	(c) 1998-99 Robert Lanziner-Furtenbach
	
*/

#include "RLFDebug.h"
#include "RLFWindow.h"
#include "RLFSearch.h"
#include "RLFSetting.h"

#include <string.h>
#include <malloc.h>

#include <interface/MenuItem.h>
#include <app/Application.h>
#include <ScrollView.h>
#include <storage/Directory.h>
#include <interface/Alert.h>
#include <storage/Path.h>
#include <storage/File.h>

RLFWindow::RLFWindow(BRect frame, entry_ref *_ref, int16 _Nr, BMessage *ValueMsg )
				: BWindow(frame, "", B_DOCUMENT_WINDOW, 0), Nr(_Nr), SavePanel(NULL),
				Messenger (NULL)
{
	BMenu 		*m_file;
	BMenu 		*m_edit;
	BMenu 		*m_extra;
	BMenuItem	*it;
	BRect		r = Bounds();

	DEBUG_PRINT ( "RLFWindow constructor" );

	mb = new BMenuBar ( r, "main menu" );							// create new menubar
	AddChild ( mb );												// add menubar to window
												// needed to find correct positions for views in window
												// views themselfs are needed as targets for messages
												// therefor menuitems are added later

// create textview and add to window
	r.top = mb->Bounds().bottom + 1;								// create border for TextView
	r.right -= B_V_SCROLL_BAR_WIDTH;								// Width without ScrollBar
	r.bottom -= B_H_SCROLL_BAR_HEIGHT;								// Height without ScrollBar

// create a status-line at the bottom of the window left to the horizontal scrollbar
	BRect r2 = r;
	r2.top = r2.bottom+2;
	r2.right = r2.left+100-1;
	r2.bottom += B_H_SCROLL_BAR_HEIGHT;
	BStringView *StatusView = new BStringView ( r2, "rflStatus", "", B_FOLLOW_LEFT | B_FOLLOW_BOTTOM, B_WILL_DRAW );
	StatusView->SetViewColor ( 216, 216, 216 );
//create a line to seperate status from text, done by a View with a changed ViewColor
	r2.top--;
	r2.bottom = r2.top;
	BView *LineView = new BView ( r2, "rlfLine", B_FOLLOW_LEFT | B_FOLLOW_BOTTOM, B_WILL_DRAW );
	LineView->SetViewColor ( 164, 164, 164, 0 );

// create a new textview
	BDirectory	dir;
	file_ref = NULL;												// clear file_ref, so Window untitled
	if ( _ref != NULL ) {											// if ref for window
		if ( dir.SetTo ( _ref ) != B_OK ) {							// if ref is a directory then ignore ref
			file_ref = new entry_ref ( *_ref );						// but if not then take ref even if file does not exist
		}
	}
	rlfTextView = new RLFTextView( r, ValueMsg, StatusView, file_ref );
	
// create a new scrollview for textview
	BScrollView *scrView = new BScrollView ( "ScrollView", rlfTextView, B_FOLLOW_ALL_SIDES, 0, true, true );
	BScrollBar *scrBar = scrView->ScrollBar ( B_HORIZONTAL );		// get pointer on horizontal scrollbar
	scrBar->ResizeBy ( -100, 0 );									// shrink size of scrollbar
	scrBar->MoveBy ( 100, 0 );										// move scrollbar to the right
	AddChild ( scrView );											// add scrollview to window

	AddChild ( StatusView );										// add Status and Line after ScrollView
	AddChild ( LineView );											// or background color will not be set (bug?)


// Menu File
	m_file = new BMenu ( "File" );									// create menu
	m_file->AddItem ( it = new BMenuItem ( "New",		new BMessage ( File_New ), 'N' ) );
	it->SetTarget ( be_app );										// new window is done by Application 
	m_file->AddItem ( it = new BMenuItem ( "Open",		new BMessage ( File_Open ), 'O' ) );
	it->SetTarget ( be_app );										// create window for file is done by Application
	m_file->AddItem ( new BMenuItem ( "Close",			new BMessage ( B_QUIT_REQUESTED ), 'W' ) );
	m_file->AddSeparatorItem();
	m_file->AddItem ( it = new BMenuItem ( "Save",		new BMessage ( File_Save ), 'S' ) );
	it->SetEnabled ( false );										// disable Save
	m_file->AddItem ( new BMenuItem ( "Save As...",		new BMessage ( File_SaveAs ) ) );
	m_file->AddSeparatorItem();
	m_file->AddItem ( it = new BMenuItem ( "Page Setup...",	new BMessage ( File_PageSetup ) ) );
	it->SetTarget ( rlfTextView );									// UNDO is done by BTextView 
	m_file->AddItem ( it = new BMenuItem ( "Print...",		new BMessage ( File_Print ), 'P' ) );
	it->SetTarget ( rlfTextView );									// UNDO is done by BTextView 
	m_file->AddSeparatorItem();
	m_file->AddItem ( it = new BMenuItem ( "About CodeWalker...", new BMessage ( B_ABOUT_REQUESTED ) ) );
	it->SetTarget ( be_app );										// Programminfo is done by Application 
	m_file->AddSeparatorItem();
	m_file->AddItem ( it = new BMenuItem ( "Quit", new BMessage ( B_QUIT_REQUESTED ), 'Q' ) );
	it->SetTarget ( be_app );										// quit is done by Application 

	mb->AddItem ( m_file );											// add menu to menubar

// Menu Edit
	m_edit = new BMenu ( "Edit" );
	m_edit->AddItem ( it = new BMenuItem ( "Undo",				new BMessage ( B_UNDO ), 'Z' ) );
	it->SetTarget ( rlfTextView );									// UNDO is done by BTextView 
	m_edit->AddSeparatorItem();
	m_edit->AddItem ( it = new BMenuItem ( "Cut",				new BMessage ( B_CUT ), 'X' ) );
	it->SetTarget ( rlfTextView );									// CUT is done by BTextView 
	m_edit->AddItem ( it = new BMenuItem ( "Copy",				new BMessage ( B_COPY ), 'C' ) );
	it->SetTarget ( rlfTextView );									// COPY is done by BTextView 
	m_edit->AddItem ( it = new BMenuItem ( "Paste",				new BMessage ( B_PASTE ), 'V' ) );
	it->SetTarget ( rlfTextView );									// PASTE is done by BTextView 
	m_edit->AddItem ( it = new BMenuItem ( "Select All",		new BMessage ( B_SELECT_ALL ), 'A' ) );
	it->SetTarget ( rlfTextView );									// SELECT ALL is done by BTextView 
	m_edit->AddSeparatorItem();
	m_edit->AddItem ( it = new BMenuItem ( "Find...",			new BMessage ( Edit_Find ), 'F' ) );
	it->SetTarget ( rlfTextView );									// is done by BTextView 
	m_edit->AddItem ( it = new BMenuItem ( "Find Again",		new BMessage ( Edit_Find_Again ), 'G' ) );
	it->SetTarget ( rlfTextView );									// is done by BTextView 
	m_edit->AddItem ( it = new BMenuItem ( "Replace...",		new BMessage ( Edit_Replace ), 'R' ) );
	it->SetTarget ( rlfTextView );									// is done by BTextView 
	m_edit->AddItem ( it = new BMenuItem ( "Replace Same",		new BMessage ( Edit_Replace_Same ), 'T' ) );
	it->SetTarget ( rlfTextView );									// is done by BTextView 

	mb->AddItem ( m_edit );											// add menu to menubar

// Menu Extras
	m_extra = new BMenu ( "Extras" );
	mb->AddItem ( m_extra );
	m_extra->AddItem ( it = new BMenuItem ( "Settings", new BMessage ( Extra_Settings ), 'E' ) );
	it->SetTarget ( be_app );

	AdaptTitle();													// set Window-Title based on Filename
	rlfTextView->MakeFocus ( true );								// make TextView active for input
	Show();															// show the window
}


bool RLFWindow::QuitRequested()
{
	status_t	what=0;
	
	if ( rlfTextView->TextChanged() == true ) {						// if text has not been saved

// create an alertbox and ask user what to do
		char s [B_FILE_NAME_LENGTH+30] ="Save changes to the document \"";
		strcat ( s, Title());
		strcat ( s, "\"?" );

		BAlert *SaveAlert = new BAlert ( "SaveAlert", s, "Don't save", "Cancel", "Save" );
		SaveAlert->SetShortcut ( 1, B_ESCAPE );						// to be able to cancel the alert by pressing esc
		what = SaveAlert->Go();										// run alert and wait for result
		if ( what == 1 ) return (false);							// if Cancel then don't close window

		if ( what == 2 ) {											// Save
			if ( file_ref == NULL ) {								// make a save as
				CreateSavePanel();
				SavePanel->SetSaveText ( Title() );
				SavePanel->SetPanelDirectory ( (BDirectory*)NULL );
				SavePanel->Show();
				return (false);										// don't close window
			} else {												// make a normal save
				if ( SaveText ( file_ref ) != B_OK ) return (false); 	// if Save didn't work don't close window
			}
		}
	}
	DeleteSavePanel();												// get rid off the SavePanel
	be_app->PostMessage ( Msg_Unreg_Win );							// tell be_app the end of the window
	return(true);													// tell window that it is time to close
}

void RLFWindow::MessageReceived(BMessage *message)
{
	entry_ref	_ref;
	BPath		_path;
	BEntry		_entry;
	BMessage	mess;
	const char	*ss;

	DEBUG_PRINT ( "RLFWindow MessageReceived" );
	DEBUG_EXEC ( message->PrintToStream() );

 	switch ( message->what ){
		case Msg_Save_Enable:
			mb->FindItem ( File_Save )->SetEnabled ( true );
			break;
 		case File_Save:
			if ( file_ref == NULL ) {
				CreateSavePanel();
				SavePanel->SetSaveText ( Title() );
				SavePanel->SetPanelDirectory ( (BDirectory*)NULL );
				SavePanel->Show();
			} else {
				if ( SaveText ( file_ref ) == B_OK ) {
					mb->FindItem ( File_Save )->SetEnabled ( false );
 					rlfTextView->TextSaved();
 				}
 			}
			break;
 		case File_SaveAs:
			CreateSavePanel();
			if ( file_ref == NULL ) {
				SavePanel->SetSaveText ( Title() );
				SavePanel->SetPanelDirectory ( (BDirectory*)NULL );
			} else {
				SavePanel->SetSaveText ( file_ref->name );
				SavePanel->SetPanelDirectory ( new BDirectory ( file_ref ) );
			}
			SavePanel->Show();
			break;
 		case File_Close:
 			Close();												// Close Window
			break;
		case B_SAVE_REQUESTED:
			DEBUG_PRINT ( "SAVE_REQUESTED" );
			DeleteSavePanel();
		    if ( message->FindRef("directory", &_ref ) == B_OK ) {	// find directory and name in message
		    	if ( message->FindString("name", &ss ) == B_OK ) {
		    		if ( _entry.SetTo ( &_ref, true ) == B_OK ) {
		    			_entry.GetPath ( &_path );					// extract directory and name out of message
		    			_path.Append ( ss );
		    			DEBUG_PRINTF ( ( "%s", _path.Path() ) );
		    			_entry.SetTo ( _path.Path() );
		    			_entry.GetRef ( &_ref );
						if ( SaveText ( &_ref ) == B_OK ) {			// save current text based on above parameter
							if ( file_ref != NULL ) {
								delete file_ref;
							}
							file_ref = new entry_ref ( _ref );
				 			rlfTextView->TextSaved();				// tell textview that text has been saved
							mb->FindItem ( File_Save )->SetEnabled ( false );	// disable menuitem "Save"
							AdaptTitle();							// update WindowTitle to new Parameters
						}
					}
				}
			}
			break;
		case B_CANCEL:												// cancel Save Panel
			DeleteSavePanel();
			break;
		case Msg_Preferences:										// forward Preferences to TextVierw
			PostMessage ( message, rlfTextView );
			break;
   		default:
   			BWindow::MessageReceived(message);						// forward all other msg to BWindow
   			break;
	}
}

void RLFWindow::AdaptTitle ( )
{
	char	s[100];
	if ( file_ref == NULL ) {										// if no ref for window
		sprintf ( s, "%s - %i", "Untitled", Nr );					// windowtitle for new file
	} else {														// if ref for window
		strcpy ( s, file_ref->name );								// windowtitle is name of file
	}	
	SetTitle ( s );													// set windowtitle
}

status_t RLFWindow::SaveText ( entry_ref *saveref )
{
	BFile	file;
	if ( saveref != NULL ) {
		if ( file.SetTo ( saveref, B_WRITE_ONLY | B_ERASE_FILE ) == B_OK ) {	// previous saving with B_Create_File which caused that an existing file was not truncated
			int32	len = rlfTextView->TextLength();
			if ( file.Write ( rlfTextView->Text(), len ) == len ) {
				return B_OK;
			}
		}
	}
	BAlert *Alert = new BAlert ( "SaveError", "Unable to save file !", "OK", NULL, NULL, B_WIDTH_AS_USUAL, B_WARNING_ALERT );
	Alert->Go();
	return B_ERROR;
}

void	RLFWindow::CreateSavePanel ()								// create a FilePanel for Saving if necessary
{
	if ( SavePanel == NULL ) {										// if no SavePanel onscreen create a new one
		Messenger = new BMessenger (this);							// create new Messenger
		SavePanel = new BFilePanel ( B_SAVE_PANEL, Messenger, NULL, B_FILE_NODE, false );
																	// create new SavePanel
	}
}

void	RLFWindow::DeleteSavePanel ()
{
	if ( SavePanel != NULL ) {
		delete SavePanel;											// get rid off SavePanel
		delete Messenger;											// get rid off Messenger
		SavePanel = NULL;											// reset pointers
		Messenger = NULL;
	}
}
