/*  CPUMon - Small AtheOS utility for displaying the CPU load.
 *  Copyright (C) 2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

#ifndef __F_GRAPHVIEW_H__
#define __F_GRAPHVIEW_H__

#include <cosmoe/kernel.h>
#include <interface/View.h>

namespace os {
    class TableView;
    class CStringView;
}
class MultiMeter;


class GraphView : public os::CView
{
public:
	GraphView( const os::CRect& cRect, int nCPUCount );
	~GraphView();

	virtual void      AttachedToWindow();
	virtual void      DetachedFromWindow();
	virtual void      TimerTick( int nID );
    
    
private:
	os::TableView*         m_pcTableView;
	MultiMeter*            m_apcMeters[MAX_CPU_COUNT];
	os::CStringView*       m_apcTempViews[MAX_CPU_COUNT];
	int                    m_nCPUCount;
	int                    m_nTimer;
	int                    m_nW8378xDevice;
};




class MultiMeter : public os::CView
{
public:
	MultiMeter( const os::CRect& cFrame, const char* pzTitle, uint32 nResizeMask, uint32 nFlags );
	~MultiMeter();


	// From View:
	virtual void GetPreferredSize( float* outWidth, float* outHeight ) const
	{
		if (outWidth)  *outWidth = 5.0f;
		if (outHeight) *outHeight = 5.0f;
	}

	virtual void Draw( os::CRect cUpdateRect );

	void AddValue( float vValue );

private:
	float m_avValues[ 1024 ];

};




#endif // __F_GRAPHVIEW_H__
