#include <interface/View.h>
#include <app/Invoker.h>
#include <storage/Path.h>

#include <string>
#include <vector>

using namespace os;

class IconView;
class BitmapView;

class Icon
{
public:
    Icon( const char* pzTitle, const char* pzPath, const struct stat& sStat );
    ~Icon();
    const std::string& 		GetName() const { return( m_cTitle ); }
    CRect                   GetBounds( CFont* pcFont );
    CRect                   GetFrame( CFont* pcFont );
    void                    Draw( CView* pcView, const CPoint& cOffset, bool bLarge, bool bBlendText );
    void                    Select( BitmapView* pcView, bool bSelected );
    void                    Select( IconView* pcView, bool bSelected );
    bool                    IsSelected() const { return( m_bSelected ); }
    CBitmap*				GetBitmap();
  
    CPoint        			m_cPosition;
    bool                	m_bSelected;
    struct stat        		m_sStat;
private:
    float 					GetStrWidth( CFont* pcFont );
    static CBitmap*			s_pcBitmap[16];
    static int				s_nCurBitmap;
    CRect        			m_cBounds;
    float        			m_vStringWidth;
    int                		m_nMaxStrLen;
    bool        			m_bBoundsValid;
    bool        			m_bStrWidthValid;
    std::string        		m_cTitle;
    uint8        			m_anSmall[16*16*4];
    uint8        			m_anLarge[32*32*4];
};

class IconView : public CView, public CInvoker
{
public:
	IconView( CRect cFrame, const char* pzPath, CBitmap* pcBitmap = NULL );
	~IconView();

	void            	LayoutIcons();
	virtual void		SetDirChangeMsg( CMessage* pcMsg );
	virtual void		Invoked();
	virtual void		DirChanged( const std::string& cNewPath );

	void				SetPath( const std::string& cPath );
	std::string			GetPath();

	Icon*               FindIcon( const CPoint& cPos );
	void                Erase( const CRect& cFrame );

	virtual void        AttachedToWindow();
	virtual void  	    KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );
	virtual void        MouseDown( CPoint cPosition );
	virtual void        MouseUp( CPoint cPosition );
	virtual void        MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
	virtual void        Draw( CRect cUpdateRect );

private:
	static int32		ReadDirectory( void* pData );
	void				ReRead();

	struct ReadDirParam
	{
		ReadDirParam( IconView* pcView ) { m_pcView = pcView; m_bCancel = false; }
		IconView* m_pcView;
		bool      m_bCancel;
	};
	CPath                m_cPath;
	ReadDirParam*        m_pcCurReadDirSession;

	CMessage*            m_pcDirChangeMsg;
	CPoint               m_cLastPos;
	CPoint               m_cDragStartPos;
	CRect                m_cSelRect;
	bigtime_t            m_nHitTime;
	CBitmap*             m_pcBitmap;
	std::vector<Icon*>   m_cIcons;
	bool                 m_bCanDrag;
	bool                 m_bSelRectActive;
};
