#include <stdio.h>
#include <cosmoe/kernel.h>

#include <interface/Window.h>
#include <interface/View.h>
#include <tableview.h>
#include <interface/TextView.h>
#include <interface/StringView.h>
#include <interface/Button.h>
#include <interface/Screen.h>
#include <app/Message.h>

using namespace os;

#define ID_OK 1
#define ID_CANCEL 2

static bool g_bRun      = true;
static bool g_bSelected = false;

static std::string g_cName;
static std::string g_cPassword;

class LoginView : public CView
{
public:
  LoginView( CRect cFrame );
  ~LoginView();
  
  virtual void AllAttached();
  virtual void FrameResized( float inWidth, float inHeight );
  
private:
  void Layout();

  TextView*       m_pcNameView;
  CStringView*    m_pcNameLabel;
  TextView*       m_pcPasswordView;
  CStringView*    m_pcPasswordLabel;
  CButton*        m_pcOkBut;
  CButton*        m_pcCancelBut;
};

class LoginWindow : public CWindow
{
public:
  LoginWindow( const CRect& cFrame );

  
  virtual bool        OkToQuit() { g_bRun = false; return( true ); }
  virtual void        MessageReceived( CMessage* pcMessage );
private:
  LoginView* m_pcView;
};

LoginWindow::LoginWindow( const CRect& cFrame ) : CWindow( cFrame, "Login:" )
{
  m_pcView = new LoginView( Bounds() );
  AddChild( m_pcView );

}

void LoginWindow::MessageReceived( CMessage* pcMsg )
{
dbprintf( "LoginWindow::MessageReceived\n" );
	switch( pcMsg->what )
	{
		case ID_OK:
dbprintf( "LoginWindow: OK selected\n" );
		g_bSelected = true;
		case ID_CANCEL:
		PostMessage( M_QUIT );
		break;
		default:
		CWindow::MessageReceived( pcMsg );
		break;
	}
}

LoginView::LoginView( CRect cFrame ) : CView( cFrame, "password_view", CF_FOLLOW_ALL )
{
	m_pcOkBut        = new CButton( CRect( 0, 0, 0, 0 ), "ok_but", "Ok", new CMessage( ID_OK ), CF_FOLLOW_NONE );
	m_pcCancelBut    = new CButton( CRect( 0, 0, 0, 0 ), "cancel_but", "Cancel", new CMessage( ID_CANCEL ), CF_FOLLOW_NONE );
	m_pcNameView     = new TextView( CRect( 0, 0, 0, 0 ), "name_view", "", CF_FOLLOW_NONE );
	m_pcPasswordView = new TextView( CRect( 0, 0, 0, 0 ), "pasw_view", "", CF_FOLLOW_NONE );

	m_pcNameLabel    = new CStringView( CRect(0,0,1,1), "string", "Login name:", ALIGN_RIGHT );
	m_pcPasswordLabel= new CStringView( CRect(0,0,1,1), "string", "Password:", ALIGN_RIGHT );

	m_pcPasswordView->SetPasswordMode( true );

	AddChild( m_pcNameView, true );
	AddChild( m_pcNameLabel );
	AddChild( m_pcPasswordView, true );
	AddChild( m_pcPasswordLabel );
	AddChild( m_pcOkBut, true );
	AddChild( m_pcCancelBut, true );

	Layout();
}

LoginView::~LoginView()
{
	g_cName     = m_pcNameView->GetBuffer()[0];
	g_cPassword = m_pcPasswordView->GetBuffer()[0];
}

void LoginView::AllAttached()
{
	m_pcNameView->MakeFocus();
	Window()->SetDefaultButton( m_pcOkBut );
}


void LoginView::FrameResized( float inWidth, float inHeight )
{
	Layout();
}


void LoginView::Layout()
{
	CRect cBounds = Bounds();

	CRect cButFrame(0,0,0,0);
	CRect cLabelFrame(0,0,0,0);

	float buttonH, buttonV;
	float labelH, labelV;
	float nameViewH, nameViewV;

	m_pcCancelBut->GetPreferredSize( &buttonH, &buttonV );
	m_pcNameLabel->GetPreferredSize( &labelH, &labelV );
	m_pcNameView->GetPreferredSize( &nameViewH, &nameViewV );

	if ( nameViewV > labelV )
	{
		labelV = nameViewV;
	}

	cButFrame.right = buttonH - 1;
	cButFrame.bottom = buttonV - 1;

	cLabelFrame.right = labelH - 1;
	cLabelFrame.bottom = labelV - 1;

	CRect cEditFrame( cLabelFrame.right + 10, cLabelFrame.top, cBounds.right - 20, cLabelFrame.bottom );

	m_pcNameLabel->SetFrame( cLabelFrame + CPoint( 10, 20 ) );
	m_pcNameView->SetFrame( cEditFrame + CPoint( 10, 20 ) );

	m_pcPasswordLabel->SetFrame( cLabelFrame + CPoint( 10, 50 + labelV ) );
	m_pcPasswordView->SetFrame( cEditFrame + CPoint( 10, 50 + labelV ) );

	m_pcOkBut->SetFrame( cButFrame + CPoint( cBounds.right - buttonH - 15, cBounds.bottom - buttonV - 10 ) );
	m_pcCancelBut->SetFrame( cButFrame + CPoint( cBounds.right - buttonH*2 - 30, cBounds.bottom - buttonV - 10 ) );
}


bool get_login( std::string* pcName, std::string* pcPassword )
{
	g_bRun = true;
	g_bSelected = false;

	CRect cFrame( 0, 0, 249, 129 );

	IPoint cScreenRes;

		// Need a new scope to reduce the time the desktop is locked.
	{ cScreenRes = CScreen().GetResolution();  }

	cFrame += CPoint( cScreenRes.x / 2 - (cFrame.Width()+1.0f) / 2, cScreenRes.y / 2 - (cFrame.Height()+1.0f) / 2 );

	CWindow* pcWnd = new LoginWindow( cFrame );

	pcWnd->Show();
	pcWnd->Activate();

	while( g_bRun )
	{
		snooze( 20000 );
	}

	*pcName     = g_cName;
	*pcPassword = g_cPassword;
	return( g_bSelected );
}




