//****************************************************************************************
//
//	File:		Prefs.cpp
//
//	Written by:	Daniel Switkin
//
//	Copyright 1999, Be Incorporated
//
//	Revised by: Steffen Yount
//
//****************************************************************************************

#include "Prefs.h"
#include "Common.h"
#include "PulseApp.h"
//#include <storage/FindDirectory.h>
#include <storage/Path.h>
#include <interface/Screen.h>
//#include <kernel/OS.h>
#include <stdio.h>
#include <string.h>



Prefs::Prefs()
{
	mode = ::NORMAL_WINDOW;
	modeset[mode].do_display.standard = true;
	modeset[mode].do_display.priority = false;

	normal_window_rect = GetDefaultNormalWindowRect();
	Set_Normal_Window_Rect();

	mini_window_rect = GetDefaultMiniWindowRect();

	normal_fade_colors = DEFAULT_NORMAL_FADE_COLORS;
	for (int i = 0; i < MODE_COUNT; i++)
	{
		modeset[i].do_display.standard = DEFAULT_DO_STANDARD_DISPLAY;
		modeset[i].do_display.priority = DEFAULT_DO_PRIORITY_DISPLAY;
		for (int j = 0; j < COLOR_COUNT; j++)
		{
			modeset[i].color[j] = DEFAULT_COLORS[j];
		}
	}
}

int32 Prefs::Max_Bar_Count()
{
	return CPU_Count() + 1;
}

int32 Prefs::Bar_Count(int mode)
{
	int32 count = 0L;

	if (modeset[mode].do_display.standard)
		count++;

	if (modeset[mode].do_display.priority)
		count++;

	count *= CPU_Count();

	return count;
}

int32 Prefs::Bar_Count()
{
	return Bar_Count(mode);
}

void Prefs::Set_Normal_Window_Rect()
{
	float height = PROGRESS_MTOP + PROGRESS_MBOTTOM + 
		Bar_Count(NORMAL_WINDOW) * ITEM_OFFSET;

	if (PULSEVIEW_MIN_HEIGHT > height)
	{
		height = PULSEVIEW_MIN_HEIGHT;
	}

	normal_window_rect.Set(normal_window_rect.left, normal_window_rect.top,
		normal_window_rect.left + PULSEVIEW_WIDTH, normal_window_rect.top + height);
}

BRect Prefs::GetDefaultNormalWindowRect()
{
	float height = PROGRESS_MTOP + PROGRESS_MBOTTOM + Bar_Count(NORMAL_WINDOW) * ITEM_OFFSET;
	if (PULSEVIEW_MIN_HEIGHT > height)
	{
		height = PULSEVIEW_MIN_HEIGHT;
	}

	// Dock the window in the lower right hand corner just like the original
	BRect r(0, 0, PULSEVIEW_WIDTH, height);

	/*
	BRect screen_rect = BScreen(B_MAIN_SCREEN_ID).Frame();
	r.OffsetTo(screen_rect.right - r.Width() - 10, screen_rect.bottom - r.Height() - 30);
	*/

	r.OffsetTo(10, 30);

	return r;
}

BRect Prefs::GetDefaultMiniWindowRect()
{
	// Lower right hand corner by default
	BRect screen_rect = BScreen(B_MAIN_SCREEN_ID).Frame();
	screen_rect.left = screen_rect.right - 30;
	screen_rect.top = screen_rect.bottom - 150;
	screen_rect.OffsetBy(-10, -30);

	return screen_rect;
}

Prefs::~Prefs()
{
}
