//****************************************************************************************
//
//	File:		PulseApp.cpp
//
//	Written by:	Daniel Switkin
//
//	Copyright 1999, Be Incorporated
//
//	Revised by: Steffen Yount
//
//****************************************************************************************

#include "PulseApp.h"
#include "Common.h"
#include "PulseWindow.h"
#include <interface/Rect.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

// This function allows testing the display for cpu counts != mine
int32 CPU_Count()
{
	static int32 CPUCount = 0;
	if (!CPUCount)
	{
		system_info sys_info;
		get_system_info(&sys_info);
		CPUCount = sys_info.cpu_count;
	}
	return CPUCount;
}

// Ensure that the mini mode always shows an indicator
// for each CPU, at least one pixel wide
int GetMinimumViewWidth(Prefs * prefs, int mode)
{
	return prefs->Bar_Count(mode) * 3 + 1;
}


PulseApp::PulseApp(int mode) : BApplication(APP_SIGNATURE)
{
	prefs = new Prefs();
	prefs->mode = mode;
	
	PulseWindow *pulsewindow = NULL;
	if (prefs->mode == NORMAL_WINDOW)
	{
		pulsewindow = new PulseWindow(prefs->normal_window_rect);
	}
	else if (prefs->mode == MINI_WINDOW)
	{
		pulsewindow = new PulseWindow(prefs->mini_window_rect);
	}

	pulsewindow->Show();
}

PulseApp::~PulseApp()
{
	delete prefs;
}

int main(int argc, char **argv)
{
	// Parse arguments for backward compatibility - this mode will override
	// the setting stored in the user's settings directory
	// Color and width information is no longer accepted though
	int mode = NORMAL_WINDOW;
	for (int x = 1; x < argc; x++)
	{
		if (strstr(argv[x], "mini"))
			mode = MINI_WINDOW;
		else if (strstr(argv[x], "help"))
		{
			printf("usage: pulse [-mini]\n");
			return 0;
		}
	}

	PulseApp *pulseapp = new PulseApp(mode);
	pulseapp->Run();
	delete pulseapp;
	return 0;
}

