//****************************************************************************************
//
//	File:		PulseView.cpp
//
//	Written by:	David Ramsey and Daniel Switkin
//
//	Copyright 1999, Be Incorporated
//
//	Revised by: Steffen Yount
//
//****************************************************************************************

#include "PulseView.h"
#include "Common.h"
#include "PulseApp.h"
#include <interface/Alert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

PulseView::PulseView(BRect rect, const char *name) :
	BView(rect, name, B_FOLLOW_ALL_SIDES, B_FULL_UPDATE_ON_RESIZE |
		B_WILL_DRAW | B_PULSE_NEEDED /*| B_FRAME_EVENTS */)
{
}

void PulseView::MouseDown(BPoint point)
{
	MakeFocus(true);
}

void PulseView::Update()
{
	bigtime_t now = system_time();

	if (prefs->Do_priority_display())
	{
		// Calculate work done per priority since last run
		bigtime_t priority_active[P_PRIORITY_COUNT];
#if 0
		team_info team_inf;
		int32 team_cookie = 0;
		thread_info thread_inf;
		int32 thread_cookie;
#endif
		for (int i = 0; i < P_PRIORITY_COUNT; i++)
			priority_active[i] = 0;
#if 0
		while (get_next_team_info(&team_cookie, &team_inf) == B_OK)
		{
			thread_cookie = 0;
			while (get_next_thread_info(team_inf.team, &thread_cookie, &thread_inf) == B_OK) {
				if (thread_inf.priority >= P_URGENT_DISPLAY_PRIORITY)
				{
					if (thread_inf.priority >= P_URGENT_PRIORITY)
					{
						if (thread_inf.priority >= P_REALTIME_PRIORITY)
						{
							priority_active[0] += thread_inf.user_time +
								thread_inf.kernel_time;
						}
						else
						{
							priority_active[1] += thread_inf.user_time +
								thread_inf.kernel_time;
						}
					}
					else
					{
						if (thread_inf.priority >= P_REALTIME_DISPLAY_PRIORITY)
						{
							priority_active[2] += thread_inf.user_time +
								thread_inf.kernel_time;
						}
						else
						{
							priority_active[3] += thread_inf.user_time +
								thread_inf.kernel_time;
						}
					}
				}
				else
				{
					if (thread_inf.priority >= P_NORMAL_PRIORITY)
					{
						if (thread_inf.priority >= P_DISPLAY_PRIORITY)
						{
							priority_active[4] += thread_inf.user_time +
								thread_inf.kernel_time;
						}
						else
						{
							priority_active[5] += thread_inf.user_time +
								thread_inf.kernel_time;
						}
					}
					else
					{
						if (thread_inf.priority > 0)
						{
							priority_active[6] += thread_inf.user_time +
								thread_inf.kernel_time;
						}
					}
				}

			}
		}
#endif

		for (int i = 0; i < P_PRIORITY_COUNT; i++)
		{
			float prev_priority_times;
			prev_priority_times = priority_times[i];
			priority_times[i] = (float)(priority_active[i] -
				prev_priority_active[i]) / (now - prev_time);
			if (priority_times[i] < 0)
				priority_times[i] = prev_priority_times;
			if (priority_times[i] > CPU_Count())
				priority_times[i] = prev_priority_times;
			prev_priority_active[i] = priority_active[i];
		}
	}

	if (prefs->Do_standard_display())
	{
		// Calculate work done since last run for each CPU
		for (int x = 0; x < CPU_Count(); x++)
		{
			system_info sys_inf;
			get_system_info(&sys_inf);
			double cpu_time = (double)(sys_inf.cpu_infos[x].active_time -
				prev_active[x]) / (now - prev_time);
			prev_active[x] = sys_inf.cpu_infos[x].active_time;
			if (cpu_time < 0) cpu_time = 0;
			if (cpu_time > 1) cpu_time = 1;
			cpu_times[x] = cpu_time;
		}
	}

	prev_time = now;
}


PulseView::~PulseView()
{
}
