/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <malloc.h>

#include <cosmoe/kernel.h>

#include <interface/Bitmap.h>

#include "bitmap.h"
#include "ddriver.h"

using namespace os;


SrvBitmap::SrvBitmap( int nWidth, int nHeight, color_space eColorSpc, uint8* pRaster, int nBytesPerLine )
{
	m_nFlags      = 0;
	m_nWidth      = nWidth;
	m_nHeight     = nHeight;
	m_eColorSpc   = eColorSpc;
	m_pcDriver    = NULL;
	m_bVideoMem   = false;
	m_hArea       = -1;

	if ( nBytesPerLine != 0 )
	{
		m_nBytesPerLine = nBytesPerLine;
	}
	else
	{
		int nBitsPerPixel = BitsPerPixel( eColorSpc );
		if ( nBitsPerPixel == 15 )
		{
			nBitsPerPixel = 16;
		}
		m_nBytesPerLine = (nWidth * nBitsPerPixel ) / 8;
	}

	if ( m_nWidth == 0 || pRaster != NULL )
	{
		m_bFreeRaster = false;
		m_pRaster = pRaster;
	}
	else
	{
		m_bFreeRaster = true;
		m_pRaster = (uint8*) calloc( m_nHeight, m_nBytesPerLine );
	}
}


SrvBitmap::~SrvBitmap()
{
	if ( m_bFreeRaster )
	{
		delete[] m_pRaster;
	}

	if ( m_hArea >= 0 )
	{
		delete_area( m_hArea );
	}
}


BitmapNode::BitmapNode( SrvBitmap* pcBitmap )
{
	pcBitmap->AddRef();
	m_pcBitmap = pcBitmap;
	m_hHandle  = g_pcBitmaps->Insert( this );
}


BitmapNode::~BitmapNode()
{
	g_pcBitmaps->Remove( m_hHandle );
	m_pcBitmap->Release();
}
