/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "layer.h"

using namespace os;


CPoint Layer::ConvertToParent( const CPoint& cPoint ) const
{
	return( cPoint + GetLeftTop() );
}


void Layer::ConvertToParent( CPoint* pcPoint ) const
{
	*pcPoint += GetLeftTop();
}


CRect Layer::ConvertToParent( const CRect& cRect ) const
{
	return( cRect + GetLeftTop() );
}


void Layer::ConvertToParent( CRect* pcRect ) const
{
	*pcRect += GetLeftTop();
}


CPoint Layer::ConvertFromParent( const CPoint& cPoint ) const
{
	return( cPoint - GetLeftTop() );
}


void Layer::ConvertFromParent( CPoint* pcPoint ) const
{
	*pcPoint -= GetLeftTop();
}


CRect Layer::ConvertFromParent( const CRect& cRect ) const
{
	return( cRect - GetLeftTop() );
}


void Layer::ConvertFromParent( CRect* pcRect ) const
{
	*pcRect -= GetLeftTop();
}


CPoint Layer::ConvertToRoot( const CPoint& cPoint ) const
{
	if ( m_pcParent != NULL )
	{
		return( m_pcParent->ConvertToRoot( cPoint + GetLeftTop() ) );
	}

	return( cPoint );
}


void Layer::ConvertToRoot( CPoint* pcPoint ) const
{
	if ( m_pcParent != NULL )
	{
		*pcPoint += GetLeftTop();
		m_pcParent->ConvertToRoot( pcPoint );
	}
}


CRect Layer::ConvertToRoot( const CRect& cRect ) const
{
	if ( m_pcParent != NULL )
	{
		return( m_pcParent->ConvertToRoot( cRect + GetLeftTop() ) );
	}

	return( cRect );
}


void Layer::ConvertToRoot( CRect* pcRect ) const
{
	if ( m_pcParent != NULL )
	{
		*pcRect += GetLeftTop();
		m_pcParent->ConvertToRoot( pcRect );
	}
}


CPoint Layer::ConvertFromRoot( const CPoint& cPoint ) const
{
	if ( m_pcParent != NULL ) {
		return( m_pcParent->ConvertFromRoot( cPoint - GetLeftTop() ) );
	}
	return( cPoint );
}


void Layer::ConvertFromRoot( CPoint* pcPoint ) const
{
	if ( m_pcParent != NULL )
	{
		*pcPoint -= GetLeftTop();
		m_pcParent->ConvertFromRoot( pcPoint );
	}
}


CRect Layer::ConvertFromRoot( const CRect& cRect ) const
{
	if ( m_pcParent != NULL )
	{
		return( m_pcParent->ConvertFromRoot( cRect - GetLeftTop() ) );
	}

	return( cRect );
}


void Layer::ConvertFromRoot( CRect* pcRect ) const
{
	if ( m_pcParent != NULL )
	{
		*pcRect -= GetLeftTop();
		m_pcParent->ConvertFromRoot( pcRect );
	}
}
