
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <support/SupportDefs.h>

//#include <directfb.h>

#include <atheos/kernel.h>

#include <interface/Bitmap.h>

#include "dfbdriver.h"


using namespace os;

static area_id        g_nFrameBufArea = -1;


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

DirectFBDriver::DirectFBDriver() :
    m_nCurrentMode(0),
    m_nFrameBufferSize(0)
{
	dbprintf( "DFBDriver: In Constructor\n");
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

DirectFBDriver::~DirectFBDriver()
{
	//mFrameBuffer->Release(mFrameBuffer);
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool DirectFBDriver::InitModes( void )
{
	DFBResult				err;
	IDirectFBDisplayLayer*	layer;
	DFBDisplayLayerConfig	config;
	DFBSurfaceDescription	dsc;

	dbprintf( "DFBDriver: Entering InitModes()\n");

	if (DirectFBInit(NULL, NULL) != DFB_OK)
		return false;

	if (DirectFBCreate( &mFrameBuffer ) != DFB_OK)
		return false;

	if (mFrameBuffer->SetCooperativeLevel( mFrameBuffer, DFSCL_FULLSCREEN ))
		return false;

	dbprintf( "DFBDriver: About to set display to 16bpp\n");

	// See if we can set the display mode to 16bpp
	mFrameBuffer->GetDisplayLayer( mFrameBuffer, DLID_PRIMARY, &layer );

	config.flags = DLCONF_PIXELFORMAT;
	config.pixelformat = DSPF_RGB16;

	err = layer->SetConfiguration( layer, &config );
	if (!err)
	{
		dbprintf( "DFBDriver: Could not set video mode to 16bpp\n");
	}

	layer->Release( layer );

	// See if we can create our primary surface

	dsc.flags = DSDESC_CAPS;
	dsc.caps  = (DFBSurfaceCapabilities)(DSCAPS_PRIMARY | DSCAPS_FLIPPING);

	dbprintf( "DFBDriver: About to create surface\n");

	err = mFrameBuffer->CreateSurface( mFrameBuffer, &dsc, &mPrimarySurface );
	if (err != DFB_OK)
	{
		dbprintf( "DFBDriver: Failed creating primary surface\n");
		mFrameBuffer->Release( mFrameBuffer );
		return false;
	}

	unsigned int width, height;
	mPrimarySurface->GetSize( mPrimarySurface, &width, &height );
	mModeList.push_back( ScreenMode(width, height, width * 2, CS_RGB16) );
	//m_nFrameBufferSize = height * width * 2;	// move this for multi-mode
	m_nFrameBufferSize = 1024 * 1024 * 4;

	return( true );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

area_id DirectFBDriver::Open( void )
{
	dbprintf( "DFBDriver: in Open()\n");

	if ( InitModes() )
	{
//      m_pFrameBuffer = NULL;
		g_nFrameBufArea = create_area( "vesa_io",
										NULL/*(void**) &m_pFrameBuffer*/,
										m_nFrameBufferSize,
										AREA_FULL_ACCESS, AREA_NO_LOCK );
		return( g_nFrameBufArea );
	}

	return( -1 );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void DirectFBDriver::Close( void )
{
	mPrimarySurface->Release( mPrimarySurface );
	mFrameBuffer->Release( mFrameBuffer );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int DirectFBDriver::GetScreenModeCount()
{
	return( mModeList.size() );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool DirectFBDriver::GetScreenModeDesc( int nIndex, ScreenMode* psMode )
{
	if ( nIndex >= 0 && nIndex < int(mModeList.size()) )
	{
		*psMode = mModeList[nIndex];
		return( true );
	}

	return( false );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int DirectFBDriver::SetScreenMode( int nWidth, int nHeight,
									color_space eColorSpc,
									int nPosH, int nPosV,
									int nSizeH, int nSizeV,
									float vRefreshRate )
{
	m_nCurrentMode  = -1;
  
	for ( int i = GetScreenModeCount() - 1 ; i >= 0 ; --i )
	{
		if ( mModeList[i].m_nWidth == nWidth && mModeList[i].m_nHeight == nHeight && mModeList[i].m_eColorSpace == eColorSpc )
		{
			m_nCurrentMode = i;
			break;
		}
	}

	if ( m_nCurrentMode >= 0  )
	{
		int bpp;

		switch( eColorSpc )
		{
			case CS_RGBA32:
			case CS_RGB32:
				bpp = 32;

			case CS_RGB24:
				bpp = 24;

			case CS_RGB16:
			case CS_RGB15:
			case CS_RGBA15:
				bpp = 16;

			default:
				dbprintf( "eColorSpc invalid color space %d\n", eColorSpc );
				return( -1 );
		}

		if ( mFrameBuffer->SetVideoMode( mFrameBuffer, nWidth, nHeight, bpp ) )
		{
			return( 0 );
		}
	}

	return( -1 );
}

int DirectFBDriver::GetHorizontalRes()
{
	return( mModeList[m_nCurrentMode].m_nWidth );
}


int DirectFBDriver::GetVerticalRes()
{
	return( mModeList[m_nCurrentMode].m_nHeight );
}


int DirectFBDriver::GetBytesPerLine()
{
	return( mModeList[m_nCurrentMode].m_nBytesPerLine );
}


color_space DirectFBDriver::GetColorSpace()
{
	return( mModeList[m_nCurrentMode].m_eColorSpace );
}


void DirectFBDriver::SetCursorBitmap( mouse_ptr_mode eMode, const IPoint& cHotSpot, const void* pRaster, int nWidth, int nHeight )
{
	IDirectFBDisplayLayer*	layer;
	DFBResult				err;

	//m_cCursorHotSpot = cHotSpot;

	mFrameBuffer->GetDisplayLayer( mFrameBuffer, DLID_PRIMARY, &layer );

	// FIXME
	err = layer->SetCursorShape(layer, cursorShape, cHotSpot.x, cHotSpot.y);
	if (!err)
	{
		dbprintf( "DFBDriver: Could not set the mouse cursor bitmap");
	}

	layer->Release( layer );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void DirectFBDriver::MouseOn( void )
{
	IDirectFBDisplayLayer*	layer;
	DFBResult				err;

	mFrameBuffer->GetDisplayLayer( mFrameBuffer, DLID_PRIMARY, &layer );

	err = layer->EnableCursor(layer, 1);
	if (!err)
	{
		dbprintf( "DFBDriver: Could not turn the mouse cursor on");
	}

	layer->Release( layer );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void DirectFBDriver::MouseOff( void )
{
	IDirectFBDisplayLayer*	layer;
	DFBResult				err;

	mFrameBuffer->GetDisplayLayer( mFrameBuffer, DLID_PRIMARY, &layer );

	err = layer->EnableCursor(layer, 0);
	if (!err)
	{
		dbprintf( "DFBDriver: Could not turn the mouse cursor on" );
	}

	layer->Release( layer );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool DirectFBDriver::IntersectWithMouse( const IRect& cRect )
{
	return( false );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void DirectFBDriver::SetMousePos( IPoint cNewPos )
{
	IDirectFBDisplayLayer*	layer;
	DFBResult				err;

	mFrameBuffer->GetDisplayLayer( mFrameBuffer, DLID_PRIMARY, &layer );

	err = layer->WarpCursor(layer, cNewPos.x, cNewPos.y);
	if (!err)
	{
		dbprintf( "DFBDriver: Could not set the mouse cursor position" );
	}

	layer->Release( layer );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void DirectFBDriver::SetColor( int nIndex, const Color32_s& sColor )
{
/*
	outb_p( nIndex, 0x3c8 );
	outb_p( sColor.red >> 2, 0x3c9 );
	outb_p( sColor.green >> 2, 0x3c9 );
	outb_p( sColor.blue >> 2, 0x3c9 );
*/
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool DirectFBDriver::DrawLine( SrvBitmap* psBitMap, const IRect& cClipRect,
								const IPoint& cPnt1, const IPoint& cPnt2,
								const Color32_s& sColor, int nMode )
{
	DFBRegion	clippingRect;

	clippingRect.x1 = cClipRect.left;
	clippingRect.y1 = cClipRect.top;
	clippingRect.x2 = cClipRect.right;
	clippingRect.y2 = cClipRect.bottom;

	mPrimarySurface->SetClip( mPrimarySurface, &clippingRect );

	mPrimarySurface->SetColor( mPrimarySurface, sColor.red,
												sColor.green,
												sColor.blue,
												sColor.alpha);

	// FIXME translate nMode
	mPrimarySurface->SetPorterDuff(mPrimarySurface, DSPD_NONE);

	mPrimarySurface->DrawLine( mPrimarySurface, cPnt1.x, cPnt1.y,
												cPnt2.x, cPnt2.y );
	return true;
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool DirectFBDriver::FillRect( SrvBitmap* psBitMap, const IRect& cRect, const Color32_s& sColor )
{
	mPrimarySurface->SetColor( mPrimarySurface, sColor.red,
												sColor.green,
												sColor.blue,
												sColor.alpha);

	mPrimarySurface->FillRectangle(mPrimarySurface, cRect.left,
													cRect.top,
													cRect.Width(),
													cRect.Height());
	return true;
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool DirectFBDriver::BltBitmap( SrvBitmap* dstbm, SrvBitmap* srcbm,
								IRect cSrcRect, IPoint cDstPos, int nMode )
{
	DFBRectangle	aRect;

	aRect.x	= cSrcRect.left;
	aRect.y	= cSrcRect.top;
	aRect.w	= cSrcRect.Width();
	aRect.h = cSrcRect.Height();

	// FIXME translate nMode
	mPrimarySurface->SetPorterDuff(mPrimarySurface, DSPD_NONE);

	// FIXME that ain't gonna work
	mPrimarySurface->Blit( mPrimarySurface, source_surface,
											&aRect,
											cDstPos.x,
											cDstPos.y);

	return true;
}


bool DisplayDriver::BeginDirectAccess()
{
	return (mPrimarySurface->Lock(	mPrimarySurface,
									DSLF_READ | DSLF_WRITE,
									&data, &pitch ) == 0);
}


void DisplayDriver::EndDirectAccess()
{
     mPrimarySurface->Unlock( mPrimarySurface );
}



