/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef        __F_FONT_NODE_H__
#define        __F_FONT_NODE_H__

#include <support/SupportDefs.h>

#include <util/resource.h>
#include <support/Locker.h>

#include <list>
#include <string>

class  SFont;
class  SFontInstance;
struct Glyph;

namespace os {
    class CMessenger;
    struct font_properties;
};

class FontNode : public Resource
{
public:
    typedef std::list<std::pair<os::CMessenger*,void*> > DependencyList_t;
    FontNode();

    Glyph* GetGlyph( int nChar );
  
    SFontInstance*        GetInstance( void ) const { return( m_pcInstance ); }

    void                SnapPointSize( float* pvSize ) const;
    status_t                SetProperties( const os::font_properties& sProps );
    status_t                SetFamilyAndStyle( const std::string& cFamily, const std::string& cStyle );
    status_t                SetProperties( int nSize, int nShear, int nRotation );

    DependencyList_t::iterator AddDependency( os::CMessenger* pcTarget, void* pView );
    void                       RemoveDependency( DependencyList_t::iterator& cIterator );
    void                       NotifyDependent() const;
protected:
    ~FontNode();
private:
    DependencyList_t      m_cDependencies;
    os::CLocker           m_cDependenciesMutex;
    SFont*                m_pcFont;
    SFontInstance*        m_pcInstance;
    int                   m_nSize;
    int                   m_nShear;
    int                   m_nRotation;
};

#endif        // __F_FONT_NODE_H__
