/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>

#include <cosmoe/kernel.h>
#include <cosmoe/image.h>

#include "inputnode.h"
#include "server.h"
#include "layer.h"
#include "swindow.h"
#include "keyboard.h"
#include "bitmap.h"

#include <app/Message.h>

CMessageQueue InputNode::s_cEventQueue;
CPoint        InputNode::s_cMousePos( 0, 0 );
uint32        InputNode::s_nMouseButtons = 0;
int           InputNode::s_nMouseMoveEventCount = 0;

static thread_id g_hEventThread = -1;

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

InputNode::InputNode()
{
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

InputNode::~InputNode()
{
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void InputNode::SetMousePos( const CPoint& cPos )
{
//    if ( cPos != s_cMousePos ) {
		CMessage* pcEvent = new CMessage( M_MOUSE_MOVED );
		pcEvent->AddPoint( "delta_move", cPos - s_cMousePos );
		EnqueueEvent( pcEvent );
//    }
}

CPoint InputNode::GetMousePos()
{
	return( s_cMousePos );
}

uint32 InputNode::GetMouseButtons()
{
	return( s_nMouseButtons );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void InputNode::EnqueueEvent( CMessage* pcEvent )
{
	int32 nButton = 0;

	s_cEventQueue.Lock();

	pcEvent->AddInt32( "_qualifiers", GetQualifiers() );
	pcEvent->AddInt64( "_timestamp", system_time() );

	switch( pcEvent->what )
	{
		case M_MOUSE_DOWN:
			if ( pcEvent->FindInt32( "buttons", &nButton ) == 0 )
			{
				s_nMouseButtons |= 1 << (nButton - 1);
			}
			break;
		case M_MOUSE_UP:
			if ( pcEvent->FindInt32( "buttons", &nButton ) == 0 )
			{
				s_nMouseButtons &= ~(1 << (nButton - 1));
			}
			break;
		case M_MOUSE_MOVED:
		{
			CPoint cDeltaMove;

			if ( pcEvent->FindPoint( "delta_move", &cDeltaMove ) == 0 )
			{
				s_cMousePos += cDeltaMove;
			}
			else
			{
				dbprintf( "InputNode::EnqueueEvent() M_MOUSE_MOVED event missing delta_move entry\n" );
			}
			pcEvent->AddInt32( "buttons", s_nMouseButtons );
			ath_atomic_add( &s_nMouseMoveEventCount, 1 );
			break;
		}
	}

	if ( s_cMousePos.x < 0 )                            s_cMousePos.x = 0;
	if ( s_cMousePos.x >= g_pcScreenBitmap->m_nWidth )  s_cMousePos.x = g_pcScreenBitmap->m_nWidth - 1;
	if ( s_cMousePos.y < 0 )                            s_cMousePos.y = 0;
	if ( s_cMousePos.y >= g_pcScreenBitmap->m_nHeight ) s_cMousePos.y = g_pcScreenBitmap->m_nHeight - 1;

	pcEvent->AddPoint( "_scr_pos", s_cMousePos );

	s_cEventQueue.AddMessage( pcEvent );
	s_cEventQueue.Unlock();

//  resume_thread( g_hEventThread );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void InputNode::EventLoop()
{
	for (;;)
	{
		CMessage* pcEvent;

		for (;;)
		{
			s_cEventQueue.Lock();    
			pcEvent = s_cEventQueue.NextMessage();
			s_cEventQueue.Unlock();
			if ( pcEvent != NULL && pcEvent->what == M_MOUSE_MOVED )
			{
				if ( ath_atomic_add( &s_nMouseMoveEventCount, -1 ) != 1 )
				{
					delete pcEvent;
					continue;
				}
				g_cLayerGate.Close();
				g_pcDispDrv->SetMousePos( static_cast<IPoint>(s_cMousePos) );
				g_cLayerGate.Open();
			}
			break;
		}
		
		if ( pcEvent != NULL )
		{
			AppServer::GetInstance()->ResetEventTime();
			SrvWindow::HandleInputEvent( pcEvent );
			delete pcEvent;
		}
		else
		{
			snooze( 10000 );
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int32 InputNode::EventLoopEntry( void* pData )
{
	EventLoop();
	return( 0 );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void InitInputSystem()
{
	char    zInputDrvPath[1024];
	int     nPathLen;
	DIR*    pDir;
	bool    bMouseFound = false;

	//XXX: buffer overflow
	const char* pzBaseDir = getenv( "COSMOE_SYS" );
	if( pzBaseDir == NULL )
	{
		pzBaseDir = ".";
	}
	strcpy( zInputDrvPath, pzBaseDir );
	strcat( zInputDrvPath, "/drivers/appserver/input/" );
	nPathLen = strlen( zInputDrvPath );
	pDir = opendir( zInputDrvPath );

	if ( pDir != NULL )
	{
		struct dirent* psEntry;
		while( (psEntry = readdir( pDir )) != NULL )
		{
			if ( strcmp( psEntry->d_name, "." ) == 0 || strcmp( psEntry->d_name, ".." ) == 0 )
			{
				continue;
			}
			int nError;

			dbprintf( "Probe input node %s\n", psEntry->d_name );
			zInputDrvPath[nPathLen] = '\0';
			strcat( zInputDrvPath, psEntry->d_name );
			int nLib = load_library( zInputDrvPath, 0 );
			if ( nLib < 0 ) {
				dbprintf( "failed to load input node %s\n", zInputDrvPath );
				continue;
			}
			in_init* pInitFunc;
			in_get_node_count* pGetNodeCount;
			in_get_input_node* pGetInputNode;

			nError = get_symbol_address( nLib, "init_input_node", -1, (void**) &pInitFunc );
			if ( nError < 0 )
			{
				dbprintf( "Input node %s does not export entry point init_input_node()\n", psEntry->d_name );
				unload_library( nLib );
				continue;
			}
			nError = get_symbol_address( nLib, "get_node_count", -1, (void**) &pGetNodeCount );
			if ( nError < 0 )
			{
				dbprintf( "Input node %s does not export entry point get_node_count()\n", psEntry->d_name );
				unload_library( nLib );
				continue;
			}
			nError = get_symbol_address( nLib, "get_input_node", -1, (void**) &pGetInputNode );
			if ( nError < 0 )
			{
				dbprintf( "Input node %s does not export entry point get_input_node()\n", psEntry->d_name );
				unload_library( nLib );
				continue;
			}

			if ( pInitFunc() == false )
			{
				dbprintf( "Input node %s failed to initialize\n", psEntry->d_name );
				unload_library( nLib );
				continue;
			}
			int nCount = pGetNodeCount();

			for ( int i = 0 ; i < nCount ; ++i )
			{
				InputNode* pcNode = pGetInputNode(i);
				if ( pcNode != NULL )
				{
					if ( pcNode->Start() )
					{
						if ( pcNode->GetType() == InputNode::IN_MOUSE ) {
							bMouseFound = true;
						}
					}
				}
			}
		}
	}
	else
	{
		dbprintf( "Error: failed to open input driver directory\n" );
	}

	g_hEventThread = spawn_thread( "input_thread", (void*)InputNode::EventLoopEntry, DISPLAY_PRIORITY, 0, NULL );
	resume_thread( g_hEventThread );
}
