/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef __INPUTNODE_H__
#define __INPUTNODE_H__

#include <cosmoe/mouse.h>
#include <interface/Region.h>
#include <app/MessageQueue.h>

namespace os {
  class CMessage;
};

using namespace os;

class InputNode
{
public:
	enum {
		IN_UNKNOWN,
		IN_MOUSE,
		IN_KEYBOARD,
		IN_JOYSTICK
	};

	InputNode();
	virtual ~InputNode();

	static void   SetMousePos( const CPoint& cPos );
	static CPoint GetMousePos();
	static uint32 GetMouseButtons();

	virtual bool Start() = 0;
	virtual int  GetType() = 0;
	
protected:
	static void EnqueueEvent( CMessage* pcEvent );
	
private:
	friend void InitInputSystem();
	static int32 EventLoopEntry( void* pData );
	static void EventLoop();
	static int32 Loop( void* pData );

	static CMessageQueue s_cEventQueue;
	static int              s_nMouseMoveEventCount;
	static CPoint              s_cMousePos;
	static uint32              s_nMouseButtons;
};



void InitInputSystem();

typedef bool in_init();
typedef void in_uninit();
typedef int in_get_node_count();
typedef InputNode* in_get_input_node( int nIndex );

#endif // __INPUTNODE_H__
