/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef        __F_LAYER_H__
#define        __F_LAYER_H__

#include <support/SupportDefs.h>

#include <util/resource.h>
#include <interface/Region.h>
#include <interface/View.h>
#include <interface/Font.h>
#include <string>

#include "fontnode.h"

namespace os {
    class Gate;
    struct GRndCopyRect_s;
}

class FontNode;
class SrvBitmap;
class SrvWindow;


os::Color32_s GetDefaultColor( int nIndex );

extern os::Gate g_cLayerGate;

#define NUM_FONT_GRAYS 256

class Layer
{
public:
    Layer( SrvWindow* pcWindow, Layer* pcParent, const char* pzName,
           const os::CRect& cFrame, int nFlags, void* pUserObj );
    Layer( SrvBitmap* pcBitmap );
    void        Init();
  
    virtual ~Layer();

    const char*         GetName( void ) const             { return( m_cName.c_str() ); }

    int                 GetHandle( void ) const           { return( m_hHandle );                }
    void*               GetUserObject( void ) const       { return( m_pUserObject );        }
    void                SetUserObject( void* pObj )       { m_pUserObject = pObj;         }
    void                SetWindow( SrvWindow* pcWindow );
    SrvWindow*          GetWindow( void ) const           { return( m_pcWindow ); }

    void                Show( bool bFlag );
    bool                IsVisible() const                 { return( m_nHideCount == 0 ); }
    void                AddChild( Layer* pcChild, bool bTopmost );
    void                RemoveChild( Layer* pcChild );
    void                RemoveThis( void );
    void                Added( int nHideCount );
    int                 GetLevel() const                  { return( m_nLevel ); }
    Layer*              GetChildAt( os::CPoint cPos );
    Layer*              GetBottomChild( void ) const      { return( m_pcBottomChild ); }
    Layer*              GetTopChild( void ) const         { return( m_pcTopChild ); }
    Layer*              GetHigherSibling( void ) const    { return( m_pcHigherSibling ); }
    Layer*              GetLowerSibling( void ) const     { return( m_pcLowerSibling ); }

    Layer*              GetParent( void ) const           { return( m_pcParent );        }

    void                UpdateIfNeeded( bool bForce );

    void                SetBitmap( SrvBitmap* pcBitmap );
    SrvBitmap*          GetBitmap( void ) const           { return( m_pcBitmap );                }

    os::CRegion*        GetRegion();
    void                PutRegion( os::CRegion* pcReg );
    bool                IsBackdrop() const                { return( m_bBackdrop ); }
    void                MoveToFront();
    void                MoveToBack();
    int                 ToggleDepth();

    virtual void        SetFrame( const os::CRect& cRect );
    os::CRect           GetFrame( void ) const            { return( m_cFrame ); }
    os::IRect           GetIFrame( void ) const           { return( m_cIFrame ); }
    os::CRect           GetBounds( void ) const           { return( m_cFrame.Bounds() ); }
    os::IRect           GetIBounds( void ) const          { return( m_cIFrame.Bounds() ); }

    void                ScrollBy( const os::CPoint& cOffset );
  
    os::CPoint          GetLeftTop( void ) const          { return( os::CPoint( m_cFrame.left, m_cFrame.top ) );        }
    os::IPoint          GetILeftTop( void ) const         { return( os::IPoint( m_cIFrame.left, m_cIFrame.top ) );        }

    void                Invalidate( const os::IRect& cRect );
    void                Invalidate( bool bReqursive = false );
    void                SetDirtyRegFlags();

    void                DeleteRegions();
    void                UpdateRegions( bool bForce = true, bool bRoot = true );

    void                MarkModified( const os::IRect& cRect ); // Invalidate the region of views below us, inside the rectangle
    void                BeginUpdate( void );
    void                EndUpdate( void );

      // Render functions:
    virtual void        Paint( const os::IRect& cUpdateRect, bool bUpdate = false );


    void                SetFont( FontNode* pcFont );
    os::font_height     GetFontHeight() const;
    void                SetFgColor( int nRed, int nGreen, int nBlue, int nAlpha );
    void                SetFgColor( os::Color32_s sColor );
    void                SetBgColor( int nRed, int nGreen, int nBlue, int nAlpha );
    void                SetBgColor( os::Color32_s sColor );
    void                SetEraseColor( int nRed, int nGreen, int nBlue, int nAlpha );
    void                                        SetEraseColor( os::Color32_s sColor );
    void                SetDrawindMode( int nMode ) { m_nDrawingMode = nMode; }
    void                DrawFrame( const os::CRect& cRect, uint32 nStyle );
    void                MovePenTo( float x, float y )             { m_cPenPos.x = x; m_cPenPos.y = y;  }
    void                MovePenTo( const os::CPoint& cPos )        { m_cPenPos = cPos;  }
    void                MovePenBy( const os::CPoint& cPos )        { m_cPenPos += cPos; }
    os::CPoint          GetPenPosition() const { return( m_cPenPos ); }
    void                DrawLine( const os::CPoint& cFromPnt, const os::CPoint& cToPnt ) { m_cPenPos = cFromPnt; DrawLine( cToPnt ); }
    void                DrawLine( const os::CPoint& cToPnt );

    void                DrawString( const char* pzString, int nLength );

    void                CopyRect( SrvBitmap* pcBitmap, os::GRndCopyRect_s* psCmd );

    void                FillRect( os::CRect cRect );
    void                FillRect( os::CRect cRect, os::Color32_s sColor );
    void                EraseRect( os::CRect cRect );
    void                DrawBitMap( SrvBitmap* pcDstBitmap, SrvBitmap* pcSrcBitmap, os::CRect cSrcRect, os::CPoint cDstPos );
    void                BitBlit( SrvBitmap* pcBitmap, os::CRect cSrcRect, os::CPoint cDstPos );
    void                ScrollRect( SrvBitmap* pcBitmap, os::CRect cSrcRect, os::CPoint cDstPos );

      // Mouse functions:
    bool                MouseOverlap( os::CPoint cOffset );
    void                MouseOn();
    void                MouseOff();

      // Coordinate conversions:
    os::CPoint          ConvertToParent( const os::CPoint& cPoint ) const;
    void                ConvertToParent( os::CPoint* pcPoint ) const;
    os::CRect           ConvertToParent( const os::CRect& cRect ) const;
    void                ConvertToParent( os::CRect* pcRect ) const;
    os::CPoint          ConvertFromParent( const os::CPoint& cPoint ) const;
    void                ConvertFromParent( os::CPoint* pcPoint ) const;
    os::CRect           ConvertFromParent( const os::CRect& cRect ) const;
    void                ConvertFromParent( os::CRect* pcRect ) const;
    os::CPoint          ConvertToRoot( const os::CPoint& cPoint )        const;
    void                ConvertToRoot( os::CPoint* pcPoint ) const;
    os::CRect           ConvertToRoot( const os::CRect& cRect ) const;
    void                ConvertToRoot( os::CRect* pcRect ) const;
    os::CPoint          ConvertFromRoot( const os::CPoint& cPoint ) const;
    void                ConvertFromRoot( os::CPoint* pcPoint ) const;
    os::CRect           ConvertFromRoot( const os::CRect& cRect ) const;
    void                ConvertFromRoot( os::CRect* pcRect ) const;
    os::CPoint          GetScrollOffset() const { return( m_cScrollOffset ); }
    os::IPoint          GetIScrollOffset() const { return( m_cIScrollOffset ); }
private:
    friend class SrvWindow;
  
    void                InvalidateNewAreas( void );
    void                ClearDirtyRegFlags();
    void                SwapRegions( bool bForce );
    void                RebuildRegion( bool bForce );
    void                MoveChilds();
  
protected:
public:
    std::string        m_cName;
    int                m_hHandle;                // The handle passed to applications.
    void*        m_pUserObject;                // Pointer to our partner in the application (in the apps address space)
    int                m_nLevel;                // Hierarchy level. Starting with root as 0
    int                m_nHideCount;
    SrvWindow*        m_pcWindow;                // The windows that own us, or NULL if we are at screen level
    SrvBitmap*        m_pcBitmap;                // The bitmap we are supposed to render to

    os::CRect        m_cFrame;
    os::IRect        m_cIFrame;                // Frame rectangle relative to our parent
    os::CPoint        m_cScrollOffset;
    os::IPoint        m_cIScrollOffset;

    Layer*        m_pcTopChild;                // Top level child
    Layer*        m_pcBottomChild;        // Bottom level child

    Layer*        m_pcLowerSibling;        // Sibling directly below us
    Layer*        m_pcHigherSibling;        // Sibling directly above us

    Layer*        m_pcParent;                // Daddy
    
    os::IPoint        m_cDeltaMove;                // Relative movement since last region update
    os::IPoint        m_cDeltaSize;                // Relative sizing since last region update

    os::CRegion*        m_pcVisibleReg;                // Visible areas, not including non-transparent childrens
    os::CRegion*        m_pcFullReg;                // All visible areas, including childrens
    os::CRegion*        m_pcPrevVisibleReg;        // Temporary storage for m_pcVisibleReg during region rebuild
    os::CRegion*        m_pcPrevFullReg;        // Temporary storage for m_pcFullReg during region rebuild
    os::CRegion*        m_pcDrawReg;                // Only valid between BeginPaint()/EndPaint()
    os::CRegion*        m_pcDamageReg;                // Containes areas made visible since the last M_PAINT message sendt
    os::CRegion*        m_pcActiveDamageReg;

    uint32        m_nFlags;
    bool          m_bHasInvalidRegs;        // True if someting made our clipping region invalide
    bool          m_bIsUpdating;                // True while we paint areas from the damage list
    bool          m_bBackdrop;
      // Render state:
    os::CPoint          m_cPenPos;
    os::Color32_s       m_sFgColor;
    os::Color32_s       m_sBgColor;
    os::Color32_s       m_sEraseColor;
    os::Color32_s       m_asFontPallette[NUM_FONT_GRAYS];
    int                 m_nDrawingMode;
    int                 m_nRegionUpdateCount;
    int                 m_nMouseOffCnt;
    FontNode*           m_pcFont;
    bool                m_bIsMouseOn;
    bool                m_bFontPalletteValid;
    bool                m_bIsAddedToFont;
    FontNode::DependencyList_t::iterator        m_cFontViewListIterator;
};

Layer* FindLayer( int nToken );

#endif        //        __F_LAYER_H__
