/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 
#ifndef  __SERVER_H__
#define  __SERVER_H__

#include <support/SupportDefs.h>
#include <cosmoe/mouse.h>

#include <interface/Region.h>
#include <interface/GraphicsDefs.h>
#include <interface/Bitmap.h>
#include <util/array.h>
#include <app/MessageQueue.h>

#include "ddriver.h"
#include "windowdecorator.h"

typedef        struct TextFont        TextFont_s;

using namespace os;

class SrvWindow;
class Layer;
class AppServer;
class SFontInstance;
class DisplayDriver;
class SrvBitmap;

namespace os {
    class CLocker;
    class CMessage;
}

extern ::Layer*          g_pcTopView;

extern DisplayDriver*    g_pcDispDrv;

extern Array<Layer>*     g_pcLayers;
extern CLocker*          g_pcWinListLock;

void config_changed();

class AppServer
{
public:
							AppServer();
							~AppServer();

	static AppServer*		GetInstance();
	os::WindowDecorator*	CreateWindowDecorator( Layer* pcView, uint32 nFlags );

	void					Run( void );
	DisplayDriver*			GetDisplayDriver( int nIndex )        { return( m_pcDispDriver ); }

	void					SendKeyCode( int nKeyCode, int nQualifiers );

	FontNode*				GetWindowTitleFont() const { return( m_pcWindowTitleFont ); }
	FontNode*				GetToolWindowTitleFont() const { return( m_pcToolWindowTitleFont ); }

	void					R_ClientDied( thread_id hThread );

	void					ResetEventTime();
	bigtime_t				GetIdleTime() const { return( system_time() - m_nLastEvenTime ); }
	int						LoadWindowDecorator( const std::string& cPath );

	port_id					m_hWndReplyPort;
	
private:
	int						LoadDecorator( const std::string& cPath, op_create_decorator** ppfCreate );

	void					DispatchMessage( CMessage* pcReq );

	static AppServer*		s_pcInstance;
	port_id					m_hRequestPort;
	DisplayDriver*			m_pcDispDriver;
	bigtime_t				m_nLastEvenTime;
	int						m_hCurrentDecorator;
	op_create_decorator*	m_pfDecoratorCreator;

	FontNode*				m_pcWindowTitleFont;
	FontNode*				m_pcToolWindowTitleFont;
};

#endif // __SERVER_H__
