/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef  __SWINDOW_H__
#define  __SWINDOW_H__

#include "layer.h"
#include <support/Locker.h>
#include <app/Message.h>

#include <string>

namespace os {
    class CMessenger;
    class CLocker;
    class WindowDecorator;
  
//    struct WR_ToggleViewDepth_s;
    struct WR_Render_s;
}

class SrvApplication;
class WndBorder;
class SrvBitmap;
class SrvSprite;


class SrvWindow
{
public:
					SrvWindow( const char*		pzTitle,
							   void*			pTopView,
							   uint32			nFlags,
							   uint32			nDesktopMask,
							   const os::CRect&	cRect,
							   SrvApplication*	pcApp,
							   port_id			hEventPort );
					
					SrvWindow( SrvApplication*	pcApp,
							   SrvBitmap*		pcBitmap );
					
					~SrvWindow();

	void			PostUsrMessage( os::CMessage* pcMsg );
	void			ReplaceDecorator();
	void			NotifyWindowFontChanged( bool bToolWindow );
	void			BeginRegionUpdate();
	void			Quit();
	
	const char*		GetTitle() const            { return( m_cTitle.c_str() ); }
	SrvApplication*	GetApp() const              { return( m_pcApp ); }
	os::CMessenger*	GetAppTarget( void ) const  { return( m_pcAppTarget ); }
	
	void			Show( bool bShow );
	void			MakeFocus( bool bFocus );
	bool			HasFocus() const;
	void			SetBitmap( SrvBitmap* pcBitmap ) { m_pcTopView->SetBitmap( pcBitmap ); }

	void			DesktopActivated( int nNewDesktop, const os::IPoint cNewRes, os::ColorSpace eColorSpace );
	void			WindowActivated( bool bActive );
	void			ScreenModeChanged( const os::IPoint cNewRes, os::ColorSpace eColorSpace );

	void			SetFrame( const os::CRect& cFrame );
	os::CRect		GetFrame( bool bClient = true ) const;

	Layer*			GetTopView() const;
	Layer*			GetClientView() const { return( m_pcTopView ); }
	uint32			GetDesktopMask() const;
	void			SetDesktopMask( uint32 nMask );

	thread_id		Run();
	status_t		Lock();
	status_t		Unlock();
	bool			IsLocked( void );
	port_id			GetMsgPort( void )        { return( m_hMsgPort ); }

	uint32			GetFlags() const         { return( m_nFlags ); }
	bool			DispatchMessage( os::CMessage* pcReq );
	bool			DispatchMessage( const void* psMsg, int nCode );
	bool			IsOffScreen() const        { return( m_bOffscreen ); }

	bool			HasPendingSizeEvents( Layer* pcLayer );

	static void		SendKbdEvent( int nKeyCode, uint32 nQual, const char* pzConvString, const char* zRawString );
	static void		HandleInputEvent( os::CMessage* pcEvent );
	static void		HandleMouseTransaction();

	struct DeskTopState_s
	{
		DeskTopState_s() { m_pcNextWindow = NULL; }
		os::CRect   m_cFrame;
		SrvWindow* m_pcNextWindow;
		SrvWindow* m_pcNextSystemWindow;
	} m_asDTState[32];

private:
	void			R_Render( os::WR_Render_s* psPkt );
	void			MouseMoved( os::CMessage* pcEvent, int nTransit );
	void			MouseDown( os::CMessage* pcEvent );
	void			MouseUp( os::CMessage* pcEvent, bool bSendDragMsg );

	static void		HandleMouseDown( const os::CPoint& cMousePos, int nButton, os::CMessage* pcEvent );
	static void		HandleMouseMoved( const os::CPoint& cMousePos, os::CMessage* pcEvent );
	static void		HandleMouseUp( const os::CPoint& cMousePos, int nButton, os::CMessage* pcEvent );

	SrvBitmap*		GetBitmap() const { return( m_pcTopView->GetBitmap() ); }

	void			Loop( void );
	static uint32	Entry( void* pData );

	static SrvWindow*    s_pcLastMouseWindow;
	static SrvWindow*    s_pcDragWindow;
	static SrvSprite*    s_pcDragSprite;
	static os::CMessage  s_cDragMessage;
	static bool          s_bIsDragging;

	SrvWindow*		m_pcNextGlobal;

	std::string		m_cTitle;
	uint32			m_nFlags;
	uint32			m_nDesktopMask;
	uint32			m_nDeskTopMask;
	SrvBitmap*		m_pcUserBitmap;
	SrvApplication*	m_pcApp;
	Layer*			m_pcTopView;
	WndBorder*		m_pcWndBorder;
	os::WindowDecorator* m_pcDecorator;
	bigtime_t		m_nLastHitTime;      // Time of last mouse click
	bool			m_bBorderHit;
	bool			m_bOffscreen;        // True for bitmap windows
	bool			m_bNeedRegionUpdate;
	thread_id		m_hThread;
	port_id			m_hMsgPort;          // Requests from application
	port_id			m_hEventPort;        // Events to application
	os::CLocker		m_cMutex;
	os::CMessenger*	m_pcAppTarget;
};

#endif // __SWINDOW_H__
