/*
 *  The Cosmoe application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

 
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <support/SupportDefs.h>
#include <cosmoe/isa_io.h>

#include <cosmoe/vesa_gfx.h>
#include <cosmoe/kernel.h>

#include <interface/Bitmap.h>

#include "vesadrv.h"


using namespace os;


static area_id        g_nFrameBufArea = -1;


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

VesaDriver::VesaDriver() :
	m_nCurrentMode(0),
	m_nFrameBufferSize(0),
	m_nFrameBufferOffset(0)
{
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

VesaDriver::~VesaDriver()
{
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool VesaDriver::InitModes( void )
{
	Vesa_Info_s      sVesaInfo;
	VESA_Mode_Info_s sModeInfo;
	uint16           anModes[1024];
	int              nModeCount;
	int              i=0;

	strcpy( sVesaInfo.VesaSignature, "VBE2" );

	nModeCount = get_vesa_info( &sVesaInfo, anModes, 1024 );

	if ( nModeCount <= 0 )
	{
		dbprintf( "Error: VesaDriver::InitModes() no VESA20 modes found\n" );
		return( false );
	}

//    dbprintf( "Found %d vesa modes\n", nModeCount );

	int nPagedCount  = 0;
	int nPlanarCount = 0;
	int nBadCount    = 0;

	for( i = 0 ; i < nModeCount ; ++i )
	{
		get_vesa_mode_info( &sModeInfo, anModes[i] );

		if( sModeInfo.PhysBasePtr == 0 ) { // We must have linear frame buffer
			nPagedCount++;
			continue;
		}

		if( sModeInfo.BitsPerPixel < 8 )
		{
			nPlanarCount++;
			continue;
		}

		if ( sModeInfo.NumberOfPlanes != 1 )
		{
			nPlanarCount++;
			continue;
		}

		if ( sModeInfo.BitsPerPixel != 15 && sModeInfo.BitsPerPixel != 16 && sModeInfo.BitsPerPixel != 32 )
		{
			nBadCount++;
			continue;
		}

		if ( sModeInfo.RedMaskSize == 0 && sModeInfo.GreenMaskSize == 0 && sModeInfo.BlueMaskSize == 0 &&
			sModeInfo.RedFieldPosition == 0 && sModeInfo.GreenFieldPosition == 0 && sModeInfo.BlueFieldPosition == 0 )
		{
			m_cModeList.push_back( VesaMode( sModeInfo.XResolution, sModeInfo.YResolution, sModeInfo.BytesPerScanLine,
											CS_CMAP8, anModes[i] | 0x4000, sModeInfo.PhysBasePtr ) );
		}
		else if ( sModeInfo.RedMaskSize == 5 && sModeInfo.GreenMaskSize == 5 && sModeInfo.BlueMaskSize == 5 &&
					sModeInfo.RedFieldPosition == 10 && sModeInfo.GreenFieldPosition == 5 && sModeInfo.BlueFieldPosition == 0 )
		{
			m_cModeList.push_back( VesaMode( sModeInfo.XResolution, sModeInfo.YResolution, sModeInfo.BytesPerScanLine,
											CS_RGB15, anModes[i] | 0x4000, sModeInfo.PhysBasePtr ) );
		}
		else if ( sModeInfo.RedMaskSize == 5 && sModeInfo.GreenMaskSize == 6 && sModeInfo.BlueMaskSize == 5 &&
					sModeInfo.RedFieldPosition == 11 && sModeInfo.GreenFieldPosition == 5 && sModeInfo.BlueFieldPosition == 0 )
		{
			m_cModeList.push_back( VesaMode( sModeInfo.XResolution, sModeInfo.YResolution, sModeInfo.BytesPerScanLine,
											CS_RGB16, anModes[i] | 0x4000, sModeInfo.PhysBasePtr ) );
		}
		else if ( sModeInfo.BitsPerPixel == 32 && sModeInfo.RedMaskSize == 8 && sModeInfo.GreenMaskSize == 8 && sModeInfo.BlueMaskSize == 8 &&
					sModeInfo.RedFieldPosition == 16 && sModeInfo.GreenFieldPosition == 8 && sModeInfo.BlueFieldPosition == 0 )
		{
			m_cModeList.push_back( VesaMode( sModeInfo.XResolution, sModeInfo.YResolution, sModeInfo.BytesPerScanLine,
											CS_RGB32, anModes[i] | 0x4000, sModeInfo.PhysBasePtr ) );
		}
		else
		{
			dbprintf( "Found unsupported video mode: %dx%d %d BPP %d BPL - %d:%d:%d, %d:%d:%d\n",
					sModeInfo.XResolution, sModeInfo.YResolution, sModeInfo.BitsPerPixel, sModeInfo.BytesPerScanLine,
					sModeInfo.RedMaskSize, sModeInfo.GreenMaskSize, sModeInfo.BlueMaskSize,
					sModeInfo.RedFieldPosition, sModeInfo.GreenFieldPosition, sModeInfo.BlueFieldPosition );
		}
#if 0
		dbprintf( "Mode %04x: %dx%d %d BPP %d BPL - %d:%d:%d, %d:%d:%d (%p)\n", anModes[i],
				sModeInfo.XResolution, sModeInfo.YResolution, sModeInfo.BitsPerPixel, sModeInfo.BytesPerScanLine,
				sModeInfo.RedMaskSize, sModeInfo.GreenMaskSize, sModeInfo.BlueMaskSize,
				sModeInfo.RedFieldPosition, sModeInfo.GreenFieldPosition, sModeInfo.BlueFieldPosition, (void*)sModeInfo.PhysBasePtr );
#endif        
	}
	dbprintf( "Found total of %d VESA modes. Valid: %d, Paged: %d, Planar: %d, Bad: %d\n",
			nModeCount, m_cModeList.size(), nPagedCount, nPlanarCount, nBadCount );
	return( true );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

area_id VesaDriver::Open( void )
{
	if ( InitModes() )
	{
		m_nFrameBufferSize = 1024 * 1024 * 4;
//      m_pFrameBuffer = NULL;
		g_nFrameBufArea = create_area( "vesa_io", NULL/*(void**) &m_pFrameBuffer*/, m_nFrameBufferSize,
									AREA_FULL_ACCESS, AREA_NO_LOCK );
		return( g_nFrameBufArea );
	}
	return( -1 );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void VesaDriver::Close( void )
{
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int VesaDriver::GetScreenModeCount()
{
	return( m_cModeList.size() );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool VesaDriver::GetScreenModeDesc( int nIndex, ScreenMode* psMode )
{
	if ( nIndex >= 0 && nIndex < int(m_cModeList.size()) )
	{
		*psMode = m_cModeList[nIndex];
		return( true );
	}

	return( false );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int VesaDriver::SetScreenMode( int nWidth, int nHeight, ColorSpace eColorSpc,
							int nPosH, int nPosV, int nSizeH, int nSizeV, float vRefreshRate )
{
	m_nCurrentMode  = -1;

	for ( int i = GetScreenModeCount() - 1 ; i >= 0 ; --i )
	{
		if ( m_cModeList[i].m_nWidth == nWidth && m_cModeList[i].m_nHeight == nHeight && m_cModeList[i].m_eColorSpace == eColorSpc ) {
			m_nCurrentMode = i;
			break;
		}
	}

	if ( m_nCurrentMode >= 0  )
	{
		remap_area( g_nFrameBufArea, (void*)(m_cModeList[m_nCurrentMode].m_nFrameBuffer & PAGE_MASK) );
		m_nFrameBufferOffset = m_cModeList[m_nCurrentMode].m_nFrameBuffer & ~PAGE_MASK;
		if ( SetVesaMode( m_cModeList[m_nCurrentMode].m_nVesaMode ) )
		{
			return( 0 );
		}
	}
	return( -1 );
}

int VesaDriver::GetHorizontalRes()
{
	return( m_cModeList[m_nCurrentMode].m_nWidth );
}

int VesaDriver::GetVerticalRes()
{
	return( m_cModeList[m_nCurrentMode].m_nHeight );
}

int VesaDriver::GetBytesPerLine()
{
	return( m_cModeList[m_nCurrentMode].m_nBytesPerLine );
}

color_space VesaDriver::GetColorSpace()
{
	return( m_cModeList[m_nCurrentMode].m_eColorSpace );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool VesaDriver::IntersectWithMouse( const IRect& cRect )
{
//  if ( NULL != m_pcMouse ) {
//    return( cRect.DoIntersect( m_pcMouse->GetFrame() ) );
//  } else {
	return( false );
//  }
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void VesaDriver::SetColor( int nIndex, const Color32_s& sColor )
{
	outb_p( nIndex, 0x3c8 );
	outb_p( sColor.red >> 2, 0x3c9 );
	outb_p( sColor.green >> 2, 0x3c9 );
	outb_p( sColor.blue >> 2, 0x3c9 );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool VesaDriver::SetVesaMode( uint32 nMode )
{
	return ( set_vesa_mode( nMode ) == 0 );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool VesaDriver::DrawLine( SrvBitmap* psBitMap, const IRect& cClipRect,
						const IPoint& cPnt1, const IPoint& cPnt2, const Color32_s& sColor, int nMode )
{
	DisplayDriver::DrawLine( psBitMap, cClipRect, cPnt1, cPnt2, sColor, nMode );
	return( true );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool VesaDriver::FillRect( SrvBitmap* psBitMap, const IRect& cRect, const Color32_s& sColor )
{
	return( DisplayDriver::FillRect( psBitMap, cRect, sColor ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool VesaDriver::BltBitmap( SrvBitmap* dstbm, SrvBitmap* srcbm, IRect cSrcRect, IPoint cDstPos, int nMode )
{
	return( DisplayDriver::BltBitmap( dstbm, srcbm, cSrcRect, cDstPos, nMode ) );
}
