 /*
 *  The AtheOS application server
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  Modification of NextDecorator by Edwin de Jonge
 *  BeIsh 0.1
 */

#include "beishdecorator.h"
#include "gradient.h"

#include <layer.h>
#include "sfont.h"
#include <math.h>
#include <interface/Window.h>
#include <macros.h>

using namespace os;

BeIshDecorator::BeIshDecorator( Layer* pcLayer, uint32 nWndFlags ) :
    WindowDecorator( pcLayer, nWndFlags )
{
    m_nFlags = nWndFlags;

    m_bHasFocus   = false;
    m_bCloseState = false;
    m_bZoomState  = false;
    m_bDepthState = false;

    m_sFontHeight = pcLayer->GetFontHeight();
  
    CalculateBorderSizes();
}

BeIshDecorator::~BeIshDecorator()
{
}

void BeIshDecorator::CalculateBorderSizes()
{
    if ( m_nFlags & WND_NO_BORDER ) 
    {
	m_vLeftBorder   = 0;
	m_vRightBorder  = 0;
	m_vTopBorder    = 0;
	m_vBottomBorder = 0;
    }
    else
    {
	if ( (m_nFlags & (WND_NO_TITLE | WND_NO_CLOSE_BUT | WND_NO_ZOOM_BUT | WND_NO_DEPTH_BUT | WND_NOT_MOVEABLE)) ==
	     (WND_NO_TITLE | WND_NO_CLOSE_BUT | WND_NO_ZOOM_BUT | WND_NO_DEPTH_BUT | WND_NOT_MOVEABLE) ) 
	{
	    m_vTopBorder = 5;
	}
        else
	{
	   //m_vTopBorder = float(m_sFontHeight.ascender + m_sFontHeight.descender + 6);
//	   m_vTopBorder = 23.0f;
	   m_vTopBorder = 26.0f;
	}
	m_vLeftBorder   = 5;
	m_vRightBorder  = 5;
	if ( (m_nFlags & WND_NOT_RESIZABLE) != WND_NOT_RESIZABLE ) 
	{
	    m_vBottomBorder = 5;
	}
        else
	{
	    m_vBottomBorder = 5;
	}
    }
    
}

void BeIshDecorator::FontChanged()
{
    Layer* pcView = GetLayer();
    m_sFontHeight = pcView->GetFontHeight();
    CalculateBorderSizes();
    pcView->Invalidate();
    Layout();
}

void BeIshDecorator::SetFlags( uint32 nFlags )
{
    Layer* pcView = GetLayer();
    m_nFlags = nFlags;
    CalculateBorderSizes();
    pcView->Invalidate();
    Layout();
}

CPoint BeIshDecorator::GetMinimumSize()
{
    CPoint cMinSize( 0.0f, m_vTopBorder + m_vBottomBorder );

    if ( (m_nFlags & WND_NO_CLOSE_BUT) == 0 ) {
	cMinSize.x += m_cCloseRect.Width();
    }
    if ( (m_nFlags & WND_NO_ZOOM_BUT) == 0 ) {
	cMinSize.x += m_cZoomRect.Width();
    }
    if ( (m_nFlags & WND_NO_DEPTH_BUT) == 0 ) {
	cMinSize.x += m_cToggleRect.Width();
    }
    if ( cMinSize.x < m_vLeftBorder + m_vRightBorder ) {
	cMinSize.x = m_vLeftBorder + m_vRightBorder;
    }
    return( cMinSize );
}

WindowDecorator::hit_item BeIshDecorator::HitTest( const CPoint& cPosition )
{
    if ( (m_nFlags & WND_NOT_RESIZABLE) != WND_NOT_RESIZABLE ) 
     {
  
	if ( cPosition.y < 4 ) 
	{
	    if ( cPosition.x < 4 ) 
	    {
		return( HIT_SIZE_LT );
	    } 
	    else if ( cPosition.x > m_cBounds.right - 4 ) 
            {
		return( HIT_SIZE_RT );
	    }
	    else
	    {
		return( HIT_SIZE_TOP );
	    }
	} 
	else if ( cPosition.y > m_cBounds.bottom - 8 ) 
	{
	    if ( cPosition.x < 16 ) 
	    {
		return( HIT_SIZE_LB );
	    }
	    else if ( cPosition.x > m_cBounds.right - 16 ) 
	    {
		return( HIT_SIZE_RB );
	    }
	    else
	    {
		return( HIT_SIZE_BOTTOM );
	    }
	}
	else if ( cPosition.x < 2 ) 
	{
	    return( HIT_SIZE_LEFT );
	}
	else if ( cPosition.x > m_cBounds.right - 2 ) 
	{
	    return( HIT_SIZE_RIGHT );
	}
    }

  
    if ( m_cCloseRect.Contains( cPosition ) ) 
    {
	return( HIT_CLOSE );
    }
    else if ( m_cZoomRect.Contains( cPosition ) ) 
    {
	return( HIT_ZOOM );
    }
    else if ( m_cToggleRect.Contains( cPosition ) ) 
    {
	return( HIT_DEPTH );
    }
    else if ( m_cDragRect.Contains( cPosition ) ) 
    {
	return( HIT_DRAG );
    }
    return( HIT_NONE );
}

void BeIshDecorator::DrawGradientRect(const CRect& r,const Color32_s& sHighlight,const Color32_s &sNormal,const Color32_s &sShadow)
{
   Layer *pcView = GetLayer();
   CRect rGradient(r.left + 2.0,r.top + 2.0,r.right - 2.0,r.bottom - 2.0);
   int height = floor(rGradient.bottom - rGradient.top);
   Gradient gradientHN(sHighlight,sNormal,height);
   Gradient gradientSN(sShadow,sNormal,height);
   for (int i = 0; i < height; i++)
   {
      CPoint left(rGradient.left,rGradient.top + float(i));
      CPoint top(rGradient.left + float(i),rGradient.top);
      pcView->SetFgColor(gradientHN.color[i]);
      pcView->DrawLine(left,top);
      CPoint right(rGradient.right,rGradient.bottom - float(i));
      CPoint bottom(rGradient.right - float(i), rGradient.bottom);
      if (bottom == top)
	continue;
      pcView->SetFgColor(gradientSN.color[i]);
      pcView->DrawLine(right,bottom);
   }
   pcView->SetFgColor(sHighlight);
   pcView->DrawLine(CPoint(r.left + 1,r.top + 1),CPoint(r.left + 1, r.bottom));
   pcView->DrawLine(CPoint(r.left + 2,r.top + 1),CPoint(r.right, r.top + 1));
   pcView->DrawLine(CPoint(r.right,r.top + 1),CPoint(r.right,r.bottom));
   pcView->DrawLine(CPoint(r.left + 1,r.bottom),CPoint(r.right,r.bottom));

   pcView->SetFgColor(sShadow);
   pcView->DrawLine(CPoint(r.left,r.top),CPoint(r.left,r.bottom));
   pcView->DrawLine(CPoint(r.left,r.top),CPoint(r.right,r.top));
   pcView->DrawLine(CPoint(r.right - 1,r.top + 2),CPoint(r.right - 1,r.bottom - 1));
   pcView->DrawLine(CPoint(r.left + 2,r.bottom - 1),CPoint(r.right - 1,r.bottom - 1));
   
}

CRect BeIshDecorator::GetBorderSize()
{
    return( CRect( m_vLeftBorder, m_vTopBorder, m_vRightBorder, m_vBottomBorder ) );
}

void BeIshDecorator::SetTitle( const char* pzTitle )
{
    m_cTitle = pzTitle;
    Render( m_cBounds );
}

void BeIshDecorator::SetFocusState( bool bHasFocus )
{
    m_bHasFocus = bHasFocus;
    Render( m_cBounds );
}

void BeIshDecorator::SetWindowFlags( uint32 nFlags )
{
    m_nFlags = nFlags;
}

void BeIshDecorator::FrameSized( const CRect& cFrame )
{
	Layer* pcView = GetLayer();
	CPoint cDelta( cFrame.Width() - m_cBounds.Width(), cFrame.Height() - m_cBounds.Height() );
	m_cBounds = cFrame.Bounds();
	Layout();

	if ( cDelta.x != 0.0f )
	{
		CRect cDamage = m_cBounds;

		cDamage.left = m_cZoomRect.left - fabs(cDelta.x)  - 2.0f;
		pcView->Invalidate( cDamage );
	}
	if ( cDelta.y != 0.0f )
	{
		CRect cDamage = m_cBounds;

		cDamage.top = cDamage.bottom - max( m_vBottomBorder, m_vBottomBorder + cDelta.y ) - 1.0f;
		pcView->Invalidate( cDamage );
	}
}

void BeIshDecorator::Layout()
{
//   dbprintf("hello"); 
   //if ( m_nFlags & WND_NO_CLOSE_BUT ) 
    {
	m_cCloseRect.left   = 0;
	m_cCloseRect.right  = 0;
	m_cCloseRect.top    = 0;
	m_cCloseRect.bottom = 0;
    } 
    //else 
    {
	m_cCloseRect.left = 4;
//	m_cCloseRect.right = m_vTopBorder - 3;
	m_cCloseRect.right = 14 + 4;
	m_cCloseRect.top = 4;
//	m_cCloseRect.bottom = m_vTopBorder - 3;
	m_cCloseRect.bottom = 14 + 4;
//        dbprintf("hello");
    }

    m_cToggleRect.right = m_cBounds.right - 5;
    if ( m_nFlags & WND_NO_DEPTH_BUT ) 
    {
	m_cToggleRect.left = m_cToggleRect.right;
    }
   else 
    {
	m_cToggleRect.left = m_cToggleRect.right - 14.0f;
    }
    m_cToggleRect.top = 4;
//    m_cToggleRect.bottom = m_vTopBorder - 3;
    m_cToggleRect.bottom = m_cToggleRect.top + 14;
    

    if ( m_nFlags & WND_NO_ZOOM_BUT ) 
    {
	m_cZoomRect.left  = m_cToggleRect.left;
	m_cZoomRect.right = m_cToggleRect.left;
    }
    else 
    {
	if ( m_nFlags & WND_NO_DEPTH_BUT ) 
	{
	    m_cZoomRect.right = m_cBounds.right;
	}
        else
	{
	    m_cZoomRect.right  = m_cToggleRect.left - 6.0f;
	}
	m_cZoomRect.left   = m_cZoomRect.right - 16.0f;
    }
    m_cZoomRect.top  = 4;
    m_cZoomRect.bottom  = m_cZoomRect.top + 14;
//    m_cZoomRect.bottom  = m_vTopBorder - 3;

    if ( m_nFlags & WND_NO_CLOSE_BUT ) 
    {
	m_cDragRect.left  = 0.0f;
    }
    else
    {
	m_cDragRect.left  = m_cCloseRect.right + 1.0f;
//	m_cDragRect.left  = 0.0f;
    }
    if ( m_nFlags & WND_NO_ZOOM_BUT ) 
    {
	if ( m_nFlags & WND_NO_DEPTH_BUT ) 
	{
	    m_cDragRect.right = m_cBounds.right;
	}
        else
	{
	    m_cDragRect.right = m_cToggleRect.left - 1.0f;
	}
    }
    else
    {
	m_cDragRect.right  = m_cZoomRect.left - 1.0f;
    }
    m_cDragRect.top  = 1;
//    m_cDragRect.bottom  = m_vTopBorder - 2;
    m_cDragRect.bottom  = m_cDragRect.top + 20.0;
}

void BeIshDecorator::SetCloseButtonState( bool bPushed )
{
    m_bCloseState = bPushed;
    if ( (m_nFlags & WND_NO_CLOSE_BUT) == 0 ) 
    {
	Color32_s sFillColor =  m_bHasFocus ? get_default_color( COL_SEL_WND_BORDER ) : get_default_color( COL_NORMAL_WND_BORDER );
	DrawClose( m_cCloseRect, sFillColor, m_bCloseState == 1 );
    }
}

void BeIshDecorator::SetZoomButtonState( bool bPushed )
{
    m_bZoomState = bPushed;
    if ( (m_nFlags & WND_NO_ZOOM_BUT) == 0 ) 
    {
        Color32_s sFillColor =  m_bHasFocus ? get_default_color( COL_SEL_WND_BORDER ) : get_default_color( COL_NORMAL_WND_BORDER );
	DrawZoom( m_cZoomRect, sFillColor, m_bZoomState == 1 );
    }
}

void BeIshDecorator::SetDepthButtonState( bool bPushed )
{
    m_bDepthState = bPushed;
    if ( (m_nFlags & WND_NO_DEPTH_BUT) == 0 ) 
    {
	Color32_s sFillColor =  m_bHasFocus ? get_default_color( COL_SEL_WND_BORDER ) : get_default_color( COL_NORMAL_WND_BORDER );
        DrawDepth( m_cToggleRect, sFillColor, m_bDepthState == 1 );
    }
}


void BeIshDecorator::DrawDepth( const CRect& cRect, const Color32_s& sFillColor, bool bRecessed )
{
    Layer* pcView = GetLayer();
    pcView->SetBgColor(sFillColor);
    CRect rBar;
    rBar.left = cRect.left - 3;
    rBar.top = cRect.top - 3;
    rBar.right = cRect.right + 3;
    rBar.bottom = cRect.bottom + 3;
    pcView->FillRect(rBar,sFillColor);
    
    Color32_s sHighlight = get_default_color( COL_SHINE );
    sHighlight.red /= 2;
    sHighlight.green /= 2;
    sHighlight.blue /= 2;
    sHighlight.red += sFillColor.red / 2;
    sHighlight.green += sFillColor.green / 2;
    sHighlight.blue += sFillColor.blue / 2;

    Color32_s sShadow = get_default_color(COL_SHADOW);
    sShadow.red /= 2;
    sShadow.green /= 2;
    sShadow.blue /= 2;
    sShadow.red += sFillColor.red / 2;
    sShadow.green += sFillColor.green / 2;
    sShadow.blue += sFillColor.blue / 2;
    
    pcView->SetFgColor(sHighlight);
    pcView->DrawLine(CPoint(rBar.left,rBar.top),CPoint(rBar.right,rBar.top));
    pcView->SetFgColor(sShadow);
    pcView->DrawLine(CPoint(rBar.right,rBar.bottom),CPoint(rBar.right,rBar.top));

    CRect	L,R;

    L.left = cRect.left;
    L.top = cRect.top;

    L.right = L.left + 11.0f;
    L.bottom = L.top + 11.0f;

    R.left = L.left + 3.0f;
    R.top = L.top + 3.0f;

    R.right = R.left + 11.0f;
    R.bottom = R.top + 11.0f;

  
    if ( bRecessed )
    {
        DrawGradientRect(L,sShadow,sFillColor,sHighlight);
        pcView->FillRect(R,sFillColor);
        DrawGradientRect(R,sShadow,sFillColor,sHighlight);
    }
    else
    {
        DrawGradientRect(L,sHighlight,sFillColor,sShadow);
        pcView->FillRect(R,sFillColor);
        DrawGradientRect(R,sHighlight,sFillColor,sShadow);
    }
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void BeIshDecorator::DrawZoom( const CRect& cRect, const Color32_s& sFillColor, bool bRecessed )
{

    Layer* pcView = GetLayer();
    CRect rBar;
    rBar.left = cRect.left - 3;
    rBar.top = cRect.top - 3;
    rBar.right = cRect.right + 3;
    rBar.bottom = cRect.bottom + 3;
    pcView->FillRect(rBar,sFillColor);
    
    Color32_s sHighlight = get_default_color(COL_SHINE);
    sHighlight.red /= 2;
    sHighlight.green /= 2;
    sHighlight.blue /= 2;
    sHighlight.red += sFillColor.red / 2;
    sHighlight.green += sFillColor.green / 2;
    sHighlight.blue += sFillColor.blue / 2;
   
    pcView->SetFgColor(sHighlight);
    //pcView->DrawLine(CPoint(rBar.left,rBar.bottom),Point(rBar.left,rBar.top));
    pcView->DrawLine(CPoint(rBar.left,rBar.top),CPoint(rBar.right,rBar.top));

    Color32_s sShadow = get_default_color(COL_SHADOW);
    sShadow.red /= 2;
    sShadow.green /= 2;
    sShadow.blue /= 2;
    sShadow.red += sFillColor.red / 2;
    sShadow.green += sFillColor.green / 2;
    sShadow.blue += sFillColor.blue / 2;
    
    CRect	L,R;

    L.left = cRect.left;
    L.top = cRect.top;

    L.right = L.left + 9.0f;
    L.bottom = L.top + 9.0f;

    R.left = L.left + 3.0f;
    R.top = L.top + 3.0f;

    R.right = R.left + 11.0f;
    R.bottom = R.top + 11.0f;

  
    if ( bRecessed )
    {
        DrawGradientRect(R,sShadow,sFillColor,sHighlight);
        pcView->FillRect(L,sFillColor);
        DrawGradientRect(L,sShadow,sFillColor,sHighlight);
    }
    else
    {
        DrawGradientRect(R,sHighlight,sFillColor,sShadow);
        pcView->FillRect(L,sFillColor);
        DrawGradientRect(L,sHighlight,sFillColor,sShadow);
    }
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void BeIshDecorator::DrawClose( const CRect& cRect, const Color32_s& sFillColor, bool bRecessed  )
{
//    CRect L;

//    L.left = cRect.left + ((cRect.Width()+1.0f) / 3);
//    L.top = cRect.top + ((cRect.Height()+1.0f) / 3);

//   L.right = cRect.right - ((cRect.Width()+1.0f) / 3);
//    L.bottom = cRect.bottom - ((cRect.Height()+1.0f) / 3);

    Layer* pcView = GetLayer();
    CRect rBar;
    rBar.left = cRect.left - 3;
    rBar.top = cRect.top - 3;
    rBar.right = cRect.right + 3;
    rBar.bottom = cRect.bottom + 3;
    pcView->FillRect(rBar,sFillColor);
   
    Color32_s sHighlight = get_default_color(COL_SHINE);
    sHighlight.red /= 2;
    sHighlight.green /= 2;
    sHighlight.blue /= 2;
    sHighlight.red += sFillColor.red / 2;
    sHighlight.green += sFillColor.green / 2;
    sHighlight.blue += sFillColor.blue / 2;

    pcView->SetFgColor(sHighlight);
    pcView->MovePenTo(CPoint(rBar.left,rBar.bottom));
//    pcView->DrawLine(CPoint(rBar.left,rBar.bottom),CPoint(rBar.left,rBar.top));
//    pcView->DrawLine(CPoint(rBar.left,rBar.top),CPoint(rBar.right,rBar.top));
    pcView->DrawLine(CPoint(rBar.left,rBar.top));
    pcView->DrawLine(CPoint(rBar.right,rBar.top));

   Color32_s sShadow = get_default_color(COL_SHADOW);
    sShadow.red /= 2;
    sShadow.green /= 2;
    sShadow.blue /= 2;
    sShadow.red += sFillColor.red / 2;
    sShadow.green += sFillColor.green / 2;
    sShadow.blue += sFillColor.blue / 2;
  
    if ( bRecessed ) 
    {
        DrawGradientRect(cRect,sShadow,sFillColor,sHighlight);
//	pcView->FillRect( cRect, sFillColor );
//	pcView->DrawFrame( cRect, FRAME_RECESSED | FRAME_TRANSPARENT );
//	pcView->DrawFrame( L, FRAME_RAISED | FRAME_TRANSPARENT );
    }
    else 
    {
        DrawGradientRect(cRect,sHighlight,sFillColor,sShadow);
//	pcView->FillRect( cRect, sFillColor );
//	pcView->DrawFrame( cRect, FRAME_RAISED | FRAME_TRANSPARENT );
//	pcView->DrawFrame( L, FRAME_RECESSED | FRAME_TRANSPARENT );
    }
}

void BeIshDecorator::Render( const CRect& cUpdateRect )
{
    if ( m_nFlags & WND_NO_BORDER ) 
    {
	return;
    }
  
    Color32_s sFillColor =  m_bHasFocus ? get_default_color( COL_SEL_WND_BORDER ) : get_default_color( COL_NORMAL_WND_BORDER );

    Layer* pcView = GetLayer();
 
    CRect  cOBounds = pcView->GetBounds();
    
    pcView->SetFgColor(get_default_color(COL_SHADOW));
    pcView->DrawLine(CPoint(cOBounds.left,cOBounds.bottom),CPoint(cOBounds.right,cOBounds.bottom));
    pcView->DrawLine(CPoint(cOBounds.right,cOBounds.top));
    //remove shadow
    cOBounds.right -= 1;
    cOBounds.bottom -= 1;
   
    //pcView->DrawRect(cOBounds,get_default_color(COL_NORMAL_WND_BORDER));
    pcView->SetFgColor(get_default_color(COL_NORMAL_WND_BORDER));
    pcView->DrawLine(CPoint(cOBounds.left,cOBounds.bottom),CPoint(cOBounds.left,cOBounds.top));
    pcView->DrawLine(CPoint(cOBounds.left,cOBounds.top),CPoint(cOBounds.right,cOBounds.top));
    pcView->DrawLine(CPoint(cOBounds.right,cOBounds.top),CPoint(cOBounds.right,cOBounds.bottom));
    pcView->DrawLine(CPoint(cOBounds.right,cOBounds.bottom),CPoint(cOBounds.left,cOBounds.bottom));

    CRect  cIBounds = cOBounds;    
    cIBounds.left += m_vLeftBorder - 3;
    cIBounds.right -= m_vRightBorder - 4;
    cIBounds.top += m_vTopBorder - 3;
    cIBounds.bottom -= m_vBottomBorder - 4;
    
    Color32_s inbetween = get_default_color(COL_NORMAL);
    inbetween.red /= 2;
    inbetween.green /= 2;
    inbetween.blue /= 2;

    Color32_s darker = get_default_color(COL_NORMAL_WND_BORDER);
    darker.red /= 2;
    darker.green /= 2;
    darker.blue /= 2;
    
    inbetween.red += darker.red;
    inbetween.blue += darker.blue;
    inbetween.green += darker.green;
   
    pcView->SetFgColor(inbetween);
    pcView->DrawLine(CPoint(cIBounds.left,cIBounds.bottom),CPoint(cIBounds.right,cIBounds.bottom));
    pcView->DrawLine(CPoint(cIBounds.right,cIBounds.top));
    pcView->DrawLine(CPoint(cIBounds.left,cIBounds.top));
    pcView->DrawLine(CPoint(cIBounds.left,cIBounds.bottom));

    pcView->SetFgColor(get_default_color(COL_NORMAL_WND_BORDER));
    pcView->DrawLine(CPoint(cIBounds.left,cIBounds.bottom + 1),CPoint(cIBounds.right+1,cIBounds.bottom + 1));
    pcView->DrawLine(CPoint(cIBounds.right + 1, cIBounds.top - 1));
    pcView->SetFgColor(get_default_color(COL_SHINE));
    pcView->DrawLine(CPoint(cIBounds.left - 1, cIBounds.top - 1));
    pcView->DrawLine(CPoint(cIBounds.left - 1, cIBounds.bottom));

    //extra line 
    pcView->SetFgColor(get_default_color(COL_NORMAL));
    pcView->DrawLine(CPoint(cIBounds.left,cIBounds.top - 1),CPoint(cIBounds.right,cIBounds.top - 1));
    pcView->DrawLine(CPoint(cIBounds.left,cIBounds.top),CPoint(cIBounds.right,cIBounds.top));
   
    CRect  cIIBounds = cOBounds;
    cIIBounds.left += m_vLeftBorder - 1;
    cIIBounds.right -= m_vRightBorder - 2;
    cIIBounds.top += m_vTopBorder - 1;
    cIIBounds.bottom -= m_vBottomBorder - 2;

    pcView->SetFgColor(get_default_color(COL_NORMAL_WND_BORDER));
    pcView->DrawLine(CPoint(cIIBounds.left,cIIBounds.bottom),CPoint(cIIBounds.right,cIIBounds.bottom));
    pcView->DrawLine(CPoint(cIIBounds.right,cIIBounds.top));
    pcView->DrawLine(CPoint(cIIBounds.left,cIIBounds.top));
    pcView->DrawLine(CPoint(cIIBounds.left,cIIBounds.bottom));

    pcView->SetFgColor(get_default_color(COL_SHINE));
    pcView->DrawLine(CPoint(cIIBounds.left,cIIBounds.bottom + 1),CPoint(cIIBounds.right+1,cIIBounds.bottom + 1));
    pcView->DrawLine(CPoint(cIIBounds.right + 1, cIIBounds.top - 1));
    pcView->SetFgColor(get_default_color(COL_NORMAL_WND_BORDER));
    pcView->DrawLine(CPoint(cIIBounds.left - 1, cIIBounds.top - 1));
    pcView->DrawLine(CPoint(cIIBounds.left - 1, cIIBounds.bottom + 1));
   

    if ( (m_nFlags & WND_NOT_RESIZABLE) != WND_NOT_RESIZABLE ) 
    {
	pcView->SetFgColor( 0, 0, 0, 255 );
	pcView->DrawLine( CPoint( 15, cIBounds.bottom + 1 ), CPoint( 15, cOBounds.bottom - 1 ) );
	pcView->DrawLine( CPoint( cOBounds.right - 16, cIBounds.bottom + 1 ), CPoint( cOBounds.right - 16, cOBounds.bottom - 1 ) );

	pcView->SetFgColor( 255, 255, 255, 255 );
	pcView->DrawLine( CPoint( 16, cIBounds.bottom + 1 ), CPoint( 16, cOBounds.bottom - 1 ) );
	pcView->DrawLine( CPoint( cOBounds.right - 15, cIBounds.bottom + 1 ), CPoint( cOBounds.right - 15, cOBounds.bottom - 1 ) );
    }  
    if ( (m_nFlags & WND_NO_CLOSE_BUT) == 0 ) 
    {
	DrawClose( m_cCloseRect, sFillColor, m_bCloseState == 1 );
    }

    if ( (m_nFlags & WND_NO_TITLE) == 0 ) 
    {
	pcView->FillRect( m_cDragRect, sFillColor );
        pcView->SetFgColor( 0, 0, 0, 0);
	//pcView->SetFgColor( 255, 255, 255, 0 );
	pcView->SetBgColor( sFillColor );
	pcView->MovePenTo( m_cDragRect.left + 17,
			   (m_cDragRect.Height()+1.0f) / 2 -
			   (m_sFontHeight.ascender + m_sFontHeight.descender) / 2 + m_sFontHeight.ascender +
			   m_sFontHeight.line_gap * 0.5f + 1);
    
       const char *title = m_cTitle.c_str();
       int titlewidth = pcView->m_pcFont->GetInstance()->GetStringWidth(title,strlen(title));
        
//        printf("titlewidth = %d",titlewidth);
        pcView->DrawString( m_cTitle.c_str(), -1 );
        Color32_s sHighlight = get_default_color(COL_SHINE);
        sHighlight.red /= 2;
        sHighlight.green /= 2;
        sHighlight.blue /= 2;
        sHighlight.red += sFillColor.red / 2;
        sHighlight.green += sFillColor.green / 2;
        sHighlight.blue += sFillColor.blue / 2;
       
        pcView->SetFgColor(sHighlight);
        pcView->DrawLine(CPoint(m_cDragRect.left,m_cDragRect.top),CPoint(m_cDragRect.right,m_cDragRect.top));
        int rightside = m_cDragRect.left + 36 + titlewidth;
        if (rightside > m_cDragRect.right)
	 rightside = m_cDragRect.right;
        //pcView->DrawLine(CPoint(rightside,m_cDragRect.top),CPoint(rightside,m_cDragRect.bottom));
	//pcView->DrawFrame( m_cDragRect, FRAME_RAISED | FRAME_TRANSPARENT );
    } else 
    {
	pcView->FillRect( CRect( cOBounds.left + 1, cOBounds.top - 1, cOBounds.right - 1, cIBounds.top + 1 ), sFillColor );
    }

    if ( (m_nFlags & WND_NO_ZOOM_BUT) == 0 ) 
    {
	DrawZoom( m_cZoomRect, sFillColor, m_bZoomState == 1 );
    }
    if ( (m_nFlags & WND_NO_DEPTH_BUT) == 0 ) {
	DrawDepth( m_cToggleRect, sFillColor, m_bDepthState == 1 );
    }
}

extern "C" int get_api_version()
{
    return( WNDDECORATOR_APIVERSION );
}

extern "C" WindowDecorator* create_decorator( Layer* pcLayer, uint32 nFlags )
{
    return( new BeIshDecorator( pcLayer, nFlags ) );
}
