/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef	__HANDLER_H__
#define	__HANDLER_H__


#define BHandler	os::CHandler


#include <support/SupportDefs.h>
#include <string>
#include <list>

namespace os
{

class CMessage;
class CLooper;
class CMessageFilter;


typedef std::list<CMessageFilter*> MsgFilterList;	// FIXME: change to a CList

class CHandler
{
public:
					CHandler( const char* inName = NULL );
	virtual 		~CHandler();

	virtual void	TimerTick( int nID );
	
	virtual void	MessageReceived( CMessage* pcMessage );
	CLooper*		Looper() const;
	
	const char*		Name() const;
	void	 		SetName( const char* cName );
	
	void	 		SetNextHandler( CHandler* pcNextHandler );
	CHandler*		NextHandler() const;

	void			AddFilter( CMessageFilter* pcFilter );
	void			RemoveFilter( CMessageFilter* pcFilter );

	const			MsgFilterList& FilterList() const;
	
	bool			LockLooper();
	status_t		LockLooperWithTimeout(bigtime_t inTimeout);
	void			UnlockLooper();


private:
	friend class CLooper;
	friend class CMessage;
	friend class CMessenger;
	friend class NodeMonitor;
	friend class CWindow;

	virtual void	FBC_Handler1();
	virtual void	FBC_Handler2();
	virtual void	FBC_Handler3();
	virtual void	FBC_Handler4();
	virtual void	FBC_Handler5();

	std::string		mName;
	CHandler*		m_pcNextHandler;	// Next handler for unknown messages.
	CLooper*		m_pcLooper;
	int		  		m_nToken;
	MsgFilterList	mFilterList;
	uint32			__res[6];
};

}

#endif
