/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef	__LOOPER_H__
#define	__LOOPER_H__

#include <support/SupportDefs.h>
#include <cosmoe/OS.h>
#include <cosmoe/msgport.h>
#include <app/Handler.h>
#include <app/MessageQueue.h>
#include <support/Locker.h>

#include <map>

#define BLooper		os::CLooper

namespace os
{



class	CLooper : public CHandler
{
public:
	typedef std::map<int,CHandler*> handler_map;

						CLooper( const char* cName,
								 int nPriority = NORMAL_PRIORITY,
								 int nPortSize = DEFAULT_PORT_SIZE );
	virtual				~CLooper();

	void				SetName( const char* cName );
	const char* 		Name() const;

	port_id				GetMsgPort() const;
	thread_id			GetThread() const;
	proc_id				GetProcess() const;

	bool				Lock();
	status_t			LockWithTimeout( bigtime_t nTimeout );
	status_t			SafeLock();
	status_t			Unlock();

	void				SetMutex( CLocker* pcMutex );
	CLocker*			GetMutex() const;

	int					CountLocks() const;
	thread_id			LockingThread() const;

	bool				IsLocked() const;

	virtual thread_id	Run();
	int					Wait() const;

	status_t			PostMessage( uint32 nCode );
	status_t			PostMessage( CMessage *pcMsg );
	status_t			PostMessage( uint32 cCode, CHandler* pcHandler, CHandler* pcReplyTo = NULL );
	status_t			PostMessage( CMessage *pcMsg, CHandler *pcHandler, CHandler* pcReplyTo = NULL );

	void				SpoolMessages();
	CMessage*			CurrentMessage() const;
	CMessage*			DetachCurrentMessage();
	virtual void		DispatchMessage( CMessage* pcMessage, CHandler* pcHandler );
	virtual void		Started();
	virtual bool		Idle();

	CMessageQueue*		MessageQueue() const;

	virtual bool		QuitRequested();
	virtual void		Quit();

	void				AddTimer( CHandler* pcTarget, int nID, bigtime_t nPeriod, bool bOneShot = true );
	bool				RemoveTimer( CHandler* pcTarget, int nID );

	const handler_map&	GetHandlerMap() const;
	
	void				AddHandler( CHandler* pcHandler );
	bool				RemoveHandler( CHandler* pcHandler );
	CHandler*			FindHandler( const std::string& cName ) const;
	int32				CountHandlers() const;

	void				SetPreferredHandler( CHandler* pcHandler );
	CHandler*			PreferredHandler() const;

	void				AddCommonFilter( CMessageFilter* pcFilter );
	void				RemoveCommonFilter( CMessageFilter* pcFilter );

	const MsgFilterList& CommonFilterList() const;
	static CLooper*		LooperForThread( thread_id hThread ) { return( _GetLooperForThread( hThread ) ); }

private:
	friend class CApplication;
	friend class CMessenger;
	friend class NodeMonitor;
	friend class CWindow;

	virtual void		FBC_Looper1();
	virtual void		FBC_Looper2();
	virtual void		FBC_Looper3();
	virtual void		FBC_Looper4();
	virtual void		FBC_Looper5();
	virtual void		FBC_Looper6();
	virtual void		FBC_Looper7();
	virtual void		FBC_Looper8();
	
	struct TimerNode
	{
		TimerNode( CHandler* pcHandler, int nID, bigtime_t nPeriod, bool bOneShot );

		TimerNode*		m_pcNext;
		bigtime_t		m_nTimeout;

		CHandler*		m_pcHandler;
		int				m_nID;
		bigtime_t		m_nPeriod;
		bool			m_bOneShot;
	};


	void				_DecodeMessage( uint32 nCode, const void* pData );
	CHandler*			_FindHandler( int nToken ) const;
	void				_Loop( void );
	static uint32		_Entry( void* pData );
	port_id				_GetPort() const;
	void				_InsertTimer( TimerNode* pcTimer );
	void				_SetThread( thread_id hThread );

	class Private;
	Private* m;

	static void			_AddLooper( CLooper* pcLooper );
	static void			_RemoveLooper( CLooper* pcLooper );
	static CLooper*		_GetLooperForThread( thread_id hThread );
	static CLooper*		_GetLooperForPort( port_id hPort );
	static bool			_IsLooperValid( CLooper* pcLooper );
};

}
#endif
