/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef	__MESSAGE_H__
#define	__MESSAGE_H__

#include <sys/types.h>
#include <app/AppDefs.h>
#include <support/SupportDefs.h>
#include <support/TypeConstants.h>

#include <string>

#define BMessage	os::CMessage

namespace os
{

class CRect;
class IRect;
class CPoint;
class IPoint;
class CLooper;
class CHandler;
class CMessenger;
class Variant;
struct EntryRef;
struct Color32_s;


/** 
 * \ingroup app
 * \par Description:
 *
 * \sa os::CLooper, os::CHandler
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CMessage
{
public:

	uint32		what;

				CMessage( uint32 inCode = 0UL );
				CMessage( const CMessage& cMsg );
				CMessage( const void* pFlattenedData, bool inThrowError );
				~CMessage();

	size_t		FlattenedSize( void ) const;
	status_t	Flatten( uint8* pBuffer, size_t nSize ) const;
	status_t	Unflatten( const uint8* pBuffer );

	status_t	AddData( const char* pzName, type_code nType, const void* pData,
						 uint32 nSize, bool bFixedSize = true, int nMaxCountHint = 1 );

	status_t	AddMessage( const char* pzName, const CMessage* pcVal );
	status_t	AddMessenger(const char *pzName, const CMessenger& cVal);
	status_t	AddPointer( const char* pzName, const void* pVal );
	status_t	AddInt8( const char* pzName, int8 nVal );
	status_t	AddInt16( const char* pzName, int16 nVal );
	status_t	AddInt32( const char* pzName, int32 nVal );
	status_t	AddInt64( const char* pzName, int64 nVal );
	status_t	AddBool( const char* pzName, bool bVal );
	status_t	AddFloat( const char* pzName, float vVal );
	status_t	AddDouble( const char* pzName, double vVal );
	status_t	AddRect( const char* pzName, const CRect& cVal );
	status_t	AddIRect( const char* pzName, const IRect& cVal );
	status_t	AddPoint( const char* pzName, const CPoint& cVal );
	status_t	AddIPoint( const char* pzName, const IPoint& cVal );
	status_t	AddColor32( const char* pzName, const Color32_s& cVal );
	status_t	AddString( const char* pzName, const char* pzString );
	status_t	AddString( const char* pzName, const std::string& cString );
	status_t	AddVariant( const char* pzName, const Variant& cVal );
	status_t	AddRef(const char *name, const EntryRef* pVal);

	status_t	FindData( const char* pzName, type_code nType, const void** ppData, size_t* pnSize, int nIndex = 0 ) const;

	status_t	FindMessage( const char* pzName, CMessage* ppcVal, int nIndex = 0 ) const;
	status_t	FindMessenger(const char *name, CMessenger* ppcVal, int nIndex = 0) const;
	status_t	FindPointer( const char* pzName, void** ppVal, int nIndex = 0 ) const;
	status_t	FindInt8( const char* pzName, int8* pnVal, int nIndex = 0 ) const;
	status_t	FindInt16( const char* pzName, int16* pnVal, int nIndex = 0 ) const;
	status_t	FindInt32( const char* pzName, int32* pnVal, int nIndex = 0 ) const;
	status_t	FindInt64( const char* pzName, int64* pnVal, int nIndex = 0 ) const;
	status_t	FindBool( const char* pzName, bool* pbVal, int nIndex = 0 ) const;
	status_t	FindFloat( const char* pzName, float* pvVal, int nIndex = 0 ) const;
	status_t	FindDouble( const char* pzName, double* pvVal, int nIndex = 0 ) const;
	status_t	FindRect( const char* pzName, CRect* pcVal, int nIndex = 0 ) const;
	status_t	FindIRect( const char* pzName, IRect* pcVal, int nIndex = 0 ) const;
	status_t	FindPoint( const char* pzName, CPoint* pcVal, int nIndex = 0 ) const;
	status_t	FindIPoint( const char* pzName, IPoint* pcVal, int nIndex = 0 ) const;
	status_t	FindColor32( const char* pzName, Color32_s* pcVal, int nIndex = 0 ) const;
	status_t	FindString( const char* pzName, const char** pzString, int nIndex = 0 ) const;
	status_t	FindString( const char* pzName, std::string* pcString, int nIndex = 0 ) const;
	status_t	FindVariant( const char* pzName, Variant* pcVal, int nIndex = 0 ) const;
	status_t	FindRef( const char* pzName, EntryRef* pcVal, int nIndex = 0 ) const;

	// Deprecated functions needed for Deskbar/Tracker
	
	int32		FindInt32(const char *pzName, int32 nIndex = 0L) const;
	const char*	FindString(const char *pzName, int32 nIndex = 0L) const;
	
	template<class T> status_t FindInt(  const char* pzName, T* pnVal, int nIndex = 0 ) const {
	int32 nValue;
	status_t nError = FindInt32( pzName, &nValue, nIndex );
	if ( nError >= 0 ) {
		*pnVal = nValue;
	}
	return( nError );
	}
	status_t	RemoveData( const char* pzName, int nIndex = 0 );
	status_t	RemoveName( const char* pzName );

	status_t	GetInfo( const char* pzName, type_code* pnType = NULL, int* pnCount = NULL ) const;
	int32		CountNames( void ) const;
	std::string GetName( int nIndex ) const;

	void		MakeEmpty( void );
	bool		IsEmpty( void ) const;

	bool		WasDelivered( void ) const;
	bool		IsSourceWaiting( void ) const;
	bool		IsSourceRemote( void ) const;
	CMessenger	ReturnAddress( void ) const;
	bool		IsReply( void ) const;

	status_t	SendReply( uint32 nCode, CHandler* pcReplyHandler = NULL );
	status_t	SendReply( CMessage* pcTheReply, CHandler* pcReplyHandler = NULL, uint nTimeOut = ~0 );
	status_t	SendReply( uint32 nCode, CMessage* pcReplyToReply );
	status_t	SendReply( CMessage* pcTheReply, CMessage* pcReplyToReply,
							int nSendTimOut = ~0, int nReplyTimeOut = ~0 );

	CMessage&	operator=(const CMessage& cOther );

private:
	friend class CLooper;
	friend class CMessageQueue;
	friend class CMessenger;
	friend class CView;

	struct DataArray_s
	{
		DataArray_s*	psNext;
		int				nMaxSize;		// Number of bytes allocated
		int				nCurSize;		// Number of bytes added
		int				nChunkSize;		// Size of each element in a fixed size array, or 0 if the size can vary between elements
		int32			nCount;			// Number of elements in array
		type_code		nTypeCode;		// What kind of data to be found in the array
		uint8			nNameLength;	// Number of bytes in the name string
		// nNameLength sized name follows
		// Array of Chunk_s structs (or raw data chunks if fixed size) follows
	};

	struct Chunk_s
	{
	int	nDataSize;
	};

	Chunk_s*	GetChunkAddr( DataArray_s* psArray, int nIndex )  const;
	uint8*		CreateArray(  const char* pzName, type_code nType, const void* pData, uint32 nSize,
							bool bFixedSize = true, int nMaxCountHint = 1 );
	uint8*		ExpandArray( DataArray_s* psArray, const void* pData, uint32 nSize );

	void		AddArray( DataArray_s* psArray );
	void		RemoveArray( DataArray_s* psArray );
	DataArray_s*	FindArray( const char* pzName, type_code nType ) const;

	void		SetReplyHandler( CHandler* pcHandler );
	status_t	Post( port_id hPort, int nTargetToken, port_id hReplyPort = -1,
						int nReplyTarget = -1, proc_id hReplyProc = -1 );
	status_t	ReadPort( port_id hPort );
	int			GetStaticSize( void ) const { return( sizeof(what) + sizeof(m_nFlags) + sizeof(m_nTargetToken) + sizeof(m_nReplyToken) + sizeof(m_hReplyProc) + sizeof(m_hReplyPort) + sizeof( int ) ); }

	// Definitions for m_nFlags
	enum { REPLY_REQUIRED = 0x01, IS_REPLY = 0x02, WAS_DELIVERED = 0x04, REPLY_SENT = 0x00010000 };

	CMessage*	m_pcNext;

	uint32		m_nFlags;
	
	int			m_nTargetToken;
	int			m_nReplyToken;
	
	port_id	 	m_hReplyPort;
	proc_id	 	m_hReplyProc;
//    bool	 	m_bReplyDone;
	int		 	m_nFlattenedSize;

	DataArray_s* m_psFirstArray;
};

}

#endif	// __MESSAGE_H__
