/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __MESSAGEFILTER_H__
#define __MESSAGEFILTER_H__

#include <list>
#include <support/SupportDefs.h>

#define BMessageFilter		os::CMessageFilter
#define B_SKIP_MESSAGE		os::kSkipMessage
#define B_DISPATCH_MESSAGE	os::kDispatchMessage
#define filter_hook			os::FilterHookFunc
#define filter_result		os::FilterResult

namespace os
{

class CLooper;
class CMessage;
class CHandler;
class CMessageFilter;

enum FilterResult
{
	kSkipMessage,
	kDispatchMessage
};

typedef FilterResult (*FilterHookFunc)
	(CMessage *message, CHandler **target, CMessageFilter *filter);

class CMessageFilter
{
public:
							CMessageFilter();
							CMessageFilter( uint32 inCommand,
										FilterHookFunc inFilterFunc = NULL );
	virtual					~CMessageFilter();

	virtual FilterResult	Filter( CMessage* pcMessage, CHandler** ppcHandler ) = 0;

	CHandler*				Handler() const			  { return mHandler;			}
	CLooper*				Looper() const			  { return mLooper;				}
	bool					FiltersAnyCommand() const { return mFiltersAnyCommand;	}
	uint32					Command() const			  { return mCommand;			}

private:
	friend class CHandler;
	friend class CLooper;

	class Private;
	Private* 				m;

	CHandler* 				mHandler;
	CLooper*				mLooper;
	std::list<CMessageFilter*>::iterator mIterator;
	uint32   				mCommand;
	bool     				mFiltersAnyCommand;
	bool					mGlobalFilter;
};


}


#endif
