/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef	__MESSENGER_H__
#define	__MESSENGER_H__


#include <support/SupportDefs.h>

// FIXME: g++ 2.95.4 does not like the forward decls for default parameters
//        below so we include the header files instead.
#include <app/Message.h>
#include <app/Looper.h>
#include <app/Handler.h>

// The messenger has some friends in the application server
class	SrvWindow;
class	SrvApplication;
class	SrvWidget;

#define	BMessenger	os::CMessenger

namespace os
{

/** 
 * \ingroup app
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class	CMessenger
{
public:
					CMessenger();
					CMessenger( const char *inMimeSig, 
								team_id inTeamID = -1,
								status_t *outErr = NULL);
					CMessenger( const CHandler* pcHandler,
								const CLooper* pcLooper = NULL );
					CMessenger( const CMessenger& cMessenger );
					CMessenger( port_id hPort );
	
					~CMessenger();

	CHandler*		Target( CLooper** ppcLooper ) const;
	bool			IsTargetLocal( void ) const;

	bool			LockTarget( void ) const;
	status_t		LockTargetWithTimeout(bigtime_t inDuration) const;

	bool			IsValid( void ) const;
	
	status_t		SendMessage( CMessage* pcMessage, CMessage* pcReply, bigtime_t nSendTimeOut = INFINITE_TIMEOUT,
								bigtime_t nReplyTimeOut = INFINITE_TIMEOUT ) const;
	status_t		SendMessage( CMessage* pcMessage, CHandler* pcReplyHandler = NULL,
								bigtime_t nTimeOut = INFINITE_TIMEOUT ) const;
	status_t		SendMessage( uint32 nCode, CMessage* pcReply = NULL ) const;
	status_t		SendMessage( uint32 nCode, CHandler* pcReplyHandler ) const;


	CMessenger&		operator =(const CMessenger& cMessenger);
	bool			operator ==(const CMessenger& cMessenger) const;

private:
	friend class CMessage;
	friend class SrvWidget;
	friend class SrvWindow;
	friend class SrvApplication;
	friend class NodeMonitor;

					CMessenger( port_id hPort,
								int nHandlerID,
								proc_id hDestProc );

	port_id			m_hPort;
	int				m_hHandlerID;
	bool			m_bPreferredTarget;
	proc_id			m_hDestProc;
};

}

#endif	//	__MESSENGER_H__
