/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Copyright (C) 2001-2002 Bill Hayden
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __ROSTER_H__
#define __ROSTER_H__

#include <app/Messenger.h>
#include <storage/Entry.h>
#include <storage/StorageDefs.h>


#define BRoster		os::CRoster
#define be_roster	os::CApplication::c_roster
#define app_info	os::AppInfo

#define B_REQUEST_LAUNCHED		0x00000001
#define B_REQUEST_QUIT			0x00000002
#define B_REQUEST_ACTIVATED		0x00000004
#define B_SOME_APP_LAUNCHED		'BRAS'
#define B_SOME_APP_QUIT			'BRAQ'
#define B_SOME_APP_ACTIVATED	'BRAW'

#define B_LAUNCH_MASK				(0x3)

#define B_SINGLE_LAUNCH				(0x0)
#define B_MULTIPLE_LAUNCH			(0x1)
#define B_EXCLUSIVE_LAUNCH			(0x2)
#define B_BACKGROUND_APP			(0x4)
#define B_ARGV_ONLY					(0x8)

namespace os
{

class CList;
class CMessage;


struct AppInfo
{
				AppInfo();
				~AppInfo();

	thread_id	thread;
	team_id		team;
	port_id		port;
	uint32		flags;
	EntryRef	ref;
	char		signature[MIME_TYPE_LEN];
};



class CRoster
{
public:
				CRoster();
				~CRoster();

	bool		IsRunning(const char *mime_sig) const;
	bool		IsRunning(EntryRef *ref) const;
	
	team_id		TeamFor(const char *mime_sig) const;
	team_id		TeamFor(EntryRef *ref) const;
	
	void		GetAppList(CList *team_id_list) const;
	void		GetAppList(const char *sig, CList *team_id_list) const;
	
	status_t	GetAppInfo(const char *sig, AppInfo *info) const;
	status_t	GetAppInfo(EntryRef *ref, AppInfo *info) const;
	
	status_t	GetRunningAppInfo(team_id team, AppInfo *info) const;
	status_t	GetActiveAppInfo(AppInfo *info) const;
	
	status_t	FindApp(const char *mime_type, EntryRef *app) const;
	status_t	FindApp(EntryRef *ref, EntryRef *app) const;

	status_t	Broadcast(CMessage *msg) const;
	status_t	Broadcast(CMessage *msg, BMessenger reply_to) const;
	
	status_t	StartWatching(CMessenger target,
							uint32 event_mask = B_REQUEST_LAUNCHED | B_REQUEST_QUIT) const;
	status_t	StopWatching(CMessenger target) const;
	
	status_t	ActivateApp(team_id team) const;
	
	status_t	Launch(const char *mime_type, CMessage *initial_msgs = NULL,
						team_id *app_team = NULL) const;
	status_t	Launch(const char *mime_type, CList *message_list,
						team_id *app_team = NULL) const;
	status_t	Launch(const char *mime_type, int argc, char **args,
						team_id *app_team = NULL) const;
	status_t	Launch(const EntryRef *ref, const CMessage *initial_message = NULL,
						team_id *EntryRef = NULL) const;
	status_t	Launch(const EntryRef *ref, const CList *message_list,
						team_id *app_team = NULL) const;
	status_t	Launch(const EntryRef *ref, int argc, const char * const *args,
						team_id *app_team = NULL) const;

	void		GetRecentDocuments(	CMessage *refList, int32 maxCount,
									const char *ofType = NULL,
									const char *openedByAppSig = NULL) const;
	void		GetRecentDocuments( CMessage *refList, int32 maxCount,
									const char *ofTypeList[], int32 ofTypeListCount,
									const char *openedByAppSig = NULL) const;
	void		GetRecentFolders(CMessage *refList, int32 maxCount,
								const char *openedByAppSig = NULL) const;
	void		GetRecentApps(CMessage *refList, int32 maxCount) const;

	void		AddToRecentDocuments(const EntryRef *doc,
									const char *appSig = NULL) const;
	void		AddToRecentFolders( const EntryRef *folder,
									const char *appSig = NULL) const;
};


}
#endif /* __ROSTER_H__ */
