/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */



#ifndef	__APPSERVER_PROTOCOL_H__
#define	__APPSERVER_PROTOCOL_H__

#include <string.h>
#include <support/SupportDefs.h>

#include <interface/GraphicsDefs.h>
#include <interface/Font.h>
#include <interface/Region.h>
#include <interface/InterfaceDefs.h>

#include  <cosmoe/mouse.h>

class SrvWindow;

namespace os
{

enum
{
	DR_CREATE_APP = 1,
	DR_CREATE_CLIPBOARD,
	DR_GET_CLIPBOARD_DATA,
	DR_SET_CLIPBOARD_DATA,
	DR_SET_WINDOW_DECORATOR,
	DR_SET_COLOR_CONFIG,
	DR_CREATE_APP_NEW,
	DR_GET_KEYBOARD_CFG,
	DR_SET_KEYBOARD_CFG,
	DR_SET_APPSERVER_CONFIG,
	DR_GET_APPSERVER_CONFIG,
	DR_GET_DEFAULT_FONT_NAMES,
	DR_GET_DEFAULT_FONT,
	DR_SET_DEFAULT_FONT,
	DR_ADD_DEFAULT_FONT,
	DR_RESCAN_FONTS,
	DR_GET_APP_LIST,
	DR_GET_APP_TEAM,
	DR_ACTIVATE_WORKSPACE,
	DR_GET_CURRENT_WORKSPACE,

	AR_OPEN_WINDOW = 10000,
	AR_CLOSE_WINDOW,
	AR_DELETE_WINDOW,
	AR_OPEN_BITMAP_WINDOW,

	AR_CREATE_BITMAP,
	AR_DELETE_BITMAP,
	AR_GET_QUALIFIERS,
	AR_CREATE_FONT,
	AR_DELETE_FONT,
	AR_SET_FONT_FAMILY_AND_STYLE,
	AR_GET_FONT_FAMILY_AND_STYLE,
	AR_SET_FONT_PROPERTIES,
	AR_GET_STRING_WIDTHS,
	AR_GET_STRING_LENGTHS,

	AR_CREATE_SPRITE,
	AR_DELETE_SPRITE,
	AR_MOVE_SPRITE,

	AR_RESCAN_FONTS,
	AR_GET_FONT_FAMILY_COUNT,
	AR_GET_FONT_FAMILY,
	AR_GET_FONT_STYLE_COUNT,
	AR_GET_FONT_STYLE,
	AR_GET_SCREENMODE_COUNT,
	AR_GET_SCREENMODE_INFO,
	AR_LOCK_DESKTOP,
	AR_UNLOCK_DESKTOP,
	AR_SET_SCREEN_MODE,
	AR_GET_IDLE_TIME,
	AR_GET_FONT_SIZES,

	WR_GET_VIEW_FRAME = 20000,
	WR_SET_SIZE_LIMITS,
	WR_TOGGLE_VIEW_DEPTH,
	WR_RENDER,
	WR_BEGIN_DRAG,
	WR_CREATE_VIEW,
	WR_DELETE_VIEW,
	WR_MAKE_FOCUS,
	WR_SET_ALIGNMENT,
	WR_WND_MOVE_REPLY,
	WR_SET_TITLE,
	WR_UPDATE_REGIONS,
	WR_GET_PEN_POSITION,
	WR_SET_FLAGS,
	WR_GET_MOUSE,
	WR_SET_MOUSE_POS,
	WR_PUSH_CURSOR,
	WR_POP_CURSOR
};


enum
{
	DRC_PING,
	DRC_WRITE_PIXEL32,
	_DRC_unused3,
	DRC_LINE32,
	_DRC_unused4,
	DRC_FILL_RECT32,
	DRC_COPY_RECT,
	_DRC_unused5,
	DRC_SET_COLOR32,
	DRC_SET_PEN_POS,
	DRC_SET_FONT,
	DRC_DRAW_STRING,
	DRC_DRAW_BITMAP,
	DRC_SET_FRAME,
	DRC_SCROLL_VIEW,
	DRC_BEGIN_UPDATE,
	DRC_END_UPDATE,
	DRC_SET_DRAWING_MODE,
	DRC_SHOW_VIEW,
	DRC_INVALIDATE_VIEW,
	DRC_INVALIDATE_RECT
};

typedef	struct
{
	uint32 nCmd;
	uint32 nSize;
	uint32 hViewToken;
} GRndHeader_s;

struct GRndSetFrame_s : GRndHeader_s
{
    CRect cFrame;
};

struct GRndShowView_s : GRndHeader_s
{
    bool bVisible;
};

typedef	struct
{
    GRndHeader_s	sHdr;
    CPoint			sToPos;
} GRndLine32_s;

struct GRndFillRect32_s
{
    GRndHeader_s	sHdr;
    CRect			sRect;
    Color32_s		sColor;
};

struct GRndCopyRect_s
{
    GRndHeader_s	sHdr;
    CRect			cSrcRect;
    CRect			cDstRect;
};

typedef	struct
{
    GRndHeader_s	sHdr;
    int				hBitmapToken;
    CRect			cSrcRect;
    CRect			cDstRect;
} GRndDrawBitmap_s;

enum
{
    PEN_HIGH,
    PEN_LOW,
    PEN_ERASE
};

struct GRndSetDrawingMode_s : GRndHeader_s
{
    int nDrawingMode;
};

typedef	struct
{
    GRndHeader_s	sHdr;
    int				nWhichPen;
    Color32_s		sColor;
} GRndSetColor32_s;

typedef	struct
{
    GRndHeader_s	sHdr;
    bool			bRelative;
    CPoint			sPosition;
} GRndSetPenPos_s;

typedef	struct
{
    GRndHeader_s	sHdr;
    int				hFontID;
} GRndSetFont_s;

typedef	struct
{
    GRndHeader_s	sHdr;
    int				nLength;
    char			zString[1];	/* String of nLength characters, or null terminated if nLength == -1	*/
} GRndDrawString_s;

struct GRndScrollView_s : GRndHeader_s
{
    CPoint			cDelta;
};

struct GRndInvalidateRect_s : GRndHeader_s
{
    bool			m_bRecurse;
    CRect			m_cRect;
};

struct GRndInvalidateView_s : GRndHeader_s
{
    bool			m_bRecurse;
};

/***	Messages sent to main thread of the display server	***/

typedef struct
{
    MouseEvent_s	sEvent;
} DR_MouseEvent_s;

#define CLIPBOARD_FRAGMENT_SIZE (1024 * 32)

struct DR_GetClipboardData_s
{
	DR_GetClipboardData_s( const char* pzName, port_id hReply )
	{
		strcpy( m_zName, pzName ); m_hReply = hReply;
	}
	char    m_zName[64];
	port_id m_hReply;
};

struct DR_GetClipboardDataReply_s
{
	int		m_nTotalSize;
	int		m_nFragmentSize;
	uint8	m_anBuffer[CLIPBOARD_FRAGMENT_SIZE];
};


struct DR_SetClipboardData_s
{
	char    m_zName[64];
	int     m_nTotalSize;
	int	    m_nFragmentSize;
	port_id m_hReply;		// Just used as an source ID so the server wont interleave multi-package commits
	uint8	m_anBuffer[CLIPBOARD_FRAGMENT_SIZE];
};

struct DR_SetWindowDecorator_s
{
	char	m_zDecoratorPath[1024];
};

struct AR_DeleteWindow_s
{
	SrvWindow* pcWindow;
};

struct AR_LockDesktop_s
{
	AR_LockDesktop_s( port_id hReply, int nDesktop ) { m_hReply = hReply; m_nDesktop = nDesktop; }

	port_id m_hReply;
	int	  m_nDesktop;
};

struct AR_LockDesktopReply_s
{
	int	        m_nCookie;
	int	        m_nDesktop;
	IPoint      m_cResolution;
	color_space m_eColorSpace;
	area_id     m_hFrameBufferArea;
};

struct AR_UnlockDesktop_s
{
	AR_UnlockDesktop_s( int nCookie ) { m_nCookie = nCookie; }
	int m_nCookie;
};

typedef	struct
{
	int		nLength;
	char	zString[1];
} StringHeader_s;

typedef struct
{
	port_id	 hReply;
	int		 hFontToken;
	int		 nStringCount;
	StringHeader_s sFirstHeader;
	/*** nStringCount - 1, string headers follows	***/
} AR_GetStringWidths_s;


typedef struct
{
	int	nError;
	int	anLengths[1];
	/*** nStringCount - 1, lengths follow	***/
} AR_GetStringWidthsReply_s;

typedef struct
{
	port_id	 hReply;
	int		 hFontToken;
	int		 nStringCount;
	int		 nWidth;	/* Pixel width	*/
	int		 bIncludeLast;	// Should be a bool
	StringHeader_s sFirstHeader;
	/*** nStringCount - 1, string headers follows	***/
} AR_GetStringLengths_s;


typedef struct
{
	int	nError;
	int	anLengths[1];
	/*** nStringCount - 1, lengths follow	***/
} AR_GetStringLengthsReply_s;

/***	Bitmap messages	***/

struct AR_CreateBitmap_s
{
	port_id	hReply;
	int		bShareFramebuffer;	// Should be a bool
	int		nWidth;
	int		nHeight;
	color_space	eColorSpc;
};

struct AR_CreateBitmapReply_s
{
	AR_CreateBitmapReply_s() {}
	AR_CreateBitmapReply_s( int hHandle, area_id hArea ) { m_hHandle = hHandle; m_hArea = hArea; }
	int	  m_hHandle;
	area_id m_hArea;
};

struct AR_DeleteBitmap_s
{
	AR_DeleteBitmap_s( int nHandle ) { m_nHandle = nHandle; }
	int m_nHandle;
};

struct AR_CreateSprite_s
{
	AR_CreateSprite_s( port_id hReply, const CRect& cFrame, int nBitmap ) {
	m_hReply = hReply; m_cFrame = cFrame; m_nBitmap = nBitmap;
	}
	port_id m_hReply;
	int	    m_nBitmap;
	CRect   m_cFrame;
};

struct AR_CreateSpriteReply_s
{
	AR_CreateSpriteReply_s() {}
	AR_CreateSpriteReply_s( uint32 nHandle, int nError ) { m_nHandle = nHandle; m_nError = nError; }

	uint32 m_nHandle;
	int    m_nError;
};

struct AR_DeleteSprite_s
{
	AR_DeleteSprite_s( uint32 nHandle ) { m_nSprite = nHandle; }
	uint32 m_nSprite;
};

struct AR_MoveSprite_s
{
	AR_MoveSprite_s( uint32 nHandle, const CPoint& cNewPos ) {
	m_nSprite = nHandle; m_cNewPos = cNewPos;
	}
	uint32 m_nSprite;
	CPoint m_cNewPos;
};

struct AR_GetQualifiers_s
{
	AR_GetQualifiers_s( port_id hReply ) { m_hReply = hReply; }
	port_id m_hReply;
};

struct AR_GetQualifiersReply_s
{
	AR_GetQualifiersReply_s() {}
	AR_GetQualifiersReply_s( uint32 nQualifiers ) { m_nQualifiers = nQualifiers; }
	uint32	m_nQualifiers;
};

struct AR_GetScreenModeCount_s
{
	port_id	m_hReply;
};

struct AR_GetScreenModeCountReply_s
{
	int	m_nCount;
};

struct AR_GetScreenModeInfo_s
{
	port_id	m_hReply;
	int		m_nIndex;
};

struct AR_GetScreenModeInfoReply_s
{
	int	      m_nError;
	int	      m_nWidth;
	int 	  m_nHeight;
	int	      m_nBytesPerLine;
	color_space m_eColorSpace;
};

enum { SCRMF_RES = 0x0001, SCRMF_COLORSPACE = 0x0002, SCRMF_REFRESH = 0x0004, SCRMF_POS = 0x0008, SCRMF_SIZE = 0x0010 };

struct AR_SetScreenMode_s
{
	int		m_nVersion;
	int		m_nDesktop;
	uint32	m_nValidityMask;
	int		m_nWidth;
	int		m_nHeight;
	color_space	m_eColorSpace;
	float	m_vRefreshRate;
	float	m_vHPos;
	float	m_vVPos;
	float	m_vHSize;
	float	m_vVSize;
};

typedef struct
{
	int	nKeyCode;
	int	nQualifiers;
	char	zString[64];
} SR_KbdEvent_s;

/***	Messages sendt to window threads in the display server	***/

struct WR_Request_s
{
	int m_hTopView;
};

enum { RENDER_BUFFER_SIZE = 500 };

struct WR_Render_s : WR_Request_s
{
	port_id	hReply;
	int		nCount;	/* Number of render instructions	*/
	uint8	aBuffer[RENDER_BUFFER_SIZE];
};

struct WR_GetPenPosition_s : WR_Request_s
{
	port_id m_hReply;
	int	    m_hViewHandle;
};

struct WR_GetPenPositionReply_s
{
	CPoint m_cPos;
};

}
#endif	//	__APPSERVER_PROTOCOL_H__
