/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __ATOMIC_H__
#define __ATOMIC_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <support/SupportDefs.h>

typedef int atomic_t;

struct __atomic_fool_gcc_struct { int a[100]; };
#define __atomic_fool_gcc(x) (*(volatile struct __atomic_fool_gcc_struct *)x)

/** 
 * \par Description:
 *	Add nValue to *pnTarget and return the old *pnTarget. The operation
 *	is atomic and can not be affected by interrupts or other CPU's writing
 *	to *pnTarget.
 * \par Note:
 * \par Warning:
 * \param
 * \return
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

static inline atomic_t ath_atomic_add( atomic_t* pnTarget, atomic_t nValue  )
{
    register atomic_t nOldVal;

#ifdef __i386__

    register int nTmp = 0;
    __asm__ volatile( "  movl %3,%%eax;"	/* Start with the old-value in EAX */
			"0:;"
			"  movl %%eax,%%edx;"	/* Put the old-value where cmpxchgl wants it */
			"  addl %4,%%edx;"
			"  lock; cmpxchgl %%edx,%3;" /* if (*pnTarget==eax) {*pnTarget=edx;} else {eax=*pnTarget;goto 0;} */
			"  jnz 0b;"
			: "=&a" (nOldVal), "=m"(__atomic_fool_gcc(pnTarget)), "=&d"(nTmp)
			: "m" (__atomic_fool_gcc(pnTarget)), "r" (nValue)
			:  "cc" );

#elif defined(__ppc__)

	__asm__ volatile(
			"0:	lwarx	%0,0,%2\n\
				add	%0,%1,%0\n\
				stwcx.	%0,0,%2\n\
				bne-	0b\n\
				isync"
				: "=&r" (nOldVal)
				: "r" (nValue), "r" (&pnTarget->counter)
				: "cc", "memory");

#endif
	return nOldVal;
}


#ifdef __cplusplus
}
#endif

#endif /* ndef __ATOMIC_H__ */
