/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __IMAGE_H__
#define __IMAGE_H__

#include <support/SupportDefs.h>
#include <sys/types.h> /* dev_t */

#ifdef __cplusplus
extern "C" {
#endif

typedef int image_entry( char** argv, char** envv );
typedef int image_init( int nImageID );
typedef void image_term( int nImageID );

typedef struct
{
	int             ii_image_id;            // image id.
	int             ii_type;                // image type (IM_APP_SPACE, IM_ADDON_SPACE or IM_LIBRARY_SPACE).
	int             ii_open_count;          // open count private to the owner process.
	int             ii_sub_image_count;     // number of dll's linked to this image.
	int             ii_init_order;          // the larger the later to be initiated.
	image_entry*    ii_entry_point;         // entry point (_start())
	image_init*     ii_init;                // init routine. (global contructors)
	image_term*     ii_fini;                // terminate routine. (global destructors)
	dev_t           ii_device;              // device number for file.
	int             ii_inode;               // inode number for file.
	char            ii_name[ 256 ];         // full pathname of image.
	void*           ii_text_addr;           // address of text segment.
	void*           ii_data_addr;           // address of data segment.
	int             ii_text_size;           // size of text segment.
	int             ii_data_size;           // size of data segment (including BSS).
} image_info;

enum
{
    IM_KERNEL_SPACE,
    IM_APP_SPACE,
    IM_ADDON_SPACE,
    IM_LIBRARY_SPACE,
};

int load_library( const char* pzPath, uint32 nFlags );
int unload_library( int nLibrary );

int get_library_info( int nLibrary, image_info* psInfo );
int get_symbol_address( int nLibrary, const char* pzName, int nIndex, void** pPtr );

int get_image_info( int nImage, int nSubImage, image_info* psInfo );

int get_symbol_by_address( int nLibrary, const char* pAddress, char* pzName, int nMaxNamLen, void** ppAddress );
int find_module_by_addr( uint32 nAddress );

#ifdef __cplusplus
}
#endif

#endif /* __IMAGE_H__ */
