#ifndef _ATHEOS_ATHEOS_KERNEL_H_
#define _ATHEOS_ATHEOS_KERNEL_H_

#include <cosmoe/Debug.h>
#include <cosmoe/OS.h>

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_CPU_COUNT		16
#define B_MAX_CPU_COUNT		MAX_CPU_COUNT

typedef struct
{
  bigtime_t	active_time;
} CPUInfo_s;


typedef struct
{
    int64     nKernelVersion;
    char      zKernelName[ OS_NAME_LENGTH ];	 	/* Name of kernel image		*/
    char      zKernelBuildDate[ OS_NAME_LENGTH ];	/* Date of kernel built		*/
    char      zKernelBuildTime[ OS_NAME_LENGTH ];	/* Time of kernel built		*/
    bigtime_t nBootTime;				/* time of boot (# usec since 1/1/70) */
    int	      cpu_count;
    int	      cpu_type;
    CPUInfo_s cpu_infos[MAX_CPU_COUNT];
	int64	  cpu_clock_speed;
	int64     bus_clock_speed;
	int       nMaxPages;				/* total # physical pages		*/
    int       nFreePages;				/* Number of free physical pages	*/
    int	      nCommitedPages;			/* Total number of allocated pages	*/
    int	      nKernelMemSize;
    int       nPageFaults;				/* Number of page faults		*/
    int       nUsedSemaphores;			/* Number of semaphores in use		*/
    int       nUsedPorts;				/* Number of message ports in use	*/
    int       nUsedThreads;				/* Number of living threads		*/
    int       nUsedProcesses;			/* Number of living processes		*/

    int	      nLoadedImageCount;
    int	      nImageInstanceCount;

    int	      nOpenFileCount;
    int       nAllocatedInodes;
    int       nLoadedInodes;
    int       nUsedInodes;
    int	      nBlockCacheSize;
    int	      nDirtyCacheSize;
    int	      nLockedCacheBlocks;
} system_info;

typedef	struct
{
    thread_id	hThread;
} DR_ThreadDied_s;


int	reboot( void );


status_t get_system_info( system_info* psInfo );
int	 	 set_app_server_port( port_id hPort );
port_id	 get_app_server_port( void );

uint32 Delay( uint32	nMicros );

/* XXX: added for console/fbdev/linux compatibility see io.c */
int init_keyboard( void );
int restore_keyboard( void );

#ifdef __cplusplus
}
#endif

#endif /* ndef _ATHEOS_ATHEOS_KERNEL_H_ */
