/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <vector>
#include <string>

#include <interface/View.h>
#include <interface/Button.h>
#include <interface/Window.h>

#define BAlert	os::CAlert

namespace os
{

class CInvoker;


class AlertView : public CView
{
public:
	AlertView( const std::string& cText, va_list pButtons );
	AlertView( const std::string& cText, const char* inButtonOne,
										 const char* inButtonTwo,
										 const char* inButtonThree );

	virtual void	GetPreferredSize( float* outWidth, float* outHeight );
	virtual void	Draw( CRect cUpdateRect );
	virtual void	AllAttached();

	CButton*		ButtonAt(int32 inButtonIndex) const;

private:
	friend class CAlert;

	void			PrepareAlert(const std::string& cText);
	void			AddButton(const char* inButtonName, uint32 inButtonNumber);

	std::vector<std::string>	m_cLines;
	std::vector<CButton*>		mButtons;
	CPoint						mButtonSize;
	float						m_vLineHeight;
	CPoint						m_cMinSize;
};

/**
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CAlert : public CWindow
{
public:
	CAlert( const std::string& cTitle, const std::string& cText, int nFlags, ... );
	CAlert( const char* inTitle,
			const char* inText,
			const char* inButtonOne,
			const char* inButtonTwo = NULL,
			const char* inButtonThree = NULL );
	~CAlert();

	virtual void	MessageReceived( CMessage* pcMessage );

	int32			Go();
	void			Go( CInvoker* pcInvoker );

	CButton*		ButtonAt(int32 inButtonIndex) const;

private:
	AlertView*		mAlertView;
	CInvoker*		mInvoker;
	port_id			mMsgPort;
};


}
