/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__BUTTON_H__
#define	__BUTTON_H__

#include <support/SupportDefs.h>

#include <interface/Control.h>

#define BButton		os::CButton


namespace os
{

/** Simple push-button class.
 * \ingroup interface
 * \par Description:
 *	A button object renders a labeled button and accepts user-input
 *	through the mouse or keyboard. The button will Invoke() the
 *	Control if the user clicks it with the mouse or hits space or
 *	enter while the button has focus (it will take focus when
 *	clicked). Each window can have a "default" button assigned
 *	with the os::Window::SetDefaultButton() member. The default
 *	button can be invoked with the Enter key even when it doesn't
 *	have the focus.
 *	
 * \sa Control, Invoker, View
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/


class CButton : public CControl
{
public:
					CButton( CRect inFrame,
							 const char* inName,
							 const char* inLabel,
							 CMessage* inMessage,
							 uint32 inResizeMask = CF_FOLLOW_LEFT | CF_FOLLOW_TOP,
							 uint32 inFlags = WID_WILL_DRAW |
											  WID_CLEAR_BACKGROUND |
											  WID_FULL_UPDATE_ON_RESIZE );
					~CButton();

	virtual void	GetPreferredSize(float *outWidth, float *outHeight);
	virtual void	MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
	virtual void	MouseDown( CPoint cPosition );
	virtual void	MouseUp( CPoint cPosition );
	virtual void	KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );
	virtual void	KeyUp( const char* pzString, const char* pzRawString, uint32 nQualifiers );
	virtual void	Activated( bool bIsActive );
	virtual void	Draw( CRect cUpdateRect );


	virtual void	PostValueChange( const Variant& cNewValue );
	virtual void	LabelChanged( const std::string& cNewLabel );
	virtual void	EnableStatusChanged( bool bIsEnabled );
	virtual bool	Invoked( CMessage* pcMessage );

private:
	bool			m_bClicked;
};

}

#endif	//	__BUTTON_H__
