/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef	__CONTROL_H__
#define	__CONTROL_H__

#include <interface/View.h>
#include <app/Invoker.h>
#include <util/variant.h>

#define BControl	os::CControl


namespace os
{

/** Base class for GUI controls.
 * \ingroup interface
 * \par Description:
 *	
 * \sa os::Invoker
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/


class CControl	: public CView, public CInvoker
{
public:
						CControl( CRect inFrame,
								  const char* inName,
								  const char* inLabel,
								  CMessage* pcMessage,
								  uint32 inResizeMask,
								  uint32 inFlags  = WID_WILL_DRAW | WID_CLEAR_BACKGROUND );
						~CControl();

// From View:
	virtual void 		AttachedToWindow( void );

// From Control:
	virtual void		ValueChanged( Variant* pcNewValue ); // Deprecated

	virtual bool		PreValueChange( Variant* pcNewValue );
	virtual void		PostValueChange( const Variant& cNewValue );

	virtual void		LabelChanged( const std::string& cNewLabel );
	virtual void		EnableStatusChanged( bool bIsEnabled ) {}
	virtual bool		Invoked( CMessage* pcMessage );

	virtual void		SetEnabled( bool inEnabled );
	virtual bool		IsEnabled( void ) const;

	virtual void		SetLabel( const char* inLabel );
	virtual const char*	Label( void ) const;

	virtual void    	SetValue( Variant inValue, bool inInvoke = true );
	virtual Variant 	Value() const;
    
private:
	class Private;
	Private* m;

	virtual void		FBC_Control1();
	virtual void		FBC_Control2();
	virtual void		FBC_Control3();
	virtual void		FBC_Control4();
	virtual void		FBC_Control5();
};

}

#endif	//	GUI_CONTROL_HPP
