/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __DIRECTORYVIEW_H__
#define __DIRECTORYVIEW_H__

#include <sys/stat.h>
#include <interface/ListView.h>
#include <storage/Path.h>

#include <vector>
#include <string>
#include <stack>

namespace os_priv
{
    class DirKeeper;
}

namespace os
{

class CMessage;
class CBitmap;


/** Directory browser control.
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

class FileRow : public ListViewRow
{
public:
	FileRow( CBitmap* pcBitmap, const std::string& cName, const struct stat& sStat ) : m_cName(cName) {
	m_sStat = sStat; m_pcIconBitmap = pcBitmap;
	}
	virtual void	AttachToView( CView* pcView, int nColumn );
	virtual void	SetRect( const CRect& cRect, int nColumn );

	virtual float	GetWidth( CView* pcView, int nIndex );
	virtual float	GetHeight( CView* pcView );
	virtual void	Draw( const CRect& cFrame,
						  CView* pcView,
						  uint nColumn,
						  bool bSelected,
						  bool bHighlighted,
						  bool bHasFocus );
	virtual bool	HitTest( CView* pcView, const CRect& cFrame, int nColumn, CPoint cPos );
	virtual bool	IsLessThan( const ListViewRow* pcOther, uint nColumn ) const;
	const std::string& GetName() const { return( m_cName ); }
	struct stat		GetFileStat() const { return( m_sStat ); }
private:
	friend class	DirectoryView;
	std::string		m_cName;
	struct stat		m_sStat;
	uint8       	m_anIcon[16*16*4];
	CBitmap*		m_pcIconBitmap;
	float			m_avWidths[7];
};

/** Directory view suitable for file-requesters and other file browsers.
 * \ingroup interface
 * \par Description:
 *
 * \sa os::FileRequester
 * \author	Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

class DirectoryView : public ListView
{
public:
					DirectoryView( const CRect& cFrame,
								   const std::string& cPath,
								   uint32 nModeFlags = F_MULTI_SELECT |
													   F_RENDER_BORDER,
								   uint32 nResizeMask = CF_FOLLOW_LEFT |
														CF_FOLLOW_TOP,
								   uint32 nViewFlags = WID_WILL_DRAW |
													   WID_FULL_UPDATE_ON_RESIZE );
					~DirectoryView();

	void			ReRead();
	void			SetPath( const std::string& cPath );
	std::string		GetPath() const;

	FileRow*		GetFile( int nRow ) const { return( static_cast<FileRow*>(GetRow(nRow)) ); }
	void			SetDirChangeMsg( CMessage* pcMsg );
//  void	SetDirChangeTarget( const CHandler* pcHandler, const CLooper* pcLooper = NULL );
//  void	SetDirChangeTarget( const Messenger& cTarget );

	virtual void	DirChanged( const std::string& cNewPath );
	virtual void	Invoked( int nFirstRow, int nLastRow );
	virtual bool	DragSelection( const CPoint& cPos );
	virtual void	MessageReceived( CMessage* pcMessage );
	virtual void	AttachedToWindow();
	virtual void	DetachedFromWindow();
	virtual void	MouseUp( CPoint cPosition );
	virtual void	MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
	virtual void	KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );
private:
	enum { M_CLEAR, M_ADD_ENTRY, M_UPDATE_ENTRY, M_REMOVE_ENTRY };
	friend class os_priv::DirKeeper;

	static int32	ReadDirectory( void* pData );
	void			PopState();

	struct State
	{
		State( ListView* pcView, const char* pzPath );
		std::string			 		m_cPath;
		uint						m_nScrollOffset;
		std::vector<std::string>	m_cSelection;
	};
	
	struct ReadDirParam
	{
		ReadDirParam( DirectoryView* pcView ) { m_pcView = pcView; m_bCancel = false; }
		DirectoryView*	m_pcView;
		bool			m_bCancel;
	};
	
	ReadDirParam*		m_pcCurReadDirSession;
	CMessage*			m_pcDirChangeMsg;
	CPath				m_cPath;
//    Directory	      m_cDirectory;
	os_priv::DirKeeper*	m_pcDirKeeper;
	std::string			m_cSearchString;
	bigtime_t			m_nLastKeyDownTime;
	std::stack<State>	m_cStack;
	CBitmap*			m_pcIconBitmap;
};

} // End of namespace

#endif // __DIRECTORYVIEW_H__
