/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __DROPDOWNLIST_H__
#define __DROPDOWNLIST_H__

#include <interface/View.h>
#include <interface/Font.h>
#include <app/Invoker.h>

#include <vector>
#include <string>

namespace os
{

class TextView;

/** Edit box with an associated menu.
 * \ingroup interface
 * \par Description:
 *
 * \sa TextView, Invoker
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CDropDownList : public CView, public CInvoker
{
public:
					CDropDownList( CRect cFrame,
								const char* pzName,
								uint32 nResizeMask = CF_FOLLOW_LEFT | CF_FOLLOW_TOP,
								uint32 nFlags = WID_WILL_DRAW | WID_FULL_UPDATE_ON_RESIZE );
					~CDropDownList();

	void			SetEnabled( bool bEnable = true );
	bool			IsEnabled() const;
	void			SetReadOnly( bool bFlag = true );
	bool			GetReadOnly() const;

	void			AppendItem( const char* pzString );
	void			InsertItem( int nPosition, const char* pzString );
	bool			DeleteItem( int nPosition );
	int				GetItemCount() const;
	void			Clear();
	const std::string& GetItem( int nItem ) const;

	int				GetSelection() const;
	void			SetSelection( int nItem, bool bNotify = true );

	const std::string&	GetCurrentString() const;
	void			SetCurrentString( const std::string& cString );

	void 			SetMinPreferredSize( int nWidthChars );
	int				GetMinPreferredSize() const;
	void 			SetMaxPreferredSize( int nWidthChars );
	int				GetMaxPreferredSize() const;

	void			SetSelectionMessage( CMessage* pcMsg );
	CMessage*		GetSelectionMessage() const;
	void			SetSendIntermediateMsg( bool bFlag );
	bool			GetSendIntermediateMsg() const;
	void			SetEditMessage( CMessage* pcMsg );
	CMessage*		GetEditMessage() const;

	virtual void	MessageReceived( CMessage* pcMessage );

	virtual void	Draw( CRect cUpdateRect );
	virtual void	GetPreferredSize( float* outWidth, float* outHeight );
	virtual void	FrameResized( float inWidth, float inHeight );
	virtual void	MouseDown( CPoint cPosition );
	virtual void	KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );
	virtual void	AllAttached();

private:
    class DropdownView : public CView
    {
	public:
		DropdownView( CDropDownList* pcParent );

		virtual void	Draw( CRect cUpdateRect );
		virtual void	MouseDown( CPoint cPosition );
		virtual void	MouseUp( CPoint cPosition );
		virtual void	MouseMoved( CPoint cPosition, uint32 nCode, const CMessage* pcData );
		virtual void	KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );
		virtual void	AllAttached();
		virtual void	Activated( bool bIsActive );

	private:
		friend class CDropDownList;
		CDropDownList* m_pcParent;
		CPoint	      m_cContentSize;
		font_height   m_sFontHeight;
		float	      m_vGlyphHeight;
		int	          m_nOldSelection;
		int	          m_nCurSelection;
    };
    friend class DropdownView;
  
    enum { ID_MENU_CLOSED = 1, ID_SELECTION_CHANGED, ID_STRING_CHANGED };
    void	OpenMenu();
  
    CWindow*   		     m_pcMenuWindow;
    TextView* 		     m_pcEditBox;
    CRect	    	     m_cArrowRect;
    std::vector<std::string> m_cStringList;
    CMessage*		     m_pcSelectionMsg;
    CMessage*		     m_pcEditMsg;
    int			         m_nSelection;
    bool	    	     m_bMenuOpen;
    bool		         m_bSendIntermediateMsg;
    bool		         m_bIsEnabled;
};

}

#endif // __DROPDOWNLIST_H__
