/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__GRAPHICS_DEFS_H__
#define	__GRAPHICS_DEFS_H__


#include <support/SupportDefs.h>

#define rgb_color			os::Color32_s
//#define pattern			os::Pattern

#define B_SOLID_HIGH		os::SOLID_HIGH
#define B_SOLID_LOW			os::SOLID_LOW
#define B_MIXED_COLORS		os::MIXED_COLORS
#define B_TRANSPARENT_COLOR	os::Color32_s(0,0,0,0)

//#define drawing_mode		os::DrawingMode
#define B_OP_COPY			os::DM_COPY
#define B_OP_OVER			os::DM_OVER
#define B_OP_ERASE			os::DM_ERASE
#define B_OP_INVERT			os::DM_INVERT
#define B_OP_ADD			os::DM_ADD
#define B_OP_SUBTRACT		os::DM_SUBTRACT
#define B_OP_BLEND			os::DM_BLEND
#define B_OP_MIN			os::DM_MIN
#define B_OP_MAX			os::DM_MAX
#define B_OP_SELECT			os::DM_SELECT

#define color_space			os::ColorSpace
#define B_COLOR_8_BIT		os::CS_CMAP8

#define B_LIGHTEN_MAX_TINT	0.0f;
#define B_LIGHTEN_2_TINT	0.385f;
#define B_LIGHTEN_1_TINT	0.590f;

#define B_NO_TINT			1.0f;

#define B_DARKEN_1_TINT		1.147f;
#define B_DARKEN_2_TINT		1.295f;
#define B_DARKEN_3_TINT		1.407f;
#define B_DARKEN_4_TINT		1.555f;
#define B_DARKEN_MAX_TINT	2.0f;

#define B_DISABLED_LABEL_TINT		B_DARKEN_3_TINT;
#define B_HIGHLIGHT_BACKGROUND_TINT	B_DARKEN_2_TINT;
#define B_DISABLED_MARK_TINT		B_LIGHTEN_2_TINT;



namespace os
{


typedef struct Pattern
{
    uint8	data[8];
} Pattern;

extern "C" {
extern const Pattern SOLID_HIGH;
extern const Pattern SOLID_LOW;
extern const Pattern MIXED_COLORS;
}


typedef enum
{
	CS_NO_COLOR_SPACE,
	CS_RGB32,
	CS_RGBA32,
	CS_RGB24,
	CS_RGB16,
	CS_RGB15,
	CS_RGBA15,
	CS_CMAP8,
	CS_GRAY8,
	CS_GRAY1,
	CS_YUV422,
	CS_YUV411,
	CS_YUV420,
	CS_YUV444,
	CS_YUV9,
	CS_YUV12
} ColorSpace;


enum drawing_mode
{
	DM_COPY,
	DM_OVER,
	DM_INVERT,
	DM_ERASE,
	DM_BLEND,
	DM_ADD,
	DM_SUBTRACT,
	DM_MIN,
	DM_MAX,
	DM_SELECT
};


typedef struct rgbaColor
{
	uint8 red;
	uint8 green;
	uint8 blue;
	uint8 alpha;
} rgbaColor;

struct Color32_s
{
	Color32_s() {}
	Color32_s( int r, int g, int b, int a=255 ) {	red = r;
													green = g;
													blue = b;
													alpha = a; }
	
	void 	operator&=( rgbaColor aColor )
	{
		red = aColor.red;
		green = aColor.green;
		blue = aColor.blue;
		alpha = aColor.alpha;
	}

	uint8 red;
	uint8 green;
	uint8 blue;
	uint8 alpha;
};


struct screen_mode
{
	int		     m_nWidth;
	int		     m_nHeight;
	int		     m_nBytesPerLine;
	color_space  m_eColorSpace;
	float	     m_vRefreshRate;
	float	     m_vHPos;
	float	     m_vVPos;
	float	     m_vHSize;
	float	     m_vVSize;
};



} // end of namespace

#endif
