/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__INTERFACEDEFS_H__
#define	__INTERFACEDEFS_H__


#include <interface/GraphicsDefs.h>


#define B_PASTE				os::M_PASTE
#define B_CUT				os::M_CUT
#define B_COPY				os::M_COPY
#define B_ABOUT_REQUESTED	os::M_ABOUT_REQUESTED

#define B_ALIGN_CENTER		os::ALIGN_CENTER
#define B_ALIGN_RIGHT		os::ALIGN_RIGHT

#define B_CONTROL_KEY		os::QUAL_CTRL
#define B_LEFT_CONTROL_KEY	os::QUAL_LCTRL
#define B_SHIFT_KEY			os::QUAL_SHIFT
#define B_LEFT_SHIFT_KEY	os::QUAL_LSHIFT
#define B_OPTION_KEY		os::QUAL_ALT

// FIXME: command key currently not natively defined
#define B_LEFT_COMMAND_KEY	os::QUAL_LCTRL
#define B_COMMAND_KEY		os::QUAL_CTRL

// FIXME: unimplemented
#define modifiers()			0

#define B_UTF8_ELLIPSIS		"\xE2\x80\xA6"



#ifdef __cplusplus
namespace os
{
#endif

//
static const float COORD_MAX = 16000000.0f;

static const char SYS_FONT_FAMILY[] = "system_default_font";
static const char SYS_FONT_FIXED[]  = "sys_fixed";
static const char SYS_FONT_PLAIN[]  = "sys_plain";
static const char SYS_FONT_BOLD[]   = "sys_bold";


enum alignment
{
	ALIGN_LEFT,
	ALIGN_RIGHT,
	ALIGN_TOP,
	ALIGN_BOTTOM,
	ALIGN_CENTER
};

enum orientation
{
	HORIZONTAL,
	VERTICAL
};



enum mouse_ptr_mode
{
	MPTR_MONO  = 0x01,
	MPTR_CMAP8 = 0x02,
	MPTR_RGB32 = 0x04
};


enum
{
	TRANSPARENT_CMAP8 = 0xff,
	TRANSPARENT_RGB32 = 0xffffffff
};


enum
{
	MOUSE_BUT_LEFT  = 0x01,
	MOUSE_BUT_MID	= 0x02,
	MOUSE_BUT_RIGHT = 0x04,
};


enum qualifiers
{
	QUAL_LSHIFT = 0x01,
	QUAL_RSHIFT = 0x02,
	QUAL_SHIFT  = QUAL_LSHIFT | QUAL_RSHIFT,

	QUAL_LCTRL  = 0x04,
	QUAL_RCTRL  = 0x08,
	QUAL_CTRL   = QUAL_LCTRL | QUAL_RCTRL,

	QUAL_LALT   = 0x10,
	QUAL_RALT   = 0x20,
	QUAL_ALT    = QUAL_LALT | QUAL_RALT,

	QUAL_REPEAT = 0x40
};


enum
{
	VK_BACKSPACE    = 0x08,
	VK_ENTER		= 0x0a,
	VK_RETURN		= 0x0a,
	VK_SPACE		= 0x20,
	VK_TAB	    	= 0x09,
	VK_ESCAPE		= 0x1b,
	VK_LEFT_ARROW   = 0x1c,
	VK_RIGHT_ARROW  = 0x1d,
	VK_UP_ARROW		= 0x1e,
	VK_DOWN_ARROW   = 0x1f,
	VK_INSERT		= 0x05,
	VK_DELETE		= 0x7f,
	VK_HOME	    	= 0x01,
	VK_END	    	= 0x04,
	VK_PAGE_UP		= 0x0b,
	VK_PAGE_DOWN    = 0x0c,
	VK_FUNCTION_KEY = 0x10
};


extern int32	count_workspaces();
extern void		set_workspace_count(int32 inCount);
extern int32	current_workspace();
extern void		activate_workspace(int32 inWorkspace);



#ifdef __cplusplus
} // End of namespace
#endif

#endif	// __INTERFACEDEFS_H__
