/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__LAYOUTVIEW_H__
#define	__LAYOUTVIEW_H__


#include <interface/View.h>

#include <vector>

namespace os
{

#define MAX_SIZE 100000.0f


class LayoutView;

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class LayoutNode
{
public:
							LayoutNode( const char* inName,
										float vWeight = 1.0f,
										LayoutNode* pcParent = NULL,
										CView* pcView = NULL );
	virtual					~LayoutNode();

	virtual void		    SetView( CView* pcView );
	virtual void		    Layout();

	virtual void		    SetBorders( const CRect& cBorder );
	virtual CRect		    GetBorders() const;

	float			    	GetWeight() const;
	void			    	SetWeight( float vWeight );
	virtual void 		    SetFrame( const CRect& cFrame );
	virtual CRect		    Frame() const;
	virtual CRect		    Bounds() const;
	CRect		    	    GetAbsFrame() const;

	void			    	ExtendMinSize( const CPoint& cMinSize );
	void			    	LimitMaxSize( const CPoint& cMaxSize );
	void			    	ExtendMaxSize( const CPoint& cMaxSize );

	void			    	SetHAlignment( alignment eAlignment );
	void			    	SetVAlignment( alignment eAlignment );
	alignment			    GetHAlignment() const;
	alignment			    GetVAlignment() const;

	void			    	AdjustPrefSize( CPoint* pcMinSize, CPoint* pcMaxSize );
	virtual void		    GetPreferredSize( float* outWidth, float* outHeight );
	void			    	AddChild( LayoutNode* pcChild );
	LayoutNode*			    AddChild( CView* pcChild, float vWeight = 1.0f );
	void			    	RemoveChild( LayoutNode* pcChild );
	void			    	RemoveChild( CView* pcChild );

	const char*			    GetName() const;
	const std::vector<LayoutNode*>& GetChildList() const;
	LayoutNode*			    GetParent() const;
	LayoutView*			    GetLayoutView() const;
	LayoutNode*				FindNode( const char* cName, bool bRecursive = true, bool bIncludeSelf = false );

	void					SameWidth( const char* pzName1, ... );
	void					SameHeight( const char* pzName1, ... );

	void					SetBorders( const CRect& cBorders, const char* pzFirstName, ... );
	void					SetWeights( float vWeight, const char* pzFirstName, ... );
	void					SetHAlignments( alignment eAlign, const char* pzFirstName, ... );
	void					SetVAlignments( alignment eAlign, const char* pzFirstName, ... );

	void					AddToWidthRing( LayoutNode* pcRing );
	void					AddToHeightRing( LayoutNode* pcRing );

protected:    
	virtual void			CalculatePreferredSize( float* outWidth, float* outHeight );

private:
	friend class LayoutView;
	void	_AddedToView( LayoutView* pcView );
	void	_AddedToParent( LayoutNode* pcParent );
	void	_RemovedFromParent();

	struct ShareNode
	{
		LayoutNode* m_pcNext;
		LayoutNode* m_pcPrev;
	};
	std::string				m_cName;
	CRect					m_cFrame;
	CRect					m_cBorders;
	CView*					m_pcView;
	LayoutNode*				m_pcParent;
	LayoutView*				m_pcLayoutView;
	std::vector<LayoutNode*> m_cChildList;
	float					m_vWeight;
	CPoint					m_cMinSize;
	CPoint					m_cMaxSizeExtend;
	CPoint					m_cMaxSizeLimit;

	alignment				m_eHAlign;
	alignment				m_eVAlign;

	ShareNode				m_sWidthRing;
	ShareNode				m_sHeightRing;
};

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class LayoutSpacer : public LayoutNode
{
public:
					LayoutSpacer( const char* cName,
								  float vWeight = 1.0f,
								  LayoutNode* pcParent = NULL,
								  const CPoint& cMinSize = CPoint(0.0f,0.0f),
								  const CPoint& cMaxSize = CPoint(MAX_SIZE,MAX_SIZE) );

	void			SetMinSize( const CPoint& cSize );
	void			SetMaxSize( const CPoint& cSize );

private:
	virtual void    CalculatePreferredSize( float* outWidth, float* outHeight );

	CPoint			m_cMinSize;
	CPoint			m_cMaxSize;
};

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

class VLayoutSpacer : public LayoutSpacer
{
public:
    VLayoutSpacer( const char* cName,
				   float vMinHeight = 0.0f,
				   float vMaxHeight = MAX_SIZE,
				   LayoutNode* pcParent = NULL,
				   float vWeight = 1.0f ) :
	    LayoutSpacer( cName, vWeight, pcParent, CPoint( 0.0f, vMinHeight ), CPoint( 0.0f, vMaxHeight ) )
    {}
};

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class HLayoutSpacer : public LayoutSpacer
{
public:
    HLayoutSpacer( const char* cName,
				   float vMinWidth = 0.0f,
				   float vMaxWidth = MAX_SIZE,
				   LayoutNode* pcParent = NULL, float vWeight = 1.0f ) :
	    LayoutSpacer( cName, vWeight, pcParent, CPoint( vMinWidth, 0.0f ), CPoint( vMaxWidth, 0.0f ) )
    {}
};

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class HLayoutNode : public LayoutNode
{
public:
    HLayoutNode( const char* cName, float vWeight = 1.0f, LayoutNode* pcParent = NULL, CView* pcView = NULL );
//    virtual void GetPreferredSize( float* outWidth, float* outHeight );
    virtual void	Layout();
private:
    virtual void	CalculatePreferredSize( float* outWidth, float* outHeight );
};

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class VLayoutNode : public LayoutNode
{
public:
    VLayoutNode( const char* cName, float vWeight = 1.0f, LayoutNode* pcParent = NULL, CView* pcView = NULL );
//    virtual void GetPreferredSize( float* outWidth, float* outHeight );
    virtual void  Layout();

protected:
    virtual void    CalculatePreferredSize( float* outWidth, float* outHeight );
private:
};

/** Main class in the dynamic layout system
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class LayoutView : public CView
{
public:
					LayoutView( CRect inFrame,
								const char* inTitle,
								LayoutNode* pcRoot = NULL,
								uint32 inResizeMask = CF_FOLLOW_ALL,
								uint32 inFlags = WID_WILL_DRAW | WID_CLEAR_BACKGROUND );

	LayoutNode* 	GetRoot() const;
	void			SetRoot( LayoutNode* pcRoot );

	LayoutNode* 	FindNode( const char* cName, bool bRequrcive = true );

	void			InvalidateLayout();

	virtual void	GetPreferredSize( float* outWidth, float* outHeight );
	virtual void	AllAttached();
	virtual void	FrameResized( float inWidth, float inHeight );

private:
	LayoutNode* m_pcRoot;
};



} // end of namespace


#endif // __F_GUI_LAYOUTVIEW_H__
