/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef	__MENU_H__
#define	__MENU_H__


#include <interface/View.h>
#include <interface/Window.h>
#include <app/Invoker.h>
#include <support/Locker.h>
#include <support/List.h>

#define BMenu				os::CMenu
#define menu_layout			os::MenuLayout_e
#define B_ITEMS_IN_ROW		os::ITEMS_IN_ROW
#define B_ITEMS_IN_COLUMN	os::ITEMS_IN_COLUMN
#define B_ITEMS_IN_MATRIX	os::ITEMS_CUSTOM_LAYOUT
 
#define add_state			os::CMenu::AddState
#define B_INITIAL_ADD		os::CMenu::kStateInitialAdd
#define B_PROCESSING		os::CMenu::kStateProcessing
#define B_ABORT				os::CMenu::kStateAbort

#define menu_tracking_hook	os::MenuTrackingHookFunc


namespace os
{

class	CMenu;
class	MenuWindow;

enum MenuLayout_e
{
	ITEMS_IN_ROW,
	ITEMS_IN_COLUMN,
	ITEMS_CUSTOM_LAYOUT
};

typedef bool (*MenuTrackingHookFunc)(CMenu *, void *);


/**
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CMenu : public CView
{
public:
					CMenu( const char *inName,
						   MenuLayout_e eLayout = ITEMS_IN_COLUMN);

					CMenu( CRect inFrame,
						   const char* inName,
						   uint32 nResizeMask = CF_FOLLOW_LEFT |
						   						CF_FOLLOW_RIGHT |
												CF_FOLLOW_TOP,
						   uint32 nFlags  = WID_WILL_DRAW |
						   					WID_CLEAR_BACKGROUND |
											WID_FULL_UPDATE_ON_RESIZE,
						   MenuLayout_e eLayout = ITEMS_IN_COLUMN,
						   bool	inResizeToFit = true );

					~CMenu();
	bool			Lock( void ) const;
	void			Unlock( void ) const;

//	From Handler:
	virtual void	TimerTick( int nID );

//	From View:
	void			AttachedToWindow( void );
	void			DetachedFromWindow( void );
	void			WindowActivated( bool bIsActive );

	virtual void	GetPreferredSize(float *outWidth, float *outHeight);
	virtual void	MouseDown( CPoint cPosition );
	virtual void	MouseUp( CPoint cPosition );
	virtual void	MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
	virtual void	KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );
	virtual void	FrameResized( float inWidth, float inHeight );

	virtual void	Draw( CRect cUpdateRect );

//	From CMenu:
    std::string 	Label() const;

	bool			AddItem( const char* pzLabel, CMessage* pcMessage );
	bool			AddItem( CMenuItem* pcItem );
	bool			AddItem( CMenuItem* pcItem, int inIndex );
	bool			AddItem( CMenu* pcItem );
	bool			AddItem( CMenu* pcItem, int inIndex );
	bool			AddSeparatorItem();

	CMenuItem*		RemoveItem( int inIndex );
	bool			RemoveItem( CMenuItem* pcItem );
	bool			RemoveItem( CMenu* pcMenu );

	CMenuItem*		ItemAt( int inIndex ) const;
	CMenuItem*		ItemAt( CPoint cPos ) const;
	CMenu*			SubmenuAt( int inIndex ) const;
	int				CountItems( void ) const;

	int				IndexOf( CMenuItem* pcItem ) const;
	int				IndexOf( CMenu* pcMenu ) const;

	CMenuItem*		FindItem( uint32 nCode ) const;
	CMenuItem*		FindItem( const char* pzName ) const;

	virtual status_t SetTargetForItems( CHandler* pcTarget );
	virtual status_t SetTargetForItems( CMessenger cMessenger );
	CMenuItem*		FindMarked() const;
	
	bool			IsRedrawAfterSticky() const			{ return false; } /* FIXME */

	CMenu*			Supermenu() const;
	CMenuItem*		Superitem() const;
	void			InvalidateLayout();

	void			SetCloseMessage( const CMessage& cMsg );
	void			SetCloseMsgTarget( const CMessenger& cTarget );

	void 			Open( CPoint cScrPos );

	virtual void	SetRadioMode( bool inRadioModeOn )	{mIsRadioMode = inRadioModeOn;}
	bool			IsRadioMode() const					{ return mIsRadioMode; }
	
	virtual	void	SetEnabled(bool /* inEnabled */)	{ /* FIXME */  }
	bool			IsEnabled() const					{ return true; } /* FIXME */
	
	virtual void	SetMaxContentWidth( float /* inWidth */ )	{ } /* FIXME: unimplemented */
	
	enum AddState
	{
		kStateInitialAdd,
		kStateProcessing,
		kStateAbort
	};
	
	virtual bool	AddDynamicItem(AddState /*inState*/) { return false; } /* FIXME: unimplemented */
	virtual void	DrawBackground(CRect inRect);

	void			SetTrackingHook(MenuTrackingHookFunc inFunction, void *state)	{}

protected:

	CMenuItem*		Track( const CPoint& cScreenPos );
	MenuLayout_e	Layout() const;
	void			GetItemMargins(	float *outLeft, 
									float *outTop, 
									float *outRight, 
									float *outBottom) const;
	void			SetItemMargins(	float inLeft, 
									float inTop, 
									float inRight, 
									float inBottom);
									
	virtual CPoint	ScreenLocation();

private:
	friend class CMenuItem;
	friend class CSeparatorItem;

	void			SetRoot( CMenu* pcRoot );
	void			StartOpenTimer( bigtime_t nDelay );
	void			OpenSelection();
	void			SelectItem( CMenuItem* pcItem );
	void			SelectPrev();
	void			SelectNext();
	void			EndSession( CMenuItem* pcSelItem );
	void			Close( bool bCloseChilds, bool bCloseParent );
	void			SetCloseOnMouseUp( bool bFlag );

	CMenuItem*		FindHighlighted() const;

	static CLocker	s_cMutex;

	port_id	  		m_hTrackPort;
	MenuWindow* 	mMenuWindow;
	CMenu*			mRootMenu;
	CMenuItem*		mFirstMenuItem;
	int				mItemCount;
	MenuLayout_e	mMenuLayout;
	CMenuItem*		mSuperItem;
	CPoint			m_cSize;
	CRect			mItemMargins;

	class Private;
	Private*	m;

	bool	  		mIsTracking;
	bool	  		mHasOpenChildren;
	bool	  		mCloseOnMouseUp;
	bool			mIsRadioMode;
};

}
#endif
