/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __MENUFIELD_H__
#define __MENUFIELD_H__

#include <Menu.h>
#include <View.h>

#define CMenuField		os::CMenuField


namespace os
{


class CMenuBar;


class CMenuField : public CView
{
public:
						CMenuField(	CRect inFrame,
									const char *inName,
									const char *inLabel,
									BMenu *inMenu,
									uint32 inResizeFlags = B_FOLLOW_LEFT|B_FOLLOW_TOP,
									uint32 inFlags = B_WILL_DRAW | B_NAVIGABLE); 
						CMenuField(	CRect inFrame,
									const char *inName,
									const char *inLabel,
									BMenu *inMenu,
									bool inIsFixedSize,
									uint32 inResizeFlags = B_FOLLOW_LEFT|B_FOLLOW_TOP,
									uint32 inFlags = B_WILL_DRAW | B_NAVIGABLE); 
	
	virtual				~CMenuField();

	virtual	void		Draw(CRect inRect);
	virtual	void		AttachedToWindow();
	virtual	void		AllAttached();
	virtual	void		MouseDown(BPoint where);
	virtual	void		KeyDown(const char *bytes, int32 numBytes);
	virtual	void		MakeFocus(bool state);
	virtual void		MessageReceived(CMessage *msg);
	virtual void		WindowActivated(bool state);
	virtual	void		MouseUp(CPoint pt);
	virtual	void		MouseMoved(CPoint pt, uint32 code, const CMessage *msg);
	virtual	void		DetachedFromWindow();
	virtual	void		AllDetached();
	virtual	void		FrameMoved(CPoint new_position);
	virtual	void		FrameResized(float new_width, float new_height);

			CMenu*		Menu() const;
			CMenuBar*	MenuBar() const;
			CMenuItem*	MenuItem() const;

	virtual	void		SetLabel(const char *label);
			const char*	Label() const;

	virtual void		SetEnabled(bool on);
			bool		IsEnabled() const;

	virtual	void		SetAlignment(alignment label);
			alignment	Alignment() const;
	virtual	void		SetDivider(float dividing_line);
			float		Divider() const;

			void		ShowPopUpMarker();
			void		HidePopUpMarker();

	virtual void		GetPreferredSize(float *outWidth, float *outHeight);


private:

	virtual	void		FBC_MenuField1();
	virtual	void		FBC_MenuField2();
	virtual	void		FBC_MenuField3();

			CMenuField&	operator=(const CMenuField &);


			void		DrawLabel(CRect bounds, CRect update);
	static	void		InitMenu(CMenu *menu);
	static	long		MenuTask(void *arg);

			char*		mLabel;
			CMenu*		mMenu;
			CMenuBar*	mMenuBar;
			alignment	mAlign;
			float		mDivider;
			float		mStringWidth;
			bool		mEnabled;
			bool		mSelected;
			bool		mTransition;
			bool		mFixedSizeMB;
			thread_id	mMenuTaskID;
			uint32		FBC_Reserved[3];
};

}

#endif /* __MENUFIELD_H__ */
