/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __MENUITEM_H__
#define __MENUITEM_H__

#include <app/Invoker.h>
#include <interface/Menu.h>

#define BMenuItem		os::CMenuItem
#define BSeparatorItem	os::CSeparatorItem

namespace os
{


class CMenuItem : public CInvoker
{
public:
					CMenuItem( const char* inLabel,
							   CMessage*   inMsg,
							   char        inShortcut = 0,
							   uint32      inModifiers = 0UL );
					CMenuItem( CMenu* inMenu, CMessage* inMsg );
					~CMenuItem();

	CMenu*			Submenu( void ) const		{ return mSubMenu;		}
	CMenu*			Menu( void ) const		    { return mSuperMenu;	}

	CRect			Frame( void ) const			{ return( mFrame );		}

	virtual	void	SetLabel(const char *name);
	const char*		Label( void ) const			{ return mLabel;		}

	virtual void	GetContentSize(float *width, float *height);

	virtual void	Draw( void );
	virtual void	DrawContent( void );
	CPoint			ContentLocation( void ) const;

	virtual bool 	Invoked( CMessage* pcMessage );

	virtual	void	SetMarked(bool inState);
	bool			IsMarked() const 			{ return mIsMarked;		}

	virtual void	Highlight( bool inHighlight );
	bool			IsSelected() const			{ return mIsHighlighted; }
	
	virtual void	SetEnabled( bool inState );
	bool			IsEnabled() const			{ return mIsEnabled;	}
	
	virtual void	SetShortcut(char inShortcut, uint32 inModifiers);
	char			Shortcut(uint32 *outModifiers = NULL) const;

protected:
	friend class CMenu;
	
	CMenuItem*		m_pcNext;
	CMenu*			mSuperMenu;
	CMenu*			mSubMenu;
	CRect			mFrame;

	char*			mLabel;

	bool			mIsHighlighted;
	bool			mIsMarked;
	bool			mIsEnabled;

	char			mShortcut;
	uint32			mModifiers;
};


class CSeparatorItem : public CMenuItem
{
public:
					CSeparatorItem();
	virtual			~CSeparatorItem();

protected:

	virtual void	GetContentSize(float *outWidth, float *outHeight);
	virtual void	Draw();
	virtual void	Highlight(bool inHighlight);
};

}

#endif /* __MENUITEM_H__ */
