/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__REGION_H__
#define	__REGION_H__

#include <support/SupportDefs.h>
#include <interface/GraphicsDefs.h>

#include <interface/Rect.h>
#include <interface/Point.h>

#include <support/Locker.h>

#define ENUMCLIPLIST( list, node ) for ( node = (list)->m_pcFirst ; node != NULL ; node = node->m_pcNext )

namespace os
{

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

struct	ClipRect
{
				ClipRect() {}
				ClipRect( const IRect& cFrame, const IPoint& cOffset ) : m_cBounds(cFrame), m_cMove(cOffset) {}

	ClipRect*	m_pcNext;
	ClipRect*	m_pcPrev;
	IRect		m_cBounds;
	IPoint		m_cMove;	// Used to sort rectangles, so no blits destroy needed areas
};

/** 
 * \ingroup interface
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class ClipRectList
{
public:
				ClipRectList();
				~ClipRectList();

	void		Clear();
	void		AddRect( ClipRect* pcRect );
	void		RemoveRect( ClipRect* pcRect );
	ClipRect*	RemoveHead();
	void 		StealRects( ClipRectList* pcList );
	int			GetCount() const { return( m_nCount ); }

	ClipRect*	m_pcFirst;
	ClipRect*	m_pcLast;
	int			m_nCount;
};

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CRegion
{
public:
					CRegion();
					CRegion( const IRect& cRect );
					CRegion( const CRegion& cReg );
					CRegion( const CRegion& cReg,
							 const IRect& cRect,
							 bool bNormalize );

					~CRegion();

	void			Set( const IRect& cRect );
	void			Set( const CRegion& cRect );
	void			Clear( void );
	bool			IsEmpty() const { return( m_cRects.GetCount() == 0 ); }
	void			Include( const IRect& cRect );
	void			Intersect( const CRegion& cReg, const IPoint& cOffset );
	void			Intersect( const CRegion& cReg );

	void			Exclude( const IRect& cRect );
	void			Exclude( const CRegion& cReg );
	void			Exclude( const CRegion& cReg, const IPoint& cOffset );

	CRegion*		Clone( const IRect& cRectangle, bool bNormalize );
	void			Optimize();

	static void		FreeClipRect( ClipRect* pcRect );
	static ClipRect*	AllocClipRect();
	IRect			Frame() const;
	ClipRectList	m_cRects;
	
private:
	static void**	s_pFirstClip;
	static CLocker	s_cClipListMutex;

	ClipRect*		AddRect( const IRect& cRect );
};

} // End of namespace

#endif	// __REGION_H__
