/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__SCROLLBAR_H__
#define	__SCROLLBAR_H__

#include <limits.h>
#include <float.h>

#include <support/SupportDefs.h>
#include <interface/Control.h>

#define BScrollBar				os::CScrollBar
#define B_V_SCROLL_BAR_WIDTH	os::SB_MINSIZE
#define B_H_SCROLL_BAR_HEIGHT	os::SB_MINSIZE


namespace os
{

#define SB_MINSIZE				14.0f

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CScrollBar : public CControl
{
public:
					CScrollBar( CRect inFrame,
								const char* inName,
								CMessage* inMessage,
								float inMin = 0,
								float inMax = FLT_MAX,
								orientation inOrientation = VERTICAL,
								uint32 inResizeMask = 0 );
					~CScrollBar();

	void			SetTarget( CView* inTarget );
	CView*			Target( void ) const;


	// From Control:

	virtual void	PostValueChange( const Variant& cNewValue );
	virtual void	LabelChanged( const std::string& cNewLabel );
	virtual void	EnableStatusChanged( bool bIsEnabled );
	virtual bool	Invoked( CMessage* pcMessage );

	// From View

	virtual void	FrameResized( float inWidth, float inHeight );

	virtual void	MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
	virtual void	MouseDown( CPoint cPosition );
	virtual void	MouseUp( CPoint cPosition );

	virtual void	Draw( CRect cUpdateRect );

	virtual void	GetPreferredSize( float* outWidth, float* outHeight );

	virtual void	TimerTick( int nID );

	// From ScrollBar:

	void			SetSteps( float vSmall, float vBig );
	void			GetSteps( float* pvSmall, float* pvBig ) const;

	void			SetRange( float vMin, float vMax );
	void			GetRange( float* outMinimum, float* outMaximum) const;

	void			SetProportion( float inProportion );
	float			Proportion() const;
	
	orientation		Orientation() const;

private:
	CRect			GetKnobFrame() const;
	float			_PosToVal( CPoint cPos ) const;

	class Private;
	Private* m;    
};

}

#endif
