#ifndef	__SLIDER_H__
#define	__SLIDER_H__


#include <limits.h>

#include <support/SupportDefs.h>
#include <interface/Control.h>

namespace os
{

enum	{ SL_MINSIZE = 8 };

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CSlider : public CControl
{
public:
	enum { TICKS_ABOVE = 0x0001, TICKS_BELOW = 0x0002, TICKS_LEFT = 0x0001, TICKS_RIGHT = 0x0002 };
	enum knob_mode { KNOB_SQUARE, KNOB_TRIANGLE, KNOB_DIAMOND };

					CSlider( CRect inFrame,
							 const char* inName,
							 CMessage* pcMsg,
							 uint32 nTickFlags = TICKS_BELOW,
							 int nTickCount = 10,
							 knob_mode eKnobMode = KNOB_SQUARE,
							 orientation eOrientation = HORIZONTAL,
							 uint32 nResizeMask = 0 );
					~CSlider();

	// From Slider:

	virtual void	RenderLabels( CView* pcRenderView );
	virtual void	RenderSlider( CView* pcRenderView );
	virtual void	DrawThumb( CView* pcRenderView );
	virtual void	DrawHashMarks( CView* pcRenderView );

	virtual float   ValueForPoint( CPoint inPosition ) const;	// FIXME should return int32
	virtual CPoint  ValToPos( float vVal ) const;

	virtual CRect   ThumbFrame() const;
	virtual CRect   BarFrame() const;

	virtual	void 	SetBarColor(Color32_s inColor);
	virtual	void 	UseFillColor(bool inUseIt, const Color32_s* inColor=NULL);

	Color32_s 		BarColor() const					{ return mBarColor; }
	bool			FillColor(rgb_color* outColor) const;

	orientation		Orientation() const					{ return m_eOrientation; }

	virtual void	SetBarThickness( float vSize );
	virtual float	BarThickness() const;


	virtual void	SetProgStrFormat( const std::string& cFormat );
	virtual std::string GetProgStrFormat() const;
	virtual std::string GetProgressString() const;

	void			SetStepCount( int nCount );
	int				GetStepCount() const;

	void			SetHashMarkCount( int inCount );
	int				HashMarkCount() const;

	void			SetTickFlags( uint32 nFlags );
	uint32			GetTickFlags() const;

	void			SetLimitLabels( const char* inMinLabel, const char* inMaxLabel );
	const char*		MinLimitLabel();
	const char* 	MaxLimitLabel();

	virtual void	SetSteps( float vSmall, float vBig )	       { m_vSmallStep = vSmall; m_vBigStep = vBig; }
	virtual void	GetSteps( float* pvSmall, float* pvBig ) const { *pvSmall = m_vSmallStep; *pvBig = m_vBigStep; }

	virtual void	SetMinMax( float vMin, float vMax ) { m_vMin = vMin; m_vMax = vMax; }

	// From Control:

	virtual void	PostValueChange( const Variant& cNewValue );
	virtual void	EnableStatusChanged( bool bIsEnabled );

	bool			Invoked( CMessage* pcMessage );
//    void		SetCurValue( float nValue );
//    virtual void	SetCurValue( float nValue, bool bInvoke );
//    virtual float	GetCurValue() const;

	//	From View

	virtual void	AttachedToWindow();

	virtual void	MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
	virtual void	MouseDown( CPoint cPosition );
	virtual void	MouseUp( CPoint cPosition );

	virtual void	FrameResized( float inWidth, float inHeight );
	virtual void	Draw( CRect cUpdateRect );
	virtual void	GetPreferredSize( float* outWidth, float* outHeight );

private:
	void			_RefreshDisplay();

	std::string		m_cMinLabel;
	std::string		m_cMaxLabel;
	std::string		m_cProgressFormat;

	Color32_s		mFillColor;
	Color32_s		mBarColor;
	CView*			m_pcRenderView;
	CBitmap*		m_pcRenderBitmap;
	float			m_vSliderSize;
	int				m_nNumSteps;
	int				m_nNumTicks;
	int				m_nTickFlags;
	knob_mode		m_eKnobMode;
	float			m_vMin;
	float			m_vMax;
	float			m_vSmallStep;
	float			m_vBigStep;
	orientation		m_eOrientation;
	float			__m_vValue;

	bool			m_bChanged;
	bool			m_bTrack;
	CPoint			m_cHitPos;
	uint32			__reserved__[16];
};

}

#endif
