/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__TABVIEW_H__
#define	__TABVIEW_H__

#include <support/SupportDefs.h>

#include <interface/View.h>
#include <interface/Font.h>
#include <app/Invoker.h>

#include <vector>
#include <string>

namespace os
{

/** 
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class TabView : public CView, public CInvoker
{
public:
						TabView( CRect inFrame,
								 const char* inTitle,
								 uint32 nResizeMask = CF_FOLLOW_LEFT | CF_FOLLOW_TOP,
								 uint32 nFlags = WID_WILL_DRAW | WID_FULL_UPDATE_ON_RESIZE );

	int					AppendTab( const char* pzTitle, CView* pcView = NULL );
	int					InsertTab( uint nIndex, const char* pzTitle, CView* pcView = NULL );
	CView*				DeleteTab( uint nIndex );
	CView*				GetTabView( uint nIndex ) const;

	int 				SetTabTitle( uint nIndex, const std::string& cTitle );
	const std::string&	GetTabTitle( uint nIndex ) const;

	uint				GetSelection();
	void				SetSelection( uint nIndex, bool bNotify = true );

	virtual void		FrameResized( float inWidth, float inHeight );
	virtual void		MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
	virtual void		MouseDown( CPoint cPosition );
	virtual void		MouseUp( CPoint cPosition );
	virtual void		KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );

	virtual void	    GetPreferredSize( float* outWidth, float* outHeight );
	virtual void		Draw( CRect cUpdateRect );
	virtual void		AllAttached();

private:
	struct Tab
	{
		Tab( const char* pzTitle, CView* pcView ) : m_cTitle( pzTitle ) { m_pcView = pcView; }
		CView*			m_pcView;
		std::string 	m_cTitle;
		float	    	m_vWidth;
	};

	class TopView : public CView
	{
	public:
		TopView( CRect inFrame, TabView* pcParent ) :
			CView( inFrame, "top_view", CF_FOLLOW_NONE, WID_WILL_DRAW )
		{
			m_pcParent = pcParent;
		}

		virtual void	Draw( CRect cUpdateRect );
		
	private:
		TabView*		m_pcParent;
	};

	friend class TabView::TopView;

	uint				m_nSelectedTab;
	float				m_vScrollOffset;
	CPoint				m_cHitPos;
	bool				m_bMouseClicked;
	float				m_vTabHeight;
	float		 		m_vGlyphHeight;
	font_height  		m_sFontHeight;
	float				m_vTotalWidth;
	std::vector<Tab>	m_cTabList;
	TopView*			m_pcTopView;
};


}


#endif 
