/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __TABLEVIEW_H__
#define __TABLEVIEW_H__

#include <interface/View.h>

namespace os
{

class TableCell
{
public:
				TableCell();
				~TableCell();

	void		SetView( CView* pcView );
	CView*		GetView( void ) const;

	void		SetWeights( float vHorizontal, float vVertical );
	void		GetPreferredSize( float* outWidth, float* outHeight );
	void		SetAlignment( alignment eHorizontal, alignment hVertical );

	void		SetWidth( int nWidth );
	void		SetHeight( int nHeight );

	void		SetSize( CPoint cSize );
	void		SetPos( CPoint cPos );
	void		Layout( void );

private:
	friend class TableView;

	CView*		m_pcView;

	CPoint		m_cSize;
	CPoint		m_cPos;

	float		m_vHorWeight;
	float		m_vVerWeight;

	alignment	m_eHorAlign;
	alignment	m_eVerAlign;

	int			m_nLeftBorder;
	int			m_nRightBorder;
	int			m_nTopBorder;
	int			m_nBottomBorder;
};

class TableView : public CView
{
public:
					TableView( CRect inFrame,
							   const char* pzName,
							   const char* pzTitle,
							   int nWidth,
							   int nHeight,
							   uint32 nResizeMask = CF_FOLLOW_LEFT | CF_FOLLOW_TOP );
					~TableView();

		// From View:
	void			AttachedToWindow( void );
	void			AllAttached( void );
	void			FrameResized( float inWidth, float inHeight );
	void			Draw( CRect cUpdateRect );
	void			GetPreferredSize( float* outWidth, float* outHeight );


		// From TableView:
	void			SetColAlignment( int nCol, alignment eAlign );
	void			SetCellAlignment( int x, int y, alignment eHor, alignment eVer = ALIGN_CENTER );

	void			SetCellBorders( int x, int y, int nLeft, int nTop, int nRight, int nBottom );
	void			SetCellBorders( int x, int nLeft, int nTop, int nRight, int nBottom );
	void			SetCellBorders( int nLeft, int nTop, int nRight, int nBottom );

	CView*			SetChild( CView* pcView, int nColumn, int nRow, float vHorWheight = 1.0f, float vVerWheight = 1.0f );
	TableCell*		GetCell( int x, int y ) const;
	void			Layout();

private:
	void			CalcSizes( void ) const;
	void			Layout( CRect cBounds );

	TableCell*		m_pacCells;

	mutable float*	m_panMinWidths;
	mutable float*	m_panMaxWidths;

	mutable float*	m_panMinHeights;
	mutable float*	m_panMaxHeights;

	mutable float*	m_pavColWeights;
	mutable float*	m_pavRowWeights;

	mutable float	m_nMinWidth;
	mutable float	m_nMaxWidth;
	mutable float	m_nMinHeight;
	mutable float	m_nMaxHeight;

	float			m_nLeftBorder;
	float			m_nRightBorder;
	float			m_nTopBorder;
	float			m_nBottomBorder;
	char*			m_pzTitle;

	int				m_nNumRows;
	int				m_nNumCols;
};

}

#endif // __TABLEVIEW_H__
