/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __TEXTVIEW_H__
#define __TEXTVIEW_H__

#include <interface/View.h>
#include <interface/Font.h>
#include <interface/Control.h>
#include <support/String.h>

#include <vector>
#include <string>
#include <list>

namespace os
{

class TextEdit;


class TextView : public CControl
{
public:
    enum {
	EI_CONTENT_CHANGED	= 0x0001,
	EI_ENTER_PRESSED	= 0x0002,
	EI_ESC_PRESSED		= 0x0004,
	EI_FOCUS_LOST		= 0x0008,
	EI_CURSOR_MOVED		= 0x0010,
	EI_SELECTION_CHANGED	= 0x0020,
	EI_MAX_SIZE_REACHED	= 0x0040,
	EI_MAX_SIZE_LEFT	= 0x0080
    };
public:
    typedef std::vector<CString> buffer_type;
  
    TextView( CRect cFrame, const char* pzTitle, const char* pzBuffer,
	      uint32 nResizeMask = CF_FOLLOW_LEFT | CF_FOLLOW_TOP,
	      uint32 nFlags = WID_WILL_DRAW | WID_FULL_UPDATE_ON_RESIZE );
    ~TextView();

    virtual void	MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
    virtual void	MouseDown( CPoint cPosition );
    virtual void	MouseUp( CPoint cPosition );
    virtual void	KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );

    virtual void LabelChanged( const std::string& cNewLabel );
    virtual void EnableStatusChanged( bool bIsEnabled );
    virtual bool Invoked( CMessage* pcMessage );
    virtual void Activated( bool bIsActive );

    virtual void    SetValue( Variant cValue, bool bInvoke = true );
    virtual Variant Value() const;
    
    const CView* GetEditor() const;

//    void   SetEnable( bool bEnabled = true );
//    bool   IsEnabled() const;
    
    void  SetMultiLine( bool bMultiLine = true );
    bool  GetMultiLine();

    void  SetPasswordMode( bool bPassword = true );
    bool  GetPasswordMode() const;

    void  SetNumeric( bool bNumeric );
    bool  GetNumeric() const;
    
    void  SetReadOnly( bool bFlag = true );
    bool  GetReadOnly() const;

    int	  GetMaxUndoSize() const;
    void  SetMaxUndoSize( int nSize );


    uint32 GetEventMask() const;
    void   SetEventMask( uint32 nMask );
    
    void  GetRegion( std::string* pcBuffer );

    void   SetMinPreferredSize( int nWidthChars, int nHeightChars );
    IPoint GetMinPreferredSize() const;
    void   SetMaxPreferredSize( int nWidthChars, int nHeightChars );
    IPoint GetMaxPreferredSize() const;
    
    void MakeCsrVisible();
    void Clear( bool bSendNotify = true );
    void Set( const char* pzBuffer, bool bSendNotify = true );
    void Insert( const char* pzBuffer, bool bSendNotify = true );



    void	Select( const IPoint& cStart, const IPoint& cEnd, bool bSendNotify = true );
    void	SelectAll( bool bSendNotify = true );
    void	ClearSelection( bool bSendNotify = true );
    bool	GetSelection( IPoint* pcStart = NULL, IPoint* pcEnd = NULL );
    void	SetCursor( int x, int y, bool bSelect = false, bool bSendNotify = true );
    void	SetCursor( const IPoint& cPos, bool bSelect = false, bool bSendNotify = true  );
    void	GetCursor( int* x, int* y ) const;
    IPoint	GetCursor() const;
    
    void SetMaxLength( size_t nMaxLength );
    size_t GetMaxLength() const;
    size_t TextLength() const;
    
    void Cut( bool bSendNotify = true );
    void Copy();
    void Paste( bool bSendNotify = true );
    void Delete( bool bSendNotify = true );
  
    const buffer_type& GetBuffer() const;

    virtual void	SetTabOrder( int nOrder );
    virtual void	GetPreferredSize( float* outWidth, float* outHeight );
    virtual bool	FilterKeyStroke( const std::string* pcString );
  
    virtual void	FontChanged( CFont* pcNewFont );
    virtual void	FrameResized( float inWidth, float inHeight );
    virtual void	Draw( CRect cUpdateRect );

    virtual void __TV_reserved1__();
    virtual void __TV_reserved2__();
    virtual void __TV_reserved3__();
    virtual void __TV_reserved4__();
    
private:
    void	AdjustScrollbars( bool bDoScroll = true, float vDeltaX = 0.0f, float vDeltaY = 0.0f );
    friend class TextEdit;
    TextEdit* m_pcEditor;
};


} // end of namespace os

#endif // __F_STRINGEDIT_H__
