/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__VIEW_H__
#define	__VIEW_H__


#include <support/SupportDefs.h>

#include <app/Handler.h>
#include <interface/InterfaceDefs.h>
#include <interface/GraphicsDefs.h>
#include <interface/Region.h>

#define BView	os::CView

#define B_PRIMARY_MOUSE_BUTTON		0x01
#define B_SECONDARY_MOUSE_BUTTON	0x02
#define B_TERTIARY_MOUSE_BUTTON		0x04

#define B_FOLLOW_NONE 			os::CF_FOLLOW_NONE
#define B_FOLLOW_ALL_SIDES		os::CF_FOLLOW_ALL
#define B_FOLLOW_ALL  			os::CF_FOLLOW_ALL

#define B_FOLLOW_LEFT			os::CF_FOLLOW_LEFT
#define B_FOLLOW_RIGHT			os::CF_FOLLOW_RIGHT
#define B_FOLLOW_H_CENTER		os::CF_FOLLOW_H_CENTER

#define B_FOLLOW_TOP			os::CF_FOLLOW_TOP
#define B_FOLLOW_BOTTOM			os::CF_FOLLOW_BOTTOM
#define B_FOLLOW_V_CENTER		os::CF_FOLLOW_V_CENTER

#define B_FOLLOW_LEFT_RIGHT		(os::CF_FOLLOW_LEFT | os::CF_FOLLOW_RIGHT)

#define B_FULL_UPDATE_ON_RESIZE	os::WID_FULL_UPDATE_ON_RESIZE
#define B_WILL_DRAW 			os::WID_WILL_DRAW
#define B_DRAW_ON_CHILDREN		os::WID_DRAW_ON_CHILDREN
#define B_PULSE_NEEDED			os::WID_PULSE_NEEDED
#define B_FRAME_EVENTS			os::WID_FRAME_EVENTS

#define B_ENTERED_VIEW			os::MOUSE_ENTERED
#define B_INSIDE_VIEW			os::MOUSE_INSIDE
#define B_EXITED_VIEW			os::MOUSE_EXITED
#define B_OUTSIDE_VIEW			os::MOUSE_OUTSIDE

#define ui_color				os::GetStdColor

#define B_MENU_BACKGROUND_COLOR	os::COL_MENU_BACKGROUND

// FIXME: make-this-sucka-compile hacks:
#define tint_color(x,y)			x
#define DrawBitmapAsync			DrawBitmap
#define AddLine(x,y,z)			StrokeLine(x,y)

namespace os
{

Color32_s GetStdColor( int i );

class CBitmap;
class CScrollBar;
struct font_height;

enum
{
	PEN_DETAIL,	      /* primary pen (Draw, PutBixel, ... )					*/
	PEN_BACKGROUND,   /* secondary pen ( EraseRect, text background, ...)	*/
	PEN_SHINE,	      /* bright side of 3D object 							*/
	PEN_SHADOW,	      /* dark side of 3D object								*/

	PEN_BRIGHT,       /* same as PEN_SHINE, but a bit darker				*/
	PEN_DARK,	      /* same as PEN_SHADOW, but a bit brighter				*/
	PEN_WINTITLE,     /* Fill inside window title of a unselected window	*/
	PEN_WINBORDER,    /* Window border fill when unselected					*/

	PEN_SELWINTITLE,  /* Fill inside window title of a selected window		*/
	PEN_SELWINBORDER, /* Window border fill when selected					*/
	PEN_WINDOWTEXT,
	PEN_SELWNDTEXT,

	PEN_WINCLIENT,    /* Used to clear areas inside client area of window	*/
	PEN_GADGETFILL,
	PEN_SELGADGETFILL,
	PEN_GADGETTEXT,

	PEN_SELGADGETTEXT
};

enum default_color_t
{
	COL_NORMAL,
	COL_SHINE,
	COL_SHADOW,
	COL_SEL_WND_BORDER,
	COL_NORMAL_WND_BORDER,
	COL_MENU_TEXT,
	COL_SEL_MENU_TEXT,
	COL_MENU_BACKGROUND,
	COL_SEL_MENU_BACKGROUND,
	COL_SCROLLBAR_BG,
	COL_SCROLLBAR_KNOB,
	COL_LISTVIEW_TAB,
	COL_LISTVIEW_TAB_TEXT,
	COL_COUNT
};

Color32_s get_default_color( default_color_t nColor );
void __set_default_color( default_color_t nColor, const Color32_s& sColor );
void set_default_color( default_color_t nColor, const Color32_s& sColor );

enum
{
	FRAME_RECESSED    = 0x000008,
	FRAME_RAISED      = 0x000010,
	FRAME_THIN	      = 0x000020,
	FRAME_WIDE	      = 0x000040,
	FRAME_ETCHED      = 0x000080,
	FRAME_FLAT	      = 0x000100,
	FRAME_DISABLED    = 0x000200,
	FRAME_TRANSPARENT = 0x010000
};

class	CWindow;
class	CMessage;
class	CFont;


enum
{
	MOUSE_INSIDE,
	MOUSE_OUTSIDE,
	MOUSE_ENTERED,
	MOUSE_EXITED
};


/** \brief Flags controlling a View
 * \ingroup interface
 * \sa os::view_resize_flags, os::CView
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *//////////////////////////////////////////////////////////////////////////////

enum view_flags
{
	WID_FULL_UPDATE_ON_H_RESIZE	= 0x0001,	// Cause the entire view to be invalidated if made higher
	WID_FULL_UPDATE_ON_V_RESIZE	= 0x0002,	// Cause the entire view to be invalidated if made wider
	WID_FULL_UPDATE_ON_RESIZE	= 0x0003,	// Cause the entire view to be invalidated if resized
	WID_WILL_DRAW				= 0x0004,	// Tell the appserver that you want to render stuff to it
	WID_TRANSPARENT				= 0x0008,	// Allow the parent view to render in areas covered by this view
	WID_CLEAR_BACKGROUND		= 0x0010,	// Automatically clear new areas when windows are moved/resized
	WID_DRAW_ON_CHILDREN		= 0x0020,	// Setting this flag allows the view to render atop of all its childs
	WID_PULSE_NEEDED	 		= 0x0040,	// Periodically call the Pulse() function
	WID_FRAME_EVENTS			= 0x0080	// Unused
};


/** \brief Flags controlling how to resize/move a view when the parent is resized.
 * \ingroup gui
 * \sa os::view_flags, os::CView
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

enum view_resize_flags
{
	CF_FOLLOW_NONE	= 0x0000, ///< Neither the size nor the position is changed.
	CF_FOLLOW_LEFT	= 0x0001, ///< Left edge follows the parents left edge.
	CF_FOLLOW_RIGHT	= 0x0002, ///< Right edge follows the parents right edge.
	CF_FOLLOW_LEFT_RIGHT	= CF_FOLLOW_LEFT | CF_FOLLOW_RIGHT,
	CF_FOLLOW_TOP	= 0x0004, ///< Top edge follows the parents top edge.
	CF_FOLLOW_BOTTOM	= 0x0008, ///< Bottom edge follows the parents bottom edge.
	CF_FOLLOW_ALL	= 0x000F, ///< All edges follows the corresponding edge in the parent
	/**
	* If the CF_FOLLOW_LEFT is set the right edge follows the parents center.
	* if the CF_FOLLOW_RIGHT is set the left edge follows the parents center.
	*/
	CF_FOLLOW_H_CENTER	= 0x0010,
	/**
	* If the CF_FOLLOW_TOP is set the bottom edge follows the parents center.
	* if the CF_FOLLOW_BOTTOM is set the top edge follows the parents center.
	*/
	CF_FOLLOW_V_CENTER	= 0x0020,
	CF_FOLLOW_SPECIAL	= 0x0040,
	CF_FOLLOW_MASK	= 0x007f
};


/** Base class for all GUI components.
 * \ingroup interface
 *
 * The View class is the work horse in the GUI. Before you can render any graphics
 * into a window, you have to create one or more views and attatch them to it.
 * Views can be added to a window, or to another view to create a hierarchy. To render
 * someting into a window you normaly inherit this class and overload the Pain() function.
 * Each view has it's own graphical environment consisting of three colors (foreground
 * background and erase), a pen position and the drawing mode. Each view lives in a
 * coordinate system relative to its parent. Views are also responsible for receiving
 * user input. The active view in the active window receives keybord and mouse events.
 * 
 *  \sa os::CWindow, os::CHandler, \ref os::view_flags
 *  \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CView : public CHandler
{
public:
					CView( CRect inFrame,
						   const char* inTitle,
						   uint32 nResizeMask = CF_FOLLOW_LEFT | CF_FOLLOW_TOP,
						   uint32 nFlags = WID_WILL_DRAW | WID_CLEAR_BACKGROUND );

	virtual			~CView();


	// Hook functions
	virtual void	AttachedToWindow();
	virtual void	AllAttached();
	virtual void	DetachedFromWindow();
	virtual void	AllDetached();
	virtual void	Activated( bool bIsActive );
	virtual void	WindowActivated( bool bIsActive );

	virtual void	Draw( CRect cUpdateRect );

	virtual void	MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData );
	virtual void	MouseDown( CPoint cPosition );
	virtual void	MouseUp( CPoint cPosition );
	virtual void	KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers );
	virtual void	KeyUp( const char* pzString, const char* pzRawString, uint32 nQualifiers );

	virtual	void	Pulse() {};

	virtual void	FrameMoved( CPoint cDelta );
	virtual void	FrameResized( float inWidth, float inHeight );
	virtual void	ViewScrolled( const CPoint& cDelta );
	virtual void	FontChanged( CFont* pcNewFont );

	virtual void	GetPreferredSize( float* outWidth, float* outHeight );
	virtual void	ResizeToPreferred();

	virtual CPoint	GetContentSize() const;

	// End of hook functions

	virtual void	AddChild( CView* pcView, bool bAssignTabOrder = false );
	void 			RemoveChild( CView* pcChild );
	void			RemoveSelf();

	int32			CountChildren() const;
	CView*			ChildAt( const CPoint& cPos ) const;
	CView*			ChildAt( int nIndex ) const;
	CView*			Parent() const;
	CScrollBar*		GetScrollBar(orientation inAxis) const;
	CWindow*		Window() const { return( (CWindow*)Looper() ); }
	const char*		Title() const;

	CView*			FindView(const char *name) const;
	CView*			NextSibling() const;
	CView*			PreviousSibling() const;

	virtual int		GetTabOrder();
	virtual void	SetTabOrder( int nOrder );

	uint32			GetQualifiers() const;
	void			GetMouse( CPoint* pcPosition, uint32* pnButtons, bool inCheckQueue = true );
	void			SetMousePos( const CPoint& cPosition );
	
	void			WheelMoved( CPoint& inPoint );

	void			BeginDrag( CMessage* pcData, const CPoint& cOffset, const CBitmap* pcBitmap,
								CHandler* pcReplyTarget = NULL );
	void			BeginDrag( CMessage* pcData, const CPoint& cOffset, const CRect& cBounds,
								CHandler* pcReplyTarget = NULL );

	void			SetFlags( uint32 nFlags );
	uint32			Flags( uint32 nMask = ~0L )	const;

	void			SetResizeMask( uint32 nFlags );
	uint32			GetResizeMask()	const;

	void			Show( bool bVisible = true );
	void			Hide() {Show(false);}
	bool			IsHidden() const;
	virtual void	MakeFocus( bool bFocus = true );
	virtual bool	IsFocus() const;

	CRect			Frame() const;
	CRect			Bounds() const;
	CRect			GetNormalizedBounds() const;
	float			Width() const;
	float			Height() const;
	CPoint			GetLeftTop() const;

	virtual void	SetFrame( const CRect& cRect, bool bNotifyServer = true );
	virtual void	MoveBy( const CPoint& cDelta );
	virtual void	MoveBy( float vDeltaX, float vDeltaY );
	virtual void	MoveTo( CPoint cPos );
	virtual void	MoveTo( float x, float y );

	virtual void	ResizeBy( const CPoint& cDelta );
	virtual void	ResizeBy( float vDeltaW, float vDeltaH );
	virtual void	ResizeTo( const CPoint& cSize );
	virtual void	ResizeTo( float W, float H );

	virtual int		ToggleDepth();

	CPoint			ConvertToParent( CPoint cPoint ) const;
	void			ConvertToParent( CPoint* cPoint ) const;
	CRect			ConvertToParent( CRect cRect ) const;
	void			ConvertToParent( CRect* cRect ) const;

	CPoint			ConvertFromParent( CPoint cPoint ) const;
	void			ConvertFromParent( CPoint* cPoint ) const;
	CRect			ConvertFromParent( CRect cRect ) const;
	void			ConvertFromParent( CRect* cRect ) const;

	CPoint			ConvertToWindow( CPoint cPoint ) const;
	void			ConvertToWindow( CPoint* cPoint ) const;
	CRect			ConvertToWindow( CRect cRect ) const;
	void			ConvertToWindow( CRect* cRect ) const;

	CPoint			ConvertFromWindow( CPoint cPoint ) const;
	void			ConvertFromWindow( CPoint* cPoint ) const;
	CRect			ConvertFromWindow( CRect cRect ) const;
	void			ConvertFromWindow( CRect* cRect ) const;


	CPoint			ConvertToScreen( CPoint cPoint ) const;
	void			ConvertToScreen( CPoint* cPoint ) const;
	CRect			ConvertToScreen( CRect cRect ) const;
	void			ConvertToScreen( CRect* cRect ) const;

	CPoint			ConvertFromScreen( CPoint cPoint ) const;
	void			ConvertFromScreen( CPoint* cPoint ) const;
	CRect			ConvertFromScreen( CRect cRect ) const;
	void			ConvertFromScreen( CRect* cRect ) const;

	void			Invalidate( CRect cRect, bool bRecurse = false );
	void			Invalidate( bool bRecurse = false );

	void			Flush();
	void			Sync();

	void			SetDrawingMode( drawing_mode inMode );
	drawing_mode	DrawingMode() const;
	
	void			SetFont( CFont* pcFont );
	CFont*			GetFont() const;

	void			SetFontSize(float size);

	void			SetHighColor( int nRed, int nGreen, int nBlue, int nAlpha = 255 );
	void			SetHighColor( Color32_s sColor );
	Color32_s		HighColor() const;

	void			SetLowColor( int nRed, int nGreen, int nBlue, int nAlpha = 255 );
	void			SetLowColor( Color32_s sColor );
	Color32_s		LowColor() const;

	void			SetViewColor( int nRed, int nGreen, int nBlue, int nAlpha = 255 );
	void			SetViewColor( Color32_s sColor );
	Color32_s		ViewColor() const;

	void			SetOrigin(CPoint inPt);
	void			SetOrigin(float x, float y)			{ SetOrigin(CPoint(x,y)); }
	CPoint			Origin() const;
	
	void			SetScale(float inScale) const;
	
	void			PushState();
	void			PopState();

	void			MovePenTo( CPoint cPos );
	void			MovePenTo( float x, float y )		{ MovePenTo( CPoint( x, y ) ); }
	void			MovePenBy( CPoint cPos );
	void			MovePenBy( float x, float y )		{ MovePenBy( CPoint( x, y ) ); }
	CPoint			PenLocation() const;
	void			StrokeLine( CPoint cToPoint /*, Pattern inPatern = SOLID_HIGH */ );
	void			StrokeLine( CPoint cFromPnt, CPoint cToPnt /*, Pattern inPattern = SOLID_HIGH */ );
	virtual void	ScrollBy( CPoint cDelta );
	virtual void	ScrollBy( float x, float y )		{ ScrollBy( CPoint( x, y ) );	}
	virtual void	ScrollTo( CPoint cTopLeft );
	virtual void	ScrollTo( float x, float y )		{ ScrollTo( CPoint( x, y ) );	}
	CPoint			GetScrollOffset() const;
	void			ScrollRect( const CRect& cSrcRect, const CRect& cDstRect );
	
	void			BeginLineArray(int32 inLineCount)	{ /* FIXME */ }
	void			EndLineArray() 						{ /* FIXME */ }
//	void			AddLine(CPoint inFromPt, CPoint inToPt, Color32_s inColor);

	void			StrokeRect( CRect cRect );
	void			FillRect( CRect cRect );
	void			FillRect( CRect cRect, Color32_s sColor );	// WARNING: Will leave HiColor at sColor
	void			DrawBitmap( const CBitmap* inBitmap, CRect inSrcRect, CRect inDstRect );
	void			DrawBitmap( const CBitmap* inBitmap, CRect inDstRect );
	void			DrawBitmap( const CBitmap *inBitmap, CPoint inWhere );
	void			DrawBitmap( const CBitmap *inBitmap );
	void			EraseRect( const CRect& cRect );
	void			DrawFrame( const CRect& cRect, uint32 nFlags );
	void			DrawChar( int nChar )		   { char zStr[] = {nChar}; DrawString( zStr, 1 ); }
	void			DrawChar( int nChar, const CPoint& cPos ) { char zStr[] = {nChar}; DrawString( zStr, 1, cPos ); }

	void			DrawString( const char* inString );
	void			DrawString( const char* inString, int inLength );

	void			DrawString( const char* inString, CPoint inPosition );
	void			DrawString( const char* inString, int inLength, CPoint inPosition );

	// Font functions.
	void            GetTruncatedStrings( const char** pazStringArray,
										 int    nStringCount,
										 uint32 nMode,
										 float  nWidth,
										 char** pazResultArray ) const;

	float			StringWidth( const char* pzString) const;
	float			StringWidth( const char* pzString, int nLength ) const;
	float			StringWidth( const std::string& cString ) const;
	void			GetStringWidths( const char** apzStringArray, const int* anLengthArray,
					int nStringCount, float* avWidthArray ) const;

	int				GetStringLength( const char* pzString, float vWidth, bool bIncludeLast = false ) const;
	int				GetStringLength( const char* pzString, int nLength, float vWidth, bool bIncludeLast = false ) const;
	int				GetStringLength( const std::string& cString, float vWidth, bool bIncludeLast = false ) const;
	void			GetStringLengths( const char** apzStringArray, const int* anLengthArray, int nStringCount,
					float vWidth, int* anMaxLengthArray, bool bIncludeLast = false ) const;

	void			GetFontHeight( font_height* psHeight ) const;


	void			Ping( int nSize = 0 );

private:
	friend class CWindow;
	friend class CScrollBar;
	friend class CFont;

	static int		s_nNextTabOrder;

	void			_LinkChild( CView* pcChild, bool bTopmost ); // Add a view to the child list
	void			_UnlinkChild( CView* pcChild );				 // Remove a view from the child list
	void			_Attached( CWindow* pcFrame, CView* pcParent, int hHandle, int nHideCount );
	void			_Detached( bool bFirst, int nHideCount );
	void 			_IncHideCount( bool bVisible );
	int				_GetHandle() const;

	void			_SetMouseMoveRun( int nValue );
	int				_GetMouseMoveRun() const;

	void			_SetMouseMode( int nMode );
	int				_GetMouseMode() const;

	void			_SetHScrollBar( CScrollBar* pcScrollBar );
	void			_SetVScrollBar( CScrollBar* pcScrollBar );

	void			_BeginUpdate();
	void			_EndUpdate();

	void			_ParentSized( const CPoint& cDelta );
	void			_WindowActivated( bool bIsActive );
	void			_ReleaseFont();
	void			_ConstrictRectangle( CRect* pcRect, const CPoint& cOffset );

	virtual void	FBC_View1();
	virtual void	FBC_View2();
	virtual void	FBC_View3();
	virtual void	FBC_View4();
	virtual void	FBC_View5();
	virtual void	FBC_View6();
	virtual void	FBC_View7();
	virtual void	FBC_View8();
	virtual void	FBC_View9();
	virtual void	FBC_View10();
	virtual void	FBC_View11();
	virtual void	FBC_View12();
	virtual void	FBC_View13();
	virtual void	FBC_View14();
	virtual void	FBC_View15();
    
	class Private;
    Private* m;
};

}

#endif	// __F_GUI_VIEW_H__
