/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef	__WINDOW_H__
#define	__WINDOW_H__


#include <interface/View.h>
#include <app/Looper.h>

#define BWindow						os::CWindow
#define B_UNTYPED_WINDOW			os::WT_UNTYPED_WINDOW
#define B_TITLED_WINDOW				os::WT_TITLED_WINDOW
#define B_MODAL_WINDOW				os::WT_MODAL_WINDOW
#define B_DOCUMENT_WINDOW			os::WT_DOCUMENT_WINDOW
#define B_BORDERED_WINDOW			os::WT_BORDERED_WINDOW
#define B_FLOATING_WINDOW			os::WT_FLOATING_WINDOW

#define B_NOT_RESIZABLE				os::WND_NOT_RESIZABLE
#define B_NOT_H_RESIZABLE			os::WND_NOT_H_RESIZABLE
#define B_NOT_V_RESIZABLE			os::WND_NOT_V_RESIZABLE
#define B_NOT_ZOOMABLE				os::WND_NO_ZOOM_BUT
#define B_NOT_CLOSABLE				os::WND_NO_CLOSE_BUT
#define B_NOT_MOVABLE				os::WND_NOT_MOVEABLE
#define B_ASYNCHRONOUS_CONTROLS		os::WND_ASYNC_CONTROLS
#define B_WILL_ACCEPT_FIRST_CLICK	os::WND_ACCEPT_FIRST_CLICK
#define B_AVOID_FRONT				os::WND_AVOID_FRONT
#define B_NOT_MINIMIZABLE			os::WND_NOT_MINIMIZABLE

#define B_NORMAL_WINDOW_FEEL		os::WF_NORMAL
#define B_MODAL_APP_WINDOW_FEEL		os::WF_APP_MODAL
#define B_MODAL_ALL_WINDOW_FEEL		os::WF_SYSTEM_MODAL
#define B_FLOATING_APP_WINDOW_FEEL	os::WF_APP_FLOATING
#define B_FLOATING_ALL_WINDOW_FEEL	os::WF_SYSTEM_FLOATING

#define B_BORDERED_WINDOW_LOOK		os::WL_BORDERED_WINDOW
#define B_NO_BORDER_WINDOW_LOOK		os::WL_NO_BORDER_WINDOW
#define B_TITLED_WINDOW_LOOK		os::WL_TITLED_WINDOW
#define B_DOCUMENT_WINDOW_LOOK		os::WL_DOCUMENT_WINDOW
#define B_MODAL_WINDOW_LOOK			os::WL_MODAL_WINDOW
#define B_FLOATING_WINDOW_LOOK		os::WL_FLOATING_WINDOW

#define B_ALL_WORKSPACES			os::ALL_DESKTOPS


namespace os
{

class CMenu;
class View;
struct WR_Render_s;

enum window_type
{
	WT_UNTYPED_WINDOW	= 0,
	WT_TITLED_WINDOW 	= 1,
	WT_MODAL_WINDOW 	= 2,
	WT_DOCUMENT_WINDOW	= 3,
	WT_BORDERED_WINDOW	= 4,
	WT_FLOATING_WINDOW	= 5
};

enum window_feel
{
	WF_NORMAL			= 1,
	WF_APP_MODAL		= 2,
	WF_SYSTEM_MODAL		= 3,
	WF_APP_FLOATING		= 5,
	WF_SYSTEM_FLOATING	= 6
};

enum window_look
{
	WL_BORDERED_WINDOW	= 20,
	WL_NO_BORDER_WINDOW	= 19,
	WL_TITLED_WINDOW	= 1,
	WL_DOCUMENT_WINDOW	= 11,
	WL_MODAL_WINDOW		= 3,
	WL_FLOATING_WINDOW	= 7
};

enum
{
	WND_NO_CLOSE_BUT		= 0x00000001,
	WND_NO_ZOOM_BUT			= 0x00000002,
	WND_NO_DEPTH_BUT		= 0x00000004,
	WND_NOT_MOVEABLE		= 0x00000008,
	WND_NOT_H_RESIZABLE 	= 0x00000010,
	WND_NOT_V_RESIZABLE 	= 0x00000020,
	WND_NOT_RESIZABLE   	= 0x00000030,
	WND_NO_TITLE			= 0x00000040,
	WND_NO_BORDER			= 0x00000080,
	WND_FRONTMOST			= 0x00000100,
	WND_BACKMOST			= 0x00000200,
	WND_MODAL				= 0x00000400,
	WND_SYSTEM				= 0x00010000,
	WND_ASYNC_CONTROLS		= 0x00020000,	// FIXME: unimplemented
	WND_ACCEPT_FIRST_CLICK	= 0x00040000,	// FIXME: unimplemented
	WND_AVOID_FRONT			= 0x00080000,	// FIXME: unimplemented
	WND_NOT_MINIMIZABLE		= 0x00100000	// FIXME: unimplemented
};

enum
{
	CURRENT_DESKTOP 	= 0,
	ALL_DESKTOPS		= ~0
};

/**
 * \ingroup interface
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class	CWindow : public CLooper
{
public:
					CWindow( CRect inFrame,
							 const char* inTitle,
							 window_type inType = WT_UNTYPED_WINDOW,
							 uint32 inFlags = 0,
							 uint32 inWorkspace = CURRENT_DESKTOP );
					CWindow( CRect inFrame,
							 const char* inTitle,
							 window_look inType,
							 window_feel inFeel,
							 uint32 inFlags,
							 uint32 inWorkspace = CURRENT_DESKTOP );
					~CWindow();

	// Hook functions
	virtual void	FrameMoved( CPoint cDelta );
	virtual void	FrameResized( float inWidth, float inHeight );
	virtual void	ScreenModeChanged( const IPoint& cNewRes, color_space eColorSpace );
	virtual void	WorkspaceActivated( int32 inWorkspace, bool inActive );
	virtual void	WorkspacesChanged( uint32 inOldWorkspace, uint32 inNewWorkspace );
	virtual	void	MessageReceived(CMessage *inMessage);
	
	virtual void	MenusBeginning(void)	{};
	virtual void	MenusEnded(void)		{};

	void			SetWorkspaces(uint32 inWorkspaceMask);
	uint32			Workspaces() const;
	
	status_t		SetType( window_type inType );
	window_type		Type() const;

	status_t		SetFeel( window_feel inFeel );
	window_feel		Feel() const;
	
	status_t		SetLook( window_look inLook );
	window_look		Look() const;

	void			SetFlags( uint32 inFlags );
	uint32			GetFlags() const;

	void			SetSizeLimits( float minX, float minY, float maxX, float maxY);
	void			SetAlignment( const IPoint& cSize = IPoint(1,1),
								  const IPoint& cSizeOffset = IPoint(0,0),
								  const IPoint& cPos = IPoint(1,1),
								  const IPoint& cPosOffset = IPoint(0,0) );

	void			AddChild( CView* pcChild, bool bAssignTabOrder = false );
	void			RemoveChild( CView* pcChild );

	void			Activate( bool bFocus = true );
	void			Close() { Quit(); }
	
	void			Show( bool bMakeVisible = true );
	void			Hide() { Show( false ); }
	bool			IsHidden() const;

	void			Start();

	void			SetTitle( const char* inTitle );
	const char*		Title( void ) const;

//  void			SetMenuBar( CMenu* pcMenu );

	void			Flush();
	void			Sync();

	CView*			FindView( const std::string& cName ) const	{ return( dynamic_cast<CView*>(FindHandler( cName )) );	}
	CView*			FindView( const CPoint& cPos ) const;
	CView*			CurrentFocus( void ) const			{ return( dynamic_cast<CView*>(PreferredHandler()) );	}
	CView*			SetFocusChild( CView* pcChild );
	void			SetDefaultButton( CView* pcView );
	CView*			DefaultButton() const;

	CRect			Frame() const;
	CRect			Bounds() const;

	void			SetFrame( const CRect& cRect, bool bNotifyServer = true );

	void			MoveBy( float vDeltaX, float vDeltaY );
	void			MoveTo( CPoint inPoint )				{ MoveTo(inPoint.x, inPoint.y); }
	void			MoveTo( float x, float y );

	void			ResizeBy( float vDeltaW, float vDeltaH );
	void			ResizeTo( float w, float h );

	bool			IsActive() const;
	int				ToggleDepth();

	void			SetPulseRate(bigtime_t rate);
	bigtime_t		PulseRate() const;
	
	void			SetDefaultWheelView( CView* pcView );
	CView*			GetDefaultWheelView() const;

	void			TimerTick( int inID );

	void			UpdateIfNeeded() { Sync(); }
	
	void			AddShortcut( uint32 /*inKey*/,
								 uint32 /*inModifiers*/,
								 CMessage* /*inMessage*/,
								 CHandler* inTarget=NULL)	{ /* FIXME */ }

	
	//	From Looper:

	void			DispatchMessage( CMessage* pcMessage, CHandler* pcHandler );

private:
	friend	class	CView;
	friend	class	CBitmap;
	friend	class	CMenuItem;

					CWindow( CBitmap* pcBitmap );
	void			_ViewDeleted( CView* pcView );
	void			_Init( const char* inTitle,
						   CRect inFrame,
						   window_look inLook,
						   window_feel inFeel,
						   uint32 inFlags,
						   uint32 inWorkspaces);
	void			_Cleanup();
	void			_InterpretWindowType( window_type inType, window_look* outLook, window_feel* outFeel );
	void*			_AllocRenderCmd( uint32 nCmd, CView* pcView, uint32 nSize );
	void			_HandleActivate( bool bIsActive, const CPoint& cMousePos );
	void			_CallMouseMoved( CView* pcView, uint32 nButtons, int nWndTransit, const CMessage* pcData );
	void			_MouseEvent( const CPoint& cNewPos, uint32 nButtons, const CMessage* pcData, bool bReEntered );

	void			_SetMenuOpen( bool bOpen );

	void			_DeleteViewFromServer( CView* pcView );
	CPoint			_GetPenPosition( int nViewHandle );
	CView*			_GetPrevTabView( CView* pcCurrent );
	CView*			_GetNextTabView( CView* pcCurrent );
	void			_DefaultColorsChanged();

	CView*			_GetTopView() const;
	port_id			_GetAppserverPort() const;

	void			_AddPulser(CView* inView);
	void			_RemovePulser(CView* inView);

	virtual void	FBC_Window1();
	virtual void	FBC_Window2();
	virtual void	FBC_Window3();
	virtual void	FBC_Window4();
	virtual void	FBC_Window5();
	virtual void	FBC_Window6();
	
	class Private;
	Private* m;
};

} // end of namespace

#endif	// __WINDOW_H__
