/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __DESKBAR_H__
#define __DESKBAR_H__

#include <interface/Rect.h>
#include <storage/Entry.h>


#define BDeskbar				os::CDeskbar
#define deskbar_location		os::DeskbarLocation
#define B_DESKBAR_TOP			os::kDeskbarTop
#define B_DESKBAR_BOTTOM		os::kDeskbarBottom
#define B_DESKBAR_LEFT_TOP		os::kDeskbarLeftTop
#define B_DESKBAR_RIGHT_TOP		os::kDeskbarRightTop
#define B_DESKBAR_LEFT_BOTTOM	os::kDeskbarLeftBottom
#define B_DESKBAR_RIGHT_BOTTOM	os::kDeskbarRightBottom


namespace os
{

class CMessenger;
class CView;

enum DeskbarLocation
{
	kDeskbarTop,
	kDeskbarBottom,
	kDeskbarLeftTop,
	kDeskbarRightTop,
	kDeskbarLeftBottom,
	kDeskbarRightBottom
};

class CDeskbar
{
public:
						CDeskbar();
						~CDeskbar();

	CRect				Frame() const;

	DeskbarLocation		Location(bool* isExpanded=NULL) const;
	status_t			SetLocation(DeskbarLocation inLocation, bool expanded=false);

	bool				IsExpanded() const;
	status_t			Expand(bool yn);

	status_t			GetItemInfo(int32 id, const char **name) const;
	status_t			GetItemInfo(const char* name, int32 *id) const;

	bool				HasItem(int32 id) const;
	bool				HasItem(const char* name) const;

	uint32				CountItems() const;

	status_t			AddItem(CView* archivableView, int32* id=NULL);
	status_t			AddItem(EntryRef* addon, int32* id=NULL);

	status_t			RemoveItem(int32 id);
	status_t			RemoveItem(const char* name);

private:
	CMessenger*			mMessenger;
	uint32				_reserved[12];
};

}

#endif
