/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __DIRECTORY_H__
#define __DIRECTORY_H__

#include <sys/types.h>
#include <dirent.h>

#include <storage/Node.h>
#include <storage/EntryList.h>

#define BDirectory	os::CDirectory

namespace os {

class CEntry;
class CFile;
class CSymLink;

/** Filesystem directory class
 * \ingroup storage
 * \par Description:
 *	This class lets you iterate over the contents of a directory.
 *
 *	Unlike other CNode-derived classes it is possible to ask a
 *	os::CDirectory to retrieve its own path.
 *
 * \sa CNode
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CDirectory : public CNode, public CEntryList
{
public:
						CDirectory();
						CDirectory( const char* cPath, int nOpenMode = O_RDONLY );
						CDirectory( const CDirectory* cDir, const char* cName, int nOpenMode = O_RDONLY );
						CDirectory( const CEntry* cRef, int nOpenMode = O_RDONLY );
						CDirectory( const CNode& cNode );
						CDirectory( const CDirectory& cDir );
	virtual				~CDirectory();

	virtual status_t	FDChanged( int nNewFD );

	virtual status_t	GetNextEntryName( std::string* inName );
	virtual status_t	GetNextEntry( CEntry* inRef, bool inTraverse=false  );
	virtual status_t	Rewind();

	status_t 			CreateFile( const char* cName, CFile* pcFile, int nAccessMode = S_IRWXU);
	status_t 			CreateDirectory( const char* cName, CDirectory* pcDir, int nAccessMode = S_IRWXU );
	status_t 			CreateSymLink( const char* cName, const char* cDestination, CSymLink* pcLink );
	status_t 			GetPath( std::string* pcPath ) const;

	CDirectory&			operator=( const CDirectory& inDirectory );

private:
	DIR*				mDirIterator;
};


} // end of namespace

#endif // __DIRECTORY_H__
