/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __FILE_H__
#define __FILE_H__

#include <support/DataIO.h>
#include <storage/Node.h>
#include <support/Locker.h>

#define BFile	os::CFile


namespace os {

/** 
 * \ingroup storage
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CFile : public CPositionIO, public CNode
{
public:
	enum { DEFAULT_BUFFER_SIZE=32768 };

						CFile();
						CFile( const char* cPath,
							   int nOpenMode = O_RDONLY );
						CFile( const CDirectory* cDir,
							   const char* cName,
							   int nOpenMode = O_RDONLY );
						CFile( const CEntry* cRef,
							   int nOpenMode = O_RDONLY );
						CFile( const CNode& cNode );
						CFile( const CFile& cFile );
	virtual				~CFile();

	// From Node:
	virtual status_t	FDChanged( int nNewFD );
	virtual status_t	GetSize( off_t* outSize ) const;

	// From DataIO:
	virtual ssize_t		Read( void* outBuffer, size_t inSize );
	virtual ssize_t		Write( const void* inBuffer, size_t inSize );

	// From PositionIO:
	virtual ssize_t		ReadAt( off_t inPos, void* outBuffer, size_t inSize );
	virtual ssize_t		WriteAt( off_t inPos, const void* inBuffer, size_t inSize );

	virtual off_t		Seek( off_t inPosition, uint32 inSeekMode );
	
	// From File:
	status_t			SetBufferSize( int nSize );
	int					GetBufferSize() const;
	status_t			Flush();
    
private:
	void				_Init();
	status_t			_FillBuffer( off_t nPos );

	mutable CLocker		m_cMutex;
	uint8*				m_pBuffer;
	int					m_nBufferSize;
	int					m_nValidBufSize;
	off_t				m_nBufPos;
	bool				m_bDirty;
};


} // end of namespace

#endif // __FILE_H__
