/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef	__FILEPANEL_H__
#define	__FILEPANEL_H__

#include <sys/stat.h>
#include <interface/Window.h>

#define BFilePanel		os::CFilePanel

namespace os
{

class DirectoryView;
class TextView;
class CButton;

class FileFilter
{
public:
  virtual bool FileOk( const char* pzPath, const struct stat* psStat ) = 0;
};

/** Generic file requester.
 * \ingroup interface
 * \par Description:
 *
 * \sa os::DirectoryView
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CFilePanel : public CWindow
{
public:
    enum file_req_mode_t { LOAD_REQ, SAVE_REQ };
    enum { NODE_FILE = 0x01, NODE_DIR = 0x02 };
  
					CFilePanel( file_req_mode_t nMode = LOAD_REQ,
								CMessenger* pcTarget = NULL,
								const char* pzPath = NULL,
								uint32 nNodeType = NODE_FILE,
								bool bMultiSelect = true,
								CMessage* pcMessage = NULL,
								FileFilter* pcFilter = NULL,
								bool  bModal = false,
								bool bHideWhenDone = true,
								const char* pzOkLabel = NULL,
								const char* pzCancelLabel = NULL );
    virtual void	MessageReceived( CMessage* pcMessage );
    virtual void	FrameResized( float inWidth, float inHeight );

    void			SetPath( const std::string& cPath );
    std::string		GetPath() const;
	
private:
	void Layout();
	
	enum { ID_PATH_CHANGED = 1,
		   ID_SEL_CHANGED,
		   ID_INVOKED,
		   ID_CANCEL,
		   ID_OK,
		   ID_ALERT };

	CMessage*	    m_pcMessage;
	CMessenger*	    m_pcTarget;

	file_req_mode_t m_nMode;
	uint32	        m_nNodeType;
	bool	        m_bHideWhenDone;
	DirectoryView*  m_pcDirView;
	TextView*	    m_pcPathView;
	CButton*	    m_pcOkButton;
	CButton*	    m_pcCancelButton;
};

}

#endif // __FILEPANEL_H__
