/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Copyright (C) 2002 Bill Hayden
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __FINDDIRECTORY_H__
#define __FINDDIRECTORY_H__

#include <support/SupportDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	/* ---
		Per volume directories.  When asking for these
		directories, a volume must be specified, or the call will assume
		the boot volume.
	--- */

	B_DESKTOP_DIRECTORY				= 0,
	B_TRASH_DIRECTORY,

	/* ---
		BeOS directories.  These are mostly accessed read-only.
	--- */

	B_BEOS_DIRECTORY				= 1000,
	B_BEOS_SYSTEM_DIRECTORY,
	B_BEOS_ADDONS_DIRECTORY,
	B_BEOS_BOOT_DIRECTORY,
	B_BEOS_FONTS_DIRECTORY,
	B_BEOS_LIB_DIRECTORY,
 	B_BEOS_SERVERS_DIRECTORY,
	B_BEOS_APPS_DIRECTORY,
	B_BEOS_BIN_DIRECTORY,
	B_BEOS_ETC_DIRECTORY,
	B_BEOS_DOCUMENTATION_DIRECTORY,
	B_BEOS_PREFERENCES_DIRECTORY,
	B_BEOS_TRANSLATORS_DIRECTORY,
	B_BEOS_MEDIA_NODES_DIRECTORY,
	B_BEOS_SOUNDS_DIRECTORY,

	/* ---
		Common directories, shared among all users.
	--- */

	B_COMMON_DIRECTORY				= 2000,
	B_COMMON_SYSTEM_DIRECTORY,
	B_COMMON_ADDONS_DIRECTORY,
	B_COMMON_BOOT_DIRECTORY,
	B_COMMON_FONTS_DIRECTORY,
	B_COMMON_LIB_DIRECTORY,
	B_COMMON_SERVERS_DIRECTORY,
	B_COMMON_BIN_DIRECTORY,
	B_COMMON_ETC_DIRECTORY,
	B_COMMON_DOCUMENTATION_DIRECTORY,
	B_COMMON_SETTINGS_DIRECTORY,
	B_COMMON_DEVELOP_DIRECTORY,
	B_COMMON_LOG_DIRECTORY,
	B_COMMON_SPOOL_DIRECTORY,
	B_COMMON_TEMP_DIRECTORY,
	B_COMMON_VAR_DIRECTORY,
	B_COMMON_TRANSLATORS_DIRECTORY,
	B_COMMON_MEDIA_NODES_DIRECTORY,
	B_COMMON_SOUNDS_DIRECTORY,


	/* ---
		User directories.  These are interpreted in the context
		of the user making the find_directory call.
	--- */

	B_USER_DIRECTORY				= 3000,
	B_USER_CONFIG_DIRECTORY,
	B_USER_ADDONS_DIRECTORY,
	B_USER_BOOT_DIRECTORY,
	B_USER_FONTS_DIRECTORY,
	B_USER_LIB_DIRECTORY,
	B_USER_SETTINGS_DIRECTORY,
	B_USER_DESKBAR_DIRECTORY,
	B_USER_PRINTERS_DIRECTORY,
	B_USER_TRANSLATORS_DIRECTORY,
	B_USER_MEDIA_NODES_DIRECTORY,
	B_USER_SOUNDS_DIRECTORY,

	/* ---
		Global directories.
	--- */

	B_APPS_DIRECTORY				= 4000,
	B_PREFERENCES_DIRECTORY,
	B_UTILITIES_DIRECTORY

} directory_which;

/* ---
	The C interface
--- */

extern status_t find_directory (
	directory_which	which,				/* what directory to return path for */
	dev_t			device,				/* device w/volume, for vol-specific directories */
	bool			create_it,			/* create directory if need be */
	char*			returned_path,		/* buffer for returned path */
	int32			path_length			/* buffer size */
);

#ifdef __cplusplus
}
#endif

#ifdef __cplusplus

class CVolume;
class CPath;

/* ---
	C++ interface
--- */

extern status_t find_directory (
	directory_which	which,
	CPath*			path,
	bool			and_create_it = false,
	CVolume*		vol=NULL
);

#endif

#endif
