/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __PATH_H__
#define __PATH_H__

#include <sys/types.h>
#include <support/SupportDefs.h>

#include <string>

#define BPath	os::CPath

namespace os
{

class CEntry;

/** 
 * \ingroup storage
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CPath
{
public:
				CPath();
				CPath( const CPath& cPath );
				CPath( const CEntry* cPath );
				CPath( const char* pzPath );
				CPath( const char* pzPath, int nLen );
				~CPath();

	status_t	InitCheck() const;

	void 		operator =(const CPath& cPath );
	void		operator =(const char* pzPath );
	bool		operator==(const CPath& cPath) const;
	bool		operator==(const char* pzPath) const;
	bool		operator!=(const CPath& cPath) const;
	bool		operator!=(const char* pzPath) const;

	void		SetTo( const char* pzPath, bool inNormalize = false );
	void		SetTo( const char* pzPath, int nLen, bool inNormalize = false );
	status_t	Append( const char* pzName, bool inNormalize = false );

    const char*	Leaf() const;
    const char*	Path() const;

    CPath		GetParent() const;
	status_t	GetParent(CPath* ioParent);

    operator std::string() const;

private:
	void		_Normalize();

//    std::string m_cBuffer;
	char*		m_pzPath;
	uint		m_nMaxSize;
	status_t	mStatus;
};

} // End of namespace

#endif // __PATH_H__
