/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __SYMLINK_H__
#define __SYMLINK_H__

#include <storage/Node.h>

#include <string>

#define BSymLink	os::CSymLink

namespace os {

class CPath;

/** Symbolic link handling class.
 * \ingroup storage
 * \par Description:
 *
 *  \sa CNode, CEntry
 *  \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/


class CSymLink : public CNode
{
public:
						CSymLink();
						CSymLink( const char* cPath,
								  int nOpenMode = O_RDONLY );
						CSymLink( const CDirectory* cDir,
								  const char* cName,
								  int nOpenMode = O_RDONLY );
						CSymLink( const CEntry* cRef,
								  int nOpenMode = O_RDONLY );
						CSymLink( const CNode& cNode );
						CSymLink( const CSymLink& cNode );
	virtual				~CSymLink();

	virtual status_t	SetTo( const char* cPath, int nOpenMode = O_RDONLY );
	virtual status_t	SetTo( const CDirectory* cDir, const char* cPath, int nOpenMode = O_RDONLY );
	virtual status_t	SetTo( const CEntry* cRef, int nOpenMode = O_RDONLY );
	virtual status_t	SetTo( const CNode& cNode );
	virtual status_t	SetTo( const CSymLink& cLink );

	status_t			ReadLink( std::string* pcBuffer );
	status_t			ConstructPath( const CDirectory* cParent, CPath* pcBuffer );
	status_t			ConstructPath( const char* cParent, CPath* pcBuffer );
};


} // end of namespace

#endif // __SYMLINK_H__
