/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Copyright (C) 2001-2002 Bill Hayden
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __VOLUME_H__
#define __VOLUME_H__

#include <sys/types.h>
#include <support/SupportDefs.h>
#include <storage/Directory.h>
#include <storage/Path.h>
#include <string>
#include <mntent.h>

#define BVolume	os::CVolume


namespace os {

class	CDirectory;
//class	Bitmap;

class CVolume
{
public:
					CVolume();
					CVolume(dev_t inDevice);
					CVolume(const CVolume &inVolume);

	virtual			~CVolume();

	status_t		InitCheck() const	{ return mInitStatus; }

	status_t		SetTo(dev_t dev);
	void			Unset(void);

	dev_t			Device() const		{ return mDevice; }

	status_t		GetRootDirectory(CDirectory *dir) const;

	off_t			Capacity() const;
	off_t			FreeBytes() const;

	status_t		GetName(char *name) const;
	status_t		SetName(const char *name);

	//status_t		GetIcon(Bitmap *icon, icon_size which) const;

	bool			IsRemovable() const;
	bool			IsReadOnly() const;
	bool			IsPersistent() const;
	bool			IsShared() const;

// Cosmoe says: The current plan is to not support any of these
	bool			KnowsMime() const	{ return false; }
	bool			KnowsAttr() const	{ return false; }
	bool			KnowsQuery() const	{ return false; }

	bool			operator==(const CVolume &vol) const;
	bool			operator!=(const CVolume &vol) const;
	CVolume &		operator=(const CVolume &vol);

private:

friend class CVolumeRoster;

					CVolume(struct mntent* inMountEntry);

	void			_LoadVolumeProperties() const;

	virtual	void	FBC_Volume1();
	virtual	void	FBC_Volume2();
	virtual	void	FBC_Volume3();
	virtual	void	FBC_Volume4();
	virtual	void	FBC_Volume5();
	virtual	void	FBC_Volume6();
	virtual	void	FBC_Volume7();
	virtual	void	FBC_Volume8();

	dev_t			mDevice;
	status_t		mInitStatus;

// These are mutable so we can do lazy loading from const members
// Worthwhile?  Skanky?  You decide.

	mutable	bool		mPropertiesLoaded;
	mutable bool		mIsShared;
	mutable bool		mIsRemovable;
	mutable bool		mIsReadOnly;
	mutable bool		mIsPersistent;
	mutable off_t		mCapacity;
	mutable off_t		mFreeBytes;
	mutable std::string	mName;
	mutable CPath		mDevicePath;
	mutable CPath		mMountPath;

	int32				FBC_Reserved[8];
};

}

#endif
