/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __DATAIO_H__
#define __DATAIO_H__

#include <sys/types.h>
#include <support/SupportDefs.h>

#define BDataIO			os::CDataIO
#define BPositionIO		os::CPositionIO
#define BMallocIO		os::CMallocIO
#define BMemoryIO		os::CMemoryIO


namespace os {

/** 
 * \ingroup support
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CDataIO
{
public:
    virtual			~CDataIO();

    virtual ssize_t	Read( void* pBuffer, size_t nSize ) = 0;
    virtual ssize_t	Write( const void* pBuffer, size_t nSize ) = 0;
};


/** 
 * \ingroup support
 * \par Description:
 *
 * \sa
 * \author	Kurt Skauen (kurt@atheos.cx), Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CPositionIO : public CDataIO
{
public:
	virtual			~CPositionIO();

    virtual ssize_t	Read( void* outBuffer, size_t inSize );
    virtual ssize_t	Write( const void* inBuffer, size_t inSize );
	
	virtual ssize_t	ReadAt( off_t inPos, void* outBuffer, size_t inSize ) = 0;
	virtual ssize_t	WriteAt( off_t inPos, const void* inBuffer, size_t inSize ) = 0;

	virtual off_t	Seek( off_t inPosition, uint32 inSeekMode ) = 0;
	
	virtual off_t	Position() const;
	
protected:
	off_t			mPosition;
	size_t			mLength;
};



/** 
 * \ingroup support
 * \par Description:
 *
 * \sa
 * \author	Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CMallocIO : public CPositionIO
{
public:
						CMallocIO();
	virtual				~CMallocIO();

	virtual	ssize_t		ReadAt(off_t inPosition, void *outData, size_t inSize);
	virtual	ssize_t		WriteAt(off_t inPosition, const void *inData, size_t inSize);

	virtual	off_t		Seek(off_t inPosition, uint32 inSeekMode);
	virtual status_t	SetSize(off_t inSize);

	void				SetBlockSize(size_t inSize);

	const	void*		Buffer() const				{ return mData;		}
	size_t				BufferLength() const		{ return mLength;	}
	
private:
	virtual	void		FBC_MallocIO1();
	virtual	void		FBC_MallocIO2();

	size_t				mBlockSize;
	size_t				mAllocatedLength;
	char*				mData;
	int32				FBC_Reserved[1];
};


/** 
 * \ingroup support
 * \par Description:
 *
 * \sa
 * \author	Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

class CMemoryIO : public CPositionIO
{
public:
						CMemoryIO(void *inData, size_t inSize);
						CMemoryIO(const void *inData, size_t inSize);
	virtual				~CMemoryIO();

	virtual	ssize_t		ReadAt(off_t inPosition, void *outData, size_t inSize);
	virtual	ssize_t		WriteAt(off_t inPosition, const void *inData, size_t inSize);

	virtual	off_t		Seek(off_t inPosition, uint32 inSeekMode);
	
	virtual off_t		Position() const			{ return mPosition; }

private:
	virtual	void		FBC_MemoryIO1();
	virtual	void		FBC_MemoryIO2();

	bool				mIsReadOnly;
	char*				mData;

	int32				FBC_Reserved[1];
};


} // end of namespace

#endif // __DATAIO_H__
