/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Copyright (C) 2001-2002 Bill Hayden
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __LIST_H__
#define __LIST_H__

#include <deque>
#include <support/SupportDefs.h>

#define BList	os::CList

namespace os {

class CList
{
public:
					CList(int32 ignored = 0L);
					CList(const CList& inList);
virtual				~CList();

	CList&			operator=(const CList& inList);

	// Adding, removing, and replacing items
	bool			AddItem(void* inItem);
	bool			AddItem(void* inItem, int32 inIndex);
	bool			AddList(CList* inNewItems);
	bool			AddList(CList* inNewItems, int32 inIndex);

	bool			RemoveItem(void* inItem);
	void*			RemoveItem(int32 inIndex);
	bool			RemoveItems(int32 inIndex, int32 count);
	bool			ReplaceItem(int32 inIndex, void *newItem);

	void			MakeEmpty() 					{mList.clear();}

	// Arranging items
	void			SortItems(int (*cmp)(const void *, const void *));
	bool			SwapItems(int32 inItemA, int32 inItemB);
	bool			MoveItem(int32 fromIndex, int32 toIndex);

	// Retrieving items
	void*			ItemAt(int32) const;
	void*			ItemAtFast(int32 inIndex) const	{return mList[inIndex];}
	void*			FirstItem() const 				{return mList.front();}
	void*			LastItem() const  				{return mList.back();}

	// Finding properties
	int32			IndexOf(void *inItem) const;
	bool			HasItem(void *inItem) const		{return (IndexOf(inItem) >= 0L);}
	int32			CountItems() const 				{return mList.size();}
	bool			IsEmpty() const    				{return mList.size() == 0;}

	// Accessing all items
	void			DoForEach(bool (*func)(void *));
	void			DoForEach(bool (*func)(void *, void *), void *);

private:

	virtual	void	FBC_CList1();
	virtual	void	FBC_CList2();

	std::deque<void*>	mList;

	uint32			FBC_Reserved[2];
};

}

#endif /* _LIST_H */
