/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __STRING_H__
#define __STRING_H__

#include <string>
#include <stdarg.h>

#define BString		os::CString


namespace os
{

class CString
{
public:
	typedef std::string::iterator iterator;
	typedef std::string::const_iterator const_iterator;
	enum { npos = -1 };

					CString();
					CString( int nLen, char nFiller );
					CString( const char* pzString );
					CString( const char* pzString, int nLen );
					CString( const std::string& cString );
					CString( const std::string& cString, int nPos, int nLen );
					CString( const CString& cString );
					CString( const CString& cString, int nPos, int nLen );
					CString( const_iterator cBegin, const_iterator cEnd );

	const char*		String() const;
	size_t			Length() const;
	size_t			CountChars() const;

	CString&		Resize( int nNewLen );

	CString&		Format( const char* pzFormat, va_list pArgs );
	CString&		Format( const char* pzFormat, ... );

	CString&		Strip();
	CString&		LStrip();
	CString&		RStrip();

	CString&		Lower();
	CString&		Upper();

	int				Compare( const char* pzString ) const;
	int				Compare( const std::string& cOther ) const;
	int				Compare( const CString& cOther ) const;

	int				CompareNoCase( const char* pzString ) const;
	int				CompareNoCase( const std::string& cOther ) const;
	int				CompareNoCase( const CString& cOther ) const;


	CString&		operator=( const char* pzString );
	CString&		operator=( const std::string& cString );
	CString&		operator=( const CString& cString );

	CString&		operator+=( const char* pzString );
	CString&		operator+=( const std::string& cString );
	CString&		operator+=( const CString& cString );

	CString			operator+( const char* pzString );
	CString			operator+( const std::string& cString );
	CString			operator+( const CString& cString );

	bool			operator==( const char* pzString ) const;
	bool			operator==( const std::string& cString ) const;
	bool			operator==( const CString& cString ) const;

	bool			operator<( const char* pzString ) const;
	bool			operator<( const std::string& cString ) const;
	bool			operator<( const CString& cString ) const;

	bool			operator>( const char* pzString ) const;
	bool			operator>( const std::string& cString ) const;
	bool			operator>( const CString& cString ) const;

	char			operator[]( size_t nPos ) const;
	char&			operator[]( size_t nPos );

	operator const	std::string&() const;
	std::string&	str();
//    operator std::string&();

	iterator		begin();
	iterator		end();

	const_iterator	begin() const;
	const_iterator	end() const;

	size_t			size() const;
	bool			empty() const;
	void			resize( size_t nLen );
	void			resize( size_t nLen, char nFiller );
	void			reserve( size_t nLen );

	CString& 		erase( size_t nPos = 0, size_t nLen = npos );
	iterator 		erase( iterator i );
	iterator 		erase( iterator cFirst, iterator cLast );

private:
	std::string		mString;
};



}

#endif
