/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#ifndef __CIRCULARBUFFER_H__
#define __CIRCULARBUFFER_H__

#include <sys/types.h>
#include <support/SupportDefs.h>

namespace os
{


/** Circular (FIFO) data container.
 * \ingroup util
 * \par Description:
 *
 * \since Atheos 0.3.7
 * \sa os::CWindow, os::CView, os::CLooper
 * \author Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

class CircularBuffer
{
public:
				CircularBuffer( int nMaxReserved = 3, int nBlockSize = 8192 );
				~CircularBuffer();
	status_t	Write( const void* pBuffer, int nSize );
	ssize_t		Read( void* pBuffer, int nSize );
	void		Clear();
	ssize_t		Size() const;
	
private:
	struct Block
	{
		Block* m_psNext;
		int    m_nStart;
		int    m_nEnd;
	};

	Block*		_AllocBlock();

	int			m_nBlockSize;
	int			m_nMaxReserved;
	int			m_nNumReserved;
	Block*		m_psFirstBlock;
	Block*		m_psLastBlock;
	Block*		m_psFirstReserved;
	int			m_nTotalSize;
};

} // end of namespace

#endif // __CIRCULARBUFFER_H__
