/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __TRANSLATOR_H__
#define __TRANSLATOR_H__

#include <sys/types.h>

#include <interface/Rect.h>
#include <interface/GraphicsDefs.h>

#include <string>

namespace os
{

class CMessenger;
class CMessage;

enum
{
    BMF_MULTIPASS = 0x0001
};


static const char* TRANSLATOR_TYPE_IMAGE = "image/x-atheos_trans";

struct BitmapHeader
{
    size_t	bh_header_size;
    size_t	bh_data_size;
    uint32	bh_flags;
    IRect	bh_bounds;
    int		bh_frame_count;
    int		bh_bytes_per_row;
    ColorSpace bh_color_space;
};

struct BitmapFrameHeader
{
    size_t 	bf_header_size;
    size_t 	bf_data_size;
    uint32  	bf_time_stamp;
    ColorSpace bf_color_space;
    int		bf_bytes_per_row;
    IRect	bf_frame;
};


class DataReceiver
{
public:
    DataReceiver();
    virtual ~DataReceiver();

    virtual void DataReady( void* pData, size_t nLen, bool bFinal ) = 0;
private:
    void* m;
};

class Translator
{
public:
    enum { ERR_OK = 0, ERR_INVALID_DATA = -1, ERR_SUSPENDED = -2 };
public:
    Translator();
    virtual ~Translator();

    
    void    SetMessage( const CMessage& cMsg );
    void    SetTarget( const CMessenger& cTarget, bool bSendData );
    void    SetTarget( DataReceiver* pcTarget );

    virtual void     SetConfig( const CMessage& cConfig ) = 0;
    virtual status_t AddData( const void* pData, size_t nLen, bool bFinal );
    virtual ssize_t  AvailableDataSize() = 0;
    virtual ssize_t  Read( void* pData, size_t nLen ) = 0;
    virtual void     Abort() = 0;
    virtual void     Reset() = 0;
protected:
    virtual status_t DataAdded( void* pData, size_t nLen, bool bFinal );
    void Invoke( void* pData, size_t nSize, bool bFinal );
private:
    struct Internal;

    Internal* m;
};

struct TranslatorInfo
{
    std::string source_type;
    std::string dest_type;
    float	quality;
};

class TranslatorNode
{
public:
    TranslatorNode();
    virtual ~TranslatorNode();

    virtual int Identify( const std::string& cSrcType, const std::string& cDstType, const void* pData, int nLen ) = 0;
    virtual TranslatorInfo GetTranslatorInfo() = 0;
    virtual Translator*	   CreateTranslator() = 0;
private:
    void* m;
};



class TranslatorFactory
{
public:
    enum { ERR_NOT_ENOUGH_DATA = -1, ERR_UNKNOWN_FORMAT = -2, ERR_NO_MEM = -3 };
public:
    TranslatorFactory();
    ~TranslatorFactory();

    static TranslatorFactory* GetDefaultFactory();
    
    void LoadAll();

    status_t FindTranslator( const std::string& cSrcType, const std::string& cDstType,
			     const void* pData, size_t nLen, Translator** ppcTrans );

    int             GetTranslatorCount();
    TranslatorNode* GetTranslatorNode( int nIndex );
    TranslatorInfo  GetTranslatorInfo( int nIndex );
    Translator*     CreateTranslator( int nIndex );

private:
    struct Internal;
    Internal* m;
};

    
} // end of namespace





#endif // __F_TRANSLATION_TRANSLATOR_H__
