/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#ifndef __VARIANT_H__
#define __VARIANT_H__

#include <support/SupportDefs.h>
#include <support/String.h>
#include <support/TypeConstants.h>

#include <interface/Rect.h>
#include <interface/Point.h>
#include <interface/GraphicsDefs.h>

namespace os
{

class CPoint;
class IPoint;
class CRect;
class IRect;
class CMessage;
struct Color32_s;

class Variant
{
public:
	Variant();
	Variant( int nValue );
	Variant( int64 nValue );
	Variant( float vValue );
	Variant( double vValue );
	Variant( bool bValue );
	Variant( const CString& cString );
	Variant( const CPoint& cValue );
	Variant( const IPoint& cValue );
	Variant( const CRect& cValue );
	Variant( const IRect& cValue );
	Variant( const CMessage& cValue );
	Variant( Color32_s cValue );
	Variant( void* pValue );
	Variant( void* pData, int nSize );
	Variant( const Variant& cOther );
	~Variant();

	void SetInt8( int8 nValue );
	void SetInt16( int16 nValue );
	void SetInt32( int32 nValue );
	void SetInt64( int64 nValue );
	void SetFloat( float vValue );
	void SetDouble( double vValue );
	void SetBool( bool bValue );
	void SetString( const CString& cValue );
	void SetPoint( const CPoint& cValue );
	void SetIPoint( const IPoint& cValue );
	void SetRect( const CRect& cValue );
	void SetIRect( const IRect& cValue );
	void SetPointer( void* pValue  );
	void SetRaw( const void* pData, int nSize );

	int8	AsInt8() const;
	int16	AsInt16() const;
	int32	AsInt32() const;
	int64	AsInt64() const;
	float	AsFloat() const;
	double	AsDouble() const;
	bool	AsBool() const;
	CString	AsString() const;
	CPoint	AsPoint() const;
	IPoint	AsIPoint() const;
	CRect	AsRect() const;
	IRect	AsIRect() const;
	void*	AsPointer() const;
	void*	AsRaw( size_t* pnSize );

	Variant& operator=( int8 nValue );
	Variant& operator=( int16 nValue );
	Variant& operator=( int32 nValue );
	Variant& operator=( int64 nValue );
	Variant& operator=( float vValue );
	Variant& operator=( double vValue );
	Variant& operator=( bool bValue );
	Variant& operator=( const CString& cValue );
	Variant& operator=( const CPoint& cValue );
	Variant& operator=( const IPoint& cValue );
	Variant& operator=( const CRect& cValue );
	Variant& operator=( const IRect& cValue );
	Variant& operator=( const Variant& cValue );

	operator int() const;
	operator int64() const;
	operator float() const;
	operator double() const;
	operator bool() const;
	operator CString() const;
	operator CPoint() const;
	operator IPoint() const;
	operator CRect() const;
	operator IRect() const;

/*
	bool operator==( int8 nValue ) const;
	bool operator==( int16 nValue ) const;
	bool operator==( int32 nValue ) const;
	bool operator==( int64 nValue ) const;
	bool operator==( float vValue ) const;
	bool operator==( double vValue ) const;
	bool operator==( bool bValue ) const;
	bool operator==( const CString& cValue ) const;
	bool operator==( const CPoint& cValue ) const;
	bool operator==( const IPoint& cValue ) const;
	bool operator==( const CRect& cValue ) const;
	bool operator==( const IRect& cValue ) const;
*/    
	bool operator==( const Variant& cValue ) const;
/*
	bool operator!=( int8 nValue ) const;
	bool operator!=( int16 nValue ) const;
	bool operator!=( int32 nValue ) const;
	bool operator!=( int64 nValue ) const;
	bool operator!=( float vValue ) const;
	bool operator!=( double vValue ) const;
	bool operator!=( bool bValue ) const;
	bool operator!=( const CString& cValue ) const;
	bool operator!=( const CPoint& cValue ) const;
	bool operator!=( const IPoint& cValue ) const;
	bool operator!=( const CRect& cValue ) const;
	bool operator!=( const IRect& cValue ) const;
*/    
	bool operator!=( const Variant& cValue ) const;

	size_t FlattenedSize() const;
	status_t Flatten( void* pBuffer, size_t nBufSize ) const;
	status_t Unflatten( const void* pBuffer, size_t nBufSize );

private:
	void _Clear();

	type_code m_eType;
	
	union
	{
		int32  nInt32;
		int64  nInt64;
		float  vFloat;
		double vDouble;
		bool   bBool;
		void*  pData;
		char*  pzString;
		struct
		{
			size_t nSize;
			void*  pBuffer;
		} sBuffer;
	} m_uData;
};





} // end of namespace



#endif // __VARIANT_H__
