/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#include <app/MessageRunner.h>
#include <support/SupportDefs.h>


using namespace os;


CMessageRunner::CMessageRunner(CMessenger target, const CMessage *msg,
								bigtime_t inInterval, int32 inCount)
{
	mCount = inCount;
	mInterval = inInterval;
	
	mThread = spawn_thread( "MessageRunner", (void*)_Entry, NORMAL_PRIORITY, 0, this );
	if ( mThread != -1 )
	{
		resume_thread( mThread );
	}
	
	mTarget = target;
	mReplyTo = target;
	mMessage = msg;
}


CMessageRunner::CMessageRunner(CMessenger target, const CMessage *msg,
								bigtime_t inInterval, int32 inCount,
								CMessenger reply_to)
{
	mCount = inCount;
	mInterval = inInterval;
	
	mThread = spawn_thread( "MessageRunner", (void*)_Entry, NORMAL_PRIORITY, 0, this );
	if ( mThread != -1 )
	{
		resume_thread( mThread );
	}
	
	mTarget = target;
	mReplyTo = reply_to;
	mMessage = msg;
}


CMessageRunner::~CMessageRunner()
{
	if (mThread != -1)
		wait_for_thread(mThread);
}


status_t	CMessageRunner::InitCheck() const
{
	return EOK;
}


status_t	CMessageRunner::SetInterval(bigtime_t inInterval)
{
	mInterval = inInterval;

	return EOK;
}


status_t	CMessageRunner::SetCount(int32 inCount)
{
	mCount = inCount;

	return EOK;
}


status_t	CMessageRunner::GetInfo(bigtime_t *outInterval, int32 *outCount) const
{
	if (outInterval)
		*outInterval = mInterval;
	
	if (outCount)
		*outCount = mCount;

	return EOK;
}


uint32	CMessageRunner::_Entry( void* pData )
{
	CMessageRunner* This = (CMessageRunner*) pData;
	
	This->_Loop();
	
	return 0;
}


void	CMessageRunner::_Loop( void )
{
	// FIXME: This is untested code
	
	for(;;)
	{
		CMessage aMessage(*mMessage);
		
		mTarget.SendMessage( &aMessage );
		
		if (mCount > 0)
		{
			mCount--;
		}
		else if (mCount == 0)
		{
			break;
		}
		
		snooze(mInterval);
	}
}


void	CMessageRunner::FBC_MessageRunner1() {};
void	CMessageRunner::FBC_MessageRunner2() {};
void	CMessageRunner::FBC_MessageRunner3() {};
void	CMessageRunner::FBC_MessageRunner4() {};
void	CMessageRunner::FBC_MessageRunner5() {};
void	CMessageRunner::FBC_MessageRunner6() {};
