/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#include <support/SupportDefs.h>

#include <app/Looper.h>
#include <app/Message.h>
#include <interface/Button.h>
#include <interface/Window.h>
#include <interface/Font.h>

#include <macros.h>

using namespace os;

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CButton::CButton( CRect inFrame, const char* inName, const char* inLabel,
                  CMessage* inMessage, uint32 inResizeMask , uint32 inFlags )
    : CControl( inFrame, inName, inLabel, inMessage, inResizeMask, inFlags )
{
    m_bClicked = false;
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CButton::~CButton()
{
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CButton::GetPreferredSize( float *outWidth, float *outHeight )
{
	if (outWidth)
		*outWidth = StringWidth( Label() ) + 16;

	if (outHeight)
	{
		font_height sHeight;

		GetFontHeight( &sHeight );

		*outHeight = sHeight.ascender + sHeight.descender + sHeight.line_gap + 8;
	}
}


void CButton::PostValueChange( const Variant& cNewValue )
{
	Invalidate();
	Flush();
}


void CButton::LabelChanged( const std::string& cNewLabel )
{
	Invalidate();
	Flush();
}


void CButton::EnableStatusChanged( bool bIsEnabled )
{
	Invalidate();
	Flush();
}


bool CButton::Invoked( CMessage* pcMessage )
{
	CControl::Invoked( pcMessage );
	return( true );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CButton::KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers )
{
	if ( IsEnabled() == false )
	{
		CView::KeyDown( pzString, pzRawString, nQualifiers );
		return;
	}

	if ( pzString[1] == '\0' && (pzString[0] == VK_ENTER || pzString[0] == ' ')  )
	{
		SetValue(1, false );
	}
	else
	{
		SetValue(0, false);
		CControl::KeyDown( pzString, pzRawString, nQualifiers );
	}
}

void CButton::KeyUp( const char* pzString, const char* pzRawString, uint32 nQualifiers )
{
	if ( IsEnabled() == false )
	{
		CView::KeyUp( pzString, pzRawString, nQualifiers );
		return;
	}

	if ( pzString[1] == '\0' && (pzString[0] == VK_ENTER || pzString[0] == ' ')  )
	{
		if ( Value().AsBool() == true )
		{
			SetValue( false );
		}
	}
	else
	{
		CControl::KeyDown( pzString, pzRawString, nQualifiers );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CButton::MouseDown( CPoint cPosition )
{
	int32       nButton = 0x01;

	Window()->CurrentMessage()->FindInt32("buttons", &nButton);

	if ( nButton != 1 || IsEnabled() == false )
	{
		CView::MouseDown( cPosition );
		return;
	}

	MakeFocus( true );

	m_bClicked = Bounds().Contains( cPosition );
	SetValue( m_bClicked, false );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CButton::MouseUp( CPoint cPosition )
{
	int32       nButton = 0x01;

	Window()->CurrentMessage()->FindInt32("buttons", &nButton);

	if ( nButton != 1 || IsEnabled() == false )
	{
		CView::MouseUp( cPosition );
		return;
	}

	if ( Value().AsBool() != false )
	{
		SetValue( false );
	}
	
	m_bClicked = false;
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CButton::MouseMoved( CPoint cPosition, uint32 nCode, const CMessage* pcData )
{
	if ( IsEnabled() == false )
	{
		CView::MouseMoved( cPosition, nCode, pcData );
		return;
	}

	if ( m_bClicked )
	{
		uint32        nButtons;

		GetMouse( NULL, &nButtons );

		if ( nButtons == 0 )
		{
			return;
		}

		if ( nCode == MOUSE_OUTSIDE )
		{
			SetValue( false, false );
		}
		else
		{
			SetValue( true, false );
		}
	}
}

void CButton::Activated( bool bIsActive )
{
	Invalidate();
	Flush();
}

void CButton::Draw( CRect cUpdateRect )
{
	CRect cBounds = Bounds();


	float vStrWidth = StringWidth( Label() );

	font_height sHeight;
	GetFontHeight( &sHeight );

	float vCharHeight = sHeight.ascender + sHeight.descender/* + sHeight.line_gap*/;
	float y = floor( 2.0f + (cBounds.Height()+1.0f)*0.5f - vCharHeight*0.5f + sHeight.ascender /*+ sHeight.line_gap / 2*/ );
	float x = floor((cBounds.Width()+1.0f) / 2.0f - vStrWidth / 2.0f);

	if ( Value().AsBool() )
	{
		y += 1.0f;
		x += 1.0f;
	}

	SetViewColor( get_default_color( COL_NORMAL ) );

	uint32 nFrameFlags = (Value().AsBool() && IsEnabled()) ? FRAME_RECESSED : FRAME_RAISED;

	if ( IsEnabled() && Window()->DefaultButton() == this )
	{
		SetHighColor( 0, 0, 0 );
		StrokeLine( CPoint( cBounds.left, cBounds.top ), CPoint( cBounds.right, cBounds.top ) );
		StrokeLine( CPoint( cBounds.right, cBounds.bottom ) );
		StrokeLine( CPoint( cBounds.left, cBounds.bottom ) );
		StrokeLine( CPoint( cBounds.left, cBounds.top ) );
		cBounds.left += 1;
		cBounds.top += 1;
		cBounds.right -= 1;
		cBounds.bottom -= 1;
		DrawFrame( cBounds, nFrameFlags );
	}
	else
	{
		DrawFrame( cBounds, nFrameFlags );
	}

	if ( IsEnabled() )
	{
		SetHighColor( 0, 0, 0 );
	}
	else
	{
		SetHighColor( 255, 255, 255 );
	}

	SetLowColor( get_default_color( COL_NORMAL ) );

	MovePenTo( x, y );

	DrawString( Label() );
	if ( IsEnabled() == false )
	{
		SetHighColor( 100, 100, 100 );
		MovePenTo( x - 1.0f, y - 1.0f );
		SetDrawingMode( DM_OVER );
		DrawString( Label() );
		SetDrawingMode( DM_COPY );
	}

	if ( IsEnabled() && IsFocus() ) {
		StrokeLine( CPoint( (cBounds.Width()+1.0f)*0.5f - vStrWidth*0.5f, y + sHeight.descender - sHeight.line_gap / 2 - 1.0f ),
					CPoint( (cBounds.Width()+1.0f)*0.5f + vStrWidth*0.5f, y + sHeight.descender - sHeight.line_gap / 2 - 1.0f ) );
	}
}
