/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#include <support/SupportDefs.h>

#include <interface/CheckBox.h>
#include <interface/Font.h>

#include <macros.h>

using namespace os;

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CCheckBox::CCheckBox( CRect cFrame, const char* pzName, const char* pzLabel,
                        CMessage* pcMessage, uint32 nResizeMask, uint32 nFlags )
    : CControl( cFrame, pzName, pzLabel, pcMessage, nResizeMask, nFlags )
{
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CCheckBox::~CCheckBox()
{
}

void CCheckBox::PostValueChange( const Variant& cNewValue )
{
	Invalidate();
	Flush();
}

void CCheckBox::LabelChanged( const std::string& cNewLabel )
{
	Invalidate();
	Flush();
}

void CCheckBox::EnableStatusChanged( bool bIsEnabled )
{
	Invalidate();
	Flush();
}

bool CCheckBox::Invoked( CMessage* pcMessage )
{
	CControl::Invoked( pcMessage );
	return( true );
}

void CCheckBox::GetPreferredSize( float* outWidth, float* outHeight )
{
	float height = 13.0f;
	float width = 13.0f;
	
	std::string cLabel = Label();

	if ( cLabel.empty() == false )
	{
		font_height sHeight;
		GetFontHeight( &sHeight );
		if ( sHeight.ascender + sHeight.descender > 13.0f )
		{
			height = sHeight.ascender + sHeight.descender;
		}
		width += 3.0f + StringWidth( cLabel );
	}

	if (outWidth)
		*outWidth = width;

	if (outHeight)
		*outHeight = height;
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CCheckBox::MouseDown( CPoint cPosition )
{
	SetValue( !Value().AsBool() );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CCheckBox::MouseUp( CPoint cPosition )
{
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CCheckBox::Draw( CRect cUpdateRect )
{
	CRect cBounds = Bounds();

	std::string cLabel = Label();

	SetHighColor( get_default_color( COL_NORMAL ) );
	FillRect( cUpdateRect );

	if ( cLabel.empty() == false )
	{
		font_height sHeight;

		GetFontHeight( &sHeight );
		MovePenTo( 16, (cBounds.Height()+1.0f) * 0.5f - (sHeight.ascender + sHeight.descender) * 0.5f + sHeight.ascender );

		SetHighColor( 0, 0, 0 );
		SetLowColor( get_default_color( COL_NORMAL ) );
		DrawString( cLabel.c_str() );
	}

	SetViewColor( get_default_color( COL_NORMAL ) );

	CRect cButFrame( 0, 0, 13, 13 );

	float nDelta = (cBounds.Height() - cButFrame.Height()) / 2;
	cButFrame.top += nDelta;
	cButFrame.bottom += nDelta;

	DrawFrame( cButFrame, FRAME_RAISED | FRAME_TRANSPARENT );

	if ( Value().AsBool() != false )
	{
		CPoint        cPnt1;
		CPoint        cPnt2;

		SetHighColor( 0, 0, 0 );

		cPnt1 = CPoint( cButFrame.left + 3, cButFrame.top + 3 );
		cPnt2 = CPoint( cButFrame.right - 3, cButFrame.bottom - 3 );

		StrokeLine( cPnt1, cPnt2 );
		StrokeLine( cPnt1 + CPoint( 1, 0 ), cPnt2  + CPoint( 1, 0 ) );

		cPnt1 = CPoint( cButFrame.left + 3, cButFrame.bottom - 3 );
		cPnt2 = CPoint( cButFrame.right - 3, cButFrame.top + 3 );

		StrokeLine( cPnt1, cPnt2 );
		StrokeLine( cPnt1 + CPoint( 1, 0 ), cPnt2  + CPoint( 1, 0 ) );
	}
}
