/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#include <interface/Control.h>
#include <interface/Window.h>
#include <app/Message.h>

using namespace os;

class CControl::Private
{
public:
	std::string	mLabel;
	Variant		m_cValue;
	bool        mIsEnabled;
};


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CControl::CControl( CRect       inFrame,
					const char* inName,
					const char* inLabel,
					CMessage*   inMessage,
					uint32      inResizeMask,
					uint32      inFlags )
    : CView( inFrame, inName, inResizeMask, inFlags ),
      CInvoker( inMessage, NULL )
{
	m = new Private;

	m->mIsEnabled = true;
	m->mLabel     = inLabel;
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CControl::~CControl()
{
	delete m;
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CControl::AttachedToWindow( void )
{
	CView::AttachedToWindow();

	if ( Messenger().IsValid() == false )
	{
		SetTarget( static_cast<CHandler*>(Window()) );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool CControl::Invoked( CMessage* pcMessage )
{
	pcMessage->AddVariant( "value", m->m_cValue );
	return( true );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CControl::SetEnabled( bool inEnabled )
{
	if ( m->mIsEnabled != inEnabled )
	{
		m->mIsEnabled = inEnabled;
		EnableStatusChanged( inEnabled );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool CControl::IsEnabled( void ) const
{
	return( m->mIsEnabled );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CControl::SetLabel( const char* inLabel )
{
	if ( inLabel != m->mLabel )
	{
		m->mLabel = inLabel;
		LabelChanged( m->mLabel );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

const char* CControl::Label( void ) const
{
	return( m->mLabel.c_str() );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CControl::SetValue( Variant inValue, bool inInvoke )
{
	if ( m->m_cValue != inValue )
	{
		if ( PreValueChange( &inValue ) )
		{
			m->m_cValue = inValue;
			PostValueChange( inValue );
			
			if ( inInvoke )
			{
				Invoke();
			}
		}
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

Variant CControl::Value( void ) const
{
	return( m->m_cValue );
}


void CControl::ValueChanged( Variant* pcNewValue )
{
}


bool CControl::PreValueChange( Variant* pcNewValue )
{
	return( true );
}


void CControl::PostValueChange( const Variant& cNewValue )
{
}


void CControl::LabelChanged( const std::string& cNewLabel )
{
}


void CControl::FBC_Control1() {}
void CControl::FBC_Control2() {}
void CControl::FBC_Control3() {}
void CControl::FBC_Control4() {}
void CControl::FBC_Control5() {}
