/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <interface/Rect.h>


using namespace os;

CPoint	c_origin(0.0f,0.0f);


void	CPoint::ConstrainTo(CRect inRect)
{
	if (x < inRect.left)
		x = inRect.left;
	else if (x > inRect.right)
		x = inRect.right;
	
	if (y < inRect.top)
		y = inRect.top;
	else if (y > inRect.bottom)
		y = inRect.bottom;
}


//
// CPoint operators
//


CPoint	CPoint::operator-( void ) const
{
	return( CPoint( -x, -y ) );
}


CPoint	CPoint::operator+( const CPoint& cPoint ) const
{
	return( CPoint( x + cPoint.x, y + cPoint.y ) );
}


CPoint	CPoint::operator-( const CPoint& cPoint ) const
{
	return( CPoint( x - cPoint.x, y - cPoint.y ) );
}


const CPoint&	CPoint::operator+=( const CPoint& cPoint )
{
	x += cPoint.x; y += cPoint.y; return( *this );
}


const CPoint&	CPoint::operator-=( const CPoint& cPoint )
{
	x -= cPoint.x; y -= cPoint.y; return( *this );
}


bool	CPoint::operator<( const CPoint& cPoint ) const
{
	return( y < cPoint.y || ( y == cPoint.y && x < cPoint.x ) );
}


bool	CPoint::operator>( const CPoint& cPoint ) const
{
	return( y > cPoint.y || ( y == cPoint.y && x > cPoint.x ) );
}


bool	CPoint::operator==( const CPoint& cPoint ) const
{
	return( y == cPoint.y && x == cPoint.x );
}


bool	CPoint::operator!=( const CPoint& cPoint ) const
{
	return( y != cPoint.y || x != cPoint.x );
}


//
// IPoint operators
//


IPoint	IPoint::operator-( void ) const
{
	return( IPoint( -x, -y ) );
}


IPoint	IPoint::operator+( const IPoint& cPoint ) const
{
	return( IPoint( x + cPoint.x, y + cPoint.y ) );
}


IPoint	IPoint::operator-( const IPoint& cPoint ) const
{
	return( IPoint( x - cPoint.x, y - cPoint.y ) );
}


const IPoint&	IPoint::operator+=( const IPoint& cPoint )
{
	x += cPoint.x; y += cPoint.y; return( *this );
}


const IPoint&	IPoint::operator-=( const IPoint& cPoint )
{
	x -= cPoint.x; y -= cPoint.y; return( *this );
}


bool	IPoint::operator<( const IPoint& cPoint ) const
{
	return( y < cPoint.y || ( y == cPoint.y && x < cPoint.x ) );
}


bool	IPoint::operator>( const IPoint& cPoint ) const
{
	return( y > cPoint.y || ( y == cPoint.y && x > cPoint.x ) );
}


bool	IPoint::operator==( const IPoint& cPoint ) const
{
	return( y == cPoint.y && x == cPoint.x );
}


bool	IPoint::operator!=( const IPoint& cPoint ) const
{
	return( y != cPoint.y || x != cPoint.x );
}

