/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */

 
#include <support/SupportDefs.h>
#include <interface/StringView.h>
#include <interface/Font.h>

#include <macros.h>
#include <string>

using namespace os;

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CStringView::CStringView( CRect inFrame, const char* inName, const char* inString,
							uint32 inResizeMask, uint32 inFlags )
    : CView( inFrame, inName, inResizeMask, inFlags )
{
	m = new data;
	m->m_nMinSize = 0;
	m->m_nMaxSize = 0;
	m_eAlign      = ALIGN_LEFT;
	SetHighColor( 0, 0, 0 );
	SetText( inString );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CStringView::~CStringView()
{
	delete m;
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CStringView::SetText( const char* pzString )
{
	if ( pzString == NULL )
	{
		m->m_cString.resize(0);
	}
	else
	{
		m->m_cString = pzString;
	}
	Invalidate();
	Flush();
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

const char* CStringView::Text( void ) const
{
	return( m->m_cString.c_str() );
}

void CStringView::SetMinPreferredSize( int nWidthChars )
{
	m->m_nMinSize = nWidthChars;
}

void CStringView::SetMaxPreferredSize( int nWidthChars )
{
	m->m_nMaxSize = nWidthChars;
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CStringView::GetPreferredSize( float* outWidth, float* outHeight )
{
	font_height sHeight;

	GetFontHeight( &sHeight );
	float vStrWidth = StringWidth( m->m_cString );

	/*
	if ( bLargest )
	{
		if ( m->m_nMaxSize > 0 )
		{
			vStrWidth = float(m->m_nMaxSize) * StringWidth( "M" );
		}
	}
	else
	{
		if ( m->m_nMinSize > 0 )
		{
			vStrWidth = float(m->m_nMinSize) * StringWidth( "M" );
		}
	}
	*/

	if (outWidth)
		*outWidth = vStrWidth;

	if (outHeight)
		*outHeight = sHeight.ascender + sHeight.descender;
}


void CStringView::AttachedToWindow()
{
	CView* pcParent = Parent();

	if ( pcParent != NULL )
	{
		SetLowColor( pcParent->LowColor() );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CStringView::Draw( CRect cUpdateRect )
{
	CRect        cBounds = Bounds();

	FillRect( cBounds, get_default_color( COL_NORMAL ) );

	font_height sHeight;
	float vStrWidth = StringWidth( m->m_cString );

	GetFontHeight( &sHeight );

	float x;

	if ( m_eAlign == ALIGN_LEFT )
	{
		x = 0.0f;
	}
	else if ( m_eAlign == ALIGN_RIGHT )
	{
		x = cBounds.Width() + 1.0f - vStrWidth;
	}
	else
	{
		x = (cBounds.Width()+1.0f) / 2.0f - vStrWidth / 2.0f;
	}

	DrawString( m->m_cString.c_str(),
				CPoint( x, (cBounds.Height()+1.0f) / 2 - (sHeight.ascender + sHeight.descender) / 2 + sHeight.ascender ) );
}
