/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <stdio.h>
#include <support/SupportDefs.h>

#include <interface/Window.h>
#include <interface/View.h>
#include <interface/ScrollBar.h>
#include <app/Application.h>
#include <app/Message.h>

#include <interface/Font.h>
#include <interface/Bitmap.h>

#include <appserver/protocol.h>

#include <macros.h>

using namespace os;

int CView::s_nNextTabOrder = 0;

/*
const Pattern SOLID_HIGH = {{0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}};
const Pattern SOLID_LOW =  {{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}};
const Pattern MIXED_COLORS={{0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55}};
*/

static Color32_s g_asDefaultColors[] =
{
	Color32_s( 0xaa, 0xaa, 0xaa, 0xff ),        // COL_NORMAL
	Color32_s( 0xff, 0xff, 0xff, 0xff ),        // COL_SHINE
	Color32_s( 0x00, 0x00, 0x00, 0xff ),        // COL_SHADOW
	Color32_s( 0x66, 0x88, 0xbb, 0xff ),        // COL_SEL_WND_BORDER
	Color32_s( 0x78, 0x78, 0x78, 0xff ),        // COL_NORMAL_WND_BORDER
	Color32_s( 0x00, 0x00, 0x00, 0xff ),        // COL_MENU_TEXT
	Color32_s( 0x00, 0x00, 0x00, 0xff ),        // COL_SEL_MENU_TEXT
	Color32_s( 0xcc, 0xcc, 0xcc, 0xff ),        // COL_MENU_BACKGROUND
	Color32_s( 0x66, 0x88, 0xbb, 0xff ),        // COL_SEL_MENU_BACKGROUND
	Color32_s( 0x78, 0x78, 0x78, 0xff ),        // COL_SCROLLBAR_BG
	Color32_s( 0xaa, 0xaa, 0xaa, 0xff ),        // COL_SCROLLBAR_KNOB
	Color32_s( 0x78, 0x78, 0x78, 0xff ),        // COL_LISTVIEW_TAB
	Color32_s( 0xff, 0xff, 0xff, 0xff )         // COL_LISTVIEW_TAB_TEXT
};


typedef struct StateStruct
{
	CPoint			origin;	// FIXME: origin saved/restored but not implemented
	float			scale;	// FIXME: scale saved/restored but not implemented
	drawing_mode	drawingMode;
	
// Line Cap/Join Mode FIXME
// Miter limit FIXME
// Pen size FIXME

	CPoint			penLocation;
	Color32_s		highColor;
	Color32_s   	lowColor;
	
// stipple pattern FIXME
// Local/Global clipping regions FIXME
// Font context FIXME

	StateStruct* nextState;
} StateStruct;

/** Get the value of one of the standard system colors.
 * \par Description:
 *        Call this function to obtain one of the user-configurable
 *        system colors. This should be used whenever possible instead
 *        of hardcoding colors to make it possible for the user to
 *        customize the look of the operating system.
 * \par Note:
 * \par Warning:
 * \param
 *        nColor - One of the COL_xxx enums from default_color_t
 * \return
 *        The current color for the given system pen.
 * \sa
 * \author        Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

Color32_s os::get_default_color( default_color_t nColor )
{
	return( g_asDefaultColors[nColor] );
}

void os::__set_default_color( default_color_t nColor, const Color32_s& sColor )
{
	g_asDefaultColors[nColor] = sColor;
}

void os::set_default_color( default_color_t nColor, const Color32_s& sColor )
{
	g_asDefaultColors[nColor] = sColor;
}

static Color32_s Tint( const Color32_s& sColor, float vTint )
{
	int r = int( (float(sColor.red) * vTint + 127.0f * (1.0f - vTint)) );
	int g = int( (float(sColor.green) * vTint + 127.0f * (1.0f - vTint)) );
	int b = int( (float(sColor.blue) * vTint + 127.0f * (1.0f - vTint)) );
	if ( r < 0 ) r = 0; else if (r > 255) r = 255;
	if ( g < 0 ) g = 0; else if (g > 255) g = 255;
	if ( b < 0 ) b = 0; else if (b > 255) b = 255;
	return( Color32_s( r, g, b, sColor.alpha ) );
}


class CView::Private
{
public:
	int          m_hViewHandle; 	// Our bridge to the server
	port_id      m_hReplyPort;      // Used as reply address when talking to server
	CView*       m_pcTopChild;
	CView*       m_pcBottomChild;

	CView*       m_pcLowerSibling;
	CView*       m_pcHigherSibling;

	CView*       m_pcParent;

	CView*       m_pcPrevFocus;

	CScrollBar*  m_pcHScrollBar;
	CScrollBar*  m_pcVScrollBar;
	CRect        m_cFrame;
	CPoint       mOrigin;
	float        mScale;
	CPoint       m_cScrollOffset;
	std::string  m_cTitle;

	drawing_mode m_eDrawingMode;
	Color32_s    m_sFgColor;
	Color32_s    m_sBgColor;
	Color32_s    m_sEraseColor;

	CFont*        m_pcFont;
	StateStruct* mStateStack;

	int          m_nBeginPaintCount;
	int          __unused;

	uint32       m_nResizeMask;
	uint32       m_nFlags;
	int          m_nHideCount;
	int          m_nMouseMoveRun;
	int          m_nMouseMode;        /* Record whether the mouse was inside or outside
								* the bounding box during the last mouse event */
	int          m_nTabOrder; // Sorting order for keyboard manouvering
};


/**
 * View constructor
 *  \param        cFrame The frame rectangle in the parents coordinate system.
 *  \param        pzTitle The logical name of the view. This parameter is newer
 *                   rendered anywhere, but is passed to the Handler::Handler()
 *                constructor to identify the view.
 *  \param        nResizeMask Flags defining how the views frame rectangle is
 *                affected if the parent view is resized.
 *  \param        nFlags Various flags to control the views behavour.
 *
 *  \sa os::view_flags, os::view_resize_flags, Handler::Handler()
 * \author        Kurt Skauen (kurt@atheos.cx)
 *//////////////////////////////////////////////////////////////////////////////

CView::CView( CRect inFrame, const char* inTitle, uint32 inResizeMask, uint32 inFlags )
        : CHandler( inTitle )
{
	m = new Private;

	m->m_hReplyPort = create_port( "view_reply", DEFAULT_PORT_SIZE );

	m->m_cFrame					= inFrame;
	m->m_cTitle					= inTitle;
	m->m_nResizeMask			= inResizeMask;
	m->m_nFlags					= inFlags;

	m->m_pcParent				= NULL;
	m->m_pcPrevFocus			= NULL;

	m->m_pcTopChild				= NULL;
	m->m_pcBottomChild			= NULL;

	m->m_pcLowerSibling			= NULL;
	m->m_pcHigherSibling		= NULL;

	m->m_pcFont					= NULL;
	m->m_hViewHandle			= -1;
	m->m_nBeginPaintCount		= 0;

	m->m_sFgColor.red			= 255;
	m->m_sFgColor.green			= 255;
	m->m_sFgColor.blue			= 255;

	m->m_eDrawingMode			= DM_COPY;
	m->m_nMouseMode				= MOUSE_OUTSIDE;
	m->m_nMouseMoveRun			= 0;
	m->m_nHideCount				= 0;

	m->m_pcHScrollBar			= NULL;
	m->m_pcVScrollBar			= NULL;
	m->mStateStack				= NULL;

	m->m_nTabOrder				= -1;
	m->m_sBgColor				= get_default_color( COL_NORMAL );
	m->m_sEraseColor			= get_default_color( COL_NORMAL );
	
	m->mOrigin					= CPoint(0,0);
	m->mScale					= 1.0f;

	CFont* pcFont = new CFont();

	font_properties sProp;
	CApplication::GetInstance()->GetDefaultFont( DEFAULT_FONT_REGULAR, &sProp );

	if ( pcFont->SetFamilyAndStyle( sProp.m_cFamily.c_str(), sProp.m_cStyle.c_str() ) == 0 )
	{
		pcFont->SetProperties( sProp.m_vSize, sProp.m_vShear, sProp.m_vRotation );
		SetFont( pcFont );
		pcFont->Release();
		Flush();
	}
	else
	{
		dbprintf( "Error : CView::View() unable to create font\n" );
	}
}

/** View destructor
 * \par Description:
 *        The destructor will release all resources used by the view
 *        itseld, and then recursively delete all child views.
 *
 *        To prevent a children to be deleted you must unlink it with
 *        CView::RemoveChild() before deleting it's parent.
 * \author        Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

CView::~CView()
{
	CView* pcChild;

	if ( m->m_pcVScrollBar != NULL )
	{
		m->m_pcVScrollBar->SetTarget( NULL );
	}

	if ( m->m_pcHScrollBar != NULL )
	{
		m->m_pcHScrollBar->SetTarget( NULL );
	}

	CWindow* pcWnd = Window();

	if ( pcWnd != NULL )
	{
		pcWnd->_ViewDeleted( this );
	}

	while( (pcChild = m->m_pcTopChild) /*  GetChildAt( 0 ) */ )
	{
		RemoveChild( pcChild );
		delete pcChild;
	}

	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->RemoveChild( this );
	}
	_ReleaseFont();
	delete_port( m->m_hReplyPort );
	delete m;
}

/** Get the keybord manouvering order.
 * \par Description:
 *        This member is called by the system to decide which view to select next
 *        when the <TAB> key is pressed. The focus is given to the next view with
 *        higher or equal tab-order as the current. You can overload this member
 *        to decide the order whenever it is called, or rely on the default
 *        implementation that will return whatever was set by SetTabOrder().
 *        A negative return value means that the view should not be skipped when
 *        searching for the next view to activate.
 *
 * \return The views sorting order for keyboard manouvering.
 * \sa SetTabOrder()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *//////////////////////////////////////////////////////////////////////////////

int CView::GetTabOrder()
{
	return( m->m_nTabOrder );
}

/** Set the keyboard manouvering sorting order.
 * \par Description:
 *        Set the value that will be returned by GetTabOrder().
 * \param
 *        nOrder - The sorting order.
 * \sa GetTabOrder()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *//////////////////////////////////////////////////////////////////////////////

void CView::SetTabOrder( int nOrder )
{
	m->m_nTabOrder = nOrder;
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_LinkChild( CView* pcChild, bool bTopmost )
{
	if ( NULL == pcChild->m->m_pcParent )
	{
		pcChild->m->m_pcParent        = this;
		if ( NULL == m->m_pcBottomChild && NULL == m->m_pcTopChild )
		{
			m->m_pcBottomChild        = pcChild;
			m->m_pcTopChild        = pcChild;
			pcChild->m->m_pcLowerSibling         = NULL;
			pcChild->m->m_pcHigherSibling = NULL;
		}
		else
		{
			if ( bTopmost )
			{
				if ( NULL != m->m_pcTopChild )
				{
					m->m_pcTopChild->m->m_pcHigherSibling = pcChild;
				}

				pcChild->m->m_pcLowerSibling = m->m_pcTopChild;
				m->m_pcTopChild = pcChild;
				pcChild->m->m_pcHigherSibling = NULL;
			}
			else
			{
				if ( NULL != m->m_pcBottomChild )
				{
					m->m_pcBottomChild->m->m_pcLowerSibling = pcChild;
				}
				pcChild->m->m_pcHigherSibling = m->m_pcBottomChild;
				m->m_pcBottomChild                   = pcChild;
				pcChild->m->m_pcLowerSibling  = NULL;
			}
		}
	}
	else
	{
		dbprintf( "ERROR : Attempt to add an view already belonging to a window\n" );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_UnlinkChild( CView* pcChild )
{
	if( pcChild->m->m_pcParent == this )
	{
		pcChild->m->m_pcParent = NULL;

		if ( pcChild == m->m_pcTopChild )
		{
			m->m_pcTopChild = pcChild->m->m_pcLowerSibling;
		}

		if ( pcChild == m->m_pcBottomChild )
		{
			m->m_pcBottomChild = pcChild->m->m_pcHigherSibling;
		}

		if ( NULL != pcChild->m->m_pcLowerSibling )
		{
			pcChild->m->m_pcLowerSibling->m->m_pcHigherSibling = pcChild->m->m_pcHigherSibling;
		}

		if ( NULL != pcChild->m->m_pcHigherSibling )
		{
			pcChild->m->m_pcHigherSibling->m->m_pcLowerSibling = pcChild->m->m_pcLowerSibling;
		}

		pcChild->m->m_pcLowerSibling  = NULL;
		pcChild->m->m_pcHigherSibling = NULL;
	}
	else
	{
		dbprintf( "ERROR : Attempt to remove a view not belonging to this window\n" );
	}
}


/** Start a drag and drop operation.
 * \par Description:
 *
 *        This member is normally called from the MouseMoved() member to
 *        initiate a drag and drop operation. The function takes a
 *        os::CMessage containing the data to drag, and information about
 *        how to visually represent the dragged data to the user. The
 *        visual representation can be either a bitmap (possibly with
 *        an alpha-channel to make it half-transparent) or a simple
 *        rectangle.
 *
 *        When a drag and drop operation is in progress the application
 *        server will include the message given here in \p pcData in
 *        M_MOUSE_MOVED (os::CView::MouseMoved()) messages sent to views
 *        as the user moves the mouse and also in the M_MOUSE_UP
 *        (os::CView::MouseUp()) message that terminate the operation.
 * 
 * \param pcData
 *        A os::CMessage object containing the data to be dragged.
 * \param cHotSpot
 *        Mouse position relative to the visible representation
 *        of the dragged object.
 * \param pcBitmap
 *        A bitmap that will be used as the visible representation
 *        of the dragged data. The bitmap can have an alpha channel.
 * \param cBounds
 *        And alternate way to visually represent the dragged data.
 *        If a rectangle is used instead of an bitmap the appserver
 *        will render a rectangle instead of a bitmap.
 * \param pcReplyHandler
 *        The handler that should receive replies sent by the receiver
 *        of the dragged data.
 * \sa MouseMoved(), MouseUp(), os::CMessage, os::CHandler
 * \author        Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

void CView::BeginDrag( CMessage* pcData, const CPoint& cHotSpot, const CBitmap* pcBitmap, CHandler* pcReplyHandler )
{
	CWindow* pcWindow = Window();
	if ( pcWindow == NULL )
	{
		dbprintf( "Error: CView::BeginDrag() called on view not attached to a window\n" );
		return;
	}

	CMessage cReq( WR_BEGIN_DRAG );

	pcData->SetReplyHandler( (pcReplyHandler==NULL) ? this : pcReplyHandler );

	cReq.AddInt32( "bitmap", pcBitmap->mHandle );
	cReq.AddPoint( "hot_spot", cHotSpot );
	cReq.AddMessage( "data", pcData );

	CMessage cReply;
	if ( CMessenger( pcWindow->_GetAppserverPort() ).SendMessage( &cReq, &cReply ) < 0 )
	{
		dbprintf( "CView::BeginDrag() failed to send request to server\n" );
	}
}


void CView::BeginDrag( CMessage* pcData, const CPoint& cHotSpot, const CRect& cBounds, CHandler* pcReplyHandler )
{
	CWindow* pcWindow = Window();
	if ( pcWindow == NULL )
	{
		dbprintf( "Error: CView::BeginDrag() called on view not attached to a window\n" );
		return;
	}

	CMessage cReq( WR_BEGIN_DRAG );

	pcData->SetReplyHandler( (pcReplyHandler==NULL) ? this : pcReplyHandler );

	cReq.AddInt32( "bitmap", -1 );
	cReq.AddRect( "bounds", cBounds );
	cReq.AddPoint( "hot_spot", cHotSpot );
	cReq.AddMessage( "data", pcData );

	CMessage cReply;
	if ( CMessenger( pcWindow->_GetAppserverPort() ).SendMessage( &cReq, &cReply ) < 0 )
	{
		dbprintf( "CView::BeginDrag() failed to send request to server\n" );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetFlags( uint32 nFlags )
{
	m->m_nFlags = nFlags;
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

uint32 CView::Flags( uint32 nMask ) const
{
	return( m->m_nFlags & nMask );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetResizeMask( uint32 nFlags )
{
	m->m_nResizeMask = nFlags;
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

uint32 CView::GetResizeMask() const
{
	return( m->m_nResizeMask );
}


/** Return the ideal size for the view.
 * \par Description:
 *
 *        This member returns the ideal size for the view.  It should be
 *        overridden by your subclass to determine the ideal size for your
 *        particular type of view.  This base implementation just returns
 *        some generic minimum values.
 * 
 * \param outWidth
 *        The address of a float which will contain the ideal width on return.
 *        It can be null if you don't need the width.
 * \param outHeight
 *        The address of a float which will contain the ideal height on return.
 *        It can be null if you don't need the height.
 * \sa ResizeToPreferred()
 * \author Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

void CView::GetPreferredSize( float* outWidth, float* outHeight )
{
	if (outWidth)
		*outWidth = 50.0f;

	if (outHeight)
		*outHeight = 20.0f;
}


/** Resize the view to the preferred size
 * \par Description:
 *
 *        This member is a convenience function which resizes the view to the
 *        ideal size (as returned by GetPreferredSize).
 *
 * \sa GetPreferredSize()
 * \author Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

void CView::ResizeToPreferred()
{
	float width, height;

	GetPreferredSize(&width, &height);
	ResizeTo(width, height);
}


CPoint CView::GetContentSize() const
{
	CRect cBounds = Frame();
	return( CPoint( cBounds.Width(), cBounds.Height() ) );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CPoint CView::GetScrollOffset( void ) const
{
	return( m->m_cScrollOffset );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CView* CView::Parent( void ) const
{
	return( m->m_pcParent );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_Attached( CWindow* pcWindow, CView* pcParent, int hHandle, int nHideCount )
{
	m->m_nHideCount += nHideCount;
	if ( pcWindow != NULL )
	{
		pcWindow->Flush();
		pcWindow->AddHandler( this );

		if ( hHandle != -1 )
		{
			m->m_hViewHandle = hHandle;
		}
		else
		{
			CMessage cReq( WR_CREATE_VIEW );

			cReq.AddInt32( "top_view", pcWindow->_GetTopView()->m->m_hViewHandle );
			cReq.AddInt32( "flags", m->m_nFlags );
			cReq.AddPointer( "user_object", this );
			cReq.AddString( "name", Name() );
			cReq.AddRect( "frame", Frame() );
			cReq.AddInt32( "parent_view", (pcParent) ? pcParent->m->m_hViewHandle : -1 );
			cReq.AddColor32( "fg_color", m->m_sFgColor );
			cReq.AddColor32( "bg_color", m->m_sBgColor );
			cReq.AddColor32( "er_color", m->m_sEraseColor );
			cReq.AddInt32( "hide_count", (hHandle==-1) ? m->m_nHideCount : 0 );
			cReq.AddPoint( "scroll_offset", m->m_cScrollOffset );

			CMessage cReply;

			CMessenger( pcWindow->_GetAppserverPort() ).SendMessage( &cReq, &cReply );

			cReply.FindInt( "handle", &m->m_hViewHandle );
		}
		SetNextHandler( pcParent );

		CFont* pcFont;

		if ( NULL != m->m_pcFont )
		{
			GRndSetFont_s* psCmd = static_cast<GRndSetFont_s*>( pcWindow->_AllocRenderCmd( DRC_SET_FONT, this, sizeof( GRndSetFont_s ) ));
				if ( psCmd != NULL )
				{
				psCmd->hFontID = m->m_pcFont->GetFontID();
			}
		}
		else
		{
			pcFont = new CFont();
			pcFont->SetFamilyAndStyle( SYS_FONT_FAMILY, SYS_FONT_PLAIN );
			pcFont->SetProperties( 8.0f, 0.0f, 0.0f );
			SetFont( pcFont );
			pcFont->Release();
		}

		Flush();
		AttachedToWindow();
	}

	CView* pcChild;
	for ( pcChild = m->m_pcBottomChild ; NULL != pcChild ; pcChild = pcChild->m->m_pcHigherSibling )
	{
		pcChild->_Attached( pcWindow, this, -1, nHideCount );
	}

	if ( pcWindow != NULL )
	{
		AllAttached();
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_Detached( bool bFirst, int nHideCount )
{
	CView* pcChild;

	m->m_nHideCount -= nHideCount;

	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		DetachedFromWindow();
	}

	for ( int i = 0 ; (pcChild = ChildAt( i )) ; ++i )
	{
		pcChild->_Detached( false, nHideCount );
	}

	if ( pcWindow != NULL )
	{
		AllDetached();
	}

	if ( pcWindow != NULL && m->m_hViewHandle != -1 )
	{
		pcWindow->Flush();
		pcWindow->_DeleteViewFromServer( this );
		m->m_hViewHandle = -1;
		if ( pcWindow->RemoveHandler( this ) == false )
		{
			dbprintf( "CView::_Detached() failed to remove view from Looper\n" );
		}
		pcWindow->_ViewDeleted( this );
	}

	if ( bFirst )
	{
		m->m_pcParent = NULL;
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_WindowActivated( bool bIsActive )
{
	CWindow* pcWnd = Window();
	CView*   pcChild;

	assert( pcWnd != NULL );

	WindowActivated( bIsActive );
	if ( pcWnd->CurrentFocus() == this )
	{
		Activated( bIsActive );
	}

	for ( pcChild = m->m_pcBottomChild ; NULL != pcChild ; pcChild = pcChild->m->m_pcHigherSibling )
	{
		pcChild->_WindowActivated( bIsActive );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::RemoveChild( CView* pcChild )
{
	CWindow* pcWindow = Window();

	_UnlinkChild( pcChild );

	if ( NULL != pcWindow && -1 != m->m_hViewHandle )
	{
		pcChild->_Detached( true, m->m_nHideCount );
		pcWindow->Sync();
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::RemoveSelf( void )
{
	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->RemoveChild( this );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::AddChild( CView* pcChild, bool bAssignTabOrder )
{
	_LinkChild( pcChild, true );

	if ( bAssignTabOrder )
	{
		pcChild->SetTabOrder( s_nNextTabOrder++ );
	}

	pcChild->_Attached( Window(), this, -1, m->m_nHideCount );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CView* CView::ChildAt( const CPoint& cPos ) const
{
	CView* pcChild;

	for ( pcChild = m->m_pcTopChild ; NULL != pcChild ; pcChild = pcChild->m->m_pcLowerSibling )
	{
		if ( pcChild->m->m_nHideCount > 0 )
		{
			continue;
		}

		if ( !( (cPos.x < pcChild->m->m_cFrame.left)  ||
				(cPos.x > pcChild->m->m_cFrame.right) ||
				(cPos.y < pcChild->m->m_cFrame.top)   ||
				(cPos.y > pcChild->m->m_cFrame.bottom)) )
		{
			return( pcChild );
		}
	}
	return( NULL );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CView* CView::ChildAt( int inIndex ) const
{
	CView* pcChild;

	if ( inIndex >= 0 )
	{
		for ( pcChild = m->m_pcTopChild ; NULL != pcChild ; pcChild = pcChild->m->m_pcLowerSibling )
		{
			if ( 0 == inIndex )
			{
				return( pcChild );
			}
			inIndex--;
		}
	}
	return( NULL );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int32 CView::CountChildren( ) const
{
	int32 theCount = 0L;
    CView* pcChild;

	for ( pcChild = m->m_pcTopChild; NULL != pcChild; pcChild = pcChild->m->m_pcLowerSibling )
	{
		theCount++;
	}

	return theCount;
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CView* CView::FindView( const char* inName ) const
{
    CView* pcChild;

    for ( pcChild = m->m_pcTopChild ; NULL != pcChild ; pcChild = pcChild->m->m_pcLowerSibling )
    {
		if ( strcmp(inName, pcChild->m->m_cTitle.c_str()) == 0 )
		{
			return( pcChild );
		}
    }

    return( NULL );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CView* CView::NextSibling() const
{
	return( m->m_pcLowerSibling );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CView* CView::PreviousSibling() const
{
	return( m->m_pcHigherSibling );
}


/** Get either of the CScrollBar object associated with this view
 * \par Description:
 *
 *        This member returns the CScrollBar object specified by the inAxis
 *        parameter.
 *
 * \param
 *        inAxis - the desired axis, either HORIZONTAL or VERTICAL
 * \author Bill Hayden (hayden@haydentech.com)
 *****************************************************************************/

CScrollBar* CView::GetScrollBar( orientation inAxis ) const
{
	switch (inAxis)
	{
		case HORIZONTAL:
			return( m->m_pcHScrollBar );

		case VERTICAL:
			return( m->m_pcVScrollBar );
	}

	return NULL;
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_IncHideCount( bool bVisible )
{
	if ( bVisible )
	{
		m->m_nHideCount--;
	}
	else
	{
		m->m_nHideCount++;
	}

	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndShowView_s* psCmd;
		psCmd = (GRndShowView_s*) pcWindow->_AllocRenderCmd( DRC_SHOW_VIEW, this, sizeof(GRndShowView_s) );

		if ( psCmd != NULL )
		{
			psCmd->bVisible = bVisible;
		}
	}

	for ( CView* pcChild = m->m_pcBottomChild ; pcChild != NULL ; pcChild = pcChild->m->m_pcHigherSibling )
	{
		pcChild->_IncHideCount( bVisible );
	}
}

/** Show/hide a view and all its children.
 * \par Description:
 *        Calling show with bVisible == false on a view have the same effect as
 *        unlinking it from it's parent, except that the view will remember it's
 *        entire state like the font, colors, position, depth, etc etc. A subsequent
 *        call to Show() with bVisible == true will restore the view compleatly.
 *
 *        Calls to Show() can be nested. If it is called twice with bVisible == false
 *        it must be called twice with bVisible == true before the view is made
 *        visible. It is safe to call Show(false) on a view before it is attached to a
 *        window. The view will then remember it's state and will not be visible
 *        when attached to a window later on.
 *
 *        A hidden view will not receive draw messages or input events from keybord
 *        or mouse. It will still receive messages targeted directly at the view.
 * \par Note:
 *        It is NOT legal to show a visible view! Calling Show(true) before
 *        Show(false) is an error.
 * \param
 *        bVisible - A boolean telling if the view should be hidden or viewed.
 * \sa IsVisible(), Window::Show()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *//////////////////////////////////////////////////////////////////////////////


void CView::Show( bool bVisible )
{
	if ( bVisible )
	{
		m->m_nHideCount--;
	}
	else
	{
		m->m_nHideCount++;
	}

	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndShowView_s* psCmd;
		psCmd = (GRndShowView_s*) pcWindow->_AllocRenderCmd( DRC_SHOW_VIEW, this, sizeof(GRndShowView_s) );
		if ( psCmd != NULL )
		{
			psCmd->bVisible = bVisible;
		}
	}
	
	for ( CView* pcChild = m->m_pcBottomChild ; pcChild != NULL ; pcChild = pcChild->m->m_pcHigherSibling )
	{
		pcChild->_IncHideCount( bVisible );
	}
	Flush();
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool CView::IsHidden() const
{
	return( m->m_nHideCount > 0 );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MakeFocus( bool bFocus )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		if ( false == bFocus )
		{
			if ( pcWindow->CurrentFocus() == this )
			{
				pcWindow->SetFocusChild( NULL );
			}
		}
		else
		{
			pcWindow->SetFocusChild( this );
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

bool CView::IsFocus( void ) const
{
	CWindow* pcWindow = Window();

	if ( NULL != pcWindow )
	{
		if ( pcWindow->IsActive() == false )
			return( false );

		CView* pcFocus = pcWindow->CurrentFocus();
		return( pcFocus == this );
	}

	return( false );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::GetMouse( CPoint* pcPosition, uint32* pnButtons, bool inCheckQueue )
{
	CWindow* pcWindow = Window();
	if ( pcWindow == NULL )
	{
		dbprintf( "Error: CView::GetMouse() called on view not attached to a window\n" );
		return;
	}

	CMessage cReq( WR_GET_MOUSE );

	CMessage cReply;
	if ( CMessenger( pcWindow->_GetAppserverPort() ).SendMessage( &cReq, &cReply ) < 0 )
	{
		dbprintf( "CView::GetMouse() failed to send request to server\n" );
	}

	if ( pcPosition != NULL )
	{
		cReply.FindPoint( "position", pcPosition );
		ConvertFromScreen( pcPosition );
	}

	if ( pnButtons != NULL )
	{
		cReply.FindInt( "buttons", pnButtons );
	}
}


void CView::SetMousePos( const CPoint& cPosition )
{
	CWindow* pcWindow = Window();
	if ( pcWindow == NULL )
	{
		dbprintf( "Error: CView::GetMouse() called on view not attached to a window\n" );
		return;
	}

	CMessage cReq( WR_SET_MOUSE_POS );

	cReq.AddPoint( "position", ConvertToScreen( cPosition ) );

	if ( CMessenger( pcWindow->_GetAppserverPort() ).SendMessage( &cReq ) < 0 )
	{
		dbprintf( "CView::SetMousePos() failed to send request to server\n" );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

uint32 CView::GetQualifiers() const
{
	if ( CApplication::GetInstance() == NULL )
		return( 0 );

	return( CApplication::GetInstance()->GetQualifiers() );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MouseMoved( CPoint cNewPos, uint32 nCode, const CMessage* pcData )
{
	CWindow* pcWnd = Window();

	if ( pcWnd != NULL )
	{
		int32 nButton = 0x01;

		pcWnd->CurrentMessage()->FindInt32("buttons", &nButton);

		pcWnd->_MouseEvent( ConvertToScreen( cNewPos ), nButton, pcData, true );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MouseDown( CPoint cPosition )
{
	// FIXME: BeOS did not send to parent automatically

	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->MouseDown( ConvertToParent( cPosition ) );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MouseUp( CPoint cPosition )
{
	// FIXME: BeOS did not send to parent automatically

	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->MouseUp( ConvertToParent( cPosition ) );
	}
}


void CView::WheelMoved( CPoint& inPoint )
{
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::KeyDown( const char* pzString, const char* pzRawString, uint32 nQualifiers )
{
	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->KeyDown( pzString, pzRawString, nQualifiers );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::KeyUp( const char* pzString, const char* pzRawString, uint32 nQualifiers )
{
	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->KeyUp( pzString, pzRawString, nQualifiers );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int CView::ToggleDepth( void )
{
	CWindow* pcWindow = Window();

	if ( NULL != pcWindow )
	{
		CMessage cReq( WR_TOGGLE_VIEW_DEPTH );

		cReq.AddInt32( "top_view", pcWindow->_GetTopView()->m->m_hViewHandle );
		cReq.AddInt32( "view", m->m_hViewHandle );

		CMessage cReply;
		if ( CMessenger( pcWindow->_GetAppserverPort() ).SendMessage( &cReq, &cReply ) < 0 )
		{
			dbprintf( "Error: Window::ToggleDepth() failed to send WR_TOGGLE_VIEW_DEPTH request to the server\n" );
		}
	}

	if ( NULL != m->m_pcParent )
	{
		if ( m->m_pcParent->m->m_pcTopChild != this )
		{
			m->m_pcParent->_UnlinkChild( this );
			m->m_pcParent->_LinkChild( this, true );
		}
		else
		{
			m->m_pcParent->_UnlinkChild( this );
			m->m_pcParent->_LinkChild( this, false );
		}
	}
	return( false );
}

/** Set the size and position relative to the parent view.
 * \par Description:
 *        Set the frame-rectangle of the view relative to its parent.
 *        If this cause new areas of this view or any of its
 *        siblings/childs the affected views will receive a draw
 *        message to update the newly exposed areas.
 *
 *        If the upper-left corner of the view is moved the
 *        CView::FrameMoved() hook will be called and if the lower-right
 *        corner move relative to the upper-left corner the
 *        CView::FrameResized() hook will be called.
 *
 * \param cRect
 *        The new frame rectangle.
 * \sa ResizeTo(), ResizeBy(), MoveTo(), MoveBy()
 * \author        Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

void CView::SetFrame( const CRect& cRect, bool bNotifyServer )
{
	CPoint cMove = CPoint( cRect.left, cRect.top ) - CPoint( m->m_cFrame.left, m->m_cFrame.top );
	CPoint cSize = CPoint( cRect.Width(), cRect.Height() ) - CPoint( m->m_cFrame.Width(), m->m_cFrame.Height() );

	m->m_cFrame = cRect;

	CWindow* pcWindow = Window();

	if ( cSize.x || cSize.y || cMove.x || cMove.y )
	{
		if ( bNotifyServer && pcWindow != NULL )
		{
			GRndSetFrame_s* psCmd = static_cast<GRndSetFrame_s*>(pcWindow->_AllocRenderCmd( DRC_SET_FRAME, this, sizeof(GRndSetFrame_s) ));

			if ( psCmd != NULL )
			{
				psCmd->cFrame = cRect;
			}
		}

		if ( cSize.x || cSize.y )
		{
			CView* pcChild;

			for ( pcChild = m->m_pcBottomChild ; NULL != pcChild ; pcChild = pcChild->m->m_pcHigherSibling )
			{
				pcChild->_ParentSized( cSize );
			}
		}
	}

	if ( cSize.x || cSize.y )
		FrameResized( cSize.x, cSize.y );

	if ( cMove.x || cMove.y )
		FrameMoved( cMove );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MoveBy( const CPoint& cDelta )
{
	SetFrame( m->m_cFrame + cDelta );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MoveBy( float vDeltaX, float vDeltaY )
{
	SetFrame( m->m_cFrame + CPoint( vDeltaX, vDeltaY ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MoveTo( CPoint cPos )
{
	SetFrame( m->m_cFrame - GetLeftTop() + cPos );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MoveTo( float x, float y )
{
	SetFrame( m->m_cFrame - GetLeftTop() + CPoint( x, y ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ResizeBy( const CPoint& cDelta )
{
	SetFrame( CRect( m->m_cFrame.left, m->m_cFrame.top, m->m_cFrame.right + cDelta.x, m->m_cFrame.bottom + cDelta.y ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ResizeBy( float vDeltaW, float vDeltaH )
{
	SetFrame( CRect( m->m_cFrame.left, m->m_cFrame.top, m->m_cFrame.right + vDeltaW, m->m_cFrame.bottom + vDeltaH ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ResizeTo( const CPoint& cSize )
{
	SetFrame( CRect( m->m_cFrame.left, m->m_cFrame.top, m->m_cFrame.left + cSize.x, m->m_cFrame.top + cSize.y ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ResizeTo( float w, float h )
{
	SetFrame( CRect( m->m_cFrame.left, m->m_cFrame.top, m->m_cFrame.left + w, m->m_cFrame.top + h ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::FrameMoved( CPoint cPoint )
{
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::FrameResized( float inWidth, float inHeight )
{
}

void CView::ViewScrolled( const CPoint& cDelta )
{
}

/** Add a rectangle to the damage list.
 * \par Description:
 *        To avoid rendering the whole view when a new area is made
 *        visible, the appserver maintain a damage list containing
 *        areas made visible since the last draw message was sent
 *        to the view. When the View respond to the draw message
 *        the appserver will clip the rendering to the damage list
 *        to avoid rendering any pixels that are still intact.
 *        By calling Invalidate() you can force an area
 *        into the damage list, and make the appserver send a
 *        draw message to the view. This will again make the
 *        Draw() member be called to revalidate the damaged
 *        areas.
 * \param cRect
 *        The rectangle to invalidate.
 * \param bRecurse
 *        If true cRect will also be converted into each children's
 *        coordinate system and added to their damage list.
 * 
 * \sa Invalidate( bool ), Draw()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *****************************************************************************/

void CView::Invalidate( CRect cRect, bool bRecurse )
{
	CWindow* pcWnd = Window();

	if ( pcWnd == NULL )
	{
		return;
	}

	GRndInvalidateRect_s* psCmd = static_cast<GRndInvalidateRect_s*>(pcWnd->_AllocRenderCmd( DRC_INVALIDATE_RECT, this, sizeof(GRndInvalidateRect_s) ));

	if ( psCmd != NULL )
	{
		psCmd->m_cRect    = cRect;
		psCmd->m_bRecurse = bRecurse;
	}
}

/** Invalidate the whole view.
 * \par Description:
 *        Same as calling Invalidate( Bounds() ), but more efficient since
 *        the appserver know that the whole view is to be invalidated, and
 *        does not have to merge the rectangle into the clipping region.
 * \param bRecurse - True if all childs should be invalidated recursivly as well.
 * \sa Invalidate( CRect, bool ), Draw()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *****************************************************************************/

void CView::Invalidate( bool bRecurse )
{
	CWindow* pcWnd = Window();
	if ( pcWnd == NULL )
	{
		return;
	}

	GRndInvalidateView_s* psCmd = static_cast<GRndInvalidateView_s*>(pcWnd->_AllocRenderCmd( DRC_INVALIDATE_VIEW, this, sizeof(GRndInvalidateView_s) ));

	if ( psCmd != NULL )
	{
		psCmd->m_bRecurse = bRecurse;
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_ParentSized( const CPoint& cDelta )
{
	CRect cNewFrame = m->m_cFrame;

	if ( m->m_nResizeMask & CF_FOLLOW_H_CENTER )
	{
		if ( m->m_nResizeMask & CF_FOLLOW_LEFT )
		{
			cNewFrame.right = (cNewFrame.right * 2 + cDelta.x + 1) / 2;
		}

		if ( m->m_nResizeMask & CF_FOLLOW_RIGHT )
		{
			cNewFrame.left = (cNewFrame.left * 2 + cDelta.x + 1) / 2;

			cNewFrame.right += cDelta.x;
		}
	}
	else
	{
		if ( m->m_nResizeMask & CF_FOLLOW_RIGHT )
		{
			if ( (m->m_nResizeMask & CF_FOLLOW_LEFT) == 0 )
			{
				cNewFrame.left += cDelta.x;
			}
			cNewFrame.right += cDelta.x;
		}
	}
	if ( m->m_nResizeMask & CF_FOLLOW_BOTTOM )
	{
		if ( (m->m_nResizeMask & CF_FOLLOW_TOP) == 0 )
		{
			cNewFrame.top += cDelta.y;
		}
		cNewFrame.bottom += cDelta.y;
	}

	SetFrame( cNewFrame );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_BeginUpdate()
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		if ( m->m_hViewHandle != -1 )
		{
			if ( m->m_nBeginPaintCount++ == 0 )
			{
				pcWindow->_AllocRenderCmd( DRC_BEGIN_UPDATE, this, sizeof(GRndHeader_s) );
			}
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::_EndUpdate( void )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		if ( --m->m_nBeginPaintCount == 0 )
		{
			pcWindow->_AllocRenderCmd( DRC_END_UPDATE, this, sizeof(GRndHeader_s) );
		}
	}
}

/** Called by the system update "damaged" areas of the view.
 * \par Description:
 * \par Note:
 * \par Warning:
 * \param cUpdateRect
 *        A rectangle enclosing all damaged areas. This is just a rough
 *        "worst-case", further fine-grained clipping will be performed
 *        by the Application Server to avoid updating non-damaged pixels
 *        and make the update as fast and flicker-free as possible.
 *
 * \sa Invalidate(), Flush()
 * \author        Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/


void CView::Draw( CRect cUpdateRect )
{
	SetViewColor( get_default_color( COL_NORMAL ) );
	EraseRect( cUpdateRect );
}

/** Flush the render queue.
 * \par Description:
 *        Call the window's Flush() member to flush the render queue.
 *
 *        If the view is not attached to a window, this member do nothing.
 *
 *        To get more info take a look at Window::Flush()
 * \sa Sync(), Window::Flush()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *****************************************************************************/

void CView::Flush( void )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		pcWindow->Flush();
	}
}

/** Flush the render queue.
 * \par Description:
 *        Call the window's Flush() member to flush the render queue and
 *        syncronize the view with the appserver.
 *
 *        If the view is not attached to a window, this member do nothing.
 *
 *        To get more info take a look at Window::Flush()
 * \sa Sync(), Window::Flush()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *****************************************************************************/

void CView::Sync( void )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		pcWindow->Sync();
	}
}



CPoint CView::PenLocation() const
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		return( pcWindow->_GetPenPosition( m->m_hViewHandle ) );
	}

	return( CPoint( 0, 0 ) );
}


void CView::SetOrigin(CPoint inPoint)
{
	m->mOrigin = inPoint;
}


CPoint CView::Origin(void) const
{
	return m->mOrigin;
}


void CView::SetScale(float inScale) const
{
	m->mScale = inScale;
}


void CView::PushState()
{
	StateStruct* aState = (StateStruct*)malloc(sizeof(StateStruct));

	aState->origin = Origin();
	aState->scale = m->mScale;
	aState->drawingMode = DrawingMode();
	aState->penLocation = PenLocation();
	aState->highColor = HighColor();
	aState->lowColor = LowColor();
	aState->nextState = m->mStateStack;

	m->mStateStack = aState;
}


void CView::PopState()
{
	if (m->mStateStack != NULL)
	{
		StateStruct* aState = m->mStateStack;
		m->mStateStack = aState->nextState;

		SetOrigin(aState->origin);
		SetScale(aState->scale);
		SetDrawingMode(aState->drawingMode);
		SetHighColor(aState->highColor);
		SetLowColor(aState->lowColor);
		MovePenTo(aState->penLocation);

		free(aState);
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MovePenTo( CPoint cPos )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndSetPenPos_s* psCmd = static_cast<GRndSetPenPos_s*>(pcWindow->_AllocRenderCmd( DRC_SET_PEN_POS, this, sizeof( GRndSetPenPos_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->bRelative        = false;
			psCmd->sPosition        = cPos;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::MovePenBy( CPoint cPos )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndSetPenPos_s* psCmd = static_cast<GRndSetPenPos_s*>(pcWindow->_AllocRenderCmd( DRC_SET_PEN_POS, this, sizeof( GRndSetPenPos_s )));
		if ( psCmd != NULL )
		{
			psCmd->bRelative = true;
			psCmd->sPosition = cPos;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::DrawString( const char* pString, int nLength )
{
	if ( nLength == -1 )
	{
		nLength = strlen( pString );
	}

	if ( nLength > 0 )
	{
		CWindow* pcWindow = Window();

		if ( pcWindow != NULL )
		{
			GRndDrawString_s* psCmd = static_cast<GRndDrawString_s*>( pcWindow->_AllocRenderCmd( DRC_DRAW_STRING, this,
																								sizeof( GRndDrawString_s ) + nLength - 1 ) );
			if ( psCmd != NULL )
			{
				psCmd->nLength = nLength;
				memcpy( psCmd->zString, pString, nLength );
			}
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::DrawString( const char* inString )
{
	DrawString( inString, strlen( inString ) );
}

//void CView::DrawString( const std::string& cString )
//{
//	DrawString( cString.c_str(), cString.size() );
//}

void CView::DrawString( const char* inString, CPoint inPosition )
{
	MovePenTo( inPosition );
	DrawString( inString, strlen( inString ) );
}

void CView::DrawString( const char* inString, int nLength, CPoint inPosition )
{
	MovePenTo( inPosition );
	DrawString( inString, nLength );
}

//void CView::DrawString( const std::string& cString, const CPoint& cPos )
//{
//	MovePenTo( cPos );
//	DrawString( cString.c_str(), cString.size() );
//}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetDrawingMode( drawing_mode inMode )
{
	if ( inMode != m->m_eDrawingMode )
	{
		CWindow* pcWnd = Window();
		m->m_eDrawingMode = inMode;

		if ( pcWnd == NULL )
		{
			return;
		}

		GRndSetDrawingMode_s* psMsg;
		psMsg = static_cast<GRndSetDrawingMode_s*>( pcWnd->_AllocRenderCmd( DRC_SET_DRAWING_MODE, this,
																		sizeof( GRndSetDrawingMode_s ) ) );
		if ( psMsg != NULL )
		{
			psMsg->nDrawingMode = inMode;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

drawing_mode CView::DrawingMode() const
{
	return( m->m_eDrawingMode );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetHighColor( int nRed, int nGreen, int nBlue, int nAlpha )
{
	SetHighColor( Color32_s( nRed, nGreen, nBlue, nAlpha ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetHighColor( Color32_s sColor )
{
	CWindow* pcWindow = Window();
	m->m_sFgColor = sColor;

	if ( pcWindow != NULL )
	{
		GRndSetColor32_s* psCmd = static_cast<GRndSetColor32_s*>(pcWindow->_AllocRenderCmd( DRC_SET_COLOR32, this, sizeof( GRndSetColor32_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->nWhichPen     = PEN_HIGH;
			psCmd->sColor        = sColor;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetLowColor( int nRed, int nGreen, int nBlue, int nAlpha )
{
	SetLowColor( Color32_s( nRed, nGreen, nBlue, nAlpha ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetLowColor( Color32_s sColor )
{
	CWindow* pcWindow = Window();

	m->m_sBgColor = sColor;

	if ( pcWindow != NULL )
	{
		GRndSetColor32_s* psCmd = static_cast<GRndSetColor32_s*>(pcWindow->_AllocRenderCmd( DRC_SET_COLOR32, this, sizeof( GRndSetColor32_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->nWhichPen        = PEN_LOW;
			psCmd->sColor        = sColor;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetViewColor( int nRed, int nGreen, int nBlue, int nAlpha )
{
	SetViewColor( Color32_s( nRed, nGreen, nBlue, nAlpha ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::SetViewColor( Color32_s sColor )
{
	CWindow* pcWindow = Window();

	m->m_sEraseColor = sColor;

	if ( pcWindow != NULL )
	{
		GRndSetColor32_s* psCmd = static_cast<GRndSetColor32_s*>(pcWindow->_AllocRenderCmd( DRC_SET_COLOR32, this, sizeof( GRndSetColor32_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->nWhichPen = PEN_ERASE;
			psCmd->sColor    = sColor;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

Color32_s CView::HighColor() const
{
	return( m->m_sFgColor );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

Color32_s CView::LowColor() const
{
	return( m->m_sBgColor );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

Color32_s CView::ViewColor() const
{
	return( m->m_sEraseColor );
}

void CView::_ReleaseFont()
{
	m->m_pcFont->Release();
}

/** Change the views text font.
 * \par Description:
 *        Replace the font used when rendering text and notify the view
 *        itself about the change by calling the CView::FontChanged()
 *        hook function.
 *
 *        Font's are reference counted and SetFont() will call
 *        Font::AddRef() on the new font and Font::Release() on the old
 *        font. This means that you retain ownership on the font even
 *        after CView::SetFont() returns. You must therefor normally call
 *        Font::Release() on the font after setting it.
 *
 *        The view will be affected by changes applied to the font after
 *        it is set. If someone change the properties of the font later
 *        the view will again be notified through the CView::FontChanged()
 *        hook
 *
 * \param pcFont
 *        Pointer to the new font. It's reference count will be
 *        increased by one.
 *
 * \sa DrawString(), os::Font
 * \author        Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

void CView::SetFont( CFont* pcFont )
{
	CWindow* pcWindow = Window();

	if ( pcFont == m->m_pcFont )
	{
		return;
	}
	if ( m->m_pcFont != NULL )
	{
		_ReleaseFont();
	}
	if ( NULL != pcFont )
	{
		m->m_pcFont = pcFont;
		m->m_pcFont->AddRef();
	}
	else
	{
		m->m_pcFont = new CFont();
		if ( m->m_pcFont->SetFamilyAndStyle( SYS_FONT_FAMILY, SYS_FONT_PLAIN ) == 0 )
		{
			m->m_pcFont->SetProperties( 8.0f, 0.0f, 0.0f );
		}
	}

	if ( pcWindow != NULL )
	{
		GRndSetFont_s* psCmd = static_cast<GRndSetFont_s*>( pcWindow->_AllocRenderCmd( DRC_SET_FONT, this, sizeof( GRndSetFont_s ) ));

		if ( psCmd != NULL )
		{
			psCmd->hFontID = m->m_pcFont->GetFontID();
		}
	}
	FontChanged( m->m_pcFont );
}

CFont* CView::GetFont() const
{
	return( m->m_pcFont );
}

void CView::SetFontSize( float inSize )
{
	m->m_pcFont->SetSize(inSize);
}


void CView::StrokeRect( CRect inRect /*, Pattern inPattern */ )
{
	// FIXME Fix this obvious lameness
	
	StrokeLine( inRect.LeftTop(), inRect.RightTop() );
	StrokeLine( inRect.RightBottom() );
	StrokeLine( inRect.LeftBottom() );
	StrokeLine( inRect.LeftTop() );
}
	
	
//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::StrokeLine( CPoint cFromPnt, CPoint cToPnt /*, Pattern inPattern */ )
{
	CWindow*        pcWindow = Window();

	MovePenTo( cFromPnt );

	if ( pcWindow != NULL )
	{
		GRndLine32_s* psCmd = static_cast<GRndLine32_s*>( pcWindow->_AllocRenderCmd( DRC_LINE32, this, sizeof( GRndLine32_s )) );
		if ( psCmd != NULL )
		{
			psCmd->sToPos = cToPnt;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::StrokeLine( CPoint cToPnt /*, Pattern inPattern */ )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndLine32_s* psCmd = static_cast<GRndLine32_s*>(pcWindow->_AllocRenderCmd( DRC_LINE32, this, sizeof( GRndLine32_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->sToPos = cToPnt;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::FillRect( CRect cRect )
{
	CWindow*            pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndFillRect32_s* psCmd = static_cast<GRndFillRect32_s*>( pcWindow->_AllocRenderCmd( DRC_FILL_RECT32, this, sizeof( GRndFillRect32_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->sRect  = cRect;
			psCmd->sColor = m->m_sFgColor;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::FillRect( CRect cRect, Color32_s sColor )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndFillRect32_s* psCmd = static_cast<GRndFillRect32_s*>(pcWindow->_AllocRenderCmd( DRC_FILL_RECT32, this, sizeof( GRndFillRect32_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->sRect  = cRect;
			psCmd->sColor = sColor;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::EraseRect( const CRect& cRect )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndFillRect32_s* psCmd = static_cast<GRndFillRect32_s*>(pcWindow->_AllocRenderCmd( DRC_FILL_RECT32, this, sizeof( GRndFillRect32_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->sRect  = cRect;
			psCmd->sColor = m->m_sEraseColor;
		}
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::DrawBitmap( const CBitmap* inBitmap, CRect inDstRect )
{
	DrawBitmap(inBitmap, inDstRect, inDstRect);
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::DrawBitmap( const CBitmap* inBitmap, CRect inSrcRect, CRect inDstRect )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndDrawBitmap_s* psCmd = static_cast<GRndDrawBitmap_s*>(pcWindow->_AllocRenderCmd( DRC_DRAW_BITMAP, this, sizeof( GRndDrawBitmap_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->hBitmapToken = inBitmap->mHandle;
			psCmd->cSrcRect     = inSrcRect;
			psCmd->cDstRect     = inDstRect;
		}
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::DrawBitmap( const CBitmap* inBitmap, CPoint inWhere )
{
	DrawBitmap(inBitmap, inBitmap->mBounds + inWhere, inBitmap->mBounds + inWhere);
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::DrawBitmap( const CBitmap* inBitmap )
{
	DrawBitmap(inBitmap, inBitmap->mBounds + PenLocation(),
						 inBitmap->mBounds + PenLocation());
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::DrawFrame( const CRect& a_cRect, uint32 nStyle )
{
	CRect cRect(a_cRect);
	cRect.Floor();
	bool bSunken = false;

	if ( ((nStyle & FRAME_RAISED) == 0) && (nStyle & (FRAME_RECESSED)) )
	{
		bSunken = true;
	}

	Color32_s sFgCol = get_default_color( COL_SHINE );
	Color32_s sBgCol = get_default_color( COL_SHADOW );

	if ( nStyle & FRAME_DISABLED )
	{
		sFgCol = Tint( sFgCol, 0.6f );
		sBgCol = Tint( sBgCol, 0.4f );
	}
	Color32_s sFgShadowCol = Tint( sFgCol, 0.6f );
	Color32_s sBgShadowCol = Tint( sBgCol, 0.5f );

	if ( nStyle & FRAME_FLAT )
	{
		SetHighColor( (bSunken) ? sBgCol : sFgCol );
		MovePenTo( cRect.left, cRect.bottom );
		StrokeLine( CPoint( cRect.left, cRect.top ) );
		StrokeLine( CPoint( cRect.right, cRect.top ) );
		StrokeLine( CPoint( cRect.right, cRect.bottom ) );
		StrokeLine( CPoint( cRect.left, cRect.bottom ) );
	}
	else
	{
		if ( nStyle & FRAME_THIN )
		{
			SetHighColor( (bSunken) ? sBgCol : sFgCol );
		}
		else
		{
			SetHighColor( (bSunken) ? sBgCol : sFgShadowCol );
		}

		MovePenTo( cRect.left, cRect.bottom );
		StrokeLine( CPoint( cRect.left, cRect.top ) );
		StrokeLine( CPoint( cRect.right, cRect.top ) );

		if ( nStyle & FRAME_THIN )
		{
			SetHighColor( (bSunken) ? sFgCol : sBgCol );
		}
		else
		{
			SetHighColor( (bSunken) ? sFgCol : sBgShadowCol );
		}
		StrokeLine( CPoint( cRect.right, cRect.bottom ) );
		StrokeLine( CPoint( cRect.left, cRect.bottom ) );


		if ( (nStyle & FRAME_THIN) == 0 )
		{
			if ( nStyle & FRAME_ETCHED )
			{
				SetHighColor( (bSunken) ? sBgCol : sFgCol );

				MovePenTo( cRect.left + 1.0f, cRect.bottom - 1.0f );

				StrokeLine( CPoint( cRect.left + 1.0f, cRect.top + 1.0f ) );
				StrokeLine( CPoint( cRect.right - 1.0f, cRect.top + 1.0f ) );

				SetHighColor( (bSunken) ? sFgCol : sBgCol );

				StrokeLine( CPoint( cRect.right - 1.0f, cRect.bottom - 1.0f ) );
				StrokeLine( CPoint( cRect.left + 1.0f, cRect.bottom - 1.0f ) );
			}
			else
			{
				SetHighColor( (bSunken) ? sBgShadowCol : sFgCol );

				MovePenTo( cRect.left + 1.0f, cRect.bottom - 1.0f );

				StrokeLine( CPoint( cRect.left + 1.0f, cRect.top + 1.0f ) );
				StrokeLine( CPoint( cRect.right - 1.0f, cRect.top + 1.0f ) );

				SetHighColor( (bSunken) ? sFgShadowCol : sBgCol );

				StrokeLine( CPoint( cRect.right - 1.0f, cRect.bottom - 1.0f ) );
				StrokeLine( CPoint( cRect.left + 1.0f, cRect.bottom - 1.0f ) );
			}
			if ( (nStyle & FRAME_TRANSPARENT) == 0 )
			{
				EraseRect( CRect( cRect.left + 2.0f, cRect.top + 2.0f, cRect.right - 2.0f, cRect.bottom - 2.0f) );
			}
		}
		else
		{
			if ( (nStyle & FRAME_TRANSPARENT) == 0 )
			{
				EraseRect( CRect( cRect.left + 1.0f, cRect.top + 1.0f, cRect.right - 1.0f, cRect.bottom - 1.0f) );
			}
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ScrollBy( CPoint cDelta )
{
	if ( cDelta.x || cDelta.y )
	{
		m->m_cScrollOffset += cDelta;
		CWindow* pcWindow = Window();

		if ( pcWindow != NULL ) {
			GRndScrollView_s* psCmd = (GRndScrollView_s*)pcWindow->_AllocRenderCmd( DRC_SCROLL_VIEW, this,
																				sizeof( GRndScrollView_s ) );
			if ( psCmd != NULL ) {
				psCmd->cDelta = cDelta;
			}
		}
		if ( cDelta.x != 0 && m->m_pcHScrollBar != NULL ) {
			m->m_pcHScrollBar->SetValue( -m->m_cScrollOffset.x );
		}
		if ( cDelta.y != 0 && m->m_pcVScrollBar != NULL ) {
			m->m_pcVScrollBar->SetValue( -m->m_cScrollOffset.y );
		}
		ViewScrolled( cDelta );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ScrollTo( CPoint cTopLeft )
{
	ScrollBy( cTopLeft - m->m_cScrollOffset );
}

/** Copy a rectangle from one location to another within the view.
 * \par Description:
 *        ScrollRect() will copy the source rectangle to the destination
 *        rectangle using the blitter. If parts of the source rectangle
 *        is obscured (by another window/view or the screen edge) and
 *        the same area in the destination area is not that area in the
 *        destination rectangle will be invalidated (see Invalidate()).
 *
 * \par Note:
 *        Scrolling an area within the view might affect the current
 *        "damage-list" of the view and cause a repaint message to be
 *        sent to the view. If ScrollRect() is called repeatadly and
 *        the application for some reason can't handle the generated
 *        repaint messages fast enough more and more damage-rectangles
 *        will accumulate in the views damage list slowing things
 *        further down until the appserver is broght to a grinding halt.
 *        To avoid this situation the next draw message received after
 *        a call to ScrollRect() will cause Sync() instead of Flush() to
 *        be called when Draw() returns to syncronize the application
 *        with the appserver. So even though the ScrollRect() member
 *        itself is asyncronous it might cause Sync() to be called and
 *        will have the performance implications mentioned in the
 *        documentation of Sync()
 * 
 * \param cSrcRect
 *        The source rectangle in the views coordinate system.
 * \param cSrcRect
 *        The destination rectangle in the views coordinate system.
 *        This rectangle should have the same size but a difference
 *        position than the \p cSrcRect. In a future version it might be
 *        possible to scale the rectangle by using a different size so
 *        make sure they don't differ or you might get a surprice
 *        some day.
 *
 * \sa ScrollTo(), ScrollBy(), Sync()
 * \author        Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/


void CView::ScrollRect( const CRect& cSrcRect, const CRect& cDstRect )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		GRndCopyRect_s* psCmd = static_cast<GRndCopyRect_s*>(pcWindow->_AllocRenderCmd( DRC_COPY_RECT, this, sizeof( GRndCopyRect_s ) ));
		if ( psCmd != NULL )
		{
			psCmd->cSrcRect = cSrcRect;
			psCmd->cDstRect = cDstRect;
		}
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

float CView::StringWidth( const char* pzString) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return( 0 );
	}
	return( m->m_pcFont->GetStringWidth( pzString ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

float CView::StringWidth( const char* pzString, int nLength ) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return( 0 );
	}
	return( m->m_pcFont->GetStringWidth( pzString, nLength ) );
}

float CView::StringWidth( const std::string& cString) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return( 0 );
	}
	return( m->m_pcFont->GetStringWidth( cString ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::GetStringWidths( const char* apzStringArray[], const int* anLengthArray,
                            int nStringCount, float* avWidthArray ) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return;
	}
	m->m_pcFont->GetStringWidths( apzStringArray, anLengthArray, nStringCount, avWidthArray );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int CView::GetStringLength( const char* pzString, float vWidth, bool bIncludeLast ) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return( 0 );
	}
	return( m->m_pcFont->GetStringLength( pzString, vWidth, bIncludeLast ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

int CView::GetStringLength( const char* pzString, int nLength, float vWidth, bool bIncludeLast ) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return( 0 );
	}

	return( m->m_pcFont->GetStringLength( pzString, nLength, vWidth, bIncludeLast ) );
}

int CView::GetStringLength( const std::string& cString, float vWidth, bool bIncludeLast ) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return( 0 );
	}
	return( m->m_pcFont->GetStringLength( cString, vWidth, bIncludeLast ) );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::GetStringLengths( const char** apzStringArray, const int* anLengthArray, int nStringCount,
                             float vWidth, int* anMaxLengthArray, bool bIncludeLast ) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return;
	}
	m->m_pcFont->GetStringLengths( apzStringArray, anLengthArray, nStringCount, vWidth,
								anMaxLengthArray, bIncludeLast );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::GetFontHeight( font_height* psHeight ) const
{
	if ( m->m_pcFont == NULL )
	{
		dbprintf( "Warning: %s() View %s has no font\n", __FUNCTION__, m->m_cTitle.c_str() );
		return;
	}
	m->m_pcFont->GetHeight( psHeight );
}

/** @name ConvertToParent
 * \brief Convert a point or rectangle from local to parent coordinate system
 * \par Description:
 *        Convert a point or rectangle from local to parent coordinate system
 * \par Note:
 * \par Warning:
 * \param
 * \return
 * \sa
 * \author        Kurt Skauen (kurt@atheos.cx)
 */
//@{
/** Translate a os::CPoint into our parents coordinate system. Returning the the result */

CPoint CView::ConvertToParent( CPoint cPoint ) const
{
	return( cPoint + GetLeftTop() + m->m_cScrollOffset );
}

/** Translate a os::CPoint into our parent's coordinate system. Modifying the passed CPoint object */

void CView::ConvertToParent( CPoint* pcPoint ) const
{
	*pcPoint += GetLeftTop() + m->m_cScrollOffset;
}

CRect CView::ConvertToParent( CRect cRect ) const
{
	return( cRect + GetLeftTop() + m->m_cScrollOffset );
}


void CView::ConvertToParent( CRect* pcRect ) const
{
	*pcRect += GetLeftTop() + m->m_cScrollOffset;
}
//@}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CPoint CView::ConvertFromParent( CPoint cPoint ) const
{
	return( cPoint - GetLeftTop() - m->m_cScrollOffset );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertFromParent( CPoint* pcPoint ) const
{
	*pcPoint -= GetLeftTop() + m->m_cScrollOffset;
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CRect CView::ConvertFromParent( CRect cRect ) const
{
	return( cRect - GetLeftTop() - m->m_cScrollOffset );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertFromParent( CRect* pcRect ) const
{
	*pcRect -= GetLeftTop() + m->m_cScrollOffset;
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CPoint CView::ConvertToWindow( CPoint cPoint ) const
{
	if ( m->m_pcParent != NULL )
		return( m->m_pcParent->ConvertToWindow( cPoint + GetLeftTop() + m->m_cScrollOffset ) );

	return( cPoint );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertToWindow( CPoint* pcPoint ) const
{
	if ( m->m_pcParent != NULL )
	{
		*pcPoint += GetLeftTop() + m->m_cScrollOffset;
		m->m_pcParent->ConvertToWindow( pcPoint );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CRect CView::ConvertToWindow( CRect cRect ) const
{
	if ( m->m_pcParent != NULL )
		return( m->m_pcParent->ConvertToWindow( cRect + GetLeftTop() + m->m_cScrollOffset ) );

	return( cRect );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertToWindow( CRect* pcRect ) const
{
	if ( m->m_pcParent != NULL )
	{
		*pcRect += GetLeftTop() + m->m_cScrollOffset;
		m->m_pcParent->ConvertToWindow( pcRect );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CPoint CView::ConvertFromWindow( CPoint cPoint ) const
{
	if ( m->m_pcParent != NULL )
		return( m->m_pcParent->ConvertFromWindow( cPoint - GetLeftTop() - m->m_cScrollOffset ) );

	return( cPoint );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertFromWindow( CPoint* pcPoint ) const
{
	if ( m->m_pcParent != NULL )
	{
		*pcPoint -= GetLeftTop() + m->m_cScrollOffset;
		m->m_pcParent->ConvertFromWindow( pcPoint );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CRect CView::ConvertFromWindow( CRect cRect ) const
{
	if ( m->m_pcParent != NULL )
		return( m->m_pcParent->ConvertFromWindow( cRect - GetLeftTop() - m->m_cScrollOffset ) );

	return( cRect );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertFromWindow( CRect* pcRect ) const
{
	if ( m->m_pcParent != NULL )
	{
		*pcRect -= GetLeftTop() + m->m_cScrollOffset;
		m->m_pcParent->ConvertFromWindow( pcRect );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CPoint CView::ConvertToScreen( CPoint cPoint ) const
{
	if ( m->m_pcParent != NULL )
		return( m->m_pcParent->ConvertToScreen( cPoint + GetLeftTop() + m->m_cScrollOffset ) );

	return( cPoint + GetLeftTop() + m->m_cScrollOffset );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertToScreen( CPoint* pcPoint ) const
{
	*pcPoint += GetLeftTop() + m->m_cScrollOffset;

	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->ConvertToScreen( pcPoint );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CRect CView::ConvertToScreen( CRect cRect ) const
{
	if ( m->m_pcParent != NULL )
		return( m->m_pcParent->ConvertToScreen( cRect + GetLeftTop() + m->m_cScrollOffset ) );

	return( cRect + GetLeftTop() );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertToScreen( CRect* pcRect ) const
{
	*pcRect += GetLeftTop() + m->m_cScrollOffset;

	if ( m->m_pcParent != NULL )
		m->m_pcParent->ConvertToScreen( pcRect );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CPoint CView::ConvertFromScreen( CPoint cPoint ) const
{
	if ( m->m_pcParent != NULL )
		return( m->m_pcParent->ConvertFromScreen( cPoint - GetLeftTop() - m->m_cScrollOffset ) );

	return( cPoint - GetLeftTop() );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertFromScreen( CPoint* pcPoint ) const
{
	*pcPoint -= GetLeftTop() + m->m_cScrollOffset;

	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->ConvertFromScreen( pcPoint );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CRect CView::ConvertFromScreen( CRect cRect ) const
{
	if ( m->m_pcParent != NULL )
		return( m->m_pcParent->ConvertFromScreen( cRect - GetLeftTop() - m->m_cScrollOffset ) );

	return( cRect - GetLeftTop() );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::ConvertFromScreen( CRect* pcRect ) const
{
	*pcRect -= GetLeftTop() + m->m_cScrollOffset;

	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->ConvertFromScreen( pcRect );
	}
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CRect CView::Frame() const
{
    return( m->m_cFrame );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CRect CView::Bounds() const
{
	return( m->m_cFrame - CPoint( m->m_cFrame.left, m->m_cFrame.top ) - m->m_cScrollOffset );
}


CRect CView::GetNormalizedBounds() const
{
	return( m->m_cFrame.Bounds() );
}


void CView::_ConstrictRectangle( CRect* pcRect, const CPoint& cOffset )
{
	CPoint cOff = cOffset - CPoint( m->m_cFrame.left, m->m_cFrame.top ) - m->m_cScrollOffset;
	*pcRect &= m->m_cFrame + cOff;
	if ( m->m_pcParent != NULL )
	{
		m->m_pcParent->_ConstrictRectangle( pcRect, cOff );
	}
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

float CView::Width() const
{
	return( m->m_cFrame.Width() );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

float CView::Height() const
{
	return( m->m_cFrame.Height() );
}

//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

CPoint CView::GetLeftTop() const
{
	return( m->m_cFrame.LeftTop() );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

const char* CView::Title() const
{
	return( m->m_cTitle.c_str() );
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::AttachedToWindow()
{
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::AllAttached()
{
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::DetachedFromWindow()
{
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::AllDetached()
{
}

/** Hook called when the view gain or loose focus.
 * \par Description:
 *        This is a callback member that can be overloaded by derived classes to
 *        learn when the view is activated and when it is deactivated. The
 *        bIsActive parameter tell whether the focus was lost or gained.
 *
 *        The view has focus when it is the active view in the active window.
 * \par Note:
 *        This is a hook function that is called by the system to notify about
 *        an event. You should never call this member yourself.
 *
 *        The window is locked when this member is called.
 * \param bIsActive - true if the view gain and false if it loose focus.
 * \sa MakeFocus(), WindowActivated()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *//////////////////////////////////////////////////////////////////////////////


void CView::Activated( bool bIsActive )
{
}

/** Hook called when the window hosting this view gain or loose focus.
 * \par Description:
 *        This is a callback member that can be overloaded by derived classes to
 *        learn when the window that this view is hosted by is activated and when
 *        it is deactivated. The bIsActive parameter tell whether the focus was
 *        lost or gained. This member is called whenever the window changes focus
 *        independent of whether the view is active or not.
 *        
 * \par Note:
 *        This is a hook function that is called by the system to notify about
 *        an event. You should never call this member yourself.
 *
 *        The window is locked when this member is called.
 * \param bIsActive - true if the window gain and false if it loose focus.
 * \sa MakeFocus(), Activated()
 * \author        Kurt Skauen (kurt.skauen@c2i.net)
 *//////////////////////////////////////////////////////////////////////////////

void CView::WindowActivated( bool bIsActive )
{
}


/** Called to notify the view that the font has changed
 * \par Description:
 *        FontChanged() is a virtual hook function that can be
 *        overloaded by inheriting classes to track changes to the
 *        view's font.
 *
 *        This hook function is called whenver the font is replaced
 *        through the CView::SetFont() member or if the currently
 *        assigned font is modified in a way that whould alter the
 *        graphical appearance of the font.
 *
 * \par Note:
 *        CView::SetFont() will call FontChanged() syncronously and will
 *        cause FontChanged() to be called even if the view is not yet
 *        added to a window. Changes done to the font-object cause a
 *        message to be sent to the window thread and FontChanged()
 *        will then be called asyncronously from the window thread when
 *        the message arrive. For this reason it is only possible to
 *        track changes done to the font object itself when the view is
 *        added to a window.
 *        
 * \param pcNewFont
 *        Pointer to the affected font (same as returned by GetFont()).
 *
 * \sa SetFont(), os::Font
 * \author        Kurt Skauen (kurt@atheos.cx)
 *****************************************************************************/

void CView::FontChanged( CFont* pcNewFont )
{
}


//----------------------------------------------------------------------------
// NAME:
// DESC:
// NOTE:
// SEE ALSO:
//----------------------------------------------------------------------------

void CView::Ping( int nSize )
{
	CWindow* pcWindow = Window();

	if ( pcWindow != NULL )
	{
		pcWindow->_AllocRenderCmd( DRC_PING, this, sizeof( GRndHeader_s ) + nSize );
	}
}


int CView::_GetHandle() const
{
	return( m->m_hViewHandle );
}


void CView::_SetMouseMoveRun( int nValue )
{
    m->m_nMouseMoveRun = nValue;
}


int CView::_GetMouseMoveRun() const
{
	return( m->m_nMouseMoveRun );
}


void CView::_SetMouseMode( int nMode )
{
	m->m_nMouseMode = nMode;
}


int CView::_GetMouseMode() const
{
	return( m->m_nMouseMode );
}


void CView::_SetHScrollBar( CScrollBar* pcScrollBar )
{
	m->m_pcHScrollBar = pcScrollBar;
}


void CView::_SetVScrollBar( CScrollBar* pcScrollBar )
{
	m->m_pcVScrollBar = pcScrollBar;
}


void CView::FBC_View1()  {}
void CView::FBC_View2()  {}
void CView::FBC_View3()  {}
void CView::FBC_View4()  {}
void CView::FBC_View5()  {}
void CView::FBC_View6()  {}
void CView::FBC_View7()  {}
void CView::FBC_View8()  {}
void CView::FBC_View9()  {}
void CView::FBC_View10() {}
void CView::FBC_View11() {}
void CView::FBC_View12() {}
void CView::FBC_View13() {}
void CView::FBC_View14() {}
void CView::FBC_View15() {}
