/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <storage/FilePanel.h>
#include <directoryview.h>
#include <interface/TextView.h>
#include <interface/Button.h>
#include <interface/Alert.h>
#include <requesters.h>
#include <app/Message.h>
#include <app/Application.h>

#include <strstream>

using namespace os;

CFilePanel::CFilePanel( file_req_mode_t   nMode,
                        CMessenger*       pcTarget,
                        const char*       pzPath,
                        uint32            nNodeType,
                        bool              bMultiSelect,
                        CMessage*         pcMessage,
                        FileFilter*       pcFilter,
                        bool              bModal,
                        bool              bHideWhenDone,
                        const char*       pzOkLabel,
                        const char*       pzCancelLabel ) : CWindow( CRect(0,0,1,1), "" )

{
	Lock();

	if ( pzOkLabel == NULL )
	{
		if ( nMode == LOAD_REQ )
		{
			pzOkLabel = "Load";
		}
		else
		{
			pzOkLabel = "Save";
		}
	}

	if ( pzCancelLabel == NULL )
	{
		pzCancelLabel = "Cancel";
	}


	m_nMode         = nMode;
	m_nNodeType     = nNodeType;
	m_bHideWhenDone = bHideWhenDone;
	m_pcTarget      = (pcTarget != NULL ) ? pcTarget : new CMessenger( CApplication::GetInstance() );
	m_pcMessage     = pcMessage;

	std::string cPath;
	std::string cFile;

	if ( pzPath == NULL )
	{
		const char* pzHome = getenv( "HOME" );
		if ( pzHome != NULL )
		{
			cPath = pzHome;
			if ( cPath[cPath.size()-1] != '/' )
			{
				cPath += '/';
			}
		}
		else
		{
			cPath = "/tmp/";
		}
	}
	else
	{
		cPath = pzPath;
		if ( cPath.find( '/' ) == std::string::npos )
		{
			const char* pzHome = getenv( "HOME" );
			if ( pzHome != NULL ) {
				cPath = pzHome;
				if ( cPath[cPath.size()-1] != '/' )
				{
					cPath += '/';
				}
			}
			else
			{
				cPath = "/tmp/";
			}
			cFile = pzPath;
		}
		else
		{
			cPath = pzPath;
		}
	}

	if ( cFile.empty() )
	{
		if ( nNodeType == NODE_FILE )
		{
			struct stat sStat;
			if ( cPath[cPath.size()-1] != '/' || (stat( cPath.c_str(), &sStat ) < 0 || S_ISREG( sStat.st_mode )) ) {
				uint nSlash = cPath.rfind( '/' );
				if ( nSlash != std::string::npos )
				{
					cFile = cPath.c_str() + nSlash + 1;
					cPath.resize( nSlash );
				}
				else
				{
					cFile = cPath;
					cPath = "";
				}
			}
		}
	}
	m_pcDirView      = new DirectoryView( Bounds(), cPath );
	m_pcPathView     = new TextView( CRect(0,0,1,1), "path_edit", cFile.c_str() );
	m_pcCancelButton = new CButton( CRect(0,0,1,1), "cancel", pzCancelLabel, new CMessage( ID_CANCEL ) );
	m_pcOkButton     = new CButton( CRect(0,0,1,1), "ok", pzOkLabel, new CMessage( ID_OK ) );

	AddChild( m_pcDirView );
	AddChild( m_pcPathView );
	AddChild( m_pcCancelButton );
	AddChild( m_pcOkButton );

	m_pcDirView->SetMultiSelect( bMultiSelect );
	m_pcDirView->SetSelChangeMsg( new CMessage( ID_SEL_CHANGED ) );
	m_pcDirView->SetInvokeMsg( new CMessage( ID_INVOKED ) );
	m_pcDirView->SetDirChangeMsg( new CMessage( ID_PATH_CHANGED ) );
	m_pcDirView->MakeFocus();
	SetTitle( m_pcDirView->GetPath().c_str() );

	CRect cFrame( 250, 150, 699, 499 );
	SetFrame( cFrame );
	Unlock();
}


void CFilePanel::Layout()
{
	CRect cBounds = Bounds();

	CPoint cSize1;
	CPoint cSize2;

	m_pcOkButton->GetPreferredSize( &cSize1.x, &cSize1.y );
	m_pcCancelButton->GetPreferredSize( &cSize2.x, &cSize2.y );

	if ( cSize2.x > cSize1.x )
	{
		cSize1 = cSize2;
	}

	CRect cOkRect = cBounds;

	cOkRect.bottom -= 5;
	cOkRect.top = cOkRect.bottom - cSize1.y;
	cOkRect.right -= 10;
	cOkRect.left = cOkRect.right - cSize1.x;

	CRect cCancelRect = cOkRect;
	cCancelRect.left -= cSize1.x + 15;
	cCancelRect.right -= cSize1.x + 15;

	float height;
	m_pcPathView->GetPreferredSize( NULL, &height );
	CRect cPathFrame = cBounds;

	cPathFrame.bottom = cOkRect.top - 10;
	cPathFrame.top = cPathFrame.bottom - height;
	cPathFrame.left += 10;
	cPathFrame.right -= 10;

	CRect cDirFrame = cBounds;
	cDirFrame.bottom = cPathFrame.top - 10;
	cDirFrame.top += 10;
	cDirFrame.left += 10;
	cDirFrame.right -= 10;

	m_pcPathView->SetFrame( cPathFrame );
	m_pcDirView->SetFrame( cDirFrame );
	m_pcOkButton->SetFrame( cOkRect );
	m_pcCancelButton->SetFrame( cCancelRect );
}


void CFilePanel::FrameResized( float inWidth, float inHeight )
{
	Layout();
}


void CFilePanel::MessageReceived( CMessage* pcMessage )
{
	switch( pcMessage->what )
	{
		case ID_PATH_CHANGED:
			SetTitle( m_pcDirView->GetPath().c_str() );
			break;

		case ID_SEL_CHANGED:
		{
			int nSel = m_pcDirView->GetFirstSelected();
			if ( nSel >= 0 )
			{
				FileRow* pcFile = m_pcDirView->GetFile( nSel );
				if ( (m_nNodeType & NODE_DIR) || S_ISDIR( pcFile->GetFileStat().st_mode ) == false ) {
					m_pcPathView->Set( pcFile->GetName().c_str() );
				}
			}
//            else {
//                m_pcPathView->Set( "" );
//            }
			break;
		}
		case ID_ALERT: // User has answered the "Are you sure..." requester
		{
			int32 nButton;
			if ( pcMessage->FindInt32( "which", &nButton ) != 0 )
			{
				dbprintf( "CFilePanel::MessageReceived() message from alert "
						"requester does not contain a 'which' element!\n" );
				break;
			}

			if ( nButton == 0 )
			{
				break;
			}
			/*** FALL THROUGH ***/
		}
		case ID_OK:
		case ID_INVOKED:
		{
			CMessage* pcMsg;
			if ( m_pcMessage != NULL )
			{
				pcMsg = new CMessage( *m_pcMessage );
			}
			else
			{
				pcMsg = new CMessage( (m_nMode == LOAD_REQ) ? M_LOAD_REQUESTED : M_SAVE_REQUESTED );
			}
			pcMsg->AddPointer( "source", this );

			if ( m_nMode == LOAD_REQ )
			{
				for ( int i = m_pcDirView->GetFirstSelected() ; i <= m_pcDirView->GetLastSelected() ; ++i ) {
					if ( m_pcDirView->IsSelected( i ) == false )
					{
						continue;
					}
					CPath cPath( m_pcDirView->GetPath().c_str() );
					cPath.Append( m_pcDirView->GetFile(i)->GetName().c_str() );
					pcMsg->AddString( "file/path", cPath.Path() );
				}
				m_pcTarget->SendMessage( pcMsg );
				Show( false );
			}
			else
			{
				CPath cPath( m_pcDirView->GetPath().c_str() );
				cPath.Append( m_pcPathView->GetBuffer()[0].String() );

				struct stat sStat;
				if ( pcMessage->what != ID_ALERT && stat( cPath.Path(), &sStat ) >= 0 ) {
					std::strstream cMsg;
					cMsg << "The file " << cPath.Path() << " already exists\n"
						<< "Do you want to overwrite it?\n";

					CAlert* pcAlert = new CAlert( "Alert:", cMsg.str(), 0, "No", "Yes", NULL );
					pcAlert->Go( new CInvoker( new CMessage( ID_ALERT ), this ) );
				}
				else
				{
					pcMsg->AddString( "file/path", cPath.Path() );
					m_pcTarget->SendMessage( pcMsg );
					Show( false );
				}
			}
			delete pcMsg;
			break;
		}

		case ID_CANCEL:
			Show( false );
			break;

		default:
			CWindow::MessageReceived( pcMessage );
			break;
	}
}

void CFilePanel::SetPath( const std::string& a_cPath )
{
	std::string cPath = a_cPath;
	std::string cFile;

	if ( cPath.find( '/' ) == std::string::npos )
	{
		const char* pzHome = getenv( "HOME" );
		if ( pzHome != NULL )
		{
			cPath = pzHome;
			if ( cPath[cPath.size()-1] != '/' )
			{
				cPath += '/';
			}
		}
		else
		{
			cPath = "/tmp/";
		}

		cFile = a_cPath;
	}
	if ( cFile.empty() )
	{
		if ( m_nNodeType == NODE_FILE )
		{
			struct stat sStat;
			if ( cPath[cPath.size()-1] != '/' || (stat( cPath.c_str(), &sStat ) < 0 || S_ISREG( sStat.st_mode )) ) {
				uint nSlash = cPath.rfind( '/' );
				if ( nSlash != std::string::npos )
				{
					cFile = cPath.c_str() + nSlash + 1;
					cPath.resize( nSlash );
				}
				else
				{
					cFile = cPath;
					cPath = "";
				}
			}
		}
	}

	m_pcPathView->Set( cFile.c_str(), false );
	m_pcDirView->SetPath( cPath );
}

std::string CFilePanel::GetPath() const
{
	return( m_pcDirView->GetPath() );
}
