/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <storage/Node.h>
#include <storage/Volume.h>


using namespace os;


bool CStatable::IsFile() const
{
	struct stat	aStat;
	return ( GetStat(&aStat) == EOK ? S_ISREG( aStat.st_mode ) : false );
}


bool CStatable::IsDirectory() const
{
	struct stat	aStat;
	return ( GetStat(&aStat) == EOK ? S_ISDIR( aStat.st_mode ) : false );
}


bool CStatable::IsSymLink() const
{
	struct stat	aStat;
	return ( GetStat(&aStat) == EOK ? S_ISLNK( aStat.st_mode ) : false );
}


status_t CStatable::GetNodeRef(NodeRef *ioNodeRef) const
{
	struct stat	aStat;
	status_t	err;

	err = GetStat(&aStat);
	if (err == EOK && ioNodeRef != NULL)
	{
		ioNodeRef->device = aStat.st_dev;
		ioNodeRef->node = aStat.st_ino;
	}

	return ( err );
}

#define STAT_WRAPPER(var,member) \
								 \
do{								 \
	status_t	err;			 \
	struct stat	aStat;			 \
	err = GetStat(&aStat);		 \
	if (!err && var) *var = aStat.##member; \
	return( err ); \
} while(0)


status_t CStatable::GetCreationTime( time_t* outTime ) const
{
	STAT_WRAPPER(outTime, st_ctime);
}


status_t CStatable::GetModificationTime( time_t* outTime ) const
{
	STAT_WRAPPER(outTime, st_mtime);
}


status_t CStatable::GetAccessTime( time_t* outTime ) const
{
	STAT_WRAPPER(outTime, st_atime);
}


status_t CStatable::GetSize(off_t *outSize) const
{
	STAT_WRAPPER(outSize, st_size);
}


status_t CStatable::GetGroup( gid_t* outGroup ) const
{
	STAT_WRAPPER(outGroup, st_gid);
}


status_t CStatable::GetOwner(uid_t *outOwner) const
{
	STAT_WRAPPER(outOwner, st_uid);
}


status_t CStatable::GetVolume(CVolume *ioVolume) const
{
	struct stat	aStat;
	status_t	err;

	err = GetStat(&aStat);
	if (err == EOK && ioVolume != NULL)
		ioVolume->SetTo(aStat.st_dev);

	return ( err );
}


ino_t CStatable::GetInode() const
{
	struct stat	aStat;
	return ( GetStat(&aStat) == EOK ? aStat.st_ino : -1 );
}
