/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <unistd.h>
#include <assert.h>
#include <limits.h>
#include <storage/SymLink.h>
#include <storage/Entry.h>
#include <storage/Path.h>

#include <util/exceptions.h>
using namespace os;


CSymLink::CSymLink()
{
}


CSymLink::CSymLink( const char* cPath, int nOpenMode ) : CNode( cPath, nOpenMode | O_NOTRAVERSE )
{
	if ( IsSymLink() == false )
	{
		throw errno_exception( "Node not a symlink", EINVAL );
	}
}


CSymLink::CSymLink( const CDirectory* cDir, const char* cName, int nOpenMode ) : CNode( cDir, cName, nOpenMode | O_NOTRAVERSE )
{
	if ( IsSymLink() == false )
	{
		throw errno_exception( "Node not a symlink", EINVAL );
	}
}


CSymLink::CSymLink( const CEntry* cRef, int nOpenMode ) : CNode( cRef, nOpenMode | O_NOTRAVERSE )
{
	if ( IsSymLink() == false )
	{
		throw errno_exception( "Node not a symlink", EINVAL );
	}
}

CSymLink::CSymLink( const CNode& cNode ) : CNode( cNode )
{
	if ( IsSymLink() == false )
	{
		throw errno_exception( "Node not a symlink", EINVAL );
	}
}


CSymLink::CSymLink( const CSymLink& cLink ) : CNode( cLink )
{
}


CSymLink::~CSymLink()
{
}


status_t CSymLink::SetTo( const char* cPath, int nOpenMode )
{
	try
	{
		return( SetTo( CNode( cPath, nOpenMode | O_NOTRAVERSE ) ) );
	}
	catch( errno_exception& cExc )
	{
		errno = cExc.error();
		return( -1 );
	}
}


status_t CSymLink::SetTo( const CDirectory* cDir, const char* cPath, int nOpenMode )
{
	try
	{
		return( SetTo( CNode( cDir, cPath, nOpenMode | O_NOTRAVERSE ) ) );
	}
	catch( errno_exception& cExc )
	{
		errno = cExc.error();
		return( -1 );
	}
}


status_t CSymLink::SetTo( const CEntry* cRef, int nOpenMode )
{
	try
	{
		return( SetTo( CNode( cRef, nOpenMode | O_NOTRAVERSE ) ) );
	} catch( errno_exception& cExc ) {
		errno = cExc.error();
		return( -1 );
	}
}


status_t CSymLink::SetTo( const CNode& cNode )
{
	if ( cNode.IsSymLink() == false )
	{
		errno = EINVAL;
		return( -1 );
	}
	return( CNode::SetTo( cNode ) );
}


status_t CSymLink::SetTo( const CSymLink& cLink )
{
	return( CNode::SetTo( cLink ) );
}


status_t CSymLink::ReadLink( std::string* pcBuffer )
{
	std::string cBuffer;

	cBuffer.resize( PATH_MAX );
	for (;;)
	{
		ssize_t nLen;
		
		// FIXME: freadlink unimplemented 
		//nLen = freadlink( GetFileDescriptor(), cBuffer.c_str(), cBuffer.size() );
		if ( nLen == ssize_t(cBuffer.size()) )
		{
			cBuffer.resize( cBuffer.size() * 2 );
		}
		else if ( nLen < 0 )
		{
			return( -1 );
		}
		else
		{
			cBuffer.resize( nLen );
			break;
		}
	}
	*pcBuffer = cBuffer;
	return( 0 );
}


status_t CSymLink::ConstructPath( const char* cParent, CPath* pcPath )
{
	std::string cBuffer;
	if ( ReadLink( &cBuffer ) < 0 )
	{
		return( -1 );
	}

	if ( cBuffer[0] == '/' )
	{
		*pcPath = cBuffer.c_str();
	}
	else
	{
		*pcPath = cParent;
		pcPath->Append( cBuffer.c_str() );
	}

	return( 0 );
}


status_t CSymLink::ConstructPath( const CDirectory* cParent, CPath* pcPath )
{
	std::string cParentPath;
	if ( cParent->GetPath( &cParentPath ) < 0 )
	{
		return( -1 );
	}

	return( ConstructPath( cParentPath.c_str(), pcPath ) );
}

