/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <stdio.h>
#include <unistd.h>
#include <storage/tempfile.h>

using namespace os;

class TempFile::Private
{
public:
    std::string m_cPath;
    bool        m_bDeleteFile;
};


TempFile::TempFile( const char* pzPrefix, const char* pzPath, int nAccess ) :
        CFile()
{
    m = new Private;
    m->m_bDeleteFile = true;
    while( true )
	{
        m->m_cPath = tempnam( pzPath, pzPrefix );
        if ( SetTo( m->m_cPath.c_str(), O_RDWR | O_CREAT | O_EXCL ) < 0 )
		{
            if ( errno == EEXIST ) {
                continue;
            } else {
                throw errno_exception( "Failed to create file" );
            }
        }
		else
		{
            break;
        }
    }
}

TempFile::~TempFile()
{
    if ( m->m_bDeleteFile )
	{
        unlink( m->m_cPath.c_str() );
    }
    delete m;
}

void TempFile::Detatch()
{
    m->m_bDeleteFile = false;
}

status_t TempFile::Unlink()
{
    if ( m->m_bDeleteFile )
	{
        unlink( m->m_cPath.c_str() );
        m->m_bDeleteFile = false;
    }
    return 0;
}

std::string TempFile::GetPath() const
{
    return( m->m_cPath );
}

