/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Copyright (C) 2001-2002 Bill Hayden
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <storage/Volume.h>
#include <support/SupportDefs.h>


using namespace os;


CVolume::CVolume()
{
	Unset();
}


CVolume::CVolume(dev_t inDevice)
{
	SetTo(inDevice);
}


CVolume::CVolume(const CVolume &inVolume)
{
	SetTo(inVolume.mDevice);
}


CVolume::CVolume(struct mntent* inMountEntry)
{
	char* deviceOptionsList;

	// Extract the device number

	deviceOptionsList = strstr(inMountEntry->mnt_opts, "dev=");

	if (deviceOptionsList)
	{
		int offset = 4;

		if (deviceOptionsList[5] == 'x' || deviceOptionsList[5] == 'X')
			offset += 2;

		mDevice = atoi(deviceOptionsList + offset);
	}
	else
		mDevice = (dev_t) -1;

	// Get the properties

	mPropertiesLoaded = true;

	mIsShared = false;	// FIXME
	mIsRemovable = false;	// FIXME
	mIsReadOnly = ( hasmntopt(inMountEntry, MNTOPT_RO) != NULL );
	mIsPersistent = true;	// FIXME
	mCapacity = 0L;	// FIXME
	mFreeBytes = 0L;	// FIXME
	mName = "Bocephus";	// FIXME
	mDevicePath.SetTo(inMountEntry->mnt_fsname);
	mMountPath.SetTo(inMountEntry->mnt_dir);

	mInitStatus = (mDevice == -1) ? -1 : 0;
}


CVolume::~CVolume()
{
}


status_t	CVolume::SetTo(dev_t inDevice)
{
	mDevice = inDevice;
	mInitStatus = (mDevice == -1) ? -1 : 0;
	mPropertiesLoaded = false;

	return mInitStatus;
}


void	CVolume::Unset(void)
{
	mDevice = -1;
	mInitStatus = -1;
	mPropertiesLoaded = false;
}


status_t	CVolume::GetRootDirectory(CDirectory *ioDirectory) const
{
	if (!mPropertiesLoaded)
		_LoadVolumeProperties();

	// FIXME: the following is not an implemented CDirectory function yet
	//ioDirectory.SetTo(mRootDirectory);

	return EOK;
}


off_t	CVolume::Capacity() const
{
	if (!mPropertiesLoaded)
		_LoadVolumeProperties();

	return mCapacity;
}


off_t	CVolume::FreeBytes() const
{
	if (!mPropertiesLoaded)
		_LoadVolumeProperties();

	return mFreeBytes;
}


status_t	CVolume::GetName(char *ioName) const
{
	if (!mPropertiesLoaded)
		_LoadVolumeProperties();

	if (mMountPath.Path() != NULL)
		strcpy(ioName, mMountPath.Path());
	else if (mDevicePath.Path() != NULL)
		strcpy(ioName, mDevicePath.Path());
	else
		return -1;

	return EOK;
}


status_t	CVolume::SetName(const char *name)
{
	mName = name;
	return EOK;
}

//status_t	CVolume::GetIcon(Bitmap *icon, icon_size which) const;

bool	CVolume::IsRemovable() const
{
	if (!mPropertiesLoaded)
		_LoadVolumeProperties();

	return mIsRemovable;
}


bool	CVolume::IsReadOnly() const
{
	if (!mPropertiesLoaded)
		_LoadVolumeProperties();

	return mIsReadOnly;
}


bool		CVolume::IsPersistent() const
{
	if (!mPropertiesLoaded)
		_LoadVolumeProperties();

	return mIsPersistent;
}


bool		CVolume::IsShared() const
{
	if (!mPropertiesLoaded)
		_LoadVolumeProperties();

	return mIsShared;
}


bool		CVolume::operator==(const CVolume &inVolume) const
{
	return (inVolume.Device() == mDevice);
}


bool		CVolume::operator!=(const CVolume &inVolume) const
{
	return (inVolume.Device() != mDevice);
}


CVolume&	CVolume::operator=(const CVolume &inVolume)
{
	mDevice = inVolume.mDevice;

	mPropertiesLoaded = inVolume.mPropertiesLoaded;
	mIsShared = inVolume.mIsShared;
	mIsRemovable = inVolume.mIsRemovable;
	mIsReadOnly = inVolume.mIsReadOnly;
	mIsPersistent = inVolume.mIsPersistent;
	mCapacity = inVolume.mCapacity;
	mFreeBytes = inVolume.mFreeBytes;

	mDevicePath = inVolume.mDevicePath;
	mMountPath = inVolume.mMountPath;

	return *this;
}


void		CVolume::_LoadVolumeProperties() const
{
	mPropertiesLoaded = true;
}

void CVolume::FBC_Volume1() {};
void CVolume::FBC_Volume2() {};
void CVolume::FBC_Volume3() {};
void CVolume::FBC_Volume4() {};
void CVolume::FBC_Volume5() {};
void CVolume::FBC_Volume6() {};
void CVolume::FBC_Volume7() {};
void CVolume::FBC_Volume8() {};
