/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Copyright (C) 2001-2002 Bill Hayden
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <support/List.h>
#include <algorithm>


using namespace os;



CList::CList(int32 ignored)
{
}


CList::CList(const CList& inList)
{
	mList = inList.mList;
}


CList::~CList()
{
}


CList&	CList::operator=(const CList &inList)
{
	mList = inList.mList;
	return *this;
}


bool	CList::AddItem(void* inItem)
{
	mList.push_back(inItem);

	return true;
}


bool	CList::AddItem(void *inItem, int32 inIndex)
{
	if (inIndex < 0L || inIndex >= (int32)mList.size())
		return false;

	mList.insert(mList.begin() + inIndex, inItem);

	return true;
}


bool	CList::AddList(CList *inNewItems)
{
	if (inNewItems == NULL)
		return false;

	mList.insert(mList.end(),
				 inNewItems->mList.begin(),
				 inNewItems->mList.end());

	return true;
}


bool	CList::AddList(CList *inNewItems, int32 inIndex)
{
	if (inNewItems == NULL || inIndex < 0L || inIndex >= (int32)mList.size())
		return false;

	mList.insert(mList.begin() + inIndex,
				 inNewItems->mList.begin(),
				 inNewItems->mList.end());

	return true;
}


bool	CList::RemoveItem(void *inItem)
{
	int32	index = IndexOf(inItem);

	if (index < 0L)
		return false;

	mList.erase(mList.begin() + index);

	return true;
}


void*	CList::RemoveItem(int32 inIndex)
{
	if (inIndex < 0L || inIndex >= (int32)mList.size())
		return NULL;

	void* itemHolder = mList[inIndex];

	mList.erase(mList.begin() + inIndex);

	return itemHolder;
}


bool	CList::RemoveItems(int32 inIndex, int32 inCount)
{
	if (inIndex < 0L || inCount <= 0L || inIndex + inCount >= (int32)mList.size())
		return false;

	mList.erase(mList.begin() + inIndex, mList.begin() + inIndex + inCount);

	return true;
}


bool	CList::ReplaceItem(int32 inIndex, void *inNewItem)
{
	if (inIndex < 0L || inIndex >= (int32)mList.size())
		return false;

	mList[inIndex] = inNewItem;

	return true;
}


void	CList::SortItems(int (*inCompareFunc)(const void *, const void *))
{
	std::sort(mList.begin(), mList.end(), inCompareFunc);
}


bool	CList::SwapItems(int32 inIndexA, int32 inIndexB)
{
	if (inIndexA < 0L || inIndexB < 0L)
		return false;

	if (inIndexA >= (int32)mList.size() || inIndexB >= (int32)mList.size())
		return false;

	void* itemA = mList[inIndexA];

	mList[inIndexA] = mList[inIndexB];
	mList[inIndexB] = itemA;

	return true;
}


bool	CList::MoveItem(int32 inFromIndex, int32 inToIndex)
{
	if (inFromIndex < 0L || inFromIndex >= (int32)mList.size())
		return false;

	if (inToIndex < 0L || inToIndex >= (int32)mList.size())
		return false;

	void* itemA = mList[inFromIndex];

	mList.erase(mList.begin() + inFromIndex);
	mList.insert(mList.begin() + inToIndex, itemA);

	return true;
}


void*	CList::ItemAt(int32 inIndex) const
{
	if (inIndex < 0L || inIndex >= (int32)mList.size())
		return NULL;

	return ItemAtFast(inIndex);
}


int32	CList::IndexOf(void *inItem) const
{
	uint32 index;

	for (index = 0UL; index < mList.size(); index++)
	{
		if (mList[index] == inItem)
			return index;
	}

    return -1L;
}


void	CList::DoForEach(bool (*inForEachFunc)(void *))
{
	uint32 index;

	for (index = 0UL; index < mList.size(); index++)
	{
		if (inForEachFunc(mList[index]))
			break;
	}
}


void	CList::DoForEach(bool (*inForEachFunc)(void *, void *), void *arg2)
{
	uint32 index;

	for (index = 0UL; index < mList.size(); index++)
	{
		if (inForEachFunc(mList[index], arg2))
			break;
	}
}


void	CList::FBC_CList1() {}
void	CList::FBC_CList2() {}

