/*  libcosmoe.so - the interface to the Cosmoe UI
 *  Portions Copyright (C) 2001-2002 Bill Hayden
 *  Portions Copyright (C) 1999-2001 Kurt Skauen
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2 as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA
 */


#include <support/StopWatch.h>
#include <cosmoe/OS.h>

#include <stdio.h>
#include <string.h>


using namespace os;


CStopWatch::CStopWatch(const char *name, bool silent)
{
	Reset();

	mSilent = silent;

	mName = new char[strlen(name)];
	strcpy(mName, name);
}


CStopWatch::~CStopWatch()
{
	if (!mSilent)
	{
		printf("StopWatch \"%s\": %lld microseconds\n", mName, ElapsedTime());

		if (mLap > 0L)
		{
			int32 i;

			for (i = 0L; i < mLap; i++)
			{
				printf("[Lap %d: %lld microseconds]\n", i, mLaps[i]);
			}
		}
	}

	delete[] mName;
}


void CStopWatch::Suspend()
{
	if (mSuspendTime == 0LL)
		mSuspendTime = get_real_time();
}


void CStopWatch::Resume()
{
	if (mSuspendTime != 0LL)
	{
		mStartTime += (get_real_time() - mSuspendTime);
		mSuspendTime = 0LL;
	}
}


bigtime_t CStopWatch::Lap()
{
	if (mLap < 9L)
	{
		mLaps[mLap] = ElapsedTime();
		return mLaps[mLap++];
	}

	return 0LL;
}


void CStopWatch::Reset()
{
	mStartTime = get_real_time();
	mSuspendTime = 0LL;
	mLap = 0L;
}


inline bigtime_t CStopWatch::ElapsedTime() const
{
	return get_real_time() - mStartTime;
}


void	CStopWatch::FBC_StopWatch1() {}
void	CStopWatch::FBC_StopWatch2() {}

