
#include <stdio.h>
#include <ctype.h>

#include <support/String.h>
#include <interface/Font.h>

using namespace os;


CString::CString()
{
}

CString::CString( int nLen, char nFiller ) : mString( nLen, nFiller )
{
}

CString::CString( const char* pzString ) : mString( pzString )
{
}

CString::CString( const char* pzString, int nLen ) : mString( pzString, nLen )
{
}

CString::CString( const std::string& cString ) : mString( cString )
{
}

CString::CString( const std::string& cString, int nPos, int nLen ) : mString( cString, nPos, nLen )
{
}

CString::CString( const CString& cString ) : mString( cString.mString )
{
}

CString::CString( const CString& cString, int nPos, int nLen ) : mString( cString.mString, nPos, nLen )
{
}

CString::CString( const_iterator cBegin, const_iterator cEnd ) : mString( cBegin, cEnd )
{
}


const char* CString::String() const
{
    return( mString.c_str() );
}

size_t CString::Length() const
{
    return( mString.size() );
}

size_t CString::CountChars() const
{
    int nNumChars = 0;
    for ( uint i = 0 ; i < mString.size() ; i += utf8_char_length( mString[i] ) ) {
        nNumChars++;
    }
    return( nNumChars );
}

CString& CString::CString::Resize( int nNewLen )
{
    mString.resize( nNewLen );
    return( *this );
}

    
CString& CString::Format( const char* pzFormat, va_list pArgs )
{
    int nMaxLen = 4096;
    char zBuffer[nMaxLen];
    int nLen = vsnprintf( zBuffer, nMaxLen, pzFormat, pArgs );

    if ( nLen < nMaxLen )
	{
        mString = zBuffer;
        return( *this );
    }

    while ( nLen >= nMaxLen )
	{
        nMaxLen *= 2;
        char* pzBuffer = new char[nMaxLen];
        nLen = vsnprintf( pzBuffer, nMaxLen, pzFormat, pArgs );
        if ( nLen < nMaxLen )
		{
            mString = pzBuffer;
        }
        delete[] pzBuffer;
    }
    return( *this );
}

CString& CString::Format( const char* pzFormat, ... )
{
    va_list sArgList;
    va_start( sArgList, pzFormat );
    Format( pzFormat, sArgList );
    va_end( sArgList );
    return( *this );
}

CString& CString::Strip()
{
    RStrip();
    LStrip();
    return( *this );
}

CString& CString::LStrip()
{
    std::string::iterator i;
    
    for ( i = mString.begin() ; i != mString.end() ; ++i )
	{
        if ( isspace( *i ) == false )
		{
            break;
        }
    }

    mString.erase( mString.begin(), i );
    return( *this );
}

CString& CString::RStrip()
{
    uint nSpaces = 0;
    for ( uint i = mString.size() - 1 ; i >= 0 ; --i )
	{
        if ( isspace( mString[i] ) == false )
		{
            break;
        }
        nSpaces++;
    }
    mString.resize( mString.size() - nSpaces );
    return( *this );
}

CString& CString::Lower()
{
    for ( uint i = 0 ; i < mString.size() ; ++i ) {
        mString[i] = tolower( mString[i] );
    }
    return( *this );
}

CString& CString::Upper()
{
    for ( uint i = 0 ; i < mString.size() ; ++i ) {
        mString[i] = toupper( mString[i] );
    }
    return( *this );
}

int CString::Compare( const char* pzString ) const
{
    return( strcmp( mString.c_str(), pzString ) );
}

int CString::Compare( const std::string& cOther ) const
{
    return( strcmp( mString.c_str(), cOther.c_str() ) );
}

int CString::Compare( const CString& cOther ) const
{
    return( strcmp( mString.c_str(), cOther.mString.c_str() ) );
}


int CString::CompareNoCase( const char* pzString ) const
{
    return( strcasecmp( mString.c_str(), pzString ) );
}

int CString::CompareNoCase( const std::string& cOther ) const
{
    return( strcasecmp( mString.c_str(), cOther.c_str() ) );
}

int CString::CompareNoCase( const CString& cOther ) const
{
    return( strcasecmp( mString.c_str(), cOther.mString.c_str() ) );
}


CString& CString::operator=( const char* pzString )
{
    mString = pzString;
    return( *this );
}

CString& CString::operator=( const std::string& cString )
{
    mString = cString;
    return( *this );
}

CString& CString::operator=( const CString& cString )
{
    mString = cString.mString;
    return( *this );
}

    
CString& CString::operator+=( const char* pzString )
{
    mString += pzString;
    return( *this );
}

CString& CString::operator+=( const std::string& cString )
{
    mString += cString;
    return( *this );
}

CString& CString::operator+=( const CString& cString )
{
    mString += cString.mString;
    return( *this );
}
    
CString CString::operator+( const char* pzString )
{
    CString cStr(*this);
    cStr += pzString;
    return( cStr );
}

CString CString::operator+( const std::string& cString )
{
    CString cStr(*this);
    cStr.mString += cString;
    return( cStr );
}

CString CString::operator+( const CString& cString )
{
    CString cStr(*this);
    cStr.mString += cString.mString;
    return( cStr );
}

bool CString::operator==( const char* pzString ) const
{
    return( mString == pzString );
}

bool CString::operator==( const std::string& cString ) const
{
    return( mString == cString );
}

bool CString::operator==( const CString& cString ) const
{
    return( mString == cString.mString );
}

bool CString::operator<( const char* pzString ) const
{
    return( mString < pzString );
}

bool CString::operator<( const std::string& cString ) const
{
    return( mString < cString );
}

bool CString::operator<( const CString& cString ) const
{
    return( mString < cString.mString );
}

bool CString::operator>( const char* pzString ) const
{
    return( mString > pzString );
}

bool CString::operator>( const std::string& cString ) const
{
    return( mString > cString );
}

bool CString::operator>( const CString& cString ) const
{
    return( mString > cString.mString );
}

char CString::operator[]( size_t nPos ) const
{
    return( mString[nPos] );
}

char& CString::operator[]( size_t nPos )
{
    return( mString[nPos] );
}
/*
String::operator std::string&()
{
    return( mString );
}
*/
CString::operator const std::string&() const
{
    return( mString );
}

std::string& CString::str()
{
    return( mString );
}



CString::iterator CString::begin()
{
    return( mString.begin() );
}


CString::iterator CString::end()
{
    return( mString.end() );
}

CString::const_iterator CString::begin() const
{
    return( mString.begin() );
}

CString::const_iterator CString::end() const
{
    return( mString.end() );
}

size_t CString::size() const
{
    return( mString.size() );
}

bool CString::empty() const
{
    return( mString.empty() );
}

void CString::resize( size_t nLen )
{
    mString.resize( nLen );
}

void CString::resize( size_t nLen, char nFiller )
{
    mString.resize( nLen, nFiller );
}

void CString::reserve( size_t nLen )
{
    mString.reserve( nLen );
}


CString& CString::erase( size_t nPos, size_t nLen )
{
    mString.erase( nPos, nLen );
    return( *this );
}

CString::iterator CString::erase( iterator i )
{
    return( mString.erase( i ) );
}

CString::iterator CString::erase( iterator cFirst, iterator cLast )
{
    return( mString.erase( cFirst, cLast ) );
}
